@@ -68,14 +68,12 @@ let setuid uid = ()
 let set_close_on_exec fd = ()
 let set_signal signal f = Sys.set_signal signal f
   
-external getdtablesize : unit -> int = "ml_getdtablesize"
-  
-let max_sockets = getdtablesize () - 10
 let max_filedescs = 50
   
 let chroot _ = ()
 
 external set_nonblock: Unix.file_descr -> unit = "ml_set_nonblock"
+external set_console_title : string -> unit = "os_set_console_title"
 
 let mini (x: int) y = if x < y then x else y
 
