@@ -1,15 +1,16 @@
-#include <caml/mlvalues.h>
-#include <caml/alloc.h>
-#include <caml/memory.h>
-#include <caml/custom.h>
-#include <caml/callback.h>
-#include <caml/fail.h>
+
 
 #include "../../utils/lib/os_stubs.h"
 
-#ifdef HAS_SIGNALS_H
-#include <signals.h>
-#endif
+
+
+/*******************************************************************
+
+
+                         os_read
+
+
+*******************************************************************/
 
 #define UNIX_BUFFER_SIZE 16384
 
@@ -33,6 +34,14 @@ extern ssize_t os_read(OS_FD fd, char *buf, size_t len)
   return numread;
 }
 
+/*******************************************************************
+
+
+                         os_ftruncate
+
+
+*******************************************************************/
+
 #include <winioctl.h>
 
 void os_ftruncate(OS_FD fd, OFF_T size, /* bool */ int sparse)
@@ -62,11 +71,27 @@ void os_ftruncate(OS_FD fd, OFF_T size, /* bool */ int sparse)
     }
 }
 
+/*******************************************************************
+
+
+                         os_getdtablesize
+
+
+*******************************************************************/
+
 int os_getdtablesize()
 {
   return 32767;
 }
 
+/*******************************************************************
+
+
+                         os_getfdsize
+                         
+
+*******************************************************************/
+
 int64 os_getfdsize(OS_FD fd)
 {
   long len_high;
@@ -76,6 +101,14 @@ int64 os_getfdsize(OS_FD fd)
   return ((int64) len_high << 32 | ret);
 }
 
+/*******************************************************************
+
+
+                         os_getfilesize
+
+
+*******************************************************************/
+
 int64 os_getfilesize(char *path)
 {
   OS_FD fd = CreateFile(path, GENERIC_READ, FILE_SHARE_READ,
@@ -97,6 +130,14 @@ int64 os_getfilesize(char *path)
   }
 }
 
+/*******************************************************************
+
+
+                         os_lseek
+
+
+*******************************************************************/
+
 OFF_T os_lseek(OS_FD fd, OFF_T ofs, int cmd)
 {
   DWORD ret;
@@ -115,6 +156,14 @@ OFF_T os_lseek(OS_FD fd, OFF_T ofs, int cmd)
   return ((OFF_T) ofs_high << 32 | ret);
 }
 
+/*******************************************************************
+
+
+                         os_set_nonblock
+
+
+*******************************************************************/
+
 #include <winsock2.h>
 
 void os_set_nonblock(OS_SOCKET fd)
@@ -130,28 +179,28 @@ void os_set_nonblock(OS_SOCKET fd)
   }
 }
 
+/*******************************************************************
 
-//http://lists.gnu.org/archive/html/bug-gnu-chess/2004-01/msg00020.html
-void gettimeofday(struct timeval* p, void* tz /* IGNORED */){
-   union {
-     long long ns100; /*time since 1 Jan 1601 in 100ns units */
-     FILETIME ft;
-   } _now;
 
-   GetSystemTimeAsFileTime( &(_now.ft) );
-   p->tv_usec=(long)((_now.ns100 / 10LL) % 1000000LL );
-   p->tv_sec= (long)((_now.ns100-(116444736000000000LL))/10000000LL);
-   return;
-}
+                         os_uname
 
 
+*******************************************************************/
+
 // http://msdn.microsoft.com/library/default.asp?url=/library/en-us/sysinfo/base/getting_the_system_version.asp
 #define BUFSIZE 80
+#define SM_SERVERR2 89
+typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
+
 
 void os_uname(char buf[])
 {
    OSVERSIONINFOEX osvi;
    BOOL bOsVersionInfoEx;
+   SYSTEM_INFO si;
+   PGNSI pGNSI;
+   char tbuf[4096];
+   char pbuf[32];
    char binull='\0';
 
    // Try calling GetVersionEx using the OSVERSIONINFOEX structure.
@@ -173,8 +222,28 @@ void os_uname(char buf[])
       case VER_PLATFORM_WIN32_NT:
 
       // Test for the specific product.
+      if ( osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 0 )
+      {
+	     if( osvi.wProductType == VER_NT_WORKSTATION )
+              strcat (buf, "Microsoft Windows Vista \0");
+         else strcat (buf, "Windows Server \"Longhorn\" \0" );
+      }    
+
       if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )
-         strcat(buf, "Microsoft Windows Server 2003, \0");
+      {
+         // Use GetProcAddress to avoid load issues on Windows 2000
+         pGNSI = (PGNSI) GetProcAddress (GetModuleHandle("kernel32.dll"), "GetNativeSystemInfo");
+         if(NULL != pGNSI)
+             pGNSI(&si);
+
+         if( GetSystemMetrics(SM_SERVERR2) )
+            strcat (buf, "Microsoft Windows Server 2003 \"R2\" \0");
+         else if( osvi.wProductType == VER_NT_WORKSTATION && si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
+         {
+            strcat (buf, "Microsoft Windows XP Professional x64 Edition \0");
+         }
+         else strcat (buf, "Microsoft Windows Server 2003, \0");
+      }
 
       if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )
          strcat(buf, "Microsoft Windows XP \0");
@@ -189,7 +258,7 @@ void os_uname(char buf[])
       if( bOsVersionInfoEx )
       {
          // Test for the workstation type.
-         if ( osvi.wProductType == VER_NT_WORKSTATION )
+         if ( osvi.wProductType == VER_NT_WORKSTATION  &&  si.wProcessorArchitecture != PROCESSOR_ARCHITECTURE_AMD64 ) 
          {
             if( osvi.dwMajorVersion == 4 )
                strcat(buf, "Workstation 4.0 \0" );
@@ -202,17 +271,38 @@ void os_uname(char buf[])
          else if ( osvi.wProductType == VER_NT_SERVER || 
                    osvi.wProductType == VER_NT_DOMAIN_CONTROLLER )
          {
-            if(osvi.dwMajorVersion==5 && osvi.dwMinorVersion==2)
+            if(osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2)
             {
-               if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
-                  strcat(buf, "Datacenter Edition \0" );
-               else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
-                  strcat(buf, "Enterprise Edition \0" );
-               else if ( osvi.wSuiteMask == VER_SUITE_BLADE )
-                  strcat(buf, "Web Edition \0" );
-               else strcat(buf, "Standard Edition \0" );
-            }
-            else if(osvi.dwMajorVersion==5 && osvi.dwMinorVersion==0)
+               if ( si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64 )
+               {
+                   if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
+                      strcat (buf, "Datacenter Edition for Itanium-based Systems \0" );
+                   else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
+                      strcat (buf, "Enterprise Edition for Itanium-based Systems \0" );
+               }
+
+               else if ( si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64 )
+               {
+                   if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
+                      strcat (buf, "Datacenter x64 Edition \0" );
+                   else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
+                      strcat (buf, "Enterprise x64 Edition \0" );
+                   else strcat (buf, "Standard x64 Edition \0" );
+               }
+
+               else
+               {
+                   if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
+                      strcat (buf, "Datacenter Edition \0" );
+                   else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
+                      strcat (buf, "Enterprise Edition \0" );
+                   else if ( osvi.wSuiteMask == VER_SUITE_BLADE )
+                      strcat (buf, "Web Edition \0" );
+                   else strcat (buf, "Standard Edition \0" );
+               }
+            }            
+
+            else if(osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0)
             {
                if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                   strcat(buf, "Datacenter Server \0" );
@@ -255,11 +345,20 @@ void os_uname(char buf[])
             strcat(buf, "Server \0" );
          if ( lstrcmpi( "SERVERNT", szProductType) == 0 )
             strcat(buf, "Advanced Server \0" );
-         printf( "%d.%d", (int)osvi.dwMajorVersion, (int)osvi.dwMinorVersion );
+         sprintf(tbuf, "%d.%d %c", (int)osvi.dwMajorVersion, (int)osvi.dwMinorVersion, binull );
       }
 
-      // Display service pack (if any) and build number.
-			char tbuf[4096];
+      // Display service pack (if any), build number and processor count (if it`s > 1).
+
+   	  GetSystemInfo(&si);
+	  
+      if( si.dwNumberOfProcessors > 1)
+         
+          sprintf(pbuf, "(CPUs: %lu)%c", si.dwNumberOfProcessors, binull);
+      else
+          pbuf[0] = binull;
+                 
+
       if( osvi.dwMajorVersion == 4 && lstrcmpi( osvi.szCSDVersion, "Service Pack 6" ) == 0 )
       { 
          HKEY hKey;
@@ -271,16 +370,16 @@ void os_uname(char buf[])
             0, KEY_QUERY_VALUE, &hKey );
          if( lRet == ERROR_SUCCESS )
 								 
-            sprintf(tbuf, "Service Pack 6a (Build %lu)%c", osvi.dwBuildNumber & 0xFFFF, binull );
+            sprintf(tbuf, "Service Pack 6a (Build %lu) %s", osvi.dwBuildNumber & 0xFFFF, pbuf );
          else // Windows NT 4.0 prior to SP6a
          {
-            sprintf(tbuf, "%s (Build %lu)%c",osvi.szCSDVersion, osvi.dwBuildNumber & 0xFFFF, binull);
+            sprintf(tbuf, "%s (Build %lu) %s",osvi.szCSDVersion, osvi.dwBuildNumber & 0xFFFF, pbuf);
          }
          RegCloseKey( hKey );
       }
       else // not Windows NT 4.0 
       {
-         sprintf(tbuf, "%s (Build %lu)%c", osvi.szCSDVersion, osvi.dwBuildNumber & 0xFFFF, binull);
+         sprintf(tbuf, "%s (Build %lu) %s", osvi.szCSDVersion, osvi.dwBuildNumber & 0xFFFF, pbuf);
       }
 
       strcat(buf, tbuf);
@@ -292,7 +391,7 @@ void os_uname(char buf[])
       if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
       {
           strcat (buf, "Microsoft Windows 95 ");
-          if (osvi.szCSDVersion[1]=='C' || osvi.szCSDVersion[1]=='B')
+          if (osvi.szCSDVersion[1] == 'C' || osvi.szCSDVersion[1] == 'B')
              strcat(buf, "OSR2 \0" );
       } 
 
@@ -317,3 +416,45 @@ void os_uname(char buf[])
    return; 
 }
 
+/*******************************************************************
+
+
+                          os_os_supported
+
+
+*******************************************************************/
+
+int os_os_supported()
+{
+   OSVERSIONINFO pVersion;
+   pVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+   int check = GetVersionEx(&pVersion);
+
+   if ( check != 0 )
+   {
+   if ( pVersion.dwPlatformId == VER_PLATFORM_WIN32_NT )
+       return 1;
+     else
+       return 0;
+   }
+   else  /* expect an NT system because GetVersionEx failed */
+   {
+     return 1;
+   }
+}
+
+/*******************************************************************
+
+
+                      os_set_console_title
+
+
+ *******************************************************************/
+
+value os_set_console_title(value buf)
+{
+   char *pbuf = String_val (buf);
+
+   SetConsoleTitle((LPCTSTR)pbuf);
+   return Val_unit;
+}
