@@ -24,6 +24,11 @@ open Options
 open CommonTypes
 open CommonGlobals
 
+let log_prefix = "[cCl]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
 type 'a client_impl = {
     mutable impl_client_type : client_type;
     mutable impl_client_state : host_state;
@@ -124,10 +129,6 @@ let client_to_option (client : client) =
   let client = as_client_impl client in
   client.impl_client_ops.op_client_to_option client.impl_client_val
 
-let client_network (client : client) =
-  let client = as_client_impl client in
-  client.impl_client_ops.op_client_network
-
 let client_info (client : client) =
   let client = as_client_impl client in
   client.impl_client_ops.op_client_info client.impl_client_val
@@ -173,6 +174,8 @@ let client_clear_files client=
   client.impl_client_ops.op_client_clear_files client.impl_client_val
 
 let client_browse client immediate =
+  if CommonNetwork.network_is_enabled
+     ((as_client_impl client).impl_client_ops.op_client_network) then
   let client = as_client_impl client in
   client.impl_client_ops.op_client_browse client.impl_client_val immediate
 
@@ -286,18 +289,27 @@ let is_friend c =
 let is_contact c =
   (client_type c) land client_contact_tag <> 0
 
+let is_must_browse c =
+  (client_type c) land client_must_browse_tag <> 0
+
 let set_friend c =
   set_client_type c (client_type c lor client_friend_tag)
 
 let set_contact c =
   set_client_type c (client_type c lor client_contact_tag)
 
+let set_must_browse c =
+  set_client_type c (client_type c lor client_must_browse_tag)
+
 let set_not_friend c =
   set_client_type c (client_type c land (lnot client_friend_tag))
 
 let set_not_contact c =
   set_client_type c (client_type c land (lnot client_contact_tag))
 
+let set_not_must_browse c =
+  set_client_type c (client_type c land (lnot client_must_browse_tag))
+
 let is_nolimit c =
   (client_type c) land client_nolimit_tag <> 0
 
@@ -381,6 +393,10 @@ let client_remove c =
   H.remove clients_by_num c;
   set_client_state c RemovedHost
 
+let disconnect_all_clients () =
+  H.iter (fun c -> client_disconnect c) clients_by_num;
+  Intmap.iter (fun _ c -> client_disconnect c) !uploaders
+
 let _ =
   Heap.add_memstat "CommonClient" (fun level buf ->
       let counter = ref 0 in
@@ -407,43 +423,40 @@ module G = GuiTypes
 
 let client_print_html c o =
   let impl = as_client_impl c in
-  let i = client_info c in
   let n = impl.impl_client_ops.op_client_network in
   let info = client_info c in
   let buf = o.conn_buf in
 	html_mods_td buf [
 	("", "sr", n.network_name);
 	("", "sr", (match info.G.client_kind with
-          Indirect_location (name, _) -> Printf.sprintf "I"
+          Indirect_location (_ , _, _, _) -> Printf.sprintf "I"
         | Known_location (ip, port) -> Printf.sprintf "D") );
 	(String.escaped info.G.client_name, "sr", client_short_name info.G.client_name); ]
 
 let client_print c o =
   let impl = as_client_impl c in
-  let i = client_info c in
   let n = impl.impl_client_ops.op_client_network in
   let info = client_info c in
   let buf = o.conn_buf in
+  let ips,cc,cn = string_of_kind_geo info.G.client_kind in
   if use_html_mods o then begin
-	html_mods_td buf [
+	html_mods_td buf ([
 	("", "sr", Printf.sprintf "%d" (client_num c));
 	("", "sr", n.network_name);
-	("", "sr", (try match info.G.client_kind with
-             Known_location (ip,port) -> Printf.sprintf "%s" (Ip.to_string ip)
-           | Indirect_location _ -> Printf.sprintf "None"
-           with _ -> ""));
-	(String.escaped info.G.client_name, "sr", client_short_name info.G.client_name); ];
+	("", "sr", ips);
+	] @ (if !Geoip.active then [(cn, "sr", cc)] else []) @ [
+	(String.escaped info.G.client_name, "sr", client_short_name info.G.client_name); ]);
     end
   else begin
       Printf.bprintf buf "[%s%6d] Name  : %-27s Rating  : %-10d  IP   : %-20s"
         n.network_name
         (client_num c)
         (shorten info.G.client_name 20)
-        i.GuiTypes.client_rating
+        info.GuiTypes.client_rating
         (match info.G.client_kind with
-            Indirect_location (name, _) -> "firewalled"
+              Indirect_location (_, _, ip, port)
             | Known_location (ip, port) ->
-            Printf.sprintf "%s:%d" (Ip.to_string ip) port)
+                Printf.sprintf "%s:%d" (Ip.to_string ip) port)
     end
 
 
@@ -458,12 +471,37 @@ let clear_upload_slots () =
       if i.GuiTypes.client_uploaded = Int64.zero && ctime > 1 then
         begin
 	  client_disconnect c;
-          if !verbose then lprintf_nl "disconnected client %d: [%s %s] %s after %d minute of silence."
+          if !verbose then lprintf_nl "disconnected client %d: [%s %s] %s after %d %s of silence."
 	    (client_num c)
 	    i.GuiTypes.client_software
 	    i.GuiTypes.client_release
 	    i.GuiTypes.client_name
 	    ctime
+	    (if ctime = 1 then "minute" else "minutes")
 	end
     with _ -> ()
   ) !uploaders
+
+let impl_client_info impl =
+  let module T = GuiTypes in
+  {
+     T.client_num = impl.impl_client_num;
+     T.client_state = impl.impl_client_state;
+     T.client_type = impl.impl_client_type;
+  
+     T.client_tags = [];
+     T.client_name = "";
+     T.client_network = 0;
+     T.client_files = None;
+     T.client_rating = 0;
+     T.client_chat_port = 0;
+     T.client_connect_time = BasicSocket.last_time ();
+     T.client_kind = Indirect_location ("", Md4.Md4.null, Ip.null, 0);
+     T.client_software = "";
+     T.client_release = "";
+     T.client_emulemod = "";
+     T.client_downloaded = 0L;
+     T.client_uploaded = 0L;
+     T.client_upload = None;
+     T.client_sui_verified = None;
+  }
