@@ -29,16 +29,18 @@ open CommonNetwork
 open CommonOptions
 open CommonTypes
 open CommonFile
+open Gettext
 
-(* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[cCO] "
-    (log_time ()); lprintf_nl2
-      
-(* prints a new logline with date, module and does not start newline *)
-let lprintf_n () =
-  lprintf "%s[cCO] "
-    (log_time ()); lprintf
+let _s x = _s "CommonComplexOptions" x
+let _b x = _b "CommonComplexOptions" x
+
+let log_prefix = "[cCO]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
 
 (*************************************************************************)
 (*                                                                       *)
@@ -66,16 +68,18 @@ module FileOption = struct
       match v with
         Options.Module assocs ->
           let get_value name conv = conv (List.assoc name assocs) in
+	  let get_value_nil name conv =
+	    try conv (List.assoc name assocs) with Not_found -> [] in
           let network = try get_value "file_network" value_to_string
             with _ -> "Donkey" in
           let network = 
             try network_find_by_name network with e ->
-                lprintf_nl ()
+                lprintf_nl
 		  "Error %s for network %s while parsing file %s"
 		    (Printexc2.to_string e)
 		    network
 		    (get_value "file_filename" value_to_string);
-		lprintf_nl () "This core is lacking support for network %s, exiting" network;
+    lprintf_nl "This core is lacking support for network %s, exiting" network;
                 exit_properly 70
           in
           let file_state = 
@@ -99,18 +103,26 @@ module FileOption = struct
                 normalize_time (get_value "file_age" value_to_int)
             with _ -> ());
           set_file_state file file_state;       
-          if !verbose then
-          (match file_state with
-              FileDownloading -> lprintf_nl () "New downloading file";
-            | FileDownloaded -> lprintf_nl () "New downloaded file";
-            | _ -> lprintf_nl () "New file with other state"
-          );
-          
+
           (try
               set_file_best_name file
-              (get_value "file_filename" value_to_string)
+              (get_value "file_filename" value_to_string) "" 0
             with _ -> ());
+
+          (try
+	    List.iter (fun s -> add_file_filenames file s)
+	      (get_value_nil "file_filenames" (value_to_list value_to_string))
+            with _ -> ());
+
           set_file_priority file priority;
+
+          if !verbose then lprintf_nl "New %s file %s"
+	      (match file_state with
+                 FileDownloading -> "downloading"
+               | FileDownloaded -> "downloaded"
+               | _ -> "other")
+	     (file_best_name file);
+
           file
       | _ -> assert false
           
@@ -123,6 +135,8 @@ module FileOption = struct
         ("file_priority", int_to_value (file_priority file)) ::
         ("file_state", state_to_value (file_state file)) ::
         ("file_filename", string_to_value (file_best_name file)) ::
+	("file_filenames", List
+	(List.map string_to_value impl.impl_file_filenames)) ::
         ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
           (file_to_option file)
         )
@@ -143,7 +157,7 @@ let done_files =
   
 let files = 
   define_option files_section ["files"] 
-    "The files currently being downloaded" (
+    "The files currently being downloaded, primary downloads must come first" (
     listiter_option (FileOption.t false)) []
 
 (*************************************************************************)
@@ -266,7 +280,7 @@ module ServerOption = struct
           in
           let network = 
             try network_find_by_name network with e ->
-                lprintf_nl () "Network %s not supported" network;
+                lprintf_nl "Network %s not supported" network;
                 raise e
               in
           let server = network_server_of_option network assocs in
@@ -930,6 +944,7 @@ shared_directories *)
 
 let load () =
   Options.load files_ini;
+  shorten_all_file_filenames !!max_filenames;
   Options.load servers_ini;
   Options.load searches_ini;
   Options.load results_ini;
@@ -940,7 +955,10 @@ let load () =
   known_uids =:= [];
   Options.load friends_ini
 
+let allow_saving_ini_files = ref true
+
 let save () =
+  if !allow_saving_ini_files then begin
   networks_iter (fun n -> network_save_complex_options n);
 
   Options.save_with_help files_ini;
@@ -962,94 +980,81 @@ let save () =
         Options.save_with_help results_ini;
         results =:= [];
     end;
-  lprintf_nl () "Options correctly saved"
+  lprintf_nl (_b "Options correctly saved")
+  end
 
 let save_sources () =
+  if !allow_saving_ini_files then begin
   networks_iter (fun n -> network_save_sources n);
-  lprintf_nl () "Sources correctly saved"
+  lprintf_nl (_b "Sources correctly saved")
+  end
 
 open Zip
 
 let backup_zip archive files =
-  begin
-    let oc = Zip.open_out archive in
-    try
-        List.iter (fun file ->
-	  begin
-	    try
-              let s = Unix.stat file in
-		Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.Unix.st_mtime file
-	    with e ->
-	      failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) file)
-	  end
-        ) files;
-      Zip.close_out oc
-    with e ->
-	Zip.close_out oc;
-	failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) archive)
-  end
+  try 
+    Unix2.tryopen_write_zip archive (fun oc ->
+      List.iter (fun file ->
+	try
+          let s = Unix.stat file in
+	  Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.Unix.st_mtime file
+	with e ->
+	  failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) file)
+      ) files)
+  with e ->
+    failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) archive)
 
 open Tar
 
 let backup_tar archive files =
-  let failed_files = ref "" in
-  let otar = Tar.open_out ~compress:`Gzip archive in
+  let failed_files = ref [] in
+  Unix2.tryopen_write_tar ~compress:`Gzip archive (fun otar ->
     List.iter (fun arg ->
-    begin
       try
-	let ic = Pervasives.open_in_bin arg in
-	let stat = Unix.stat arg in
-	let size = stat.Unix.st_size in
-	if size > Sys.max_string_length then
-	  begin
-	    Tar.close_out otar;
-	    failwith (Printf.sprintf "Tar: file %s too big" arg)
-	  end;
-	let header = 
-	  { Tar.t_name = arg;
-	    t_mode = 0o644;
-            t_uid = stat.Unix.st_uid;
-            t_gid = stat.Unix.st_gid;
-            t_size = 0;
-            t_mtime = Int32.of_float stat.Unix.st_mtime;
-            t_chksum = 0;
-            t_typeflag = REGULAR;
-            t_linkname = "";
-	    t_format = POSIX_FORMAT;
-            t_uname = "";
-            t_gname = "";
-            t_devmajor = 0;
-            t_devminor = 0;
-            t_prefix = "";
-            t_gnu = None;}
-	in
-	let s = String.create size in
-	  Pervasives.really_input ic s 0 size;
-	  Pervasives.close_in ic;
-	  Tar.output otar header s
-	with e ->
-	  let error = (Printexc2.to_string e) in
-	    if error =
-	      "Gzip.Error(\"error during compression\")"
-	      && Autoconf.windows && arg = "fasttrack.ini" then
-              (* for whatever reason this error is raised on Windows,
-                 but fasttrack.ini is stored correctly *)
-	      if !verbose_hidden_errors then
-	        lprintf_nl () "Tar: Windows specific pseudo error %s in %s" error arg
-	      else ()
-	    else
-	      begin
-		if !failed_files = "" then
-	          failed_files := arg
-		else
-    		  failed_files := Printf.sprintf "%s  %s" !failed_files arg;
-		  lprintf_nl () "Tar: error %s in %s" error arg
-              end
-    end
-    ) files;
-    Tar.close_out otar;
-    if !failed_files <> "" then
-      failwith (Printf.sprintf "Tar: error backing up %s" !failed_files)
+	let header, s =
+	  Unix2.tryopen_read_bin arg (fun ic ->
+	    let stat = Unix.stat arg in
+	    let size = stat.Unix.st_size in
+	    if size > Sys.max_string_length then
+	      failwith (Printf.sprintf "Tar: file %s too big, limit %d byte" arg Sys.max_string_length);
+	    let header = 
+	      { Tar.t_name = arg;
+	      t_mode = 0o644;
+	      t_uid = stat.Unix.st_uid;
+	      t_gid = stat.Unix.st_gid;
+	      t_size = 0;
+	      t_mtime = Int32.of_float stat.Unix.st_mtime;
+	      t_chksum = 0;
+	      t_typeflag = REGULAR;
+	      t_linkname = "";
+	      t_format = POSIX_FORMAT;
+	      t_uname = "";
+	      t_gname = "";
+	      t_devmajor = 0;
+	      t_devminor = 0;
+	      t_prefix = "";
+	      t_gnu = None;} in
+	    let s = String.create size in
+	    Pervasives.really_input ic s 0 size;
+	    header, s) in
+	Tar.output otar header s
+      with e ->
+	let error = Printexc2.to_string e in
+	if error = "Gzip.Error(\"error during compression\")"
+	  && Autoconf.windows && arg = "fasttrack.ini" then begin
+	    (* for whatever reason this error is raised on Windows,
+               but fasttrack.ini is stored correctly *)
+	    if !verbose then
+        lprintf_nl "Tar: Windows specific pseudo error %s in %s" error arg
+	  end
+	else begin
+	  failed_files := arg :: !failed_files;
+    lprintf_nl "Tar: error %s in %s" error arg
+        end
+    ) files);
+    if !failed_files <> [] then
+      failwith (Printf.sprintf "Tar: error backing up %s"
+	(String.concat " " (List.rev !failed_files)))
 
 let backup_options () =
   let counter = ref 1 in
@@ -1085,90 +1090,94 @@ let backup_options () =
           ".zip" -> backup_zip archive files
         | _ -> backup_tar archive files
       end
-    with e -> lprintf_nl () "Exception %s while options backup" (Printexc2.to_string e); raise e
+    with e -> lprintf_nl "Exception %s while options backup" (Printexc2.to_string e); raise e
   end;
-  lprintf_nl () "Options backup as %s correctly saved" format
+  lprintf_nl (_b "Options backup as %s correctly saved") format
              
-let buildinfo () =
-  (
-        "MLNet Multi-Network p2p client version " ^ Autoconf.current_version
-      ^ (if Autoconf.scm_version <> "" then "\nSCM version info: " ^ Autoconf.scm_version else "")
-      ^ "\nNetworks: " ^ !networks_string
-      ^ "\nOcaml version: " ^ Sys.ocaml_version
-      ^ "\nBuild on: " ^ Autoconf.build_system ^ " (" ^ Unix2.endianness () ^ ")"
-      ^ (if Autoconf.glibc_version = "" then "" 
-          else
-            let real_glibc_version = MlUnix.glibc_version_num () in
-            if real_glibc_version = "" || real_glibc_version = Autoconf.glibc_version 
-              then " with glibc " ^ Autoconf.glibc_version
-            else 
-              Printf.sprintf " (Warning: glibc version mismatch, %s present on your system, MlDonkey was compiled with %s)"
-              real_glibc_version Autoconf.glibc_version)  
-      ^ (if Autoconf.configure_arguments <> "" then
-           "\nConfigure arguments: " ^ Autoconf.configure_arguments else "")
-      ^ (if !patches_string <> "" then "\n" ^ !patches_string else "")
-      ^ "\nFeatures:"
-          ^ (if BasicSocket.has_threads () then " threads" else " no-threads")
-          ^ (let s = Zlib.zlib_version_num () in Printf.sprintf " zlib%s" (if s <> "" then "-" ^ s else ""))
-          ^ (if Autoconf.has_bzip2 then
-	       begin
-	         let s =
-	           (* catch the exception in the case Bzip2.bzlib_version_num returns an empty string *)
-	           try
-	             List.hd(String2.split_simplify
-		   (Misc2.bzlib_version_num ()) ',') 
-		   with e -> ""
-		 in
-	         Printf.sprintf " bzip2%s" (if s <> "" then "-" ^ s else "")
-	       end
-	     else " no-bzip2")
-          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && Autoconf.has_gd_jpg then
-	       Printf.sprintf " gd(jpg/png%s)"
-	         (let s = CommonGraphics.png_version_num () in
-		    if s <> "" then "-" ^ s else "")
-	     else "")
-          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && not Autoconf.has_gd_jpg then
-	       Printf.sprintf " gd(png%s)"
-	         (let s = CommonGraphics.png_version_num () in
-		    if s <> "" then "-" ^ s else "")
-	     else "")
-	  ^ (if Autoconf.has_gd && not Autoconf.has_gd_png && Autoconf.has_gd_jpg then " gd(jpg)" else "")
-	  ^ (if not Autoconf.has_gd then " no-gd" else "")
-          ^ (if Autoconf.has_iconv then " iconv" else " no-iconv")
-          ^ (if Autoconf.check_bounds then " check-bounds" else " no-check-bounds")
+let _ =
+  CommonBlocking.add_update_hook CommonServer.check_blocked_servers;
+  CommonBlocking.add_update_hook CommonServer.server_must_update_all;
+  option_hook ip_blocking (fun _ ->
+    try
+      CommonBlocking.set_ip_blocking_list !!ip_blocking
+    with _ -> ()
+  );
+  option_hook ip_blocking_countries (fun _ ->
+    CommonBlocking.set_ip_blocking_countries !!ip_blocking_countries
+  );
+  option_hook ip_blocking_countries_block (fun _ ->
+    CommonBlocking.set_ip_blocking_countries_block !!ip_blocking_countries_block;
+    CommonBlocking.set_ip_blocking_countries !!ip_blocking_countries
+  );
+  option_hook geoip_dat (fun _ ->
+    try
+      CommonBlocking.set_geoip_dat !!geoip_dat
+    with _ -> ()
+  );
+  option_hook max_filenames (fun _ ->
+    shorten_all_file_filenames !!max_filenames
+  );
+  option_hook max_opened_connections (fun _ ->
+  if !verbose then lprintf_nl
+    "checking max_opened_connections = %d for validity" !!max_opened_connections;
+(* original code from ./src/config/unix/MlUnix.ml
+let max_all_sockets = getdtablesize ()
+let max_sockets = max (max_all_sockets - 100) (max_all_sockets / 2)
+let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
+
+  (* ulimit open files. minimum 150, most systems have 1024 *)
+  let max_all_sockets = Unix2.c_getdtablesize () in
+
+  (* old max_sockets code: max (150 - 100) (150 / 2),
+     minimum number of max_opened_connections *)
+  let min_conns = 75 in
+
+  if min_conns > !!max_opened_connections then begin
+    lprintf_nl "max_opened_connections is set too low (%d), raising to %d"
+      !!max_opened_connections min_conns;
+    max_opened_connections =:= min_conns
+  end;
+
+  let reserved_fds = 40 in (* ini files, dynamic libs, etc. *)
+
+  let total_files = (* maximum number of files in use at the same time *)
+    (maxi (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + reserved_fds
+  in
+
+  let wanted_socks = !!max_opened_connections + total_files in
+
+  if max_all_sockets < wanted_socks then
+    if max_all_sockets < total_files + min_conns then (* check if ulimit is enough to allow total_files + min_conns *)
+      begin
+        lprintf_nl "only %d file descriptors available, raise ulimit open files to at least %d"
+          max_all_sockets wanted_socks;
+        lprintf_nl "FD info: max_opened_connections %d, number of (possible) concurrent downloads %d, = %d fd needed"
+          !!max_opened_connections total_files wanted_socks;
+        CommonGlobals.exit_properly 71
+      end
+  else
+    begin
+      let new_max_opened_connections =
+        maxi (max_all_sockets - total_files) (max_all_sockets / 2)
+      in
+      lprintf_nl "max_opened_connections is set too high (%d), reducing to %d"
+        !!max_opened_connections new_max_opened_connections;
+      max_opened_connections =:= new_max_opened_connections;
+    end;
+
+  if !verbose then lprintf_nl
+    "max_opened_connections %d, total_files %d, max_concurrent_downloads %d, !!files %d"
+      !!max_opened_connections total_files !!max_concurrent_downloads (List.length !!files);
+
+  TcpBufferedSocket.set_max_opened_connections
+    (fun _ -> !!max_opened_connections);
+
+  Unix32.max_cache_size := total_files - reserved_fds;
+  calc_real_max_indirect_connections ()
+)
+
+let _ =
+  Heap.add_memstat "CommonComplexOptions" (fun level buf ->
+      Printf.bprintf buf "  friends: %d\n" (List.length !!friends);
+      Printf.bprintf buf "  contacts: %d\n" (List.length !contacts);
   )
-  
-let runinfo () =
-  (
-        "Enabled Networks: " 
-      ^   (if Autoconf.donkey = "yes" && !!enable_donkey then " Donkey" else "")
-      ^   (if Autoconf.donkey = "yes" && !!enable_overnet then " Overnet" else "")
-      ^   (if Autoconf.donkey = "yes" && !!enable_kademlia then " Kademlia" else "")             
-      ^   (if Autoconf.bittorrent = "yes" && !!enable_bittorrent then " BitTorrent" else "")
-      ^   (if Autoconf.fasttrack = "yes" && !!enable_fasttrack then " Fasttrack" else "")
-      ^   (if Autoconf.gnutella = "yes" && !!enable_gnutella then " Gnutella" else "")
-      ^   (if Autoconf.gnutella2 = "yes" && !!enable_gnutella2 then " G2" else "")
-      ^   (if Autoconf.filetp = "yes" && !!enable_fileTP then " FileTP" else "")
-      ^  "\nServer usage: " ^ (if !!enable_servers then "enabled" else "disabled (you are not able to connect to ED2K Servers)")
-      ^ (let uname = Unix32.uname () in
-          if uname <> "" then Printf.sprintf "\nSystem info: %s" uname else "")
-      ^ "\nLanguage: " ^ Charset.default_language
-      ^ " - locale: " ^ Charset.locstr
-      ^ "\n max_string_length: " ^ string_of_int Sys.max_string_length
-      ^ " - word_size: " ^ string_of_int Sys.word_size
-      ^ " - max_array_length: " ^ string_of_int Sys.max_array_length
-      ^ "\n max file descriptors: " ^ string_of_int (Unix2.c_getdtablesize ())
-      ^ " - max useable file size: " ^ 
-	    (match Unix2.c_sizeofoff_t () with
-	     | 4 -> "2GB"
-
-	     |  _ -> Printf.sprintf "2^%d-1 bits (do the maths ;-p)" ((Unix2.c_sizeofoff_t () *8)-1)
-
-(*
-	     | _ -> Printf.sprintf "%s"
-	       ((*size_of_int64*)(string_of_float(((2. ** (float_of_int((Unix2.c_sizeofoff_t () * 8)-1))) -. 1.)/. 8.)))
-*)
-
-	     )
-  )    
-  
\ No newline at end of file
