@@ -21,8 +21,7 @@ val load : unit -> unit
 val save : unit -> unit
 val save_sources : unit -> unit
 val backup_options : unit -> unit
-val buildinfo : unit -> string
-val runinfo : unit -> string
+val allow_saving_ini_files : bool ref
   
 val done_files :  CommonTypes.file list Options.option_record
 val files :  CommonTypes.file list Options.option_record
@@ -42,6 +41,8 @@ val shared_directories :
   
 val incoming_files : unit -> CommonTypes.shared_directory
 val incoming_directories : unit -> CommonTypes.shared_directory
+val search_incoming_files : unit -> CommonTypes.shared_directory
+val search_incoming_directories : unit -> CommonTypes.shared_directory
   
 val sharing_only_directory : CommonTypes.sharing_strategy
 
