@@ -26,15 +26,13 @@ open CommonTypes
 open CommonOptions
 open CommonGlobals
 
-(* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[cF] "
-    (log_time ()); lprintf_nl2
-      
-(* prints a new logline with date, module and does not start newline *)
-let lprintf_n () =
-  lprintf "%s[cF] "
-    (log_time ()); lprintf
+let log_prefix = "[cF]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
 	    
 (*************************************************************************)
 (*                                                                       *)
@@ -46,18 +44,20 @@ type 'a file_impl = {
     mutable impl_file_update : int;
     mutable impl_file_state : file_state;
 
-    mutable impl_file_comment : string;
+    mutable impl_file_comment : string list;
     mutable impl_file_num : int;
     mutable impl_file_val : 'a;
     mutable impl_file_ops : 'a file_ops;
     mutable impl_file_size : int64;
     mutable impl_file_age : int;
-    mutable impl_file_fd : Unix32.t;
+    mutable impl_file_fd : Unix32.t option;
     mutable impl_file_downloaded : int64;
     mutable impl_file_received : int64;
     mutable impl_file_last_received : (int64 * int) list;
     mutable impl_file_last_rate : float;
     mutable impl_file_best_name : string;
+    mutable impl_file_filenames : string list;
+    mutable impl_file_magic : string option;
     mutable impl_file_priority: int; (* normal = 0, low < 0, high > 0 *)
     mutable impl_file_last_seen : int;
     mutable impl_file_probable_name : string option;
@@ -85,7 +85,6 @@ it will happen soon. *)
     mutable op_file_recover : ('a -> unit);
     mutable op_file_all_sources : ('a -> client list);
     mutable op_file_active_sources : ('a -> client list);
-    mutable op_file_comment : ('a -> string);
     mutable op_file_set_priority : ('a -> int -> unit);
     mutable op_file_print_html : ('a -> Buffer.t -> unit);
     mutable op_file_print_sources_html : ('a -> Buffer.t -> unit);
@@ -95,6 +94,7 @@ it will happen soon. *)
     mutable op_file_proposed_filenames : ('a -> string list);
   }
 
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         as_file...                                    *)
@@ -121,15 +121,17 @@ let dummy_file_impl = {
     impl_file_ops = Obj.magic 0;
     impl_file_size = Int64.zero;
     impl_file_age = 0;
-    impl_file_fd = Unix32.create_diskfile "" [Unix.O_RDONLY] 0o666;
+    impl_file_fd = None;
     impl_file_downloaded = Int64.zero;
     impl_file_received = Int64.zero;
     impl_file_last_received = [];
     impl_file_last_rate = 0.0;
     impl_file_best_name = "<UNKNOWN>";
+    impl_file_filenames = [];
+    impl_file_magic = None;
     impl_file_priority = 0;
     impl_file_last_seen = 0;
-    impl_file_comment = "";
+    impl_file_comment = [];
     impl_file_probable_name = None;
   }
 
@@ -160,7 +162,7 @@ let files_by_num = H.create 1027
 let ni n m =
   let s = Printf.sprintf "File.%s not implemented by %s"
       m n.network_name in
-  lprintf_nl () "%s" s;
+  lprintf_nl "%s" s;
   s
 
 let fni n m = failwith (ni n m)
@@ -206,10 +208,6 @@ let file_save_as (file : file) name =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_save_as file.impl_file_val name
 
-let file_comment (file : file) =
-  let file = as_file_impl file in
-  file.impl_file_ops.op_file_comment file.impl_file_val
-
 let file_network (file : file) =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_network
@@ -239,21 +237,55 @@ let set_file_state file state =
   update_file_state impl state
 
 let set_file_comment file comment =
-  let impl = as_file_impl file in
-  impl.impl_file_comment <- comment
+  if not (List.mem comment (as_file_impl file).impl_file_comment) then
+  (as_file_impl file).impl_file_comment <-
+    (as_file_impl file).impl_file_comment @ [(HashComments.merge file_comments comment)]
+
+let file_comment file =
+  (as_file_impl file).impl_file_comment
+
+let file_comment_length file =
+  List.length (as_file_impl file).impl_file_comment
 
-let file_comment file = 
-  let impl = as_file_impl file in
-  impl.impl_file_comment
-      
 let file_best_name (file : file) =
   let file = as_file_impl file in
   file.impl_file_best_name
   
-let set_file_best_name file name =
+let set_file_best_name file name fs namemax =
   let file = as_file_impl file in
-  let name = String2.replace name '/' "::" in
-  file.impl_file_best_name <- name
+  let old_name = file.impl_file_best_name in
+  let real_name = Filename2.filesystem_compliant name fs namemax in
+  if real_name = "" then
+    lprintf_nl "can not rename file \"%s\" to \"%s\""
+      (String.escaped file.impl_file_best_name) (String.escaped real_name)
+  else begin
+    file.impl_file_best_name <- real_name;
+    if name <> real_name then
+      lprintf_nl "wanted new name \"%s\" changed to \"%s\" due to system limitations"
+        (String.escaped name) (String.escaped file.impl_file_best_name);
+    if !verbose && old_name <> file.impl_file_best_name then
+      lprintf_nl "best_name of \"%s\" changed to \"%s\""
+        (String.escaped old_name) (String.escaped file.impl_file_best_name)
+  end
+
+let add_file_filenames file name =
+  let impl = as_file_impl file in
+  if name <> ""
+  && not (List.mem name impl.impl_file_filenames) 
+  && List.length impl.impl_file_filenames < !!max_filenames then
+    impl.impl_file_filenames <- name :: impl.impl_file_filenames;
+  file_must_update file
+
+let shorten_all_file_filenames length =
+  H.iter (fun file ->
+    let impl = as_file_impl file in
+    if List.length impl.impl_file_filenames > length then
+      begin
+        let new_list, _ = List2.cut !!max_filenames impl.impl_file_filenames in
+        impl.impl_file_filenames <- new_list;
+	file_must_update file
+      end
+  ) files_by_num
 
 let set_file_format (file : file) format =
   let file = as_file_impl file in
@@ -293,6 +325,7 @@ let default_file_print_sources_html file buf =
         ( "1", "srh br ac", "Client number", "Num" ) ;
         ( "0", "srh br", "Client Name", "Name" ) ;
         ( "0", "srh br", "IP address", "IP address" ) ;
+        ( "0", "srh br", "Client software", "CS" ) ;
         ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ;
         ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; ];
 
@@ -302,8 +335,8 @@ let default_file_print_sources_html file buf =
         let cinfo = client_info c in
         let addr = 
         (match cinfo.GuiTypes.client_kind with
-          Indirect_location (name, _) -> ""
-        | Known_location (ip, port) -> Printf.sprintf "%s:%d" (Ip.to_string ip) port 
+          Indirect_location (_, _, ip, port)
+        | Known_location (ip, port) -> Printf.sprintf "%s:%d" (Ip.to_string ip) port
         )
         in
 
@@ -313,6 +346,7 @@ let default_file_print_sources_html file buf =
           ("", "sr br ar", Printf.sprintf "%d" (client_num c));
           ("", "sr br", cinfo.GuiTypes.client_name);
           ("", "sr br", addr);
+          ("", "sr br", cinfo.GuiTypes.client_software);
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
           ("", "sr ar br", (size_of_int64 cinfo.GuiTypes.client_downloaded)); ];
 
@@ -398,19 +432,20 @@ let add_file_downloaded file n =
 let file_size file = 
   (as_file_impl file).impl_file_size
 
-let file_disk_name file =
-  Unix32.filename (as_file_impl file).impl_file_fd
-
 let file_fd file =
-  (as_file_impl file).impl_file_fd
+  match (as_file_impl file).impl_file_fd with
+    | Some fd -> fd
+    | None -> raise Not_found
+
+let file_disk_name file =
+  Unix32.filename (file_fd file)
 
 let set_file_fd file fd =
-  (as_file_impl file).impl_file_fd <- fd
+  (as_file_impl file).impl_file_fd <- Some fd
 
 let set_file_disk_name file filename =
   let orig_fd = file_fd file in
-  if orig_fd != Unix32.bad_fd then
-    Unix32.rename orig_fd filename
+  Unix32.rename orig_fd filename
 
 let file_downloaded file = (as_file_impl file).impl_file_downloaded
 
@@ -428,6 +463,26 @@ let set_file_priority file p =
       file_must_update file
     end
 
+let file_magic file =
+  (as_file_impl file).impl_file_magic
+  
+let set_file_magic file magic =
+  match magic with
+    None -> ()
+  | Some magic -> (as_file_impl file).impl_file_magic <- Some (HashMagic.merge files_magic magic)
+
+let check_magic file =
+  let check file =
+    match Magic.M.magic_fileinfo (file_disk_name file) false with
+      None -> ()
+    | Some magic -> set_file_magic file (Some magic)
+  in
+  let magic = file_magic file in
+    match magic with
+      None -> check file
+    | Some magic when magic = "data" || magic = "empty" -> check file
+    | _ -> ()
+
 let set_file_last_seen file age =
   let impl = as_file_impl file in
   impl.impl_file_last_seen <- age
@@ -446,7 +501,6 @@ let file_preview (file : file) =
 module G = GuiTypes
 
 let file_downloaders file o cnt =
-  let buf = o.conn_buf in
 
       let srcs = file_active_sources file in
       let counter = ref cnt in
@@ -465,7 +519,7 @@ let file_downloaders file o cnt =
           | _ -> ()
       ) srcs;
 
-	if !counter mod 2 = 0 then true else false
+	!counter mod 2 = 0
 
 (*************************************************************************)
 (*                                                                       *)
@@ -475,7 +529,7 @@ let file_downloaders file o cnt =
 
 (* Use span for Opera DOM compatibility *)
 let colored_chunks chunks =
-  let chunks_length = Array.length chunks in
+  let chunks_length = VerificationBitmap.length chunks in
   let graph_width = min !!html_vd_chunk_graph_max_width (8 * chunks_length) in
   let ostr = Buffer.create 100 in
   Printf.bprintf ostr "\\<table cellspacing=0 cellpadding=0 width=\\\"%dpx\\\"\\>\\<tr\\>" graph_width;
@@ -492,34 +546,39 @@ let colored_chunks chunks =
       current_output_bit := new_output_bit
     end in
 
-  (match !!html_vd_chunk_graph_style with
+  (let color_of_state = function
+    | VerificationBitmap.State_missing -> 0
+    | VerificationBitmap.State_partial -> 1
+    | VerificationBitmap.State_complete -> 2
+    | VerificationBitmap.State_verified -> 3 in
+    match !!html_vd_chunk_graph_style with
      | 0 ->
-	 let previous = ref 0 in
+	 let previous = ref VerificationBitmap.State_missing in
 	 let runlength = ref 0 in
 
-	 Array.iter (fun b ->
+	 VerificationBitmap.iteri (fun _ b ->
 	   if b = !previous then
 	     incr runlength
 	   else begin
-	     display_bar !previous !runlength;
+	     if !runlength > 0 then
+	       display_bar (color_of_state !previous) !runlength;
 	     previous := b;
 	     runlength := 1
 	   end
 	 ) chunks;
-	 display_bar !previous !runlength
+	 display_bar (color_of_state !previous) !runlength
      | _ ->
 	 let missing = ref 0 in
 	 let partial = ref 0 in
 	 let complete = ref 0 in
 	 let verified = ref 0 in
 
-	 Array.iter (fun b -> 
+	 VerificationBitmap.iteri (fun _ b -> 
 	   match b with
-             | 0 -> incr missing
-             | 1 -> incr partial
-             | 2 -> incr complete
-             | 3 -> incr verified
-             | _ -> ()
+             | VerificationBitmap.State_missing -> incr missing
+             | VerificationBitmap.State_partial -> incr partial
+             | VerificationBitmap.State_complete -> incr complete
+             | VerificationBitmap.State_verified -> incr verified
 	 ) chunks;
 	 match !!html_vd_chunk_graph_style with
 	   | 1 ->
@@ -546,61 +605,87 @@ let file_print file o =
 
   if use_html_mods o then begin
 
+      html_mods_cntr_init ();
       html_mods_table_header buf "sourcesInfo" "sourcesInfo" [
         ( "0", "srh br", "File Info", "Info" ) ;
         ( "0", "srh", "Value", "Value" ) ];
 
-      Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>";
+      Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
         ("File number/Network", "sr br", "[#] Network");
         ("", "sr", Printf.sprintf "[%d] %s" (file_num file) n.network_name) ];
 
-      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
         ("Downloaded/Total size", "sr br", "DLed/Size");
         ("", "sr", Printf.sprintf "%s bytes of %s bytes"
             (Int64.to_string info.G.file_downloaded) (Int64.to_string info.G.file_size) ) ];
 
-      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
         ("File priority", "sr br", "Priority");
         ("", "sr", Printf.sprintf "%d" (file_priority file)) ];
 
-      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
         ("Number of file sources", "sr br", "Sources");
         ("", "sr", Printf.sprintf "%d" (List.length srcs)) ];
 
-      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 
-      let tt = ref "0=Missing, 1=Partial, 2=Complete, 3=Verified" in
-      let tc = String.length info.G.file_chunks in
-      let c0 = ref 0 in
-      let c1 = ref 0 in
-      let c2 = ref 0 in
-      let c3 = ref 0 in   
+      (match info.G.file_chunks with 
+      | None -> ()
+      | Some chunks ->
+	  let tt = "0=Missing, 1=Partial, 2=Complete, 3=Verified" in
+	  let tc = VerificationBitmap.length chunks in
+	  let c0 = ref 0 in
+	  let c1 = ref 0 in
+	  let c2 = ref 0 in
+	  let c3 = ref 0 in   
 
-      String.iter (fun c ->
-        match c with
-        | '0' -> incr c0
-        | '1' -> incr c1
-        | '2' -> incr c2
-        | '3' -> incr c3
-        | _ -> ()
-      ) info.G.file_chunks;
+	  VerificationBitmap.iteri (fun _ c ->
+            match c with
+            | VerificationBitmap.State_missing -> incr c0
+            | VerificationBitmap.State_partial -> incr c1
+            | VerificationBitmap.State_complete -> incr c2
+            | VerificationBitmap.State_verified -> incr c3
+	  ) chunks;
       
-      let header = Printf.sprintf "%d (%d+%d+%d+%d): " tc !c0 !c1 !c2 !c3 in
+	  let header = Printf.sprintf "%d (%d+%d+%d+%d): " tc !c0 !c1 !c2 !c3 in
 
-      html_mods_td buf [
-        (!tt, "sr br", "Chunks");
-        (!tt, "sr", 
-          header ^ if !!html_vd_chunk_graph then
-          (colored_chunks (Array.init (String.length info.G.file_chunks)
-          (fun i -> ((int_of_char info.G.file_chunks.[i])-48))))
-          else
-            info.G.file_chunks
-        ) ];
+	  html_mods_td buf [
+            (tt, "sr br", "Chunks");
+            (tt, "sr", 
+            header ^ if !!html_vd_chunk_graph then
+              colored_chunks chunks
+            else
+              VerificationBitmap.to_string chunks
+            ) ]
+      );
 
+      (match file_magic file with
+        Some magic ->
+	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+	    html_mods_td buf [
+            ("File type computed by libmagic", "sr br", "File magic");
+            ("", "sr", magic) ]
+       | _ -> ());
+
+      (
+	if file_comment file <> [] then
+	  begin
+	    let list_header = ref true in
+	    List.iter (fun s ->
+	      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+	      if !list_header then html_mods_td buf [
+		("File comments", "sr br", "File comments");
+		("", "sr", s) ]
+	      else
+	        html_mods_td buf [
+		("", "sr br", "");
+		("", "sr", s) ];
+	    if !list_header then list_header := false) (file_comment file);
+	  end);
 
       file_print_html file buf;
       
@@ -609,20 +694,26 @@ let file_print file o =
 
     end else
     begin
-      Printf.bprintf buf "[%-s %5d]\n%s\n%s\nTotal   %10s\nPartial %10s\npriority %d\n"
+      Printf.bprintf buf "[%-s %5d]\n%s\n%s%s\nTotal   %10s\nPartial %10s\npriority %d\n"
         n.network_name
         (file_num file)
-        (String2.shorten 80 (file_best_name file))
+        (shorten (file_best_name file) 80)
+	(match file_magic file with
+	   Some magic -> Printf.sprintf "%s\n" magic
+	 | None -> "")
         (string_of_uids info.G.file_uids)
         (Int64.to_string info.G.file_size)
         (Int64.to_string info.G.file_downloaded)
         (file_priority file);
-      Printf.bprintf buf "Chunks: [%-s]\n" info.G.file_chunks;
+      Printf.bprintf buf "Chunks: [%-s]\n"
+	(match info.G.file_chunks with
+	| None -> ""
+	| Some chunks -> VerificationBitmap.to_string chunks);
       (match impl.impl_file_probable_name with
           None -> ()
         | Some filename ->
             Printf.bprintf buf "Probable name: %s\n" filename);
-      List.iter (fun (name,_) -> Printf.bprintf buf "    (%s)\n" name) info.G.file_names
+      List.iter (fun name -> Printf.bprintf buf "    (%s)\n" name) info.G.file_names
     end;
 
   (try
@@ -632,7 +723,6 @@ let file_print file o =
             file_print_sources_html file buf
           else begin
             Printf.bprintf buf "%d sources:\n" (List.length srcs);
-          let counter = ref 0 in
           let print_source c =
                   Printf.bprintf buf "  [%4d] " (client_num c);
                   client_bprint c buf;
@@ -643,6 +733,11 @@ let file_print file o =
     
     with _ -> ())
 
+let file_print_ed2k_link filename filesize md4hash =
+  if md4hash = Md4.null then "" else
+  Printf.sprintf "ed2k://|file|%s|%s|%s|/"
+    (Url.encode filename) (Int64.to_string filesize) (Md4.to_string md4hash)
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         add_segment (internal)                        *)
@@ -786,7 +881,6 @@ let new_file_ops network =
       op_file_set_format = (fun _ -> fni network "file_set_format");
       op_file_all_sources = (fun _ -> fni network "file_all_sources");
       op_file_active_sources = (fun _ -> fni network "file_active_sources");
-      op_file_comment = (fun _ -> ni_ok network "file_comment"; "");
       op_file_set_priority = (fun _ _ -> ni_ok network "file_set_priority");
       op_file_print_html = (fun _ _ -> ni_ok network "file_print_html");
       op_file_print_sources_html = (fun _ _ -> fni network "file_print_sources_html");
@@ -800,40 +894,40 @@ let new_file_ops network =
   f
 
 let check_file_implementations () =
-  lprintf_nl () "\n---- Methods not implemented for CommonFile ----";
+  lprintf_nl "\n---- Methods not implemented for CommonFile ----";
   List.iter (fun (c, cc) ->
       let n = c.op_file_network.network_name in
-      lprintf_nl () "\n  Network %s" n; 
+      lprintf_nl "\n  Network %s" n; 
       if c.op_file_to_option == cc.op_file_to_option then 
-        lprintf_nl () "op_file_to_option";
+        lprintf_nl "op_file_to_option";
       if c.op_file_info == cc.op_file_info then
-        lprintf_nl () "op_file_info";
+        lprintf_nl "op_file_info";
       if c.op_file_commit == cc.op_file_commit then
-        lprintf_nl () "op_file_commit";
+        lprintf_nl "op_file_commit";
       if c.op_file_save_as == cc.op_file_save_as then
-        lprintf_nl () "op_file_save_as";
+        lprintf_nl "op_file_save_as";
       if c.op_file_cancel == cc.op_file_cancel then
-        lprintf_nl () "op_file_cancel";
+        lprintf_nl "op_file_cancel";
       if c.op_file_pause == cc.op_file_pause then
-        lprintf_nl () "op_file_pause";
+        lprintf_nl "op_file_pause";
       if c.op_file_resume == cc.op_file_resume then
-        lprintf_nl () "op_file_resume";
+        lprintf_nl "op_file_resume";
 (*      if c.op_file_disk_name == cc.op_file_disk_name then
         lprintf_nl "op_file_disk_name"; *)
       if c.op_file_check == cc.op_file_check then
-        lprintf_nl () "op_file_check";
+        lprintf_nl "op_file_check";
       if c.op_file_recover == cc.op_file_recover then
-        lprintf_nl () "op_file_recover";
+        lprintf_nl "op_file_recover";
       if c.op_file_set_format == cc.op_file_set_format then
-        lprintf_nl () "op_file_set_format";
+        lprintf_nl "op_file_set_format";
       if c.op_file_all_sources == cc.op_file_all_sources then
-        lprintf_nl () "op_file_all_sources";
+        lprintf_nl "op_file_all_sources";
       if c.op_file_active_sources == cc.op_file_active_sources then
-        lprintf_nl () "op_file_active_sources";
+        lprintf_nl "op_file_active_sources";
       if c.op_file_print_html == cc.op_file_print_html then
-        lprintf_nl () "op_file_print_html";
+        lprintf_nl "op_file_print_html";
       if c.op_file_print_sources_html == cc.op_file_print_sources_html then
-        lprintf_nl () "op_file_print_sources_html";
+        lprintf_nl "op_file_print_sources_html";
   ) !files_ops;
   lprint_newline ()
 
@@ -855,6 +949,12 @@ let _ =
       ) files_by_num;
       Printf.bprintf buf "  files: %d\n" !counter;
       Printf.bprintf buf "  files_ops: %d\n" (List.length !files_ops);
+      let counter = ref 0 in
+      HashMagic.iter (fun _ -> incr counter) files_magic;
+      Printf.bprintf buf "  files_magic: %d\n" !counter;
+      let counter = ref 0 in
+      HashComments.iter (fun _ -> incr counter) file_comments;
+      Printf.bprintf buf "  files_comments: %d\n" !counter
   )
 
 
@@ -878,7 +978,7 @@ let file_write file offset s pos len =
 let file_verify file key begin_pos end_pos =
   Unix32.flush_fd (file_fd file);
   if !verbose_md4 then begin
-      lprintf_nl () "Checksum to compute: %Ld-%Ld of %s" begin_pos end_pos
+      lprintf_nl "Checksum to compute: %Ld-%Ld of %s" begin_pos end_pos
         (file_disk_name file);
     end;
   try
@@ -900,7 +1000,7 @@ let file_verify file key begin_pos end_pos =
     in
     let result = computed = key in
     if !verbose_md4 then begin
-        lprintf_nl () "Checksum computed: %s against %s = %s"
+        lprintf_nl "Checksum computed: %s against %s = %s"
           (string_of_uid key) 
         (string_of_uid computed)
         (if result then "VERIFIED" else "CORRUPTED");
@@ -908,7 +1008,7 @@ let file_verify file key begin_pos end_pos =
     result
   with
     | Not_found -> raise Not_found
-    | e -> lprintf_nl () "Checksum computation failed: Exception: %s" (Printexc2.to_string e);
+    | e -> lprintf_nl "Checksum computation failed: Exception: %s" (Printexc2.to_string e);
     false
 
 let file_mtime file = Unix32.mtime64 (file_fd file)
@@ -967,3 +1067,32 @@ let forceable_download = ref []
 
 let com_files_by_num = files_by_num
 let files_by_num = ()
+
+let impl_file_info impl =
+  let module T = GuiTypes in
+  {
+    T.file_fields = T.Fields_file_info.all;
+    T.file_comment = "";
+    T.file_name = impl.impl_file_best_name;
+    T.file_names = impl.impl_file_filenames;
+    T.file_num = impl.impl_file_num;
+    T.file_size = impl.impl_file_size;
+    T.file_downloaded = impl.impl_file_downloaded;
+    T.file_state = impl.impl_file_state;
+    T.file_download_rate = file_download_rate impl;
+    T.file_priority = impl.impl_file_priority;
+    T.file_age = impl.impl_file_age;
+    T.file_last_seen = impl.impl_file_last_seen;
+
+    T.file_network = 0;
+    T.file_md4 = Md4.null;
+    T.file_all_sources = 0;
+    T.file_active_sources = 0;
+    T.file_sources = None;
+    T.file_chunks = None;
+    T.file_availability = [];
+    T.file_format = FormatNotComputed 0;
+    T.file_chunks_age = [||];
+    T.file_uids = [];
+    T.file_sub_files = [];
+  }
