@@ -21,18 +21,20 @@ type 'a file_impl = {
     mutable impl_file_update : int;
     mutable impl_file_state : CommonTypes.file_state;
     
-    mutable impl_file_comment : string;
+  mutable impl_file_comment : string list;
   mutable impl_file_num : int;
   mutable impl_file_val : 'a;
   mutable impl_file_ops : 'a file_ops;
   mutable impl_file_size : int64;
   mutable impl_file_age : int;
-  mutable impl_file_fd : Unix32.t;
+  mutable impl_file_fd : Unix32.t option;
   mutable impl_file_downloaded : int64;
   mutable impl_file_received : int64;
   mutable impl_file_last_received : (int64 * int) list;
   mutable impl_file_last_rate : float;
   mutable impl_file_best_name : string;
+  mutable impl_file_filenames : string list;
+  mutable impl_file_magic : string option;
   mutable impl_file_priority : int;
     mutable impl_file_last_seen : int;
     mutable impl_file_probable_name : string option;
@@ -51,7 +53,6 @@ and 'a file_ops = {
   mutable op_file_recover : 'a -> unit;
   mutable op_file_all_sources : 'a -> CommonTypes.client list;
   mutable op_file_active_sources : 'a -> CommonTypes.client list;
-  mutable op_file_comment : 'a -> string;
   mutable op_file_set_priority : 'a -> int -> unit;
     mutable op_file_print_html: 'a -> Buffer.t -> unit;
     mutable op_file_print_sources_html : 'a -> Buffer.t -> unit;
@@ -74,14 +75,15 @@ val update_file_num : 'a file_impl -> unit
 val update_file_state : 'a file_impl -> CommonTypes.file_state -> unit
 val file_to_option : CommonTypes.file -> (string * Options.option_value) list
 val file_save_as : CommonTypes.file -> string -> unit
-val file_comment : CommonTypes.file -> string
 val file_network : CommonTypes.file -> CommonTypes.network
 val file_info : CommonTypes.file -> GuiTypes.file_info
 val file_pause : CommonTypes.file -> unit
 val file_resume : CommonTypes.file -> unit
 val set_file_state : CommonTypes.file -> CommonTypes.file_state -> unit
 val file_best_name : CommonTypes.file -> string
-val set_file_best_name : CommonTypes.file -> string -> unit
+val set_file_best_name : CommonTypes.file -> string -> string -> int -> unit
+val add_file_filenames : CommonTypes.file -> string -> unit
+val shorten_all_file_filenames : int -> unit
 val set_file_format : CommonTypes.file -> CommonTypes.format -> unit
 val file_check : CommonTypes.file -> unit
 val file_recover : CommonTypes.file -> unit
@@ -101,8 +103,9 @@ val sample_timer : unit -> unit
 val file_download_rate : 'a file_impl -> float
 val add_file_downloaded : CommonTypes.file -> Int64.t -> unit
 val file_downloaders : CommonTypes.file -> CommonTypes.ui_conn -> int -> bool
-val colored_chunks : int array -> string
+val colored_chunks : VerificationBitmap.t -> string
 val file_print : CommonTypes.file -> CommonTypes.ui_conn -> unit
+val file_print_ed2k_link : string -> int64 -> Md4.Md4.t -> string
 val file_size : CommonTypes.file -> int64
 val file_disk_name : CommonTypes.file -> string
 val file_fd : CommonTypes.file -> Unix32.t
@@ -115,8 +118,11 @@ val set_file_priority : CommonTypes.file -> int -> unit
 val set_file_last_seen : CommonTypes.file -> int -> unit
 val file_debug : CommonTypes.file -> string
 val set_file_comment : CommonTypes.file -> string -> unit
-val file_comment : CommonTypes.file -> string
-  
+val file_comment : CommonTypes.file -> string list
+val file_comment_length : CommonTypes.file -> int
+val file_magic : CommonTypes.file -> string option
+val set_file_magic : CommonTypes.file -> string option -> unit
+val check_magic : CommonTypes.file -> unit  
 val recover_bytes : CommonTypes.file -> (int64 * int64) list
 val file_write : CommonTypes.file -> int64 -> string -> int -> int -> unit
 val file_verify : CommonTypes.file ->
@@ -129,3 +135,4 @@ val propose_filenames : unit -> unit
 val propose_filename : CommonTypes.file -> unit
 
 val forceable_download : CommonTypes.result_info list ref
+val impl_file_info : 'a file_impl -> GuiTypes.file_info
