@@ -107,14 +107,20 @@ in an option, and then change the option accordingly. ?> *)
 let find_other_port = ref false
 
 let shorten str limit =
-  (* TODO: we schould change all strings to utf8 when
+  (* TODO: we should change all strings to utf8 when
      they come into the core instead. *)
   let name = Charset.to_utf8 (* String.escaped *) str in
-  let len = String.length name in
+  let slen = String.length str in
+  let len =
+    try
+      Charset.utf8_length str
+    with e -> slen
+  in
+  let diff_len_utf8_ascii = slen - len in
   let max_len = maxi limit 10 in
   if len > max_len then
-    let prefix = String.sub name 0 (max_len -7) in
-    let suffix = String.sub name (len-4) 4 in
+    let prefix = String.sub name 0 (max_len - 7 + diff_len_utf8_ascii) in
+    let suffix = String.sub name (len - 4 + diff_len_utf8_ascii) 4 in
     Printf.sprintf "%s...%s" prefix suffix
   else name
 
@@ -144,14 +150,6 @@ let find_port server_name bind_addr port_option handler =
 let one_day = 3600. *. 24.
 let half_day = one_day /. 2.
 
-let printf_char c =
-  if !verbose then
-    (lprint_char c)
-
-let printf_string c =
-  if !verbose then
-    (lprint_string c)
-
 let minutes25 = 25 * 60
 
 let new_connection_control () = {
@@ -215,8 +213,25 @@ let download_control = TcpBufferedSocket.create_read_bandwidth_controler
 
 let payload_bandwidth = ref 0.
 
+let check_ul_dl_ratio () =
+  if !!max_hard_upload_rate < 0 then max_hard_upload_rate =:= 0;
+  if !!max_hard_download_rate < 0 then max_hard_download_rate =:= 0;
+  let max_max_hard_download_rate =
+    match !!max_hard_upload_rate with
+    | 0 -> None
+    | x when x < 4 -> Some (x * 3)
+    | x when x < 10 -> Some (x * 4)
+    | x -> None in
+  match max_max_hard_download_rate with
+  | None -> ()
+  | Some limit ->
+      if !!max_hard_download_rate = 0 ||
+	!!max_hard_download_rate > limit then
+	  max_hard_download_rate =:= limit
+
 let _ =
   option_hook max_hard_upload_rate (fun _ ->
+      check_ul_dl_ratio ();
       TcpBufferedSocket.change_rate upload_control
         (!!max_hard_upload_rate * 1024);
       payload_bandwidth :=
@@ -226,9 +241,9 @@ let _ =
 	   maxi (!!max_hard_upload_rate * 1024) 1024) *. 0.90;
   );
   option_hook max_hard_download_rate (fun _ ->
-      let rate = !!max_hard_download_rate in
+      check_ul_dl_ratio ();
       TcpBufferedSocket.change_rate download_control
-        (rate * 1024))
+        (!!max_hard_download_rate * 1024))
 
 let udp_write_controler = UdpSocket.new_bandwidth_controler upload_control
 
@@ -467,15 +482,15 @@ let chat_message_fifo = (Fifo.create () : (int * string * int * string * string)
 let log_chat_message i num n s =
   Fifo.put chat_message_fifo (last_time(),i,num,n,s);
   try
-    let oc = open_out_gen [Open_creat; Open_wronly; Open_append] 0o600 !messages_log in
-    Printf.fprintf oc "%s: %s (%s): %s\n" (Date.simple (BasicSocket.date_of_int (last_time ()))) n i s;
-    close_out oc;
+    Unix2.tryopen_write_gen !messages_log [Open_creat; Open_wronly; Open_append] 
+      0o600 (fun oc ->
+	Printf.fprintf oc "%s: %s (%s): %s\n" (Date.simple (BasicSocket.date_of_int (last_time ()))) n i s)
   with e ->
     lprintf_nl "[ERROR] Exception %s while trying to log message to %s"
       (Printexc2.to_string e) !messages_log;
 
   while (Fifo.length chat_message_fifo) > !!html_mods_max_messages do
-    let foo = Fifo.take chat_message_fifo in ()
+    ignore(Fifo.take chat_message_fifo)
   done
 let last_message_log = ref 0
 
@@ -484,7 +499,7 @@ let last_message_log = ref 0
 let debug_clients = ref Intset.empty
 
 let default_user = {
-    ui_user_name = "admin";
+    ui_user_name = CommonUserDb.admin_user;
     ui_user_searches = [];
     ui_last_search = None;
     ui_last_results = [];
@@ -511,14 +526,6 @@ let find_ui_user user =
   in
   iter !ui_users
 
-
-let valid_password user pass =
-  let pass = Md4.Md4.string pass in
-  try
-    let password = List.assoc user !!users in
-    password = pass
-  with _ -> false
-
 (* control_: means that it is the limited bandwidth, not the unlimited one
   used by the interfaces. tcp_: the full bandwidth (limited+unlimited) *)
 
@@ -720,8 +727,10 @@ let for_string_tag tag f =
 
 let partial_chunk c =
   match c with
-    '0' | '1' -> true
-  | _ -> false
+  | VerificationBitmap.State_missing | VerificationBitmap.State_partial -> 
+      true
+  | VerificationBitmap.State_complete | VerificationBitmap.State_verified -> 
+      false
 
 module Connections = struct
 
@@ -881,3 +890,18 @@ let _ =
       activity := new_activity ()
   )
 
+module HashMagic = Weak.Make(struct
+        type t = string
+        let hash s = Hashtbl.hash s
+        let equal x y = x = y
+      end)
+
+let files_magic = HashMagic.create 100
+
+module HashComments = Weak.Make(struct
+        type t = string
+        let hash s = Hashtbl.hash s
+        let equal x y = x = y
+      end)
+
+let file_comments = HashComments.create 1000
