@@ -32,7 +32,7 @@ open CommonClient
 open CommonServer
 open CommonNetwork
 open CommonOptions
-open CommonTypes
+open CommonUserDb
 open CommonFile
 open CommonGlobals
 open CommonSearch
@@ -41,6 +41,14 @@ open CommonServer
 open CommonTypes
 open CommonComplexOptions
 
+let log_prefix = "[cInt]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
+
 (*************  ADD/REMOVE FUNCTIONS ************)
 let check_forbidden_chars (uc : Charset.uchar) =
   match uc with
@@ -76,29 +84,73 @@ let canonize_basename name =
         Charset.add_uchar buf uc'
       end
   done;
-  if Autoconf.windows then
+  if not Autoconf.windows then
     Charset.to_locale (Buffer.contents buf)
   else
-    Buffer.contents buf
+    Buffer.contents buf (* Windows uses patched Ocaml which always uses Unicode chars *)
+
+let last_sent_dir_warning = Hashtbl.create 10
+
+let hdd_full_log_closed = ref false
+let all_temp_queued = ref false
+
+let send_dirfull_warning dir full line1 =
+  let status = if full then "is full" else "has enough space again" in
+  lprintf_nl "WARNING: Directory %s %s, %s" dir status line1;
+  if (not (keep_console_output ())) then
+    Printf.eprintf "\n%s WARNING: Directory %s %s, %s\n%!" (log_time ()) dir status line1;
+  if !!hdd_send_warning_interval <> 0 then
+    let current_time = last_time () in
+    let time_threshold =
+      current_time - !!hdd_send_warning_interval * Date.hour_in_secs in
+    let send_mail_again =
+      try
+        let last = Hashtbl.find last_sent_dir_warning dir in
+	last < time_threshold
+      with Not_found -> true in
+
+    if send_mail_again then begin
+      if full then Hashtbl.replace last_sent_dir_warning dir current_time;
+      CommonEvent.add_event (Console_message_event
+        (Printf.sprintf "\nWARNING: %s %s, %s\n" dir status line1));
+      if !!mail <> "" then
+        let module M = Mailer in
+        let subject = Printf.sprintf "[mldonkey@%s] AUTOMATED WARNING: %s %s" (Unix.gethostname ()) dir status in
+        let mail = {
+          M.mail_to = !!mail; M.mail_from = !!mail;
+	  M.mail_subject = subject; M.mail_body = line1;
+        } in
+	try
+          M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
+	with _ -> ()
+    end
 
 let file_commited_name incoming_dir file =
-  let network = file_network file in
-  let best_name = file_best_name file in
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
+  let fs = Unix32.filesystem incoming_dir in
+  let namemax =
+    match Unix32.fnamelen incoming_dir with
+      None -> 0
+    | Some v -> Int64.to_int v in
+
   let new_name =
-    Filename.concat incoming_dir (canonize_basename best_name)
-  in
+    Filename2.filesystem_compliant 
+      (canonize_basename (file_best_name file)) fs namemax in
+
   let new_name =
-    if Sys.file_exists new_name then
+    if Sys.file_exists (Filename.concat incoming_dir new_name) then
       let rec iter num =
-        let new_name = Printf.sprintf "%s.%d" new_name num in
-        if Sys.file_exists new_name then
-          iter (num+1)
-        else new_name
+        let new_name =
+	  Filename2.filesystem_compliant
+            (Printf.sprintf "%s_%d" new_name num) fs namemax in
+          if Sys.file_exists (Filename.concat incoming_dir new_name) then
+            iter (num+1)
+          else new_name
       in
       iter 1
     else new_name in
-  new_name
+  set_file_best_name file (file_best_name file) "" 0;
+  Filename.concat incoming_dir new_name
 
 let script_for_file file incoming new_name =
   let info = file_info file in
@@ -106,17 +158,11 @@ let script_for_file file incoming new_name =
   let file_id = Filename.basename temp_name in
   let size = Int64.to_string (file_size file) in
   let duration =
-    Date.time_to_string_long ((BasicSocket.last_time ()) - info.G.file_age)
+    string_of_int ((BasicSocket.last_time ()) - info.G.file_age)
   in
   let network = network_find_by_num info.G.file_network in
   let filename = Filename.basename new_name in
-  let ed2k_hash =
-    if info.G.file_md4 = Md4.null then "" else
-       Printf.sprintf "ed2k://|file|%s|%s|%s|/"
-        (Url.encode filename)
-        (Int64.to_string (file_size file))
-        (Md4.to_string info.G.file_md4)
-  in
+  begin try
   MlUnix.fork_and_exec !!file_completed_cmd
       [|  (* keep those for compatibility *)
       "";
@@ -130,9 +176,14 @@ let script_for_file file incoming new_name =
 	    ("FILENAME",  filename);
 	    ("FILEHASH",  string_of_uids info.G.file_uids);
 	    ("DURATION",  duration);
+	    ("DLFILES",   string_of_int (List.length !!files)); 
 	    ("INCOMING",  incoming);
 	    ("NETWORK",   network.network_name);
-	    ("ED2K_HASH", ed2k_hash)]
+	    ("ED2K_HASH", (file_print_ed2k_link filename (file_size file) info.G.file_md4))]
+  with e -> 
+      lprintf_nl "Exception %s while executing %s"
+        (Printexc2.to_string e) !!file_completed_cmd
+  end
 
 (********
 These two functions 'file_commit' and 'file_cancel' should be the two only
@@ -144,40 +195,57 @@ file has already been moved to the incoming/ directory under its new
 name.
 *)
 
+exception Incoming_full
+
 let file_commit file =
   let impl = as_file_impl file in
   if impl.impl_file_state = FileDownloaded then
     let subfiles = file_files file in
     match subfiles with
-      file :: secondary_files ->
-        let file_name = file_disk_name file in
-        let incoming =
-          if Unix2.is_directory file_name then
-            incoming_directories ()
-          else
-            incoming_files ()
-        in
-        let new_name = file_commited_name
-            incoming.shdir_dirname file in
-        if Unix2.is_directory file_name then
-          Unix2.safe_mkdir new_name;
-        (try
+      primary :: secondary_files ->
+	if primary == file then
+       (try
+	  let file_name = file_disk_name file in
+	  let incoming =
+            if Unix2.is_directory file_name then
+              incoming_directories ()
+            else
+              incoming_files ()
+          in
+
+(* check if temp_directory and incoming are on different partitions *)
+	  if (Unix.stat incoming.shdir_dirname).Unix.st_dev <>
+	     (Unix.stat !!temp_directory).Unix.st_dev
+	  then
+	    begin
+	      match Unix32.diskfree incoming.shdir_dirname with
+	        Some v -> if v < (file_size file) then begin
+		    send_dirfull_warning incoming.shdir_dirname true
+		      (Printf.sprintf "can not commit %s" (file_best_name file));
+		    raise Incoming_full
+		  end
+	      | _ -> ()
+	    end;
+
+	  let new_name = file_commited_name incoming.shdir_dirname file in
+	    if Unix2.is_directory file_name then begin
+	      Unix2.safe_mkdir new_name;
+	      Unix2.chmod new_name (Misc.int_of_octal_string !!create_dir_mask)
+	    end;
+
 (*          the next line really moves the file *)
             set_file_disk_name file new_name;
 
             if !!file_completed_cmd <> "" then
 	      script_for_file file incoming.shdir_dirname new_name;
 
-            if Unix2.is_directory new_name then
-              Unix2.chmod new_name (Misc.int_of_octal_string !!create_dir_mask);
             let best_name = file_best_name file in
             Unix32.destroy (file_fd file);
-            if !verbose_files then lprintf_nl "[cInt] file_commit: destroyed";
-            if Unix2.is_directory file_name then Unix2.remove_all_directory file_name;
-            let impl = as_file_impl file in
 
-(* When the commit action is called, the file is supposed not to exist
-anymore. *)
+            if Unix2.is_directory file_name then Unix2.remove_all_directory file_name;
+
+            let impl = as_file_impl file in
+(* When the commit action is called, the file is supposed not to exist anymore. *)
             impl.impl_file_ops.op_file_commit impl.impl_file_val new_name;
 
             begin
@@ -187,7 +255,7 @@ anymore. *)
                       incoming.shdir_dirname incoming.shdir_priority
                       best_name new_name);
               with e ->
-                  lprintf_nl "[cInt] Exception %s while trying to share commited file"
+                  lprintf_nl "Exception %s while trying to share commited file"
                     (Printexc2.to_string e);
             end;
 
@@ -195,7 +263,6 @@ anymore. *)
             done_files =:= List2.removeq file !!done_files;
             files =:= List2.removeq file !!files;
 
-            if !verbose_files then lprintf_nl "[cInt]file_commit: going to secondaries...";
             List.iter (fun file ->
 (* Commit the file first, and share it after... *)
                 try
@@ -206,10 +273,10 @@ anymore. *)
                   files =:= List2.removeq file !!files;
 
                 with e ->
-                    lprintf_nl "[cInt] Exception %s in file_commit secondaries" (Printexc2.to_string e);
+                    lprintf_nl "Exception %s in file_commit secondaries" (Printexc2.to_string e);
             ) secondary_files
-          with e ->
-              lprintf_nl "[cInt] Exception in file_commit: %s" (Printexc2.to_string e))
+        with e ->
+              lprintf_nl "Exception in file_commit: %s" (Printexc2.to_string e))
     | _ -> assert false
 
 let file_cancel file =
@@ -225,18 +292,20 @@ let file_cancel file =
             impl.impl_file_ops.op_file_cancel impl.impl_file_val;
             files =:= List2.removeq file !!files;
           with e ->
-              lprintf_nl "[cInt] Exception %s in file_cancel" (Printexc2.to_string e);
+              lprintf_nl "Exception %s in file_cancel" (Printexc2.to_string e);
       ) subfiles;
-      (try
-          let fd = file_fd file in
-          if fd != Unix32.bad_fd then Unix32.remove (file_fd file)
-        with e ->
-            lprintf_nl "[cInt]Sys.remove %s exception %s"
-              (file_disk_name file)
-            (Printexc2.to_string e); );
-      Unix32.destroy (file_fd file);
+      try
+	let fd = file_fd file in
+	(try
+           Unix32.remove fd
+         with e ->
+           lprintf_nl "Sys.remove %s exception %s"
+             (file_disk_name file)
+             (Printexc2.to_string e));
+	Unix32.destroy fd
+      with Not_found -> ()
   with e ->
-      lprintf_nl "[cInt] Exception in file_cancel: %s" (Printexc2.to_string e)
+      lprintf_nl "Exception in file_cancel: %s" (Printexc2.to_string e)
 
 let mail_for_completed_file file =
   if !!mail <> "" then
@@ -244,21 +313,24 @@ let mail_for_completed_file file =
     let info = file_info file in
     let line1 = "mldonkey has completed the download of:\r\n\r\n" in
 
-    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\nHash: %s\r\nFile was downloaded in%s\r\n"
+    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\nHash: %s\r\nFile was downloaded in %s\r\n"
       (file_best_name file)
       (file_size file)
       (string_of_uids info.G.file_uids)
-      (let age = (BasicSocket.last_time ()) - info.G.file_age in Date.time_to_string_long age)
+      (let age = (BasicSocket.last_time ()) - info.G.file_age in Date.time_to_string age "verbose")
     in
 
-    let line3 = if (file_comment file) = "" then "" else
-        Printf.sprintf "\r\nComment: %s\r\n" (file_comment file)
+    let line3 = if (file_comment file) = [] then "" else
+	let buf = Buffer.create 1000 in
+        Printf.bprintf buf "\r\nComments:\r\n";
+	List.iter (fun s -> Printf.bprintf buf "%s\r\n" s;) (file_comment file);
+	Buffer.contents buf
     in
 
     let subject = if !!filename_in_subject then
-        Printf.sprintf "[mldonkey] file received - %s" (file_best_name file)
+        Printf.sprintf "[mldonkey@%s] file received - %s" (Unix.gethostname ()) (file_best_name file)
       else
-        Printf.sprintf "mldonkey, file received"
+        Printf.sprintf "mldonkey@%s, file received" (Unix.gethostname ())
     in
 
     let incoming =
@@ -284,9 +356,6 @@ let mail_for_completed_file file =
       } in
     M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
 
-let chat_for_completed_file file =
-  CommonChat.send_warning_for_downloaded_file (file_best_name file)
-  
 let file_completed (file : file) =
   try
     let impl = as_file_impl file in
@@ -294,19 +363,12 @@ let file_completed (file : file) =
         files =:= List2.removeq file !!files;
         done_files =:= file :: !!done_files;
         update_file_state impl FileDownloaded;
-        let file_name = file_disk_name file in
-        let file_id = Filename.basename file_name in
-        ignore (CommonShared.new_shared "completed" 0 (
-            file_best_name file ) file_name);
         (try mail_for_completed_file file with e ->
-              lprintf_nl "[cInt] Exception %s in sendmail" (Printexc2.to_string e);
+              lprintf_nl "Exception %s in sendmail" (Printexc2.to_string e);
               );
-        if !!CommonOptions.chat_warning_for_downloaded then
-          chat_for_completed_file file;
-
       end
   with e ->
-      lprintf_nl "[cInt] Exception in file_completed: %s" (Printexc2.to_string e)
+      lprintf_nl "Exception in file_completed: %s" (Printexc2.to_string e)
 
 let file_add impl state =
   try
@@ -331,13 +393,18 @@ let file_add impl state =
       lprintf_nl "[cInt] Exception in file_add: %s" (Printexc2.to_string e)
 
 let server_remove server =
+  begin
+    match server_state server with
+      NotConnected _ -> ()
+    | _ -> server_disconnect server
+  end;
   try
     let impl = as_server_impl server in
     if impl.impl_server_state <> RemovedHost then begin
         set_server_state server RemovedHost;
         (try impl.impl_server_ops.op_server_remove impl.impl_server_val
           with _ -> ());
-        servers =:= Intmap.remove (server_num server) !!servers;
+        servers =:= Intmap.remove (server_num server) !!servers
       end
   with e ->
       lprintf_nl "[cInt] Exception in server_remove: %s" (Printexc2.to_string e)
@@ -357,7 +424,8 @@ let friend_add c =
       client_must_update c;
       friends =:= c :: !!friends;
       contacts := List2.removeq c !contacts;
-      impl.impl_client_ops.op_client_browse impl.impl_client_val true
+      if network_is_enabled ((as_client_impl c).impl_client_ops.op_client_network) then
+        impl.impl_client_ops.op_client_browse impl.impl_client_val true
     end
 
 (* Maybe we should not add the client to the contact list and completely remove
@@ -379,7 +447,7 @@ let friend_remove c =
       end
 
   with e ->
-      lprintf_nl "[cInt] Exception in friend_remove: %s" (Printexc2.to_string e)
+      lprintf_nl "Exception in friend_remove: %s" (Printexc2.to_string e)
 
 let contact_add c =
   let impl = as_client_impl c in
@@ -387,7 +455,8 @@ let contact_add c =
       set_contact c;
       client_must_update c;
       contacts := c :: !contacts;
-      impl.impl_client_ops.op_client_browse impl.impl_client_val true
+      if network_is_enabled ((as_client_impl c).impl_client_ops.op_client_network) then
+        impl.impl_client_ops.op_client_browse impl.impl_client_val true
     end
 
 let contact_remove c =
@@ -400,7 +469,7 @@ let contact_remove c =
         impl.impl_client_ops.op_client_clear_files impl.impl_client_val
       end
   with e ->
-      lprintf_nl "[cInt] Exception in contact_remove: %s" (Printexc2.to_string e)
+      lprintf_nl "Exception in contact_remove: %s" (Printexc2.to_string e)
 
 let exit_counter = ref 0
 let exit_timer = ref false
@@ -433,7 +502,7 @@ let display_vd = ref false
 let display_bw_stats = ref false
 
 let start_download file =
-
+  if !!pause_new_downloads then file_pause file;
   if !!file_started_cmd <> "" then
       MlUnix.fork_and_exec  !!file_started_cmd
       [|
@@ -497,11 +566,11 @@ let print_connected_servers o =
            if use_html_mods o then begin
                Printf.bprintf buf "\\<div class=servers\\>";
                html_mods_table_one_row buf "serversTable" "servers" [
-                 ("", "srh", Printf.sprintf "--- Connected to %d servers on the %s network ---\n"
+                 ("", "srh", Printf.sprintf (_b "--- Connected to %d servers on the %s network ---\n")
                    (List.length list) r.network_name); ]
              end
            else
-             Printf.bprintf buf "--- Connected to %d servers on the %s network ---\n"
+             Printf.bprintf buf (_b "--- Connected to %d servers on the %s network ---\n")
                (List.length list) r.network_name;
            if use_html_mods o then Printf.bprintf buf "\\</div\\>";
           end;
@@ -516,8 +585,8 @@ let print_connected_servers o =
         if Autoconf.donkey = "yes" && r.network_name = "Donkey" && not !!enable_servers then
           begin
             if use_html_mods o then Printf.bprintf buf "\\<div class=servers\\>";
-            Printf.bprintf buf "You disabled server usage, therefore you are not able to connect ED2K servers.\n";
-            Printf.bprintf buf "To use servers again 'set enable_servers true'\n";
+            Printf.bprintf buf (_b "You disabled server usage, therefore you are not able to connect ED2K servers.\n");
+            Printf.bprintf buf (_b "To use servers again 'set enable_servers true'\n");
             if use_html_mods o then Printf.bprintf buf "\\</div\\>"
           end;
        with e ->
@@ -743,7 +812,7 @@ let all_active_network_opfile_network_names () =
        !names
 
 let apply_on_fully_qualified_options name f =
-  if !verbose then lprintf_nl "[cInt]Change option %s" name;
+  if !verbose then lprintf_nl "Change option %s" name;
   let rec iter prefix opfile =
     let args = simple_options prefix opfile in
     List.iter (fun o ->
@@ -766,7 +835,7 @@ let apply_on_fully_qualified_options name f =
               false
             with Exit -> true
         )) then begin
-        lprintf_nl "[cInt]Could not set option %s" name;
+        lprintf_nl "Could not set option %s" name;
         raise Not_found
       end
   with Exit -> ()
@@ -824,7 +893,7 @@ let keywords_of_query query =
           | _ -> ()
         end
     | QNone ->
-        lprintf_nl "[cInt]start_search: QNone in query";
+        lprintf_nl "start_search: QNone in query";
         ()
   in
   iter query;
@@ -881,73 +950,84 @@ open CommonFile
 
 let force_download_quotas () =
   let files = List.sort (fun f1 f2 ->
-        let v = file_priority f2 - file_priority f1 in
-        if v <> 0 then v else begin
-          (**
-            * [egs] do not start downloading
-            * a small file against an already active download
-            **)
-          let d1 = file_downloaded f1 in
-          let d2 = file_downloaded f2 in
-            if (d1=0L ) && (d2 > 0L)
-            then 1
-            else if ( d2=0L ) && (d1 > 0L)
-            then -1
-            else begin
-              (* Try to download in priority files with fewer bytes missing
-               Rationale: once completed, it may allow to recover some disk space *)
-              let r1 = file_size f1 -- d1 in
-              let r2 = file_size f2 -- d2 in
-                if r1 = r2 then 0 else
-                  if r2 < r1 then 1 else -1
-            end
-        end
-  )
-    !!CommonComplexOptions.files in
+    let v = file_priority f2 - file_priority f1 in
+    if v <> 0 then v else
+      (**
+	 * [egs] do not start downloading
+         * a small file against an already active download
+      **)
+      let d1 = file_downloaded f1 in
+      let d2 = file_downloaded f2 in
+      if d1 = 0L && d2 > 0L then 1
+      else 
+	if d1 > 0L && d2 = 0L then -1
+        else 
+          (* Try to download in priority files with fewer bytes missing
+             Rationale: once completed, it may allow to recover some disk space *)
+          let r1 = file_size f1 -- d1 in
+          let r2 = file_size f2 -- d2 in
+          if r1 = r2 then 0 else
+            if r2 < r1 then 1 else -1
+    ) !!CommonComplexOptions.files in
 
+  (** move running and queued downloads from [list] to [files]
+      accumulator, until a drop of priority (or end of list) is
+      encountered; Then submit the batches of downloads with same
+      priority to [iter_line].
+
+      @param ndownloads number of running downloads no longer in [list]
+      @param nqueued number of queued downloads in [files]
+  *)
   let rec iter list priority files ndownloads nqueued =
     match list, files with
-      [], [] -> ()
+    | [], [] -> ()
     | [], _ ->
         iter_line list priority files ndownloads nqueued
     | f :: tail , _ :: _ when file_priority f < priority ->
         iter_line list priority files ndownloads nqueued
     | f :: tail, files ->
         match file_state f with
-          FileDownloading ->
+        | FileDownloading ->
             iter tail (file_priority f) (f :: files) (ndownloads+1) nqueued
         | FileQueued ->
             iter tail (file_priority f) (f :: files) ndownloads (nqueued+1)
         | _ ->
             iter tail (file_priority f) files ndownloads nqueued
 
+  (** queue or unqueue downloads from [files] list to match quotas *)
   and iter_line list priority files ndownloads nqueued =
     if ndownloads > !!max_concurrent_downloads then
       match files with
-        [] -> assert false
+      | [] -> assert false
       | f :: tail ->
           match file_state f with
-            FileDownloading ->
+          | FileDownloading ->
               set_file_state f FileQueued;
               iter_line list priority tail (ndownloads-1) nqueued
           | _ -> iter_line list priority tail ndownloads (nqueued-1)
     else
     if ndownloads < !!max_concurrent_downloads && nqueued > 0 then
       match files with
-        [] -> assert false
+      | [] -> assert false
       | f :: tail ->
           match file_state f with
-            FileQueued ->
+	  | FileQueued ->
               set_file_state f FileDownloading;
               iter_line list priority tail (ndownloads+1) (nqueued-1)
           | _ -> iter_line list priority tail ndownloads nqueued
     else
-      iter list priority [] ndownloads nqueued
+      iter list priority [] ndownloads 0
 
   in
-  iter files max_int [] 0 0
+  if not !all_temp_queued then
+    iter files max_int [] 0 0
+  else
+    List.iter (fun f ->
+      if file_state f = FileDownloading then
+	set_file_state f FileQueued
+    ) files
 
 let _ =
   option_hook max_concurrent_downloads (fun _ ->
-      force_download_quotas ()
+      ignore (force_download_quotas ())
   )
