@@ -103,6 +103,10 @@ let check_network_implementations () =
         lprintf_nl "op_network_info";
       if c.op_network_clean_exit == cc.op_network_clean_exit then 
         lprintf_nl "op_network_clean_exit";
+      if c.op_network_reset == cc.op_network_reset then 
+        lprintf_nl "op_network_reset";
+      if c.op_network_ports == cc.op_network_ports then 
+        lprintf_nl "op_network_ports";
   ) !networks_ops;
   lprint_newline ()
 
@@ -121,6 +125,8 @@ let network_server_of_option n s = n.op_network_server_of_option s
 let network_file_of_option n f = n.op_network_file_of_option f 
 let network_client_of_option n f = n.op_network_client_of_option f
 let network_clean_exit n = try n.op_network_clean_exit () with _ -> true
+let network_reset n = try n.op_network_reset () with _ -> ()
+let network_ports n = n.op_network_ports ()
 
 let networks_iter f =
   List.iter (fun r ->
@@ -262,7 +268,7 @@ let new_network shortname name flags =
       op_network_close_search = (fun _ -> ni_ok name "close_search");
       op_network_extend_search = (fun _ _ -> ni_ok name "extend search");
       op_network_clean_servers = (fun _ -> ni_ok name "clean servers");
-      op_network_parse_url = (fun _ -> ni_ok name "parse_url"; false);
+      op_network_parse_url = (fun _ -> ni_ok name "parse_url"; "", false);
       op_network_info = (fun _ -> fni name "network_info");
       op_network_connected = (fun _ -> ni_ok name "connected"; false);
       op_network_add_server = (fun _ -> fni name "op_network_add_server");
@@ -270,6 +276,8 @@ let new_network shortname name flags =
       op_network_download = (fun _ -> fni name "network_download");
       op_network_display_stats = (fun _ _ -> ni_ok name "display_stats");
       op_network_clean_exit = (fun _ -> true);
+      op_network_reset = (fun _ -> ni_ok name "reset");
+      op_network_ports = (fun _ -> ni_ok name "ports"; []);
     }
   in
   let rr = (Obj.magic r: network) in
@@ -278,10 +286,19 @@ let new_network shortname name flags =
   networks := r :: !networks;
   Hashtbl.add networks_by_name r.network_name r;
   Hashtbl.add networks_by_num r.network_num r;
+  let s =
+    if r.network_name = "Donkey" then
+      if Autoconf.donkey_sui_works () then
+        "Donkey (SUI)"
+      else
+        "Donkey (noSUI)"
+    else
+      r.network_name
+  in
   if !networks_string = "" then
-    networks_string := r.network_name
+    networks_string := s
   else
-    networks_string := Printf.sprintf "%s  %s" !networks_string r.network_name;
+    networks_string := Printf.sprintf "%s  %s" !networks_string s;
 (*  lprintf_nl "Network %s registered" r.network_name; *)
   r
   
