@@ -25,6 +25,14 @@ open Options
 open CommonUser
 open CommonTypes
 
+let log_prefix = "[cSe]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
+
 module G = GuiTypes
 
 type 'a server_impl = {
@@ -81,6 +89,32 @@ let dummy_server_impl = {
 
 let dummy_server = as_server dummy_server_impl
 
+let impl_server_info impl =
+  let module T = GuiTypes in
+  {
+    T.server_num = impl.impl_server_num;
+    T.server_state = impl.impl_server_state;
+
+    T.server_network = 0;
+    T.server_addr = Ip.addr_of_ip Ip.null;
+    T.server_port = 0;
+    T.server_realport = 0;
+    T.server_score = 0;
+    T.server_tags = [];
+    T.server_nusers = 0L;
+    T.server_nfiles = 0L;
+    T.server_name = "";
+    T.server_description = "";
+    T.server_users = None;
+    T.server_banner = "";
+    T.server_preferred = false;
+    T.server_version = "";
+    T.server_max_users = 0L;
+    T.server_soft_limit = 0L;
+    T.server_hard_limit = 0L;
+    T.server_lowid_users = 0L;
+    T.server_ping = 0;
+  }
 
 let server_num s =
   let s = as_server_impl s in
@@ -105,7 +139,6 @@ let _ =
   )
 
 let server_must_update s =
-(*  lprintf "server_must_update ?\n";  *)
   let impl = as_server_impl s in
   if impl.impl_server_update <> 0 then
     CommonEvent.add_event (Server_info_event s);
@@ -117,7 +150,6 @@ let server_must_update_state s =
     begin
       impl.impl_server_update <- - impl.impl_server_update;
       CommonEvent.add_event (Server_info_event s);
-(*      lprintf "server_must_update YES\n";  *)
     end
 
 let server_update_num impl =
@@ -192,10 +224,10 @@ let new_server_ops network =
   s
 
 let check_server_implementations () =
-  lprintf_nl "\n----- Methods not implemented for CommonServer ----\n";
+  lprintf_nl "----- Methods not implemented for CommonServer ----";
   List.iter (fun (c, cc) ->
       let n = c.op_server_network.network_name in
-      lprintf_nl "\n  Network %s\n" n;
+      lprintf_nl "  Network %s" n;
       if c.op_server_remove == cc.op_server_remove then
         lprintf_nl "op_server_remove";
       if c.op_server_to_option == cc.op_server_to_option then
@@ -231,7 +263,7 @@ let server_find (num : int) =
 
 let server_blocked s =
   let info = server_info s in
-    Ip_set.ip_blocked (Ip.ip_of_addr info.G.server_addr)
+  !Ip.banned (Ip.ip_of_addr info.G.server_addr) <> None
 
 let server_connect s =
   if not (server_blocked s) then
@@ -246,10 +278,6 @@ let server_state c =
   let impl = as_server_impl c in
   impl.impl_server_state
 
-let server_num c =
-  let impl = as_server_impl c in
-  impl.impl_server_num
-
 let set_server_state c state =
   let impl = as_server_impl c in
   if impl.impl_server_state <> state then begin
@@ -290,24 +318,42 @@ let servers_get_all () =
 
 let servers_by_num = ()
 
-(*
-type server_info = {
-    server_num : int;
-    server_network : int;
+let check_blocked_servers () =
+  try
+    server_iter (fun s ->
+      if server_blocked s then
+        begin
+          let impl = as_server_impl s in
+          let info = server_info s in
+          (match impl.impl_server_state with
+             NotConnected _ -> ()
+         | _ -> server_disconnect s;
+              lprintf_nl "Disconnected server %s (%s:%d), IP is now blocked"
+                info.G.server_name
+                (Ip.string_of_addr info.G.server_addr)
+                info.G.server_port);
+      end;
+      server_must_update s)
+  with
+    Not_found -> ()
+  | e -> lprintf_nl "Exception in check_blocked_servers: %s" (Printexc2.to_string e)
 
-    mutable server_ip : Ip.t;
-    mutable server_port : int;
-    mutable server_score : int;
-    mutable server_tags : CommonTypes.tag list;
-    mutable server_nusers : int;
-    mutable server_nfiles : int;
-    mutable server_state : host_state;
-    mutable server_name : string;
-    mutable server_description : string;
-    mutable server_users : int list option;
-  }
-    *)
+let disconnect_all_servers () =
+  try
+    server_iter (fun s ->
+     (match (as_server_impl s).impl_server_state with
+	NotConnected _ -> ()
+      | _ -> server_disconnect s; server_must_update s))
+  with
+    Not_found -> ()
+  | e -> lprintf_nl "Exception in disconnect_all_servers: %s" (Printexc2.to_string e)
 
+let server_must_update_all () =
+  try
+    server_iter (fun s ->
+      server_must_update s)
+  with e ->
+    lprintf_nl "Exception in server_must_update_all: %s" (Printexc2.to_string e)
 
 let server_banner s o =
   let buf = o.conn_buf in
@@ -316,34 +362,42 @@ let server_banner s o =
       info.G.server_banner
 
 let server_print_html_header buf ext =
-    html_mods_table_header buf "serversTable" (Printf.sprintf "servers%s" ext) [
-		( "1", "srh", "Server number", "#" ) ;
-		( "0", "srh", "Connect|Disconnect", "C/D" ) ;
-		( "0", "srh", "Remove", "Rem" ) ;
+    html_mods_table_header buf "serversTable" (Printf.sprintf "servers%s" ext) ([
+    ( "1", "srh", "Server number", "#" ) ;
+    ( "0", "srh", "Connect|Disconnect", "C/D" ) ;
+    ( "0", "srh", "Remove", "Rem" ) ;
     ( "0", "srh", "Preferred", "P" ) ;
-		( "0", "srh", "[Hi]gh or [Lo]w ID", "ID" ) ;
-		( "0", "srh", "Network name", "Network" ) ;
-		( "0", "srh", "Connection status", "Status" ) ;
-		( "0", "srh br", "IP address", "IP address" ) ;
-		( "1", "srh ar", "Number of connected users", "Users" ) ;
-		( "1", "srh ar br", "Number of files indexed on server", "Files" ) ;
-		( "0", "srh", "Server name", "Name" ) ;
-		( "0", "srh", "Server details", "Details" ) ]
+    ( "0", "srh", "[Hi]gh or [Lo]w ID", "ID" ) ;
+    ( "0", "srh", "Network name", "Network" ) ;
+    ( "0", "srh", "Connection status", "Status" ) ;
+    ] @ (if !Geoip.active then [( "0", "srh", "Country Code/Name", "CC" )] else []) @ [
+    ( "0", "srh br", "IP address", "IP address" ) ;
+    ( "1", "srh ar", "Number of connected users", "Users" ) ;
+    ( "1", "srh ar br", "Max number of users", "MaxUsers" ) ;
+    ( "1", "srh ar br", "LowID users", "LowID" ) ;
+    ( "1", "srh ar br", "Number of files indexed on server", "Files" ) ;
+    ( "1", "srh ar", "Soft file limit", "Soft" ) ;
+    ( "1", "srh ar br", "Hard file limit", "Hard" ) ;
+    ( "0", "srh ar br", "Ping (ms)", "Ping" ) ;
+    ( "0", "srh", "Server version", "Version" ) ;
+    ( "0", "srh", "Server name", "Name" ) ;
+    ( "0", "srh", "Server details", "Details" ) ])
 
 let server_print s o =
   let impl = as_server_impl s in
   let n = impl.impl_server_ops.op_server_network in
+  if network_is_enabled n then
   try
     let info =
       try server_info s with e ->
-          lprintf "Exception %s in server_info (%s)\n"
+          lprintf_nl "Exception %s in server_info (%s)\n"
             (Printexc2.to_string e) n.network_name;
           raise e in
-
+    let cc,cn = Geoip.get_country (Ip.ip_of_addr info.G.server_addr) in
     let buf = o.conn_buf in
-	
-	if use_html_mods o then begin
-	let snum = (server_num s) in
+  
+  if use_html_mods o then begin
+  let snum = (server_num s) in
 
     Printf.bprintf buf "
     \\<tr class=\\\"dl-%d\\\"\\>
@@ -354,36 +408,41 @@ let server_print s o =
     \\<td class=\\\"sr\\\" %s\\</td\\>
     \\<td class=\\\"sr\\\"\\>%s\\</td\\>
     \\<td class=\\\"sr\\\"\\>%s\\</td\\>
+    %s
     \\<td class=\\\"sr br\\\"\\>%s:%s\\</td\\>
     \\<td class=\\\"sr ar\\\"\\>%Ld\\</td\\>
     \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%d\\</td\\>
+    \\<td class=\\\"sr br\\\"\\>%s\\</td\\>
     \\<td class=\\\"sr\\\"\\>%s\\</td\\>
     \\<td width=\\\"100%%\\\" class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>\n"
     (html_mods_cntr ())
-	  (
-        Printf.sprintf "%s"
-        (match impl.impl_server_state with
+    (match impl.impl_server_state with
         Connected _ -> Printf.sprintf "title=\\\"Server Banner\\\"
-						onMouseOver=\\\"mOvr(this);\\\"
-						onMouseOut=\\\"mOut(this);\\\"
-						onClick=\\\"location.href='submit?q=server_banner+%d'\\\"" snum
+            onMouseOver=\\\"mOvr(this);\\\"
+            onMouseOut=\\\"mOut(this);\\\"
+            onClick=\\\"location.href='submit?q=server_banner+%d'\\\"" snum
         | _ -> "")
-	  )
-	  snum
-      (if server_blocked s then
-        Printf.sprintf "\\<TD class=\\\"srb\\\"----\\</TD\\>"
-       else
+    snum
+      (
+        if server_blocked s && (match impl.impl_server_state with
+         NotConnected _ -> true
+             | _ -> false) then "\\<td class=\\\"srb\\\"\\>blk\\</td\\>" else
         Printf.sprintf
         "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
         onMouseOut=\\\"mOut(this);\\\" title=\\\"Connect|Disconnect\\\"
         onClick=\\\"parent.fstatus.location.href='submit?q=%s+%d'\\\"\\>%s\\</TD\\>"
-      (match impl.impl_server_state with
-        NotConnected _ -> "c"
-      | _ -> "x")
-      snum
-      (match impl.impl_server_state with
-        NotConnected _ -> "Conn"
-      | _ -> "Disc")
+        (match impl.impl_server_state with
+           NotConnected _ -> "c"
+         | _ -> "x")
+        snum
+        (match impl.impl_server_state with
+           NotConnected _ -> "Conn"
+         | _ -> "Disc")
       )
       (
         Printf.sprintf
@@ -438,41 +497,52 @@ let server_print s o =
        else "\\>"
       )
       n.network_name
-      (if server_blocked s then
-	"IP blocked"
-      else
-	(string_of_connection_state impl.impl_server_state))
+      (match impl.impl_server_state with
+        NotConnected _ -> if server_blocked s then "IP blocked"
+        else (string_of_connection_state impl.impl_server_state)
+      | _ -> (string_of_connection_state impl.impl_server_state))
+      (if !Geoip.active then 
+	 Printf.sprintf "\\<td class=\\\"sr\\\" title=\\\"%s\\\" \\>%s\\</td\\>" cn cc
+       else "")
       (Ip.string_of_addr info.G.server_addr)
       (Printf.sprintf "%s%s"
        (string_of_int info.G.server_port)
-       (if info.G.server_realport <> 0 then
-          "(" ^ (string_of_int info.G.server_realport) ^ ")" else ""))
+       (if info.G.server_realport <> 0 
+          then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
+          else ""))
       info.G.server_nusers
+      info.G.server_max_users
+      info.G.server_lowid_users
       info.G.server_nfiles
+      info.G.server_soft_limit
+      info.G.server_hard_limit
+      info.G.server_ping
+      info.G.server_version
       info.G.server_name
       info.G.server_description
 
-	end
+  end
    else
-	begin
+  begin
 
         Printf.bprintf buf "[%-10s%5d] %15s:%-10s %s\n%45sUsers:%-8Ld Files:%-8Ld State:%s\n"
           (n.network_name)
           (server_num s)
           (Ip.string_of_addr info.G.server_addr)
           (Printf.sprintf "%s%s"
-	   (string_of_int info.G.server_port)
-           (if info.G.server_realport <> 0 then
-	      "(" ^ (string_of_int info.G.server_realport) ^ ")" else ""))
-          (info.G.server_name) ("")
+          (string_of_int info.G.server_port)
+          (if info.G.server_realport <> 0 
+            then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
+            else ""))
+          (info.G.server_name)
+	  (if !Geoip.active then Printf.sprintf "%33s/%-2s%9s" cn cc "" else "")
           (info.G.server_nusers)
           (info.G.server_nfiles)
-          (if server_blocked s then
-	    "IP blocked"
-          else
-	    (string_of_connection_state impl.impl_server_state));
+          (if server_blocked s 
+            then "IP blocked" 
+            else (string_of_connection_state impl.impl_server_state));
       end;
 
   with e ->
-      lprintf "Exception %s in CommonServer.server_print\n"
+      lprintf_nl "Exception %s in CommonServer.server_print"
         (Printexc2.to_string e)
