@@ -213,7 +213,6 @@ module Make(M:
           mutable manager_active_sources : int;
           mutable manager_all_sources : int;
           mutable manager_file : (unit -> file);
-          mutable manager_brothers : file_sources_manager list;
         }
 
       and functions = {
@@ -467,7 +466,6 @@ let rec find_max_overloaded q managers =
 
 
        let print buf output_type =
-        let timer = Unix.localtime (float_of_int(last_time ()) +. 1000000000.) in
         let pos_to_string v =
           (if v > 0 then string_of_int(v) else "-")
         in
@@ -493,7 +491,7 @@ let rec find_max_overloaded q managers =
             Printf.bprintf buf "\\<tr\\>";
             html_mods_td buf [
               ("", "srh", "Statistics on sources ");
-              ("", "srh", "@ " ^(Date.time_to_string_short timer));
+              ("", "srh", "@ " ^ log_time ());
               ("", "srh", header); ];
             Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>\n";
 
@@ -524,7 +522,6 @@ let rec find_max_overloaded q managers =
 
         let nsources_per_queue = Array.create nqueues 0 in
         let nready_per_queue = Array.create nqueues 0 in
-        let nready_throttled_per_queue = Array.create nqueues 0 in
         let nindirect_per_queue = Array.create nqueues 0 in
         let ninvalid_per_queue = Array.create nqueues 0 in
         let nall = ref 0 in
@@ -1253,7 +1250,6 @@ let rec find_max_overloaded q managers =
             manager_all_sources = 0;
             manager_active_sources = 0;
             manager_sources = create_queues ();
-            manager_brothers = [];
           } in
         file_sources_managers := m :: !file_sources_managers;
         m
@@ -1274,9 +1270,6 @@ let rec find_max_overloaded q managers =
 
         m.manager_sources <- create_queues ();
 
-        let brothers = List.filter (fun m' -> m' <> m) m.manager_brothers in
-        List.iter (fun m -> m.manager_brothers <- brothers) brothers;
-
         file_sources_managers := List2.removeq m !file_sources_managers
 
 
@@ -1446,14 +1439,7 @@ let rec find_max_overloaded q managers =
       let set_request_result s file result =
         set_request_score s file (match result with
             File_not_found -> not_found_score
-          | File_found ->
-(* Advertise the files associated with this file that this source is
-probably interesting. Since we are already connected, it means
-we will probably query for the other file almost immediatly. *)
-              List.iter (fun m ->
-                  set_request_score s m initial_new_source_score
-              ) file.manager_brothers;
-              found_score
+          | File_found -> found_score
           | File_chunk -> chunk_score
           | File_upload -> upload_score
           | File_new_source -> initial_new_source_score
@@ -1519,35 +1505,6 @@ we will probably query for the other file almost immediatly. *)
                   ) s.source_files
 
 
-(*************************************************************************)
-(*                                                                       *)
-(*                         set_brothers                                  *)
-(*                                                                       *)
-(*************************************************************************)
-
-      let set_brothers files =
-        let brothers = ref [] in
-        let rec add_brother m =
-          if not (List.memq m !brothers) then begin
-              brothers := m :: !brothers;
-              List.iter add_brother m.manager_brothers
-            end
-        in
-        List.iter add_brother files;
-
-        List.iter (fun m ->
-            m.manager_brothers <- !brothers
-        ) !brothers
-
-(*************************************************************************)
-(*                                                                       *)
-(*                         get_brothers                                  *)
-(*                                                                       *)
-(*************************************************************************)
-
-      let get_brothers file =
-        List.map (fun m -> m.manager_uid) file.manager_brothers
-
 (*************************************************************************)
 (*                                                                       *)
 (*                         add_saved_source_request                      *)
@@ -1561,9 +1518,8 @@ we will probably query for the other file almost immediatly. *)
           try
             functions.function_string_to_manager uid
           with e ->
-              if !verbose then begin
+              if !verbose_sources > 0 then
                   lprintf_nl "[cSrc] CommonSources: add_saved_source_request -> %s not found" uid;
-                end;
               raise e
         in
         let r = add_request s file time in
@@ -2218,6 +2174,9 @@ connected if needed *)
                ((fun (_,_,_,_,_,n) -> n)(H.stats sources_by_num))
                ;
 
+            Printf.bprintf buf "Used indirect connections: %d\n"
+              !indirect_connections;
+
             let nconnected = ref 0 in
             Fifo.iter (fun (_,s) ->
                 if s.source_last_attempt = 0 then incr nconnected;
