@@ -287,7 +287,7 @@ let client_friend_tag = 1
 let client_contact_tag = 2
 let client_nolimit_tag = 4
 let client_initialized_tag = 8
-let client_browsed_tag = client_contact_tag lor client_friend_tag
+let client_must_browse_tag = 16
 
 type query_entry = 
   Q_AND of query_entry list
@@ -375,6 +375,7 @@ type sortvd_type =
 | ByASources
 | ByNet
 | ByAvail
+| ByComments
 | NotSorted
   
 type room_state = 
@@ -401,6 +402,7 @@ type result_info = {
 
     mutable result_time : int;
     mutable result_modified : bool;
+    mutable result_source_network : int;
   }
 
 type result = {
@@ -478,7 +480,7 @@ type network = {
     mutable op_network_share : (
       string -> string -> int64 -> unit);
     mutable op_network_private_message : (string -> string -> unit);
-    mutable op_network_parse_url : (string -> bool);
+    mutable op_network_parse_url : (string -> string * bool);
     mutable op_network_connect_servers : (unit -> unit);
     
     mutable op_network_search : (search -> Buffer.t -> unit);
@@ -495,6 +497,8 @@ type network = {
     mutable op_network_download : (result_info -> file);
     mutable op_network_display_stats : (Buffer.t -> ui_conn -> unit);
     mutable op_network_clean_exit : (unit -> bool);
+    mutable op_network_reset : (unit -> unit);
+    mutable op_network_ports : (unit -> (int * string) list);
   }
 
 and   ui_user = {
@@ -657,14 +661,14 @@ let ogg_tag_to_string ogg_tag =
 | Ogg_audio_avgbytespersec r -> Printf.sprintf "%.0f kbps" (r *. 8. /. 1000.)
 | Ogg_vorbis_version r -> Printf.sprintf "vorbis version: %.0f" r
 | Ogg_vorbis_sample_rate r -> Printf.sprintf "%.0f Hz" r
-| Ogg_vorbis_bitrates l -> Printf.sprintf "%s" (vorbis_bitrates_to_string l)
+| Ogg_vorbis_bitrates l -> vorbis_bitrates_to_string l
 | Ogg_vorbis_blocksize_0 n -> Printf.sprintf "blocksize_0: %d" (int_of_float (2. ** float_of_int n))
 | Ogg_vorbis_blocksize_1 n -> Printf.sprintf "blocksize_1: %d" (int_of_float (2. ** float_of_int n))
 | Ogg_video_width r -> Printf.sprintf "width: %.0f pixels" r
 | Ogg_video_height r -> Printf.sprintf "height: %.0f pixels" r
 | Ogg_video_sample_rate r -> Printf.sprintf "%.3f fps" r
 | Ogg_aspect_ratio r -> Printf.sprintf "1:%0.5f" r
-| Ogg_theora_cs cs -> Printf.sprintf "%s" (theora_cs_to_string cs)
+| Ogg_theora_cs cs -> theora_cs_to_string cs
 | Ogg_theora_quality n -> Printf.sprintf "%d [quality]" n
 | Ogg_theora_avgbytespersec n -> Printf.sprintf "%d kbps" (n / 1000)
 
@@ -700,7 +704,7 @@ and history_result = {
 
 and location_kind = 
   Known_location of Ip.t * int
-| Indirect_location of string * Md4.t
+| Indirect_location of string * Md4.t * Ip.t * int
   
 (*
 We should add information about what has already been sent to the GUI to
@@ -844,13 +848,24 @@ type arg_kind =
 | Arg_three of (string -> string -> string -> arg_handler)
 
     
-let  string_of_kind kind =
+let string_of_kind kind =
   try
     match kind with
     | Known_location (ip,port) -> Ip.to_string ip
-    | _ -> "firewalled"
+    | Indirect_location (server_ip, server_port, ip, port) -> Ip.to_string ip
   with _ -> ""
 
+let string_of_kind_geo kind =
+  try
+    match kind with
+    | Known_location (ip,port) -> 
+        let cc,cn = Geoip.get_country ip in
+        (Ip.to_string ip),cc,cn
+    | Indirect_location (server_ip, server_port, ip, port) ->
+        let cc,cn = Geoip.get_country ip in
+        (Ip.to_string ip),cc,cn
+  with _ -> "","X","Country error"
+
 type brand_stat = {
   mutable brand_seen : int;
   mutable brand_banned : int;
@@ -867,3 +882,8 @@ let dummy_stats =
     brand_download = 0L;
     brand_upload = 0L;
   }
+
+type kind_type = {
+  f : string -> string -> unit;
+  description : string
+}
