@@ -33,6 +33,14 @@ open TcpBufferedSocket
 open CommonGlobals
 open CommonOptions
 
+let log_prefix = "[cUp]"
+
+let lprintf_nl fmt = 
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
+
 (*
 PROBLEMS: most of the time, users won't share their files on all networks.
 We should provide a different directory than incoming/, where files
@@ -231,6 +239,13 @@ let _ =
         network_downloaded = Int64.zero;
         network_connected = 0;
       });
+  network.op_network_ports <- (fun _ ->
+    [
+    !!http_port, "http_port";
+    !!telnet_port, "telnet_port";
+    !!gui_port, "gui_port";
+    !!gift_port, "gift_port";
+    ]);
   network.op_network_connected_servers <- (fun _ -> [])
 
 let (shared_ops : shared_file CommonShared.shared_ops) =
@@ -241,11 +256,9 @@ let shareds_by_uid = Hashtbl.create 13
 let shareds_by_id = Hashtbl.create 13
 
 let add_by_uid uid sh =
-  let urn = Uid.to_string uid in
   Hashtbl.add shareds_by_uid uid sh
 
 let find_by_uid uid =
-  let urn = Uid.to_string uid in
   Hashtbl.find shareds_by_uid uid
 
 module SharedFilesIndex = IndexedSharedFiles.MakeIndex (struct
@@ -441,7 +454,7 @@ let rec start_job_for sh (wanted_id, handler) =
             CommonHasher.compute_sha1 (Unix32.filename sh.shared_fd)
             zero info.shared_size (fun job ->
                 if job.CommonHasher.job_error then begin
-                    lprintf_nl "[cUp] Error during hashing of %s" info.shared_fullname;
+                    lprintf_nl "Error during hashing of %s" info.shared_fullname;
                     current_job := None;
                   end else
                   begin
@@ -501,8 +514,8 @@ computation ??? *)
     | MD5EXT ->
         let md5ext =
           try
-            let fd = Unix32.create_rw info.shared_fullname in
-            let file_size = Unix32.getsize64 fd false in
+            let fd = Unix32.create_ro info.shared_fullname in
+            let file_size = Unix32.getsize64 fd in
             let len64 = min 307200L file_size in
             let len = Int64.to_int len64 in
             let s = String.create len in
@@ -522,7 +535,6 @@ computation ??? *)
     | ED2K ->
         let size = info.shared_size in
         let chunk_size = ed2k_block_size  in
-        let nhashes = Int64.to_int (size // chunk_size) + 1 in
         let rec iter pos hashes =
           if pos < size then
             try
@@ -530,7 +542,7 @@ computation ??? *)
                 pos (min (size -- pos) chunk_size)
               (fun job ->
                   if job.CommonHasher.job_error then begin
-                      lprintf_nl "[cUp] Error during hashing of %s" info.shared_fullname;
+                      lprintf_nl "Error during hashing of %s" info.shared_fullname;
                       current_job := None;
                     end else begin
 
@@ -543,7 +555,6 @@ computation ??? *)
           let list = List.rev hashes in
           let ed2k = md4_of_list list in
           let uid = Uid.create (Ed2k (ed2k)) in
-          let urn = Uid.to_string uid in
           info.shared_md4s <- Array.of_list list;
           info.shared_uids <- uid :: info.shared_uids;
           IndexedSharedFiles.update_result sh.shared_info info;
@@ -559,14 +570,13 @@ computation ??? *)
 
           let size = info.shared_size in
           let chunk_size = tiger_block_size in
-          let nhashes = Int64.to_int (size // chunk_size) + 1 in
           let rec iter pos hashes =
             if pos < size then
                 CommonHasher.compute_tiger info.shared_fullname
                   pos (min (size -- pos) chunk_size)
               (fun job ->
                     if job.CommonHasher.job_error then begin
-                      lprintf_nl "[cUp] Error during hashing of %s"
+                      lprintf_nl "Error during hashing of %s"
                       info.shared_fullname;
                         current_job := None;
                       end else begin
@@ -603,7 +613,7 @@ let shared_files_timer _ =
             [] ->  waiting_shared_files := tail;
           | uid :: tail ->
               if !verbose_share then
-                lprintf_nl "[cUp] shared_files_timer: starting job";
+                lprintf_nl "shared_files_timer: starting job";
               sh.shared_uids_wanted <- tail;
               current_job := Some sh;
               start_job_for sh uid
@@ -685,6 +695,7 @@ let add_shared full_name codedname size =
           impl_shared_ops = shared_ops;
           impl_shared_val = sh;
           impl_shared_requests = 0;
+          impl_shared_magic = None
         }
       and sh = {
           shared_info = index;
@@ -823,7 +834,7 @@ let ready_for_upload c =
 
 let add_pending_slot c =
   if client_has_a_slot c then begin
-      if !verbose_upload then lprintf_nl "[cUp] Avoided inserting an uploader in pending slots!"
+      if !verbose_upload then lprintf_nl "Avoided inserting an uploader in pending slots!"
     end
   else
   if not (Intmap.mem (client_num c) !pending_slots_map) then
@@ -864,18 +875,26 @@ let add_pending_slot c =
   let csh = client_upload c in
   let cdir = shared_dir csh in
   let cprio = ref (shared_prio csh) in
-  let cfriend = ref (if !!friends_upload_slot then 1 else 0) in
+  let cfriend = ref (if is_friend c && !!friends_upload_slot then 1 else 0) in
+  let csmallfiles = ref (match csh with 
+    | None -> 0
+    | Some sh -> if shared_size sh <= !!small_files_slot_limit then 1 else 0) in
   (* if cdir <> "" then
     lprintf "Testing cdir %s\n" cdir; *)
   Intmap.iter (fun _ c ->
     let sh = client_upload c in
     if shared_dir sh = cdir then decr cprio;
     if client_has_a_friend_slot c then decr cfriend;
+    match sh with
+      | None -> ()
+      | Some sh ->
+	  if shared_size sh <= !!small_files_slot_limit then
+	    decr csmallfiles;
   ) !CommonClient.uploaders;
   (* if cdir <> "" then
-    lprintf "Testing cprio %d cfriend %d\n" !cprio !cfriend; *)
-  if !cprio > 0 || 
-     (is_friend c && !cfriend > 0) then begin
+     lprintf "Testing cprio %d cfriend %d csmallfiles\n" 
+     !cprio !cfriend !csmallfiles; *)
+  if !cprio > 0 || !cfriend > 0 || !csmallfiles > 0 then begin
     remove_pending_slot c;
     if client_is_connected c then begin
       set_client_has_a_slot c true;
@@ -906,7 +925,7 @@ let dynamic_refill_upload_slots () =
   let open_slots n =
     let i = ref n in
     if !verbose_upload then
-      lprintf_nl "[cUp] try to allocate %d more slots" n;
+      lprintf_nl "try to allocate %d more slots" n;
     while !i > 0 do
       find_pending_slot ();
       decr i
@@ -921,7 +940,7 @@ let dynamic_refill_upload_slots () =
     (* max_hard_upload_rate lowered manually,... *)
     mini estimated_capacity (!!max_hard_upload_rate * 1024) in
   if !verbose_upload then
-    lprintf_nl "[cUp] usage: %d(%d) capacity: %d"
+    lprintf_nl "usage: %d(%d) capacity: %d"
       (short_delay_upload_usage ())
       (upload_usage ())
       estimated_capacity;
@@ -931,13 +950,13 @@ let dynamic_refill_upload_slots () =
 (* enough free bw for another slot *)
     if short_delay_upload_usage () + slot_bw < estimated_capacity then begin
       if !verbose_upload then
-	lprintf_nl "[cUp] uplink not fully used";
+  lprintf_nl "uplink not fully used";
       incr not_saturated_count
     end else reset_state ();
 
     if len < min_upload_slots then begin
       if !verbose_upload then
-	lprintf_nl "[cUp] too few upload slots";
+  lprintf_nl "too few upload slots";
       open_slots (min_upload_slots - len);
       reset_state ()
     end else if !not_saturated_count >= 2 then begin
@@ -1017,10 +1036,10 @@ let queue_download_request f len =
 let upload_download_timer () =
   (try download_engine ()
     with e ->
-        lprintf_nl "[cUp] Exception %s in download_engine" (Printexc2.to_string e)
+        lprintf_nl "Exception %s in download_engine" (Printexc2.to_string e)
   );
   (try next_uploads ()
-  with e -> lprintf_nl "[cUp] exc %s in upload" (Printexc2.to_string e))
+  with e -> lprintf_nl "exc %s in upload" (Printexc2.to_string e))
 
 let words_of_filename =
   let extension_list = [
@@ -1043,24 +1062,8 @@ let words_of_filename =
   let get_name_keywords file_name =
     match remove_short (String2.stem file_name) [] with
       [] | [_] ->
-        lprintf_nl "[cUp] Not enough keywords to recover %s" file_name;
+        lprintf_nl "Not enough keywords to recover %s" file_name;
         [file_name]
     | l -> l
   in
   get_name_keywords
-
-open LittleEndian
-
-let _ =
-  CommonWeb.add_redirector_info "SHARED" (fun buf ->
-      let module S = CommonShared in
-      let total_shared = ref Int64.zero in
-      let total_uploaded = ref Int64.zero in
-      S.shared_iter (fun s ->
-          let i = S.as_shared_impl s in
-          total_uploaded := !total_uploaded ++ i.S.impl_shared_uploaded;
-          total_shared := !total_shared ++ i.S.impl_shared_size
-      );
-      buf_int64 buf !total_shared;
-      buf_int64 buf !total_uploaded;
-  )
