@@ -79,6 +79,22 @@ let get_string s pos =
 let get_string_bin s pos =
   get_string s pos
     
+let get_ip2 proto s pos =
+  let ip,pos = get_ip s pos,(pos+4) in 
+  if proto > 37 then
+    let _ = get_uint8 s pos in 
+    ip,(pos+1)
+  else
+   ip,pos
+
+let get_hostname proto s pos =
+  let hn,pos = get_string s pos in
+  if proto > 37 then
+    let _ = get_uint8 s pos in
+    hn,(pos+1)
+  else
+    hn,pos
+    
 let get_list f s pos =
   let len = get_int16 s pos in
   let rec iter n pos =
@@ -89,6 +105,16 @@ let get_list f s pos =
   in
   iter len (pos+2)
 
+let get_list2 proto f s pos =
+  let len = get_int16 s pos in
+  let rec iter n pos =
+    if n = 0 then [],pos else
+    let head, pos = f proto s pos in
+    let tail, pos = iter (n-1) pos in
+    head :: tail, pos
+  in
+  iter len (pos+2)
+
 let get_array f s pos = 
   let list, pos = get_list f s pos in
   Array.of_list list, pos
@@ -372,7 +398,7 @@ let get_tag_name s pos =
   let tag_name = field_of_string name in
   tag_name, pos
   
-let get_tag s pos =
+let get_tag proto s pos =
   let tag_name, pos = get_tag_name s pos in
   let value, pos =
     match get_uint8 s pos with
@@ -382,7 +408,8 @@ let get_tag s pos =
         Fint64 (get_uint64_32 s (pos+1)), pos+5
     | 2 -> let s, pos = get_string s (pos+1) in
         String s, pos
-    | 3 -> Addr (get_ip s (pos+1)), pos+5
+    | 3 -> let ip, pos = get_ip2 proto s (pos+1) in
+        Addr ip, pos
     | 4 -> Uint16 (get_int16 s (pos+1)), pos+3
     | 5 -> Uint8 (get_uint8 s (pos+1)), pos+2
     | 6 -> Pair (get_uint64_32 s (pos+1), get_uint64_32 s (pos+5)), pos+9
@@ -392,7 +419,7 @@ let get_tag s pos =
 
 let get_result proto s pos =
   let num  = get_int s pos in
-  let net = get_int s (pos+4) in
+(*  let net = get_int s (pos+4) in *)
   let names, pos = get_list get_string s (pos+8) in
   let uids, pos = 
     if proto < 27 then
@@ -403,7 +430,7 @@ let get_result proto s pos =
   let size, pos = get_uint64_2 proto s pos in
   let format, pos = get_string s pos in
   let t, pos = get_string s pos in
-  let tags, pos = get_list get_tag s pos in
+  let tags, pos = get_list2 proto get_tag s pos in
   let comment, pos = get_string s pos in
   let already_done = get_bool  s pos in
   let time, pos = 
@@ -426,6 +453,7 @@ let get_result proto s pos =
     result_modified = false;
     result_force = false;
     result_time = time;
+    result_source_network = 0;
   }, pos+1
 
 let get_message s pos =
@@ -478,6 +506,13 @@ let get_int_date proto s pos =
 let get_int_pos s pos =
   get_int s pos, pos + 4
   
+let get_sub_files s pos =
+  get_list (fun s pos ->
+    let name, pos = get_string s pos in
+    let size, pos = get_int64 s pos, pos+8 in
+    (name, size, Some ""), pos
+  ) s pos
+  
 let get_file proto s pos = 
   let num = get_int s pos in
   let net = get_int s (pos+4) in
@@ -513,7 +548,6 @@ let get_file proto s pos =
   let comment, pos = if proto > 21 then
       get_string s pos
     else "", pos in
-  let names = List.map (fun name -> name, noips()) names in
   let last_seen = BasicSocket.last_time () - last_seen in
   let uids, pos = 
     if proto < 31 then
@@ -523,11 +557,7 @@ let get_file proto s pos =
   in
   let sub_files, pos =
     if proto > 35 then
-      get_list (fun s pos ->
-          let name, pos = get_string s pos in
-          let size, pos = get_int64 s pos, pos+8 in
-          (name, size), pos
-      ) s pos
+      get_sub_files s pos
     else [], pos
   in
 
@@ -564,7 +594,10 @@ let get_file proto s pos =
     file_all_sources = file_all_sources;
     file_active_sources = file_active_sources;
     file_state = state;
-    file_chunks = chunks;
+    file_chunks = 
+      if chunks <> "" then 
+	Some (VerificationBitmap.of_string chunks)
+      else None;
     file_availability = availability;
     file_download_rate = rate;
     file_chunks_age = chunks_age;
@@ -617,10 +650,10 @@ let get_addr proto s pos =
   let addr, pos =
   match get_uint8 s pos with
     0 ->
-      let ip = get_ip s (pos+1) in
-      Ip.addr_of_ip ip, pos+5
+      let ip, pos = get_ip2 proto s (pos+1) in
+      Ip.addr_of_ip ip, pos
   | 1 ->
-      let name,pos = get_string s (pos+1) in
+      let name,pos = get_hostname proto s (pos+1) in
       Ip.addr_of_string name, pos
   | _ -> assert false
   in
@@ -640,13 +673,26 @@ let get_server proto s pos =
   in
   let port = get_int16 s pos in
   let score = get_int s (pos+2) in
-  let tags, pos = get_list get_tag s (pos+6) in
+  let tags, pos = get_list2 proto get_tag s (pos+6) in
   let nusers, pos = get_int64_28 proto s pos in
   let nfiles, pos = get_int64_28 proto s pos in
   let state, pos = get_host_state proto s pos in
   let name, pos = get_string s pos in
   let description, pos = get_string s pos in
   let preferred = if proto > 28 then get_bool s pos else false in
+  let pos=pos+1 in
+  let ve,ma,lo,so,ha,pi,pos = 
+    if proto > 39 then
+      let ve,pos = get_string s pos in
+      let ma,pos = get_int64 s pos, (pos+8) in
+      let lo,pos = get_int64 s pos, (pos+8) in
+      let so,pos = get_int64 s pos, (pos+8) in
+      let ha,pos = get_int64 s pos, (pos+8) in
+      let pi,pos = get_int s pos, (pos+4) in
+      ve,ma,lo,so,ha,pi,pos
+    else
+      "",0L,0L,0L,0L,0,pos
+  in
   {
     server_num = num;
     server_network = net;
@@ -663,6 +709,12 @@ let get_server proto s pos =
     server_banner = "";
     server_users = None;
     server_preferred = preferred;
+    server_version = ve; 
+    server_max_users = ma;
+    server_lowid_users = lo;
+    server_soft_limit = so;
+    server_hard_limit = ha;
+    server_ping = pi;
   }, pos
 
 let get_client_type s pos = 
@@ -672,26 +724,34 @@ let get_client_type s pos =
   | 2 -> client_contact_tag
   | _ -> assert false
 
-let get_kind s pos =
+let get_kind proto s pos =
   match get_uint8 s pos with
     0 ->
-      let ip = get_ip s (pos+1) in
-      let port = get_int16 s (pos+5) in
-      Known_location (ip, port), pos+7
+      let ip,pos = get_ip2 proto s (pos+1) in
+      let port = get_int16 s pos in
+      Known_location (ip, port), pos+2
   | 1 ->
-      let name, pos = get_string s (pos+1) in
+      let name, pos = get_hostname proto s (pos+1) in
       let md4 = get_md4 s pos in
-      Indirect_location (name, md4), pos+16
+      let ip, port, pos = 
+        if proto > 38 then begin
+          let myip , pos = get_ip2 proto s (pos+16) in
+          let myport = get_int16 s pos in
+          myip, myport, (pos+2)
+        end else
+          Ip.null, 0, (pos+16)
+      in  
+      Indirect_location (name, md4, ip, port), pos
   | _ -> assert false
 
 let get_client proto s pos =
   if proto <= 18 then
     let num = get_int s pos in
     let net = get_int s (pos+4) in
-    let kind, pos = get_kind s (pos+8) in
+    let kind, pos = get_kind proto s (pos+8) in
     let state,pos = get_host_state proto s pos in
     let t = get_client_type s pos in
-    let tags, pos = get_list get_tag s (pos+1) in
+    let tags, pos = get_list2 proto get_tag s (pos+1) in
     let name, pos = get_string s pos in
     let rating = get_int s pos in
     let chat_port = get_int s (pos+4) in
@@ -719,10 +779,10 @@ let get_client proto s pos =
   else
   let num = get_int s pos in
   let net = get_int s (pos+4) in
-  let kind, pos = get_kind s (pos+8) in
+  let kind, pos = get_kind proto s (pos+8) in
   let state,pos = get_host_state proto s pos in
   let t = get_client_type s pos in
-  let tags, pos = get_list get_tag s (pos+1) in
+  let tags, pos = get_list2 proto get_tag s (pos+1) in
   let name, pos = get_string s pos in
   let rating = get_int s pos in
   let software, pos = get_string s (pos+4) in
@@ -822,13 +882,13 @@ let get_network proto s pos =
   }, pos
 
 
-let get_user s pos =
+let get_user proto s pos =
   let num = get_int s pos in
   let md4 = get_md4 s (pos+4) in
   let name, pos = get_string s (pos+20) in
-  let ip = get_ip s pos in
-  let port = get_int16 s (pos+4) in
-  let tags, pos = get_list get_tag s (pos+6) in
+  let ip, pos = get_ip2 proto s pos in
+  let port = get_int16 s pos in
+  let tags, pos = get_list2 proto get_tag s (pos+2) in
   let server = get_int s pos in
   {
     user_num = num;
@@ -879,6 +939,7 @@ let get_shared_info proto s pos =
     shared_uploaded = uploaded;
     shared_requests = requests;
     shared_uids = [];
+    shared_sub_files = [];
   }
 
 let get_shared_info_version_10 proto s pos =
@@ -894,6 +955,11 @@ let get_shared_info_version_10 proto s pos =
     else
       get_list get_uid s (pos+12)
   in
+  let sub_files, pos =
+    if proto > 36 then
+      get_sub_files s pos
+    else [], pos
+  in
   {
     shared_num = num;
     shared_network = network;
@@ -902,6 +968,7 @@ let get_shared_info_version_10 proto s pos =
     shared_uploaded = uploaded;
     shared_requests = requests;
     shared_uids = uids;
+    shared_sub_files = sub_files;
   }
 
 
@@ -916,7 +983,7 @@ let from_gui (proto : int array) opcode s =
 
     let proto = if opcode > from_gui_last_opcode then 0 else proto.(opcode) in
     if !verbose_gui_decoding then
-      lprintf_nl "FROM GUI: Opcode %d" opcode; 
+      lprintf_nl "[gDe] FROM_GUI: Opcode %d %s" opcode (String.escaped s);
     match opcode with
       0 -> GuiProtocol (get_int s 2)
 
@@ -931,7 +998,7 @@ let from_gui (proto : int array) opcode s =
     | 5
     | 52 ->
         if proto < 14 then
-           let pass,_ = get_string s 2 in Password ("admin", pass)
+           let pass = fst (get_string s 2) in Password (CommonUserDb.admin_user, pass)
         else
         let pass,pos = get_string s 2 in
         let login,pos = get_string s pos in
@@ -949,7 +1016,7 @@ let from_gui (proto : int array) opcode s =
         Download_query (list, result_num, false)
 
     | 8 -> let string, pos = get_string s 2 in
-        lprintf_nl "Received string: [%s]" (String.escaped string);
+        lprintf_nl "[gDe] Received string: [%s]" (String.escaped string);
         Url string 
 
     | 9 -> let int = get_int s 2 in RemoveServer_query int
@@ -1229,7 +1296,7 @@ let to_gui (proto : int array)  opcode s =
     let proto = if opcode > to_gui_last_opcode then 0 else proto.(opcode) in
     
     if !verbose_gui_decoding then
-      lprintf_nl "TO GUI: Opcode %d" opcode;
+      lprintf_nl "[gDe] TO GUI: Opcode %d" opcode;
     match opcode with
     | 0 ->
         let version = get_int s 2 in
@@ -1344,7 +1411,7 @@ let to_gui (proto : int array)  opcode s =
         Network_info network_info
     
     | 21 ->
-        let user_info, pos = get_user  s 2 in
+        let user_info, pos = get_user proto s 2 in
         User_info user_info
     
     | 22 ->
