@@ -25,25 +25,25 @@ open CommonNetwork
 open CommonSearch
 open CommonTypes
 open CommonGlobals
-open CommonGraphics
+open CommonShared
 open GuiTypes
 open CommonComplexOptions
 open CommonFile
 open Options
 open BasicSocket
 open TcpBufferedSocket
+open DriverGraphics
 open DriverInteractive
 open CommonOptions
+open CommonUserDb
 
-(* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[dCon] "
-    (log_time ()); lprintf_nl2
+let log_prefix = "[dCon]"
 
-(* prints a new logline with date, module and does not start newline *)
-let lprintf_n () =
-  lprintf "%s[EDK] "
-    (log_time ()); lprintf
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
 
 let rec dollar_escape o with_frames s =
   String2.convert false (fun b escaped c ->
@@ -78,6 +78,11 @@ let rec dollar_escape o with_frames s =
               Buffer.add_string b Terminal.ANSI.ansi_GREEN;
             false
 
+        | 'c' ->
+            if o.conn_output = ANSI then
+              Buffer.add_string b Terminal.ANSI.ansi_CYAN;
+            false
+
         | 'n' ->
             if o.conn_output = ANSI then
               Buffer.add_string b Terminal.ANSI.ansi_NORMAL;
@@ -101,6 +106,7 @@ let eval auth cmd o =
   let cmd = Url.decode cmd in
   let cmd =
     if String2.check_prefix cmd "ed2k://" ||
+       String2.check_prefix cmd "ftp://" ||
        String2.check_prefix cmd "http://" then "dllink " ^ cmd
     else if String2.check_prefix cmd "fha://" then "ovlink " ^ cmd
     else cmd in
@@ -127,7 +133,7 @@ let eval auth cmd o =
                 ("", "sr", "\\<a href=\\\"submit?q=" ^ !ncmd ^
                   "\\\"\\>" ^ !ncmd ^ "\\</a\\>");
                 ("", "srw", Str.global_replace (Str.regexp "\n") "\\<br\\>" !nhelp);
-                ("", "sr", "\\<a href=\\\"http://mldonkey.berlios.de/modules.php?name=Wiki&pagename=" ^ !ncmd ^
+                ("", "sr", "\\<a href=\\\"http://mldonkey.sourceforge.net/" ^ (String2.upp_initial !ncmd) ^
                   "\\\"\\>wiki\\</a\\>"); ];
               Printf.bprintf buf "\\</tr\\>\n";
           )
@@ -138,7 +144,7 @@ let eval auth cmd o =
           Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>";
           html_mods_td buf [
             ("", "sr", "< > : required parameter");
-            ("", "sr", "[< >] : optionnal parameter");
+            ("", "sr", "[< >] : optional parameter");
             ("", "sr", "< 1 | 2 > : alternative parameter"); ];
           Printf.bprintf buf "\\</table\\>\\</div\\>\\</div\\>"
         end else
@@ -265,7 +271,7 @@ $bDownloads:$n
           $rcancel/pause/resume <num>$n : cancel/pause/resume download <num>
 
 $bSearches:$n
-          $rs  <keywords>$n : start a search for keywords <keywords> on the network
+          $rs <keywords>$n : start a search for keywords <keywords> on the network
           $rvr$n : view results of the last search
           $rd <num>$n : download result number <num>
           $rvs$n : view previous searches
@@ -304,14 +310,18 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
         let user, pass =
           match args with
             [] -> failwith "Usage: auth <user> <password>"
-          | [s1] -> "admin", s1
+          | [s1] -> admin_user, s1
           | user :: pass :: _ -> user, pass
         in
         if valid_password user pass then begin
             auth := true;
             o.conn_user <- find_ui_user user;
+            if not !verbose_no_login then lprintf_nl "Authenticated user: %s" user;
             let module M = CommonMessages in
-            Buffer.add_string buf M.full_access
+            Buffer.add_string buf M.full_access;
+            (match DriverInteractive.real_startup_message () with
+               Some s -> Buffer.add_string buf s;
+             | None -> ());
           end else
         let module M = CommonMessages in
         Buffer.add_string buf M.bad_login
@@ -342,9 +352,9 @@ let check_calendar () =
   List.iter (fun (days, hours, command) ->
       if (List.mem tm.Unix.tm_wday days || days = []) &&
         (List.mem tm.Unix.tm_hour hours || hours = []) then begin
-          lprintf_nl () "Calendar execute: %s" command;
+          lprintf_nl "Calendar execute: %s" command;
           eval (ref true) command calendar_options;
-          lprintf_nl () "Calendar result: %s" (Buffer.contents calendar_options.conn_buf);
+          lprintf_nl "Calendar result: %s" (Buffer.contents calendar_options.conn_buf);
           Buffer.reset calendar_options.conn_buf;
         end
   ) !!calendar
@@ -398,7 +408,7 @@ let user_reader o telnet sock nread  =
             before_telnet_output o sock;
             let buf = o.conn_buf in
             Buffer.reset buf;
-            if o.conn_output = ANSI then Printf.bprintf buf "> $b%s$n\n" cmd;
+            if o.conn_output = ANSI then Printf.bprintf buf "> $c%s$n\n" cmd;
             eval telnet.telnet_auth cmd o;
             Buffer.add_char buf '\n';
             if o.conn_output = ANSI then Buffer.add_string buf "$n";
@@ -447,8 +457,6 @@ let iac_will_naws = "\255\253\031"
 
 let user_reader o telnet sock nread =
   let b = TcpBufferedSocket.buf sock in
-  let end_pos = b.pos + b.len in
-  let new_pos = end_pos - nread in
   let rec iter () =
     if b.len > 0 then
       let c = b.buf.[b.pos] in
@@ -520,7 +528,7 @@ let user_reader o telnet sock nread =
               before_telnet_output o sock;
               let buf = o.conn_buf in
               Buffer.reset buf; 
-              if o.conn_output = ANSI then Printf.bprintf buf "> $b%s$n\n" cmd;
+              if o.conn_output = ANSI then Printf.bprintf buf "> $c%s$n\n" cmd;
               eval telnet.telnet_auth cmd o;
               Buffer.add_char buf '\n';
               if o.conn_output = ANSI then Buffer.add_string buf "$n";
@@ -567,12 +575,13 @@ let telnet_handler t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
+        if not !verbose_no_login then lprintf_nl "Telnet connection from %s" (Ip.to_string from_ip);
         let token = create_token unlimited_connection_manager in
         let sock = TcpBufferedSocket.create_simple token
           "telnet connection"
           s in
         let telnet = {
-            telnet_auth = ref (empty_password "admin");
+            telnet_auth = ref (empty_password admin_user);
             telnet_iac = false;
             telnet_wait = 0;
             telnet_buffer = Buffer.create 100;
@@ -586,7 +595,8 @@ let telnet_handler t event =
             conn_width = 80;
             conn_height = 0;
           } in
-      if Ip.matches from_ip !!allowed_ips then begin
+	(match Ip_set.match_ip !allowed_ips_set from_ip with
+	| Some br -> 
         TcpBufferedSocket.prevent_close sock;
         TcpBufferedSocket.set_max_output_buffer sock !!interface_buffer;
         TcpBufferedSocket.set_reader sock (user_reader o telnet);
@@ -597,14 +607,14 @@ let telnet_handler t event =
 
         before_telnet_output o sock;
         TcpBufferedSocket.write_string sock
-	   (Printf.sprintf "Welcome to MLDonkey %s" Autoconf.current_version);
+	   (Printf.sprintf "Welcome to MLDonkey %s\n" Autoconf.current_version);
 
         TcpBufferedSocket.write_string sock (dollar_escape o false
-            "\n$bWelcome on mldonkey command-line$n\n\nUse $r?$n for help\n\n");
+            "$cWelcome on mldonkey command-line$n\n\nUse $r?$n for help\n\n");
 
         after_telnet_output o sock
-	end
-      else begin
+
+	| None ->
         before_telnet_output o sock;
 	let reject_message =
 	  Printf.sprintf "Telnet connection from %s rejected (see allowed_ips setting)\n"
@@ -612,88 +622,11 @@ let telnet_handler t event =
 	in
 	TcpBufferedSocket.write_string sock (dollar_escape o false reject_message);
 	shutdown sock Closed_connect_failed;
-	lprintf_n () "%s" reject_message;
-	Unix.close s
-      end
+        if not !verbose_no_login then lprintf_n "%s" reject_message;
+	Unix.close s)
 
   | _ -> ()
 
-(*************************************************************
-
-                  The Chat Server
-
-**************************************************************)
-
-let chat_handler t event =
-  match event with
-    TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
-      (
-        try
-          let o = {
-              conn_buf = Buffer.create 1000;
-              conn_output = TEXT;
-              conn_sortvd = NotSorted;
-              conn_filter = (fun _ -> ());
-              conn_user = default_user;
-              conn_width = 80; conn_height = 0;
-            } in
-
-         let from_ip = Ip.of_inet_addr from_ip in
-         if Ip.matches from_ip !!allowed_ips then
-           (
-            let chanin = Unix.in_channel_of_descr s in
-            let chanout = Unix.out_channel_of_descr s in
-            let paq = Chat_proto.read_packet_channel chanin in
-            let ret =
-              match paq with
-                ((v,id,(host,port)),iddest,pro) ->
-                  if v <> CommonChat.version then
-                    None
-                  else
-                    Some paq
-            in
-            close_out chanout;
-            (match ret with
-              None -> ()
-            | Some ((v,id,(host,port)),iddest,pro) ->
-                match pro with
-                  Chat_proto.Hello ->
-                    CommonChat.send_hello_ok ()
-                | Chat_proto.HelloOk -> ()
-                | Chat_proto.AddOpen _ -> ()
-                | Chat_proto.Byebye -> ()
-                | Chat_proto.RoomMessage _ ->
-                    (* A VOIR *)
-                    ()
-                | Chat_proto.Message s ->
-                    if iddest = !!CommonOptions.chat_console_id then
-                   (* we must eval the string as a command *)
-                      (
-                            let buf = o.conn_buf in
-                            Buffer.reset buf;
-                       let auth = ref true in
-                       eval auth s o;
-                       CommonChat.send_text !!CommonOptions.chat_console_id None
-                         (dollar_escape o false (Buffer.contents buf));
-                       Buffer.reset buf
-                      )
-                    else
-                   (* we must forward the message *)
-                      (networks_iter (fun r ->
-                        network_private_message r iddest s)
-                      )
-            )
-           )
-         else
-           Unix.close s
-     with
-       Failure mess ->
-         lprintf_nl () "%s" mess;
-         Unix.close s
-    )
-  | _ ->
-      ()
-
 (*************************************************************
 
                   The HTTP Server
@@ -831,16 +764,16 @@ let get_theme_page page =
         let fname = Filename.concat theme page in fname
 
 let theme_page_exists page =
-        if Sys.file_exists (get_theme_page page) then true else false
+    Sys.file_exists (get_theme_page page)
 
 (* if files are small really_input should be okay *)
 let read_theme_page page =
-        let theme_page = get_theme_page page in
-        let file = open_in theme_page in
-        let size = (Unix.stat theme_page).Unix.st_size in
-        let s = String.make size ' ' in
-        let ok = really_input file s 0 size in
-        close_in file; s
+  let theme_page = get_theme_page page in
+  Unix2.tryopen_read theme_page (fun file ->
+    let size = (Unix.stat theme_page).Unix.st_size in
+    let s = String.make size ' ' in
+    really_input file s 0 size;
+    s)
 
 let add_simple_commands buf =
   let this_page = "commands.html" in
@@ -899,7 +832,7 @@ let http_send_bin r buf filename =
   let ext_pos = (String.rindex filename '.') + 1 in
   let exten = (String.sub filename ext_pos ((String.length filename) - ext_pos)) in
   if !verbose_msg_servers then
-    lprintf_nl () "Extension found [%s] for file: [%s]" exten filename;
+    lprintf_nl "Extension found [%s] for file: [%s]" exten filename;
   let ext = extension_to_file_ext exten in
   http_add_bin_header r ext clen;
   Buffer.add_string buf file_to_send
@@ -909,15 +842,15 @@ let http_error_no_gd img_type =
     "jpg" ->
       (match Autoconf.has_gd_jpg with
         true -> false
-      | false -> lprintf_nl () "Warning: GD jpg support disabled"; true)
+      | false -> lprintf_nl "Warning: GD jpg support disabled"; true)
   | "png" ->
       (match Autoconf.has_gd_png with
         true -> false
-      | false -> lprintf_nl () "Warning: GD png support disabled"; true)
+      | false -> lprintf_nl "Warning: GD png support disabled"; true)
   | _ ->
       (match Autoconf.has_gd with
         true -> false
-      | false -> lprintf_nl () "Warning: GD support disabled"; true)
+      | false -> lprintf_nl "Warning: GD support disabled"; true)
 let any_ip = Ip.of_inet_addr Unix.inet_addr_any
 
 let html_open_page buf t r open_body =
@@ -962,14 +895,71 @@ let clear_page buf =
   Buffer.reset buf;
   http_file_type := UNK
 
+let send_preview r file fd size filename exten =
+  let (begin_pos, end_pos) =
+    try
+      let (begin_pos, end_pos) = request_range r in
+      let end_pos = match end_pos with
+          None -> size
+        | Some end_pos -> end_pos in
+      let range_size = end_pos -- begin_pos in
+      add_reply_header r "Content-Length"
+        (Int64.to_string range_size);
+      add_reply_header r "Content-Range"
+        (Printf.sprintf "bytes %Ld-%Ld/%Ld"
+          begin_pos (end_pos -- one)
+        size);
+      r.reply_head <- "206 Partial Content";
+      begin_pos, end_pos
+    with _ ->
+        add_reply_header r "Content-Length"
+          (Int64.to_string size);
+        zero, size
+  in
+  let len = String.length exten in
+  let exten = if len = 0 then exten
+      else String.lowercase (String.sub exten 1 (len - 1)) in
+  http_add_bin_stream_header r (extension_to_file_ext exten);
+
+  let s = String.create 200000 in
+  set_max_output_buffer r.sock (String.length s);
+  set_rtimeout r.sock 10000.;
+  let rec stream_file file pos sock =
+    let max = (max_refill sock) - 1 in
+    if max > 0 && !pos < end_pos then
+      let max64 = min (end_pos -- !pos) (Int64.of_int max) in
+      let max = Int64.to_int max64 in
+      Unix32.read fd !pos s 0 max;
+      pos := !pos ++ max64;
+      set_lifetime sock 60.;
+(*                      lprintf "HTTPSEND: refill %d %Ld\n" max !pos;*)
+(*                    lprintf "HTTPSEND: [%s]\n" (String.escaped
+                        (String.sub s 0 max)); *)
+      write sock s 0 max;
+      if output_buffered sock = 0 then begin
+(*                          lprintf "Recursing STREAM\n"; *)
+          stream_file file pos sock
+        end
+  in
+  r.reply_stream <- Some (stream_file file (ref begin_pos))
+
+
 let http_handler o t r =
   CommonInteractive.display_vd := false;
   CommonInteractive.display_bw_stats := false;
   clear_page buf;
+  if !Http_server.verbose && r.get_url.Url.short_file <> "" then
+    lprintf_nl "received URL %s %s"
+      r.get_url.Url.short_file
+      (let b = Buffer.create 100 in
+	 List.iter (fun (arg, value) -> Printf.bprintf b " %s %s" arg value) r.get_url.Url.args;
+	 if Buffer.contents b <> "" then Printf.sprintf "(%s)" (Buffer.contents b) else "");
 
-  let user = if r.options.login = "" then "admin" else r.options.login in
+  let user = if r.options.login = "" then admin_user else r.options.login in
   if not (valid_password user r.options.passwd) then begin
       clear_page buf;
+      http_file_type := HTM;
+      Buffer.add_string buf (snd(Http_server.error_page "401" "" "" (Ip.to_string (TcpBufferedSocket.my_ip r.sock)) (string_of_int !!http_port) None));
       need_auth r !!http_realm
     end
   else
@@ -1049,6 +1039,14 @@ let http_handler o t r =
                 read_theme_page this_page else
               if !!html_mods then !!CommonMessages.web_common_header_mods0
               else !!CommonMessages.web_common_header_old)
+        | "multidllink.html" ->
+            html_open_page buf t r true;
+            let this_page = "multidllink.html" in
+            Buffer.add_string buf (
+              if !!html_mods_theme != "" && theme_page_exists this_page then
+                read_theme_page this_page else
+              if !!html_mods then !!CommonMessages.multidllink_mods0
+              else !!CommonMessages.multidllink_old)
         | "" | "index.html" ->
             if !!use_html_frames then begin
                 html_open_page buf t r false;
@@ -1092,109 +1090,115 @@ let http_handler o t r =
 
         | "oneframe.html" ->
             html_open_page buf t r true;
-            Buffer.add_string buf !!motd_html
+            Buffer.add_string buf (Printf.sprintf "<br><div align=\"center\"><h3>%s %s</h3></div>"
+	      (Printf.sprintf (_b "Welcome to MLDonkey")) Autoconf.current_version);
+	    if !!motd_html <> "" then Buffer.add_string buf !!motd_html;
+	    if user2_is_admin o.conn_user.ui_user_name then
+	    (match DriverInteractive.real_startup_message () with
+	       Some s -> Buffer.add_string buf (Printf.sprintf "<p><pre><b><h3>%s</b></h3></pre>" s);
+	     | None -> ())
 
         | "bw_updown.png" ->
             (match http_error_no_gd "png" with
               false ->
-                do_draw_pic "Traffic" "s(kb)" "t(h:m:s)" download_history upload_history;
+                G.do_draw_pic "Traffic" "s(kb)" "t(h:m:s)" download_history upload_history;
                 http_send_bin r buf "bw_updown.png"
             | true -> raise Not_found)
 
         | "bw_updown.jpg" ->
             (match http_error_no_gd "jpg" with
               false ->
-                do_draw_pic "Traffic" "s(kb)" "t(h:m:s)" download_history upload_history;
+                G.do_draw_pic "Traffic" "s(kb)" "t(h:m:s)" download_history upload_history;
                 http_send_bin r buf "bw_updown.jpg"
             | true -> raise Not_found)
 
         | "bw_download.png" ->
             (match http_error_no_gd "png" with
               false ->
-                do_draw_down_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_history;
+                G.do_draw_down_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_history;
                 http_send_bin r buf "bw_download.png"
             | true -> raise Not_found)
 
         | "bw_download.jpg" ->
             (match http_error_no_gd "jpg" with
               false ->
-                do_draw_down_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_history;
+                G.do_draw_down_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_history;
                 http_send_bin r buf "bw_download.jpg"
             | true -> raise Not_found)
 
         | "bw_upload.png" ->
             (match http_error_no_gd "png" with
               false ->
-                do_draw_up_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_history;
+                G.do_draw_up_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_history;
                 http_send_bin r buf "bw_upload.png"
             | true -> raise Not_found)
 
         | "bw_upload.jpg" ->
             (match http_error_no_gd "jpg" with
             | false ->
-                do_draw_up_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_history;
+                G.do_draw_up_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_history;
                 http_send_bin r buf "bw_upload.jpg"
             | true -> raise Not_found)
 
         | "bw_h_updown.png" ->
             (match http_error_no_gd "png" with
             | false ->
-                do_draw_h_pic "Traffic" "s(kb)" "t(h:m:s)" download_h_history upload_h_history;
+                G.do_draw_h_pic "Traffic" "s(kb)" "t(h:m:s)" download_h_history upload_h_history;
                 http_send_bin r buf "bw_h_updown.png"
             | true -> raise Not_found)
 
         | "bw_h_updown.jpg" ->
             (match http_error_no_gd "jpg" with
             | false ->
-                do_draw_h_pic "Traffic" "s(kb)" "t(h:m:s)" download_h_history upload_h_history;
+                G.do_draw_h_pic "Traffic" "s(kb)" "t(h:m:s)" download_h_history upload_h_history;
                 http_send_bin r buf "bw_h_updown.jpg"
             | true -> raise Not_found)
 
         | "bw_h_download.png" ->
             (match http_error_no_gd "png" with
             | false ->
-                do_draw_down_h_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_h_history;
+                G.do_draw_down_h_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_h_history;
                 http_send_bin r buf "bw_h_download.png"
             | true -> raise Not_found)
 
         | "bw_h_download.jpg" ->
             (match http_error_no_gd "jpg" with
             | false ->
-                do_draw_down_h_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_h_history;
+                G.do_draw_down_h_pic "Traffic" "download" "s(kb)" "t(h:m:s)" download_h_history;
                 http_send_bin r buf "bw_h_download.jpg"
             | true -> raise Not_found)
 
         | "bw_h_upload.png" ->
             (match http_error_no_gd "png" with
             | false ->
-                do_draw_up_h_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_h_history;
+                G.do_draw_up_h_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_h_history;
                 http_send_bin r buf "bw_h_upload.png"
             | true -> raise Not_found)
 
         | "bw_h_upload.jpg" ->
             (match http_error_no_gd "jpg" with
             | false ->
-                do_draw_up_h_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_h_history;
+                G.do_draw_up_h_pic "Traffic" "upload" "s(kb)" "t(h:m:s)" upload_h_history;
                 http_send_bin r buf "bw_h_upload.jpg"
             | true -> raise Not_found)
 
         | "tag.png" ->
             (match http_error_no_gd "png" with
             | false ->
-                do_draw_tag !!html_mods_vd_gfx_tag_title download_history upload_history;
+                G.do_draw_tag !!html_mods_vd_gfx_tag_title download_history upload_history;
                 http_send_bin r buf "tag.png"
             | true -> raise Not_found)
 
         | "tag.jpg" ->
             (match http_error_no_gd "jpg" with
             | false ->
-                do_draw_tag !!html_mods_vd_gfx_tag_title download_history upload_history;
+                G.do_draw_tag !!html_mods_vd_gfx_tag_title download_history upload_history;
                 http_send_bin r buf "tag.jpg"
             | true -> raise Not_found)
 
         | "favicon.ico" ->
             if !verbose_msg_servers then
-              lprintf_nl () "favicon.ico request received by tracker";
+              lprintf_nl "favicon.ico request received by tracker";
             http_send_bin r buf "favicon.ico"
 
         | "filter" ->
@@ -1308,6 +1312,7 @@ let http_handler o t r =
                       | "A" -> o.conn_sortvd <- ByASources
                       | "N" -> o.conn_sortvd <- ByNet
                       | "Avail" -> o.conn_sortvd <- ByAvail
+                      | "Cm" -> o.conn_sortvd <- ByComments
                       | _ -> ()
                     end
                 | _ -> ()
@@ -1321,7 +1326,18 @@ let http_handler o t r =
 
         | "submit" ->
             begin
+
               match r.get_url.Url.args with
+                | [ "jvcmd", "multidllink" ; "links", links] ->
+                    html_open_page buf t r true;
+                    List.iter (fun url ->
+		      if url <> "\013" && url <> "" then
+		        begin
+                          Buffer.add_string buf (html_escaped (dllink_parse (o.conn_output = HTML) url));
+                          Buffer.add_string buf (html_escaped "\\<P\\>")
+			end
+                    ) (String2.split links '\n')
+
               | ("q", cmd) :: other_args ->
                   List.iter (fun arg ->
                       match arg with
@@ -1356,9 +1372,9 @@ let http_handler o t r =
                     | "downloaders" | "uploaders" | "scan_temp" | "cs"
                     | "version" | "rename" | "force_download" | "close_fds"
                     | "vd" | "vo" | "voo" | "upstats" | "shares" | "share"
-                    | "unshare" | "stats" | "users" -> drop_pre := true;
+                    | "unshare" | "stats" | "users" | "block_list" -> 
+			drop_pre := true;
                     | _ -> ());
-
                   Printf.bprintf buf "%s\n"
                     (if use_html_mods o && !drop_pre then s else "\n<pre>\n" ^ s ^ "</pre>");
 
@@ -1372,12 +1388,17 @@ let http_handler o t r =
 
               | [ "setoption", _ ; "option", name; "value", value ] ->
                   html_open_page buf t r true;
-                  CommonInteractive.set_fully_qualified_options name value;
-                  Buffer.add_string buf "Option value changed"
+		  if (o.conn_user == default_user) || !!enable_user_config then
+		    begin
+                      CommonInteractive.set_fully_qualified_options name value;
+                      Buffer.add_string buf "Option value changed"
+		    end
+		  else
+                    Buffer.add_string buf "You are not allowed to change options"
 
               | args ->
                   List.iter (fun (s,v) ->
-                      lprintf_nl () "[%s]=[%s]" (String.escaped s) (String.escaped v))
+                      lprintf_nl "[%s]=[%s]" (String.escaped s) (String.escaped v))
                   args;
                   raise Not_found
             end
@@ -1391,60 +1412,40 @@ let http_handler o t r =
                   let file = file_find file_num in
                   let fd = file_fd file in
                   let size = file_size file in
-
-                  let (begin_pos, end_pos) =
-                    try
-                      let (begin_pos, end_pos) = request_range r in
-                      let end_pos = match end_pos with
-                          None -> size
-                        | Some end_pos -> end_pos in
-                      let range_size = end_pos -- begin_pos in
-                      add_reply_header r "Content-Length"
-                        (Int64.to_string range_size);
-                      add_reply_header r "Content-Range"
-                        (Printf.sprintf "bytes %Ld-%Ld/%Ld"
-                          begin_pos (end_pos -- one)
-                        size);
-                      r.reply_head <- "206 Partial Content";
-                      begin_pos, end_pos
-                    with _ ->
-                        add_reply_header r "Content-Length"
-                          (Int64.to_string size);
-                        zero, size
-                  in
-		    
 		  let filename = file_best_name file in
 		  let exten = Filename2.last_extension filename in
-		  let len = String.length exten in
-		  let exten = if len = 0 then exten
-		      else String.lowercase (String.sub exten 1 (len - 1)) in
-		  http_add_bin_stream_header r (extension_to_file_ext exten);
-
-                  let s = String.create 200000 in
-                  set_max_output_buffer r.sock (String.length s);
-                  set_rtimeout r.sock 10000.;
-                  let rec stream_file file pos sock =
-                    let max = (max_refill sock) - 1 in
-                    if max > 0 && !pos < end_pos then
-                      let max64 = min (end_pos -- !pos) (Int64.of_int max) in
-                      let max = Int64.to_int max64 in
-                      Unix32.read fd !pos s 0 max;
-                      pos := !pos ++ max64;
-                      set_lifetime sock 60.;
-(*                      lprintf "HTTPSEND: refill %d %Ld\n" max !pos;*)
-(*                    lprintf "HTTPSEND: [%s]\n" (String.escaped
-                        (String.sub s 0 max)); *)
-                      write sock s 0 max;
-                      if output_buffered sock = 0 then begin
-(*                          lprintf "Recursing STREAM\n"; *)
-                          stream_file file pos sock
-                        end
-                  in
-                  r.reply_stream <- Some (stream_file file (ref begin_pos))
+		    send_preview r file fd size filename exten
 
               | args ->
                   List.iter (fun (s,v) ->
-                      lprintf_nl () "[%s]=[%s]" (String.escaped s) (String.escaped v))
+                      lprintf_nl "[%s]=[%s]" (String.escaped s) (String.escaped v))
+                  args;
+                  raise Not_found
+            end
+
+        | "preview_upload" ->
+            begin
+              clear_page buf;
+              match r.get_url.Url.args with
+                ["q", file_num] ->
+                  let file_num = int_of_string file_num in
+		  let file = shared_find file_num in
+		  let impl = as_shared_impl file in
+		  let info = shared_info file in
+		  let filename = impl.impl_shared_fullname in
+		  let exten = Filename2.last_extension impl.impl_shared_codedname in
+		  if not (Sys.file_exists filename) then
+		    begin
+          lprintf_nl "file %s not found" filename;
+		      raise Not_found
+		    end;
+		  let fd = Unix32.create_ro filename in
+		  let size = info.shared_size in
+		    send_preview r file fd size filename exten
+
+              | args ->
+                  List.iter (fun (s,v) ->
+                      lprintf_nl "[%s]=[%s]" (String.escaped s) (String.escaped v))
                   args;
                   raise Not_found
             end
@@ -1525,11 +1526,13 @@ let create_http_handler () =
       bind_addr = Ip.to_inet_addr !!http_bind_addr ;
       port = !!http_port;
       requests = [];
-      addrs = !!allowed_ips;
+      addrs = Ip_set.of_list !!allowed_ips;
+      use_ip_block_list = false;
       base_ref = "";
       default = http_handler http_options;
     } in
-  option_hook allowed_ips (fun _ -> config.addrs <- !!allowed_ips);
-  let sock = find_port "http server" !!http_bind_addr http_port
-      (Http_server.handler config) in
+  option_hook allowed_ips (fun _ ->
+    config.addrs <- Ip_set.of_list !!allowed_ips);
+  ignore(find_port "http server" !!http_bind_addr http_port
+      (Http_server.handler config));
   config.port <- !!http_port
