@@ -40,12 +40,17 @@ open BasicSocket
 open TcpBufferedSocket
 open CommonOptions
 open CommonGlobals
-  
-module P = GuiProto
-let lprintf_nl () =
-  lprintf "%s[dIface] "
-    (log_time ()); lprintf_nl2
+open CommonUserDb
 
+module P = GuiProto
+
+let log_prefix = "[dIface]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
 
 let binary_gui_send gui t =
   match gui.gui_sock with
@@ -384,11 +389,12 @@ let gui_closed gui sock  msg =
 let gui_initialize gui = 
 
   gui.gui_initialized <- true;
-  let connecting = ref true in
   networks_iter_all (fun n ->
       gui_send gui (Network_info (network_info n)));
-  gui_send gui (Console (DriverControlers.text_of_html !!motd_html));
-  
+  gui_send gui (Console (Printf.sprintf "Welcome to MLDonkey %s\n" Autoconf.current_version));
+  (match DriverInteractive.real_startup_message () with
+    Some s -> gui_send gui (Console s);
+  | None -> ());
   
   if gui.gui_poll then begin
       
@@ -464,19 +470,9 @@ let gui_initialize gui =
         P.Options_info (simple_options "" downloads_ini));
       networks_iter_all (fun r ->
           List.iter (fun opfile ->
-(*
-lprintf "options for net %s\n" r.network_name; 
-*)
               let prefix = r.network_shortname ^ "-" in
               let args = simple_options prefix opfile in
-(*
-lprintf "Sending for %s\n" prefix; 
- *)
-              gui_send gui (P.Options_info args)
-(*                            lprintf "sent for %s\n" prefix; *)
-          )
-          r.network_config_file 
-      );
+              gui_send gui (P.Options_info args)) r.network_config_file);
 
 (* Options panels defined in downloads.ini *)
       List.iter (fun s ->
@@ -502,7 +498,7 @@ lprintf "Sending for %s\n" prefix;
           List.iter (fun file ->
               
               List.iter (fun s ->
-                  let section = section_name s in
+                  let _ = section_name s in
                   List.iter (fun o ->
                       gui_send gui (
                         P.Add_plugin_option (r.network_name, o)
@@ -532,7 +528,7 @@ let gui_reader (gui: gui_record) t _ =
           (to_gui_last_opcode + 1) version;
         gui.gui_proto_from_gui_version <- Array.create 
           (from_gui_last_opcode + 1) version;
-        lprintf_nl () "GUI protocol %d" version    
+        if not !verbose_no_login then lprintf_nl "GUI protocol %d" version    
     
     | P.GuiExtensions list ->
         List.iter (fun (ext, bool) ->
@@ -564,11 +560,12 @@ let gui_reader (gui: gui_record) t _ =
     
     | P.Password (user, pass) ->
         begin
+          if not !verbose_no_login then lprintf_nl "GUI connection from user %s" user;
           match gui.gui_sock with
             Some sock when not (valid_password user pass) ->
               gui_send gui BadPassword;                  
               set_lifetime sock 5.;
-              lprintf "BAD PASSWORD\n"; 
+              if not !verbose_no_login then lprintf_nl "GUI connection BAD PASSWORD for user %s" user; 
               TcpBufferedSocket.close sock (Closed_for_error "Bad Password")
           
           | _ ->
@@ -616,7 +613,18 @@ let gui_reader (gui: gui_record) t _ =
               ) list
           
           | P.SetOption (name, value) ->
-              CommonInteractive.set_fully_qualified_options name value
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user_name || !!enable_user_config then
+		CommonInteractive.set_fully_qualified_options name value
+	      else
+	        begin
+                  let o = gui.gui_conn in
+                  let buf = o.conn_buf in
+                  Buffer.reset buf; 
+                  Buffer.add_string buf "\nYou are not allowed to change options\n";
+                  gui_send gui (P.Console (
+                      DriverControlers.dollar_escape o false
+                        (Buffer.contents buf)))
+		end
           
           | P.CloseSearch (num, forget) ->
               let s = List.assoc num gui.gui_searches in
@@ -662,7 +670,18 @@ let gui_reader (gui: gui_record) t _ =
                     network_extend_search r s e)
           
           | P.KillServer -> 
-              CommonInteractive.clean_exit 0
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user_name then
+		CommonInteractive.clean_exit 0
+	      else
+	        begin
+                  let o = gui.gui_conn in
+                  let buf = o.conn_buf in
+                  Buffer.reset buf; 
+                  Buffer.add_string buf "\nYou are not allowed to kill MLDonkey\n";
+                  gui_send gui (P.Console (
+                      DriverControlers.dollar_escape o false
+                        (Buffer.contents buf)))
+		end
           
           | P.Search_query s ->
               
@@ -698,41 +717,21 @@ let gui_reader (gui: gui_record) t _ =
                   then
                     
                     r.op_network_search search buf);
-(*
-        search.op_search_end_reply_handlers <- 
-          (fun _ -> send_waiting gui num search.search_waiting) ::
-search.op_search_end_reply_handlers;
-  *)
           
           | P.Download_query (filenames, num, force) ->
-              begin
-                let r = find_result num in
-                let files = result_download r filenames force in
-                List.iter CommonInteractive.start_download files
-              end
+              let r = find_result num in
+              let files = result_download r filenames force in
+              List.iter CommonInteractive.start_download files
           
           | P.ConnectMore_query ->
               networks_iter network_connect_servers
           
           | P.Url url ->
-              (* changed with the new header-check code from dllink, but i
-                 didn't care about the read from file functionality that
-                 perhaps worked before:
-              
-              if not (networks_iter_until_true (fun n -> network_parse_url n url)) then
-                begin
-                  let file = File.to_string url in
-                  let lines = String2.split_simplify file '\n' in
-                  List.iter (fun line ->
-                      ignore (networks_iter_until_true (fun n -> network_parse_url n line))
-                  ) lines
-                end
-              *)
               let query_networks url = 
                 if not (networks_iter_until_true
                     (fun n ->
                        try
-                         network_parse_url n url
+                         let s,r = network_parse_url n url in r
                        with e ->
                          lprintf "Exception %s for network %s\n"
                            (Printexc2.to_string e) (n.network_name);
@@ -750,8 +749,7 @@ search.op_search_end_reply_handlers;
                     H.req_url =  u;
                     H.req_proxy = !CommonOptions.http_proxy;
                     H.req_request = H.HEAD;
-                    H.req_user_agent = 
-                       Printf.sprintf "MLDonkey/%s" Autoconf.current_version;
+                    H.req_user_agent = get_user_agent ();
                 } in
                 H.whead r 
                   (fun headers ->
@@ -803,7 +801,7 @@ search.op_search_end_reply_handlers;
           
           | P.SaveFile (num, name) ->
               let file = file_find num in
-              set_file_best_name file name;
+              set_file_best_name file name "" 0;
               (match file_state file with
                   FileDownloaded ->
                     file_save_as file name;
@@ -860,7 +858,7 @@ search.op_search_end_reply_handlers;
           
           | P.RenameFile (num, new_name) ->
               let file = file_find num in
-              set_file_best_name file new_name;
+              set_file_best_name file new_name "" 0;
               addevent gui.gui_events.gui_files num true
           
           | P.ConnectFriend num 
@@ -869,8 +867,7 @@ search.op_search_end_reply_handlers;
               let c = client_find num in
               client_connect c
           
-          | P.DisconnectClient num
-            ->
+          | P.DisconnectClient num ->
               let c = client_find num in
               client_disconnect c
           
@@ -1047,8 +1044,22 @@ search.op_search_end_reply_handlers;
           | P.GetVersion ->
               gui_send gui (P.Version Autoconf.current_version)
 
-          | P.GiftAttach _ ->
-              gui_send gui (P.GiftServerAttach ("mldonkey", "1.1"))
+          | P.GiftAttach (profile, version, client) ->
+	      let user, pass =
+		try
+		  let index = String.index profile ':' in
+		    String.sub profile 0 (index), 
+		    String.sub profile (index+1) (String.length profile - index - 1)
+		with Not_found -> profile, "" in
+		(match gui.gui_sock with
+		  | Some sock when not (valid_password user pass) ->
+		      set_lifetime sock 5.;
+		      if not !verbose_no_login then lprintf_nl "BAD PASSWORD";
+		      TcpBufferedSocket.close sock (Closed_for_error "Bad Password")
+		  | _ ->
+		      gui.gui_auth <- true;
+		      gui.gui_conn.conn_user <- find_ui_user user;
+		      gui_send gui (P.GiftServerAttach ("mldonkey", "1.1")))
           | P.GiftStats ->
               let list = ref [] in
               networks_iter (fun n ->
@@ -1070,9 +1081,12 @@ search.op_search_end_reply_handlers;
     Failure s ->
       gui_send gui (Console (Printf.sprintf "Failure: %s\n" s))
   | e ->
-      gui_send gui (Console (    
-          Printf.sprintf "from_gui: exception %s for message %s\n" (
-            Printexc2.to_string e) (GuiProto.string_of_from_gui t)))
+      let error_text = Printexc2.to_string e in
+      if error_text = "BTInteractive.Torrent_can_not_be_used" then
+	gui_send gui (Console (Printf.sprintf "\nError: This torrent does not have valid tracker URLs\n"))
+      else
+	gui_send gui (Console (Printf.sprintf "from_gui: exception %s for message %s\n"
+          (Printexc2.to_string e) (GuiProto.string_of_from_gui t)))
 
 let gui_events () = 
   {
@@ -1131,8 +1145,9 @@ let gui_handler t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
-      lprintf_nl () "GUI connection from %s" (Ip.to_string from_ip);
-      if Ip.matches from_ip !!allowed_ips then 
+      if not !verbose_no_login then lprintf_nl "GUI connection from %s" (Ip.to_string from_ip);
+      (match Ip_set.match_ip !allowed_ips_set from_ip with
+      |	Some br ->
         
         let module P = GuiProto in
         let token = create_token unlimited_connection_manager in
@@ -1159,11 +1174,10 @@ let gui_handler t event =
             close sock Closed_for_overflow);
         (* sort GUIs in increasing order of their num *)
         
-      else begin
-          lprintf_nl () "GUI connection from %s rejected (see allowed_ips setting)"
+      | None ->
+          if not !verbose_no_login then lprintf_nl "GUI connection from %s rejected (see allowed_ips setting)"
             (Ip.to_string from_ip);
-          Unix.close s
-        end
+          Unix.close s)
   | _ -> ()
         
 let gift_handler t event = 
@@ -1171,7 +1185,8 @@ let gift_handler t event =
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
       lprintf "Gift: Connection from %s\n" (Ip.to_string from_ip);
-      if Ip.matches from_ip !!allowed_ips then 
+      (match Ip_set.match_ip !allowed_ips_set from_ip with
+      | Some br ->
         
         let module P = GuiProto in
         let token = create_token unlimited_connection_manager in
@@ -1197,11 +1212,10 @@ let gift_handler t event =
             close sock Closed_for_overflow);
         (* sort GUIs in increasing order of their num *)
         
-      else begin
+      | None ->
           lprintf "Connection from IP %s not allowed\n"
             (Ip.to_string from_ip);
-          Unix.close s
-        end
+          Unix.close s)
   | _ -> ()
 
       
@@ -1291,9 +1305,8 @@ let rec update_events list =
               
           | Console_message_event msg ->  
               Fifo.put console_messages event;
-              if Fifo.length console_messages > 30 then begin
-                  ignore (Fifo.take console_messages)
-                end;
+              if Fifo.length console_messages > !!gui_log_size then
+                  ignore (Fifo.take console_messages);
               add_gui_event event
               
         with _ -> ());
