@@ -28,60 +28,56 @@ open CommonInteractive
 open CommonDownloads
 open CommonTypes
 open CommonOptions
+open CommonUserDb
 open CommonGlobals
 open CommonNetwork
 
 open DriverInterface
 
-module Dp500 = DriverLink.DP500(struct
-
-      module CommonTypes = CommonTypes
-      module CommonFile = CommonFile
-      module CommonOptions = CommonOptions
-
-      let incoming_directory () =
-        (CommonComplexOptions.incoming_files ()).shdir_dirname
-      let files () = !!CommonComplexOptions.files
-
-    end)
-
 open Gettext (* open last  as most modules redefine _s and _b *)
 
 let _s x = _s "DriverMain" x
 let _b x = _b "DriverMain" x
 
-let keep_console_output = ref false
+let log_prefix = "[dMain]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
+
 let pid = ref ""
 
 let do_daily () =
   incr CommonWeb.days
 
 let minute_timer () =
+  DriverInteractive.hdd_check ();
   CommonShared.shared_check_files ();
   CommonUploads.upload_credit_timer ();
   CommonInteractive.force_download_quotas ();
   CommonResult.dummy_result.result_time <- last_time ();
   (try
-      Int64Swarmer.verify_some_chunks ()
-    with _ -> ()
-  );
-  CommonClient.clear_upload_slots ();
-  if !!auto_commit then
-    List.iter (fun file ->
-        file_commit file
-    ) !!CommonComplexOptions.done_files
+    CommonSwarming.verify_some_chunks ()
+  with _ -> ());
+  CommonClient.clear_upload_slots ()
 
 let hourly_timer timer =
   incr CommonWeb.hours;
-  CommonWeb.load_web_infos false;
+  CommonWeb.load_web_infos false false;
   if !CommonWeb.hours mod !!compaction_delay = 0 then Gc.compact ();
   if !!backup_options_delay <> 0
      && !CommonWeb.hours mod !!backup_options_delay = 0 then
        CommonComplexOptions.backup_options ();
   DriverControlers.check_calendar ();
-  CommonWeb.connect_redirector ();
   CommonFile.propose_filenames ()
 
+let ten_second_timer timer =
+  if !!auto_commit then
+    List.iter (fun file ->
+        file_commit file
+    ) !!CommonComplexOptions.done_files
 
 let second_timer timer =
   (try
@@ -98,6 +94,13 @@ let second_timer timer =
 
 let start_interfaces () =
 
+  (* option_hook(s) are not called when ini files are created the first time
+     force re-load of allowed_ips to call option_hook which fills the IP blocklist *)
+  ( 
+  match !created_new_base_directory with
+    None -> ()
+  | Some dir -> allowed_ips =:= !!allowed_ips
+  );
 
   if !!http_port <> 0 then begin try
         ignore (DriverControlers.create_http_handler ());
@@ -109,16 +112,6 @@ let start_interfaces () =
   ignore (find_port  "telnet server" !!telnet_bind_addr
       telnet_port DriverControlers.telnet_handler);
 
-  Dp500.start ();
-
-  if !!chat_port <> 0 then begin
-      ignore (find_port "chat server" !!chat_bind_addr
-          chat_port DriverControlers.chat_handler);
-      try
-        CommonChat.send_hello ()
-      with _ -> if !verbose then lprintf (_b "CommonChat.send_hello failed");
-    end;
-
   gui_server_sock := find_port "gui server"  !!gui_bind_addr
     gui_port gui_handler;
   if !!gift_port <> 0 then
@@ -129,43 +122,6 @@ let start_interfaces () =
   add_infinite_timer 1. second_timer
 
 
-let _ =
-  CommonWeb.add_web_kind "motd.html" (fun _ filename ->
-      lprintf_nl (_b "motd.html changed");
-    motd_html =:= File.to_string filename
-  );
-  CommonWeb.add_web_kind "motd.conf" (fun _ filename ->
-    let ic = open_in filename in
-    try
-      while true do
-	let line = input_line ic in
-	let cmd, args = String2.cut_at line ' ' in
-	let name, value = String2.cut_at args ' ' in
-	match cmd with
-	  "set" ->
-            CommonInteractive.set_fully_qualified_options name value
-	| "add_item" ->
-            CommonInteractive.add_item_to_fully_qualified_options name value
-	| "del_item" ->
-            CommonInteractive.del_item_from_fully_qualified_options name value
-	| _ ->
-	    lprintf_nl (_b "UNUSED LINE: %s") line
-
-      done;
-    with
-    | End_of_file ->
-	close_in ic
-    | e -> lprintf_nl (_b "Error while reading motd.conf(%s): %s") filename
-	(Printexc2.to_string e);
-	close_in ic
-  );
-  CommonWeb.add_web_kind "guarding.p2p" (fun _ filename ->
-      Ip_set.bl := Ip_set.load filename
-(*      Ip_set.bl := Ip_set.load_merge !Ip_set.bl filename *)
-  )
-
-
-
 let save_mlsubmit_reg () =
 
 (* Generate the mlsubmit.reg file *)
@@ -262,10 +218,11 @@ let load_config () =
   (try
       Options.load downloads_ini;
       Options.load users_ini;
+      ignore (DriverInteractive.verify_user_admin ());
+      DriverInteractive.hdd_check ()
     with e ->
         lprintf_nl "Exception %s during options load" (Printexc2.to_string e);
-        exit 70;
-        ());
+        exit 70);
 
   (* Here, we try to update options when a new version of mldonkey is
      used. For example, we can add new web_infos... *)
@@ -316,8 +273,8 @@ let load_config () =
           exit 0), _s " : print version number and exit";
       "-exit", Arg.Unit (fun _ -> exit 0), ": exit immediatly";
       "-format", Arg.String (fun file ->
-          let format = CommonMultimedia.get_info file in
-          ()), _s  " <filename> : check file format";
+          ignore (CommonMultimedia.get_info file)),
+          _s  " <filename> : check file format";
       "-test_ip", Arg.String (fun ip ->
           lprintf_nl "%s = %s" ip (Ip.to_string (Ip.of_string ip));
           exit 0), _s "<ip> : undocumented";
@@ -331,13 +288,13 @@ let load_config () =
           exit 0),
       _s " : display information on the implementations";
       "-stdout", Arg.Unit (fun _ ->
-          keep_console_output := true;
-          log_to_file stdout;
+	  lprintf_original_output := (Some stdout);
+          log_to_file stdout
       ),
        _s ": keep output to stdout after startup";
        "-stderr", Arg.Unit (fun _ ->
-           keep_console_output := true;
-           log_to_file stderr;
+	  lprintf_original_output := (Some stderr);
+          log_to_file stderr
        ),
        _s ": keep output to stderr after startup";
       "-daemon", Arg.Unit (fun _ ->
@@ -348,7 +305,7 @@ let load_config () =
       _s " : find another port when one is already used";
       "-pid", Arg.String (fun s -> pid := s;
       ),
-       _s ": directory for pid file (works only on non-Windows platforms)";
+       _s ": directory for pid file";
     ] @
       !more_args
       @
@@ -357,7 +314,7 @@ let load_config () =
 (*      Files.dump_file file; exit 0 *)
   ) "";
 
-  if not ini_files_exist && not !keep_console_output then log_file =:= "mlnet.log";
+  if not ini_files_exist && not (keep_console_output ()) then log_file =:= "mlnet.log";
 
   (**** CREATE DIRS   ****)
 
@@ -369,6 +326,8 @@ let load_config () =
   ) !!CommonComplexOptions.shared_directories;
   Unix2.safe_mkdir "searches";
   Unix2.can_write_to_directory "searches";
+  Unix2.safe_mkdir "web_infos";
+  Unix2.can_write_to_directory "web_infos";
   Unix2.safe_mkdir !!temp_directory;
   Unix2.can_write_to_directory !!temp_directory
 
@@ -385,9 +344,9 @@ let _ =
   (
     let hostname = "www.mldonkey.net" in
     try
-      ignore(Ip.from_name hostname)
-    with
-     e ->
+      ignore(Ip.from_name hostname);
+      DriverInteractive.dns_works := true
+    with e ->
         lprintf (_b "\nDNS resolution does not work! Looking up %s failed with %s.")
            hostname (Printexc2.to_string e);
 	lprintf "
@@ -395,7 +354,7 @@ The core therefore is unable to get eDonkey serverlists and loading
 .torrent files via dllink from websites is also impossible.
 If you are using MLDonkey in a chroot environment you should
 consider reading this article to get DNS support back:
-http://mldonkey.berlios.de/modules.php?name=Wiki&pagename=Chroot\n\n");
+http://mldonkey.sourceforge.net/Chroot\n\n");
   (
     let real_glibc_version = MlUnix.glibc_version_num () in
       if real_glibc_version <> Autoconf.glibc_version
@@ -407,6 +366,22 @@ This can lead to unexpected behaviour. Consider compiling the core yourself
 or getting a binary compiled with glibc %s.\n\n")
           real_glibc_version Autoconf.glibc_version Autoconf.glibc_version
   );
+  (
+    if Autoconf.magic then begin
+      (if Sys.file_exists "./magic/magic" then
+         try Unix.putenv "MAGIC" "./magic/magic" with _ -> ());
+      if Magic.M.magic_works () then
+        begin
+	  Autoconf.magic_works := true;
+          lprintf_nl (_b "Libmagic file-type recognition database present")
+	end
+      else
+        begin
+	  Autoconf.magic_works := false;
+          lprintf_nl (_b "Libmagic file-type recognition database not present")
+	end
+      end
+  );
   load_config ();
   
   add_infinite_option_timer download_sample_rate CommonFile.sample_timer;
@@ -424,19 +399,21 @@ or getting a binary compiled with glibc %s.\n\n")
     save_results =:= old_save_results;
   end;
 
-(*  lprintf "(3) networks_iter load_complex_options\n"; *)
   lprintf_nl (_b "Check http://www.mldonkey.net/ for updates");
   networks_iter (fun r -> network_load_complex_options r);
-  lprintf (_b "%senabling networks: ") (log_time ());
+  lprintf_nl (_b "enabling networks: ");
   networks_iter (fun r ->
-(*      lprintf "(4) networks_iter enabling\n"; *)
+      lprintf_nl (_b "---- enabling %s ----") r.network_name;
       network_enable r;
-      lprintf (_b "%s ") r.network_name;
+      List.iter (fun (p,s) -> if p <> 0 then lprintf_nl "using port %d (%s)" p s) (network_ports r);
 (* are there drawbacks to start recover_temp unconditionally here ? *)
       if !!recover_temp_on_startup then
         network_recover_temp r;
   );
-  lprintf (_b "\n%sdisabled networks: ") (log_time ());
+  lprintf_nl (_b "---- enabling interfaces ----");
+  List.iter (fun (p,s) -> if p <> 0 then lprintf_nl "using port %d (%s)" p s)
+    (network_ports (network_find_by_name "Global Shares"));
+  lprintf (_b "%sdisabled networks: ") (log_time ());
   let found = ref false in
     networks_iter_all (fun r ->
         if not (network_is_enabled r) then
@@ -450,8 +427,6 @@ or getting a binary compiled with glibc %s.\n\n")
   CommonOptions.start_running_plugins := true;
   CommonInteractive.force_download_quotas ();
 
-  TcpBufferedSocket.set_max_opened_connections
-    (fun _ -> !!max_opened_connections);
   TcpBufferedSocket.set_max_connections_per_second
     (fun _ -> !!max_connections_per_second);
 
@@ -460,8 +435,14 @@ or getting a binary compiled with glibc %s.\n\n")
   start_interfaces ();
 
   add_infinite_timer 60. minute_timer;
+  add_infinite_timer 10. ten_second_timer;
   add_infinite_timer 3600. hourly_timer;
   add_infinite_timer 0.1 CommonUploads.upload_download_timer;
+  add_infinite_timer !!buffer_writes_delay (fun _ -> Unix32.flush ());
+
+  if Autoconf.system = "mingw" then
+    add_infinite_timer 1. (fun timer ->
+        MlUnix.set_console_title (DriverInteractive.console_topic ()));
 
   List.iter
     CommonShared.shared_add_directory
@@ -473,19 +454,19 @@ or getting a binary compiled with glibc %s.\n\n")
   Options.prune_file downloads_ini;
   Options.prune_file users_ini;
 (*  Options.prune_file downloads_expert_ini; *)
-  add_timer 10. (fun _ -> try CommonWeb.load_web_infos true with _ -> ());
-  lprintf_nl (_b "To command: telnet %s %d")
+  add_timer 5. (fun _ -> try CommonWeb.load_web_infos true false with _ -> ());
+  lprintf_nl  (_b "To command: telnet %s %d")
 	(if !!telnet_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!telnet_bind_addr)  !!telnet_port;
-  lprintf_nl (_b "Or with browser: http://%s:%d")
+  lprintf_nl  (_b "Or with browser: http://%s:%d")
 	(if !!http_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!http_bind_addr)  !!http_port;
-  lprintf_nl (_b "For a GUI check out http://sancho-gui.sourceforge.net");
-  lprintf_nl (_b "Connect to IP %s, port %d")
+  lprintf_nl  (_b "For a GUI check out http://sancho-gui.sourceforge.net");
+  lprintf_nl  (_b "Connect to IP %s, port %d")
 	(if !!gui_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!gui_bind_addr)  !!gui_port;
-  lprintf_nl (_b "If you connect from a remote machine adjust allowed_ips");
-  if Autoconf.system = "cygwin" && not !keep_console_output then lprintf (_b "%s") win_message;
+  lprintf_nl  (_b "If you connect from a remote machine adjust allowed_ips");
+  if Autoconf.system = "cygwin" && not (keep_console_output ()) then lprintf (_b "%s") win_message;
 
   add_init_hook (fun _ ->
       if not !gui_included && ( !!start_gui || !!ask_for_gui ) then
@@ -531,71 +512,77 @@ or getting a binary compiled with glibc %s.\n\n")
 
   if not Autoconf.windows then
     MlUnix.set_signal  Sys.sigchld
-      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl "Received SIGCHLD, doing nothing"));
+      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl (_b "Received SIGCHLD, doing nothing")));
 
   if not Autoconf.windows then
     MlUnix.set_signal  Sys.sighup
-      (Sys.Signal_handle (fun _ ->
-	 lprintf_nl "Received SIGHUP, closing all files/sockets";
-         BasicSocket.close_all ();
-	 Unix32.close_all ()
+      (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGHUP, closing all files and client/server sockets");
+         networks_iter (fun r -> CommonNetwork.network_reset r); (* stop_all_bt *)
+	 CommonServer.disconnect_all_servers ();
+	 CommonClient.disconnect_all_clients ();
+	 Unix32.close_all () (* close all files *)
          ));
 
   if not Autoconf.windows then
     MlUnix.set_signal  Sys.sigpipe
-      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl "Received SIGPIPE, doing nothing"));
+      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl (_b "Received SIGPIPE, doing nothing")));
 
   MlUnix.set_signal  Sys.sigint
-    (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGINT, stopping MLDonkey...";
+    (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGINT, stopping MLDonkey...");
         CommonInteractive.clean_exit 0));
 
   MlUnix.set_signal  Sys.sigterm
-    (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGTERM, stopping MLDonkey...";
+    (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGTERM, stopping MLDonkey...");
         CommonInteractive.clean_exit 0));
 
-  if !verbose then lprintf_nl (_b "Activated system signal handling");
+  if not Autoconf.windows then
+  MlUnix.set_signal  Sys.sigusr1
+    (Sys.Signal_handle (fun _ -> lprintf_nl (_b "Received SIGUSR1, saving options...");
+        DriverInteractive.save_config ()));
 
-  Unix32.max_cache_size := MlUnix.max_filedescs
+  if not Autoconf.windows then
+  MlUnix.set_signal  Sys.sigusr2
+    (Sys.Signal_handle (fun _ -> lprintf_n (_b "Received SIGUSR2, starting garbage collection...");
+	Gc.compact ();
+	lprintf " finished";
+	lprint_newline ()));
+
+  if !verbose then lprintf_nl (_b "Activated system signal handling")
 
 let _ =
-  let security_space_filename = "config_files_space.tmp" in
+  let security_space_oc = ref None in
   begin
 (* Create a 'config_files_security_space' megabytes file to protect some space
 for config files at the end. *)
     try
-      let security_space_fd = Unix32.create_rw security_space_filename in
-      let _ =
+      let oc = Unix.openfile security_space_filename [Unix.O_WRONLY; Unix.O_CREAT] 0o600 in
         let len = 32768 in
-        let len64 = Int64.of_int len in
-        let s = String.create len in
+        let s = String.make len ' ' in
         let pos = ref zero in
         for i = 1 to !!config_files_security_space do
           for j = 1 to 32 do (* 32 = 1 MB / 32kB *)
-            Unix32.write security_space_fd !pos s 0 len;
-            pos := !pos ++ len64
+	    ignore(Unix2.c_seek64 oc !pos Unix.SEEK_SET);
+            Unix2.really_write oc s 0 len;
+            pos := !pos ++ (Int64.of_int len)
           done
-        done
-      in
-      Unix32.close security_space_fd;
-    with _ ->
-        lprintf_nl (_b "Cannot create Security space file:");
+        done;
+	ignore(Unix2.c_seek64 oc zero Unix.SEEK_SET);
+	(try Unix.lockf oc Unix.F_LOCK (!!config_files_security_space * 1024 * 1024) with _ -> ());
+	security_space_oc := Some oc
+    with e ->
+        lprintf_nl (_b "Cannot create Security space file: %s") (Printexc2.to_string e);
         lprintf_nl (_b " not enough space on device or bad permissions");
         lprintf_nl (_b "Exiting...");
         exit 73;
   end;
-  Unix32.external_start (CommonGlobals.version());
+  Unix32.external_start ();
 
   (
-    let pid_filename =
-      Printf.sprintf "%s.pid" (Filename.basename Sys.argv.(0))
-    in
     let pid_file, s =
         Filename.concat !pid pid_filename,
 	Printf.sprintf "%s\n" (string_of_int(Unix.getpid()))
     in
-    let oc = open_out pid_file in
-    output_string oc s;
-    close_out oc;
+    Unix2.tryopen_write pid_file (fun oc -> output_string oc s);
     CommonGlobals.do_at_exit (fun _ -> try Sys.remove pid_file with _ -> ());
     if !verbose then
       lprintf_nl (_b "Starting with pid %s") (string_of_int(Unix.getpid ()))
@@ -603,15 +590,15 @@ for config files at the end. *)
 
 (* When a core is spawned from a gui, the only way to know the startup has
    succeeded is the string token "Core started". *)
-  if not !keep_console_output then
+  if not (keep_console_output ()) then
     begin
       try
-        Printf.fprintf Pervasives.stderr "%sCore started\n" (log_time ());
-        Pervasives.flush Pervasives.stderr;
+        Printf.eprintf "%s[dMain] Core started\n%!" (log_time ());
       with _ -> ()
     end;
 
   lprintf_nl (_b "Core started");
+  pause_new_downloads =:= false;
   core_included := true;
 
   CommonGlobals.do_at_exit (fun _ ->
@@ -623,18 +610,23 @@ for config files at the end. *)
            lprintf_nl "Exception %s in do_at_exit while closing sockets."
              (Printexc2.to_string e);
       );
-      CommonGraphics.remove_files ();
+      DriverGraphics.G.remove_files ();
       (* In case we have no more space on filesystem for
          config files, remove the security space file *)
-      Sys.remove security_space_filename;
+      (match !security_space_oc with
+	None -> ()
+      | Some oc -> Unix.close oc);
+      (try Sys.remove security_space_filename with _ -> ());
+      CommonComplexOptions.allow_saving_ini_files := true;
       DriverInteractive.save_config ();
       CommonComplexOptions.save_sources ();
       CommonComplexOptions.backup_options ();
+      Geoip.close ();
       Unix32.external_exit ();
       lprintf_nl (_b "Core stopped")
     );
 
-  if not !keep_console_output then
+  if not (keep_console_output ()) then
     if !!log_file = "" then 
       begin
         lprintf_nl (_b "Option log_file is empty, disable logging completely...");
