@@ -42,7 +42,7 @@ open GuiProto
 module M = Gui_messages
 
 
-let mldonkey_cache = Filename.concat Sysenv.home ".mldonkey_cache" 
+let mldonkey_cache = Filename.concat CommonOptions.home_dir "cache" 
   
 let file_from_url url = 
   try
@@ -60,7 +60,7 @@ let file_from_url url =
       
 let wget_string r f progress =
   let module H = Http_client in
-  let file = file_from_url (Url.to_string true r.H.req_url) in
+  let file = file_from_url (Url.to_string r.H.req_url) in
   try
     let page = File.to_string file in
     BasicSocket.add_timer 0.1 (fun _ -> f page)
