@@ -299,7 +299,7 @@ class box columns friend_tab =
             Known_location _ -> M.fT_tx_direct
           | _ -> "")
       | Col_client_rating -> string_of_int f.gclient_rating
-      | Col_client_connect_time -> Date.time_to_string_long (f.gclient_connect_time)
+      | Col_client_connect_time -> Date.time_to_string (f.gclient_connect_time) "long"
       | Col_client_software -> f.gclient_software
       | Col_client_release -> f.gclient_release
       | Col_client_emulemod -> f.gclient_emulemod
@@ -470,7 +470,7 @@ class box_friends box_files friend_tab =
       }
     
     method update_friend f_new =
-      if client_browsed_tag land f_new.client_type = 0 then
+      if (client_friend_tag lor client_contact_tag) land f_new.client_type = 0 then
         self#h_remove_friend f_new.client_num
       else
       try
@@ -527,7 +527,7 @@ class box_friends box_files friend_tab =
     method update_friend_type (num, friend_kind) =
       try
         let (row, fi) = self#find_client num in
-        if client_browsed_tag land friend_kind = 0 then
+        if (client_friend_tag lor client_contact_tag) land friend_kind = 0 then
           self#h_remove_friend num
           else begin
             fi.gclient_type <- friend_kind;
