@@ -41,10 +41,6 @@ module StatsOption = struct
     let value_to_stat v =
       match v with
         Options.Module assocs ->
-          let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv =
-            try conv (List.assoc name assocs) with _ -> []
-          in
           {
             brand_seen = value_to_int (List.assoc "seen" assocs);
             brand_banned = value_to_int (List.assoc "banned" assocs);
@@ -68,11 +64,6 @@ module StatsOption = struct
     let t = define_option_class "Stat" value_to_stat stat_to_value
   end
 
-(* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[BT] "
-    (log_time ()); lprintf_nl2
-
 module ClientOption = struct
 
     let value_to_client file v =
@@ -80,9 +71,6 @@ module ClientOption = struct
       | Module assocs ->
 
           let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv =
-            try conv (List.assoc name assocs) with _ -> []
-          in
           let client_ip = get_value "client_ip" (from_value Ip.option)
           in
           let client_port = get_value "client_port" value_to_int in
@@ -108,10 +96,6 @@ module ClientOption = struct
 
 let value_to_file file_size file_state assocs =
   let get_value name conv = conv (List.assoc name assocs) in
-  let get_value_nil name conv =
-    try conv (List.assoc name assocs) with _ -> []
-  in
-
   let file_trackers =
     try
       get_value "file_trackers" (value_to_list value_to_string)
@@ -182,11 +166,9 @@ let value_to_file file_size file_state assocs =
             torrent_nodes = file_nodes;
 *)
             torrent_announce =
-            (
-              try
-                (List.hd file_trackers)
-              with _ -> ""
-            );
+            (match file_trackers with
+	      | h::q -> h
+	      | [] -> "");
             torrent_announce_list = file_trackers;
           } in
         let torrent_diskname = Filename.concat downloads_directory
@@ -212,7 +194,7 @@ let value_to_file file_size file_state assocs =
   (match file.file_swarmer with
       None -> ()
     | Some swarmer ->
-        Int64Swarmer.value_to_swarmer swarmer assocs;
+        CommonSwarming.value_to_frontend swarmer assocs;
   );
 
 (*
@@ -221,7 +203,7 @@ let value_to_file file_size file_state assocs =
         (get_value  "file_sources" (
           value_to_list (ClientOption.of_value file)))
     with e ->
-        lprintf_nl () "Exception %s while loading sources"
+        lprintf_nl "Exception %s while loading sources"
           (Printexc2.to_string e);
   );
 *)
@@ -229,7 +211,6 @@ let value_to_file file_size file_state assocs =
 
 let file_to_value file =
   try
-    let sources = Hashtbl2.to_list file.file_clients in
     let assocs =
       [
         "file_temp", string_to_value (Unix32.filename (file_fd file));
@@ -257,7 +238,7 @@ send us more clients.
       ("file_hashes", array_to_value
           (to_value Sha1.option) file.file_chunks) ::
       ("file_files", list_to_value
-          (fun (name, p1) ->
+          (fun (name, p1, _) ->
             SmallList [string_to_value name; int64_to_value p1])
         file.file_files) ::
       assocs
@@ -265,10 +246,10 @@ send us more clients.
     match file.file_swarmer with
       None -> assocs
     | Some swarmer ->
-        Int64Swarmer.swarmer_to_value swarmer assocs
+        CommonSwarming.frontend_to_value swarmer assocs
   with
     e ->
-      lprintf_nl () "exception %s in file_to_value"
+      lprintf_file_nl file "exception %s in file_to_value"
         (Printexc2.to_string e); raise e
 
 
