@@ -55,7 +55,7 @@ let as_file file = as_file file.file_file
 let file_size file = file.file_file.impl_file_size
 let file_downloaded file = file_downloaded (as_file file)
 let file_age file = file.file_file.impl_file_age
-let file_fd file = file.file_file.impl_file_fd
+let file_fd file = file_fd (as_file file)
 let file_disk_name file = file_disk_name (as_file file)
 let file_state file = file_state (as_file file)
 let file_num file = file_num (as_file file)
@@ -121,10 +121,18 @@ let max_request_len = Int64.of_int (1 lsl 16)
 let bt_download_counter = ref Int64.zero
 let bt_upload_counter = ref Int64.zero
 
-(* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[BT] "
-    (log_time ()); lprintf_nl2
+let log_prefix = "[BT]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_file_nl  file fmt  =
+  lprintf_nl2 (log_prefix^" [file_num "^(string_of_int (file_num file))^"]") fmt
+
+    
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
+
 
 let check_if_interesting file c =
 
@@ -133,26 +141,27 @@ let check_if_interesting file c =
         None -> assert false
       | Some up -> up
     in
-    let swarmer = Int64Swarmer.uploader_swarmer up in
+    let swarmer = CommonSwarming.uploader_swarmer up in
     let must_send =
 (* The client has nothing to propose to us *)
-      (not (Int64Swarmer.is_interesting up )) &&
+      (not (CommonSwarming.is_interesting up )) &&
 (* All the requested ranges are useless *)
       (List.filter (fun (_,_,r) ->
-            let x,y = Int64Swarmer.range_range r in
+            let x,y = CommonSwarming.range_range r in
             x < y) c.client_ranges_sent = []) &&
       (match c.client_range_waiting with
           None -> true
         | Some (x,y,r) ->
-            let x,y = Int64Swarmer.range_range r in
+            let x,y = CommonSwarming.range_range r in
             x < y) &&
-(* The current block is also useless *)
-      (match c.client_block with
-          None -> true
-        | Some b ->
-            let block_num = Int64Swarmer.block_num swarmer b in
-            let bitmap = Int64Swarmer.verified_bitmap swarmer in
-            bitmap.[block_num] <> '3')
+(* The current blocks are also useless *)
+      (match c.client_chunk with
+      | None -> true
+      | Some (chunk, blocks) ->
+	  List.for_all (fun b ->
+            let chunk_num = CommonSwarming.block_chunk_num swarmer b.up_block in
+            let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
+            VB.get bitmap chunk_num <> VB.State_verified) blocks)
     in
     if must_send then
       begin
@@ -162,18 +171,21 @@ let check_if_interesting file c =
       end
 
 let add_torrent_infos file trackers =
-  List.iter (fun tracker ->
-      file.file_trackers <- tracker :: file.file_trackers
-  ) trackers
+  file.file_trackers <- trackers @ file.file_trackers
 
-let create_temp_file file_temp file_files =
-  if !CommonOptions.verbose_files then lprintf_nl () "create_temp_file %s" file_temp;
+let create_temp_file file_temp file_files file_state =
+  if !verbose then lprintf_nl "create_temp_file %s - %s" file_temp (string_of_state file_state);
+  let writable =
+    if file_state = FileShared then
+      false
+    else
+      true
+  in
   let file_fd =
     if file_files <> [] then
-      Unix32.create_multifile file_temp
-        [Unix.O_RDWR; Unix.O_CREAT] 0o666 file_files
+      Unix32.create_multifile file_temp writable file_files
     else
-      Unix32.create_rw file_temp
+      Unix32.create_diskfile file_temp writable
   in
   if Unix32.destroyed file_fd then
     failwith
@@ -182,27 +194,40 @@ let create_temp_file file_temp file_files =
         file_temp);
   file_fd
 
-let set_trackers file file_trackers =
-  file.file_trackers <- (List.map (fun url -> {
-          tracker_url = url;
-          tracker_interval = 600;
-          tracker_min_interval = 600;
-          tracker_last_conn = 0;
-          tracker_last_clients_num = 0;
-          tracker_torrent_downloaded = 0;
-          tracker_torrent_complete = 0;
-          tracker_torrent_incomplete = 0;
-          tracker_torrent_total_clients_count = 0;
-          tracker_torrent_last_dl_req = 0;
-          tracker_id = "";
-          tracker_key = "";
-        } ) file_trackers) @ file.file_trackers
+let can_handle_tracker t =
+  String2.check_prefix (String.lowercase t.tracker_url) "http://"
+
+let rec set_trackers file file_trackers =
+  match file_trackers with
+    | [] -> ()
+    | url :: q ->
+	if not (List.exists (fun tracker -> 
+			       tracker.tracker_url = url
+			    ) file.file_trackers) then 
+	  let t = {
+            tracker_url = url;
+            tracker_interval = 600;
+            tracker_min_interval = 600;
+            tracker_last_conn = 0;
+            tracker_last_clients_num = 0;
+            tracker_torrent_downloaded = 0;
+            tracker_torrent_complete = 0;
+            tracker_torrent_incomplete = 0;
+            tracker_torrent_total_clients_count = 0;
+            tracker_torrent_last_dl_req = 0;
+            tracker_id = "";
+            tracker_key = "";
+	    tracker_enabled = true
+          } in
+	  t.tracker_enabled <- can_handle_tracker t;
+	  file.file_trackers <-  t :: file.file_trackers;
+	set_trackers file q
 
 let new_file file_id t torrent_diskname file_temp file_state =
   try
     Hashtbl.find files_by_uid file_id
   with Not_found ->
-      let file_fd = create_temp_file file_temp t.torrent_files in
+      let file_fd = create_temp_file file_temp t.torrent_files file_state in
       let rec file = {
           file_tracker_connected = false;
           file_file = file_impl;
@@ -219,7 +244,7 @@ let new_file file_id t torrent_diskname file_temp file_state =
           file_swarmer = None;
           file_trackers = [];
           file_chunks = t.torrent_pieces;
-          file_files = t.torrent_files;
+          file_files = (List.map (fun (file,size) -> (file,size,None)) t.torrent_files);
           file_blocks_downloaded = [];
           file_uploaded = Int64.zero;
           file_torrent_diskname = torrent_diskname;
@@ -227,7 +252,7 @@ let new_file file_id t torrent_diskname file_temp file_state =
           file_shared = None;
         } and file_impl =  {
           dummy_file_impl with
-          impl_file_fd = file_fd;
+          impl_file_fd = Some file_fd;
           impl_file_size = t.torrent_length;
           impl_file_downloaded = Int64.zero;
           impl_file_val = file;
@@ -236,17 +261,16 @@ let new_file file_id t torrent_diskname file_temp file_state =
           impl_file_best_name = t.torrent_name;
         }
       in
-      if List.length t.torrent_announce_list > 1 then
+      if t.torrent_announce_list <> [] then
         set_trackers file t.torrent_announce_list
       else
         set_trackers file [t.torrent_announce];
       if file_state <> FileShared then begin
-          let kernel = Int64Swarmer.create_swarmer file_temp (file_size file)
-            (min max_range_len file.file_piece_size) in
-          let swarmer = Int64Swarmer.create kernel (as_file file)
+          let kernel = CommonSwarming.create_swarmer file_temp (file_size file) in
+          let swarmer = CommonSwarming.create kernel (as_file file)
             file.file_piece_size in
           file.file_swarmer <- Some swarmer;
-          Int64Swarmer.set_verified swarmer (fun _ num ->
+          CommonSwarming.set_verified swarmer (fun _ num ->
               file.file_blocks_downloaded <- (num) ::
               file.file_blocks_downloaded;
               file_must_update file;
@@ -266,7 +290,7 @@ let new_file file_id t torrent_diskname file_temp file_state =
               ) file.file_clients
 
           );
-          Int64Swarmer.set_verifier swarmer (Verification
+          CommonSwarming.set_verifier swarmer (Verification
               (Array.map (fun sha1 -> Sha1 sha1) file.file_chunks));
         end;
       current_files := file :: !current_files;
@@ -292,7 +316,7 @@ let new_ft file_name =
       ft_retry = (fun _ -> ());
     } and file_impl =  {
       dummy_file_impl with
-      impl_file_fd = Unix32.bad_fd;
+      impl_file_fd = None;
       impl_file_size = zero;
       impl_file_downloaded = Int64.zero;
       impl_file_val = ft;
@@ -339,10 +363,29 @@ let dot_string_of_list s l =
   List.iter (fun i -> Buffer.add_char buf s.[i]) l;
   dot_string (Buffer.contents buf)
 
+let dot_string_of_string s =
+  let buf = Buffer.create 20 in
+  let found_non_int = ref false in
+  String.iter (fun s ->
+    match s with
+    | '0' .. '9' ->
+	if !found_non_int then Buffer.add_char buf '.';
+	found_non_int := false;
+	Buffer.add_char buf s
+    | _ -> found_non_int := true
+  ) s;
+  Buffer.contents buf
+
 let check_all s c l =
   let ch = char_of_int c in
   List.for_all (fun i -> s.[i] = ch) l
 
+let check_int s p =
+  try
+    ignore (int_of_string (String.sub s p 1));
+    true
+  with _ -> false
+
 (* from azureus/gpl *)
 let decode_az_style s =
   if check_all s 45 [0;7] then begin
@@ -356,6 +399,7 @@ let decode_az_style s =
       | "BS" -> Brand_btslave
       | "BX" -> Brand_bittorrentx
       | "CT" -> Brand_ctorrent
+      | "CD" -> Brand_ctorrent
       | "LT" -> Brand_libtorrent
       | "MT" -> Brand_moonlighttorrent
       | "SB" -> Brand_swiftbit
@@ -370,25 +414,42 @@ let decode_az_style s =
       | "lt" -> Brand_libtorrent
       | "MP" -> Brand_moopolice
       | "UT" -> Brand_utorrent
+      | "KT" -> Brand_ktorrent
+      | "LP" -> Brand_lphant
+      | "TR" -> Brand_transmission
+      | "HN" -> Brand_hydranode
+      | "RT" -> Brand_retriever
       | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
       let version = 
-        if brand = Brand_bitcomet then (String.sub s 4 1) ^ "." ^ (String.sub s 5 2) 
-        else (dot_string (String.sub s 3 4)) 
+        match brand with
+          | Brand_bitcomet -> (String.sub s 4 1) ^ "." ^ (String.sub s 5 2)
+          | Brand_utorrent -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "." ^ (String.sub s 5 1)
+          | Brand_ctorrent 
+          | Brand_transmission -> (String.sub s 3 2) ^ "." ^ (String.sub s 5 2)
+          | Brand_ktorrent -> 
+              let x = match s.[5] with 
+                | 'R' -> " RC" ^ (String.sub s 6 1)
+                | 'D' -> " Dev"
+                | _ -> ""
+              in
+              (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ x
+          | _ -> (dot_string (String.sub s 3 4)) 
       in
       Some (brand, version)
   end else
   None
 
 let decode_tornado_style s =
-  if check_all s 45 [4;5] then begin
+  if check_all s 45 [5] then begin
     let check_brand s =
      match s with
      | "T" -> Brand_bittornado
      | "S" -> Brand_shadow
      | "A" -> Brand_abc
      | "U" -> Brand_upnp
+     | "O" -> Brand_osprey
      | _ -> Brand_unknown
     in
     let bv = ref None in
@@ -409,7 +470,7 @@ let decode_tornado_style s =
   None
 
 let decode_mainline_style s =
-  if check_all s 45 [2;4;6;7] then begin
+  if check_all s 45 [2;7] && check_int s 1 then begin
     let s_id = String.sub s 0 1 in
     let brand =
       match s_id with
@@ -417,7 +478,7 @@ let decode_mainline_style s =
      | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None
-    else Some (brand, (dot_string_of_list s [1;3;5]))
+    else Some (brand, (dot_string_of_string (String.sub s 1 6)))
   end else
   None
 
@@ -431,15 +492,18 @@ let decode_simple_style s =
       (0, "S587Plus", Brand_btplus, "");
       (5, "Azureus", Brand_azureus, "2.0.3.2");
       (0, "-G3", Brand_g3torrent, "");
+      (0, "-AR", Brand_arctic, "");
       (4, "btfans", Brand_simplebt, "");
       (0, "btuga", Brand_btugaxp, "");
+      (0, "BTuga", Brand_btugaxp, "");
       (0, "DansClient", Brand_xantorrent, "");
       (0, "Deadman Walking-", Brand_deadmanwalking, "");
       (0, "346-", Brand_torrenttopia, "");
       (0, "271-", Brand_greedbt, "2.7.1");
-      (0, "BG", Brand_btgetit, "");
+      (10, "BG", Brand_btgetit, "");
       (0, "a00---0", Brand_swarmy, "");
       (0, "a02---0", Brand_swarmy, "");
+      (0, "10-------", Brand_jvtorrent, "");
       (0, "T00---0", Brand_teeweety, "") ]
   in
   let len = List.length !simple_list in
@@ -467,9 +531,17 @@ let decode_opera s =
      Some (Brand_opera, (dot_string_of_list s [2;3;4;5]))
   else None
 
+let decode_rufus s =
+  let release s =
+    let minor = Char.code s.[1] in
+    Printf.sprintf "%d.%d.%d" (Char.code s.[0]) (minor / 10) (minor mod 10) in
+  if "RS" = String.sub s 2 2 then
+     Some (Brand_rufus, release s)
+  else None
+
 let decode_bow s =
   if "BOW" = String.sub s 0 3 ||
-  (check_all s 45 [0;7] && "BOW" = String.sub s 1 4) then
+  (check_all s 45 [0;7] && "BOW" = String.sub s 1 3) then
     Some (Brand_bitsonwheels, (String.sub s 4 3))
   else None
 
@@ -513,6 +585,7 @@ let decode_bitspirit s =
     let bv = ref None in
     if s.[1] = (char_of_int 0) then bv := Some (Brand_bitspirit, "v1");
     if s.[1] = (char_of_int 2) then bv := Some (Brand_bitspirit, "v2");
+    if s.[1] = (char_of_int 3) then bv := Some (Brand_bitspirit, "v3");
     !bv
   end else
   None
@@ -618,13 +691,14 @@ let decoder_list = [
     decode_non_zero;
     decode_mldonkey_style;
     decode_opera;
+    decode_rufus;
   ]
 
-let parse_software s = 
+let parse_software s =
   let default = (Brand_unknown, "") in
   let rec iter l =
     match l with
-      [] -> lprintf_nl () "Unknown BT client software version, report the next line to http://mldonkey.berlios.de/modules.php?name=Wiki&pagename=UnknownBtClients%s\nBTUC:\"%s\"" Autoconf.current_version (String.escaped s);
+      [] -> lprintf_nl "Unknown BT client software version, report the next line to http://mldonkey.sourceforge.net/UnknownBtClients%s\nBTUC:\"%s\"" Autoconf.current_version (String.escaped s);
             default
       | d :: t -> match (d s) with 
                   | None -> iter t
@@ -658,7 +732,7 @@ let new_client file peer_id kind =
           client_chunks = [];
           client_ranges_sent = [];
           client_range_waiting = None;
-          client_block = None;
+          client_chunk = None;
           client_uid = peer_id;
           client_brand = brand;
           client_release = release;
@@ -704,7 +778,7 @@ let remove_client c =
 let remove_tracker url file =
   if !verbose_msg_servers then
     List.iter (fun tracker ->
-      lprintf_nl () "Old tracker list :%s" tracker.tracker_url
+      lprintf_nl "Old tracker list :%s" tracker.tracker_url
     ) file.file_trackers;
   List.iter (fun bad_tracker ->
     if bad_tracker.tracker_url = url then
@@ -712,10 +786,11 @@ let remove_tracker url file =
   ) file.file_trackers;
   if !verbose_msg_servers then
     List.iter (fun tracker ->
-      lprintf_nl () "New tracker list :%s" tracker.tracker_url
+      lprintf_nl "New tracker list :%s" tracker.tracker_url
     ) file.file_trackers
 
 let torrents_directory = "torrents"
+let new_torrents_directory = Filename.concat torrents_directory "incoming"
 let downloads_directory = Filename.concat torrents_directory "downloads"
 let tracked_directory = Filename.concat torrents_directory "tracked"
 let seeded_directory = Filename.concat torrents_directory "seeded"
