@@ -32,6 +32,11 @@ open BTGlobals
 open BTTypes
 open CommonTypes
 
+let log_prefix = "[bTM]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
 let is_enabled = ref false
 
 let disable enabler () =
@@ -59,17 +64,19 @@ let enable () =
     Unix2.safe_mkdir tracked_directory;
     Unix2.safe_mkdir downloads_directory;
     Unix2.safe_mkdir old_directory;
+    Unix2.safe_mkdir new_torrents_directory;
     Unix2.can_write_to_directory torrents_directory;
     Unix2.can_write_to_directory seeded_directory;
     Unix2.can_write_to_directory tracked_directory;
     Unix2.can_write_to_directory downloads_directory;
     Unix2.can_write_to_directory old_directory;
+    Unix2.can_write_to_directory new_torrents_directory;
     is_enabled := true;
     if !!BTTracker.tracker_port = !!client_port then
       begin
         lprint_newline();
-	lprintf_nl () "BT-client_port and BT-tracker_port can not be the same.";
-	lprintf_nl () "Change one of the settings and restart MLDonkey, exiting...\n";
+  lprintf_nl "BT-client_port and BT-tracker_port can not be the same.";
+  lprintf_nl "Change one of the settings and restart MLDonkey, exiting...\n";
 	Pervasives.exit 69
       end;
     if !!BTTracker.tracker_port > 0 then (
@@ -80,6 +87,10 @@ let enable () =
     add_session_timer enabler 300. (fun _ ->
         BTInteractive.share_files ();
     );
+    if !!import_new_torrents_interval <> 0. then
+    add_session_timer enabler !!import_new_torrents_interval (fun _ ->
+        BTInteractive.scan_new_torrents_directory ();
+    );
     add_timer 10. BTInteractive.share_files;
     add_session_timer enabler 600. BTInteractive.retry_all_ft;
     network.op_network_disable <- disable enabler;
