@@ -96,6 +96,14 @@ type brand =
 | Brand_torrenttopia
 | Brand_greedbt
 | Brand_btgetit
+| Brand_ktorrent
+| Brand_lphant
+| Brand_transmission
+| Brand_hydranode
+| Brand_jvtorrent
+| Brand_retriever
+| Brand_osprey
+| Brand_rufus
 
 let brand_list = [ 
    ( Brand_unknown          , "unknown"                , "unk" ) ;
@@ -147,6 +155,14 @@ let brand_list = [
    ( Brand_torrenttopia     , "TorrentTopia"           , "tt" ) ;
    ( Brand_greedbt          , "GreedBT"                , "gbt" ) ;
    ( Brand_btgetit          , "BTGetit"                , "btg" ) ;
+   ( Brand_ktorrent         , "KTorrent"               , "kto" ) ;
+   ( Brand_lphant           , "Lphant"                 , "lph" ) ;
+   ( Brand_transmission     , "Transmission"           , "tra" ) ;
+   ( Brand_hydranode        , "Hydranode"              , "hyd" ) ;
+   ( Brand_jvtorrent        , "JVtorrent"              , "jvt" ) ;
+   ( Brand_retriever        , "Retriever"              , "ret" ) ;
+   ( Brand_osprey           , "Osprey permaseed"       , "osp" ) ;
+   ( Brand_rufus            , "Rufus"                  , "ruf" ) ;
   ] 
 
 let brand_count = List.length brand_list
@@ -167,11 +183,11 @@ type client = {
     mutable client_sock : tcp_connection;
     mutable client_host : Ip.t * int;
     mutable client_chunks : (int64 * int64) list;
-    mutable client_uploader : Int64Swarmer.uploader option;
-    mutable client_ranges_sent : (int64 * int64 * Int64Swarmer.range) list;
+    mutable client_uploader : CommonSwarming.uploader option;
+    mutable client_ranges_sent : (int64 * int64 * CommonSwarming.range) list;
     mutable client_range_waiting :
-    (int64 * int64 * Int64Swarmer.range) option;
-    mutable client_block : Int64Swarmer.block option;
+    (int64 * int64 * CommonSwarming.range) option;
+    mutable client_chunk : (int * CommonSwarming.uploader_block list) option;
 
     mutable client_received_peer_id : bool;
     mutable client_sent_choke : bool; (* we sent a Choke to the client *)
@@ -217,6 +233,7 @@ and tracker_info = {
     mutable tracker_torrent_last_dl_req : int;
     mutable tracker_id : string;
     mutable tracker_key : string;
+    mutable tracker_enabled : bool;
   }
 
 and file = {
@@ -229,11 +246,11 @@ and file = {
     file_creation_date : int64;
     file_modified_by : string;
     file_encoding : string;
-    mutable file_swarmer : Int64Swarmer.t option;
+    mutable file_swarmer : CommonSwarming.t option;
     mutable file_clients : ((Ip.t*int), client) Hashtbl.t ;
     mutable file_clients_num : int ;
     mutable file_chunks : Sha1.t array;
-    mutable file_files : (string * int64) list;
+    mutable file_files : (string * int64 * string option) list;
     mutable file_blocks_downloaded : int list;
     (* vvv probably a network specific value vvv ?what about file_downloaded?*)
     mutable file_uploaded : int64;
