@@ -42,6 +42,11 @@ open DonkeyTypes
 open DonkeyOptions
 open CommonOptions
 
+let log_prefix = "[EDK]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
 (*************************************************************
 
 Define the instances of the plugin classes, that we be filled
@@ -61,7 +66,7 @@ let network = CommonNetwork.new_network "ED2K" "Donkey"
   ]
 
 let connection_manager = network.network_connection_manager
-let connections_controler = TcpServerSocket.create_connections_contoler
+let connections_controler = TcpServerSocket.create_connections_controler
     "Edonkey" (fun _ _ -> true)
 
 let (shared_ops : file CommonShared.shared_ops) =
@@ -100,7 +105,7 @@ let file_priority file = file.file_file.impl_file_priority
 let file_size file = file.file_file.impl_file_size
 let file_downloaded file = file_downloaded (as_file file)
 let file_age file = file.file_file.impl_file_age
-let file_fd file = file.file_file.impl_file_fd
+let file_fd file = file_fd (as_file file)
 let file_disk_name file = file_disk_name (as_file file)
 let file_best_name file = file_best_name (as_file file)
 
@@ -145,6 +150,29 @@ let xs_last_search = ref (-1)
 let zone_size = Int64.of_int (180 * 1024)
 let block_size = 9728000L
 
+(* Old value: *)
+(* let nchunks = Int64.to_int (Int64.pred file_size // block_size) + 1 in *)
+
+(* New value: *)
+(* From Emule: KnownFile.cpp 
+// File size       Data parts      ED2K parts      ED2K part hashs
+// ---------------------------------------------------------------
+// 1..PARTSIZE-1   1               1               0(!)
+// PARTSIZE        1               2(!)            2(!)
+// PARTSIZE+1      2               2               2
+// PARTSIZE*2      2               3(!)            3(!)
+// PARTSIZE*2+1    3               3               3
+*)
+
+let get_nchunks size = Int64.to_int (size // block_size) + 1 
+
+let get_nchunk_hashes size = 
+    let nchunk_hashes = Int64.to_int (size // block_size) in
+    let nchunk_hashes = if nchunk_hashes <> 0 
+      then nchunk_hashes + 1 
+      else nchunk_hashes in
+    nchunk_hashes
+
 let queue_timeout = ref (60. *. 10.) (* 10 minutes *)
 
 let files_queries_per_minute = 3 (* queries for 3 files cost 3*16=48 server-credits; we did get 60 (1 each second) *)
@@ -157,7 +185,6 @@ let master_server = ref (None: DonkeyTypes.server option)
 let udp_sock = ref (None: UdpSocket.t option)
 let listen_sock = ref (None : TcpServerSocket.t option)
 let reversed_sock = ref (None : TcpServerSocket.t option)
-let servers_ini_changed = ref true
 let new_shared = ref false
 
 (*************************************************************************)
@@ -183,6 +210,7 @@ let servers_list = ref ([] : server list)
 let current_files = ref ([] : file list)
 let xs_servers_list = ref ([] : server list)
 let connected_server_list = ref ([]  : server list)
+let connecting_server_list = ref ([] : server list)
 
 let (banned_ips : (Ip.t, int) Hashtbl.t) = Hashtbl.create 113
 let (old_requests : (int * int, request_record) Hashtbl.t) =
@@ -208,8 +236,6 @@ let udp_servers_replies = (Hashtbl.create 127 : (Md4.t, server) Hashtbl.t)
 
 let file_groups = (Hashtbl.create 1023 : (Md4.t, file_group) Hashtbl.t)
 
-let file_md4s_to_register = ref ([] : file list)
-
 module UdpClientWHashtbl = Weak.Make(struct
       type t = udp_client
       let hash c = Hashtbl.hash (c.udp_client_ip, c.udp_client_port)
@@ -234,13 +260,6 @@ let _ =
       List2.tail_map (fun s -> as_server s.server_server) !connected_server_list
   )
 
-let lprintf_nl () =
-  lprintf "%s[EDK] "
-  (log_time ()); lprintf_nl2
-
-let lprintf_n () =
-  lprintf "%s[EDK] "
-  (log_time ()); lprintf
 
 let hashtbl_remove table key v =
   try
@@ -249,6 +268,15 @@ let hashtbl_remove table key v =
       Hashtbl.remove table key
   with _ -> ()
 
+let add_connecting_server c =
+  connecting_server_list := c :: !connecting_server_list
+
+let remove_connecting_server c =
+  connecting_server_list := List2.removeq c !connecting_server_list
+
+let connecting_server_ips () =
+  List.rev_map (fun s -> s.server_ip) !connecting_server_list
+
 let add_connected_server c =
   connected_server_list := c :: !connected_server_list
 
@@ -280,12 +308,10 @@ let update_best_name file =
       match impl.impl_file_probable_name with
         None -> ()
       | Some best_name ->
-          let best_name = String2.replace best_name '/' "::" in
-          set_file_best_name file best_name;
-          if !verbose then lprintf_nl () "BEST NAME now IS %s" best_name;
+          set_file_best_name file best_name "" 0
     with Not_found -> ()
 
-let new_file file_diskname file_state md4 file_size filenames writable =
+let new_file file_diskname file_state md4 file_size filename writable =
 
   try
       let file = find_file md4 in
@@ -298,13 +324,13 @@ let new_file file_diskname file_state md4 file_size filenames writable =
             then
               begin
                 if !verbose_share then
-                  lprintf_nl () "New file with changed filename %s to %s"
+                  lprintf_nl "New file with changed filename %s to %s"
                     file.file_diskname file_diskname;
                 file.file_diskname <- file_diskname;
               end
           else
             if !verbose_share then
-              lprintf_nl () "New file with not changed different filename %s and %s"
+              lprintf_nl "New file with not changed different filename %s and %s"
                 file.file_diskname file_diskname;
         end;
       if Unix32.destroyed (file_fd file)
@@ -312,53 +338,63 @@ let new_file file_diskname file_state md4 file_size filenames writable =
           && file.file_diskname = file_diskname
         then
           file.file_file.impl_file_fd <-
-            Unix32.create_diskfile file.file_diskname Unix32.rw_flag 0o666;
+            Some (Unix32.create_diskfile file.file_diskname true);
       if Unix32.destroyed (file_fd file) then
-          lprintf_nl () "New Edonkey file with %b && %b remaining destroyed fd %s"
+          lprintf_nl "New Edonkey file with %b && %b remaining destroyed fd %s"
             (not writable) (file.file_diskname = file_diskname) file.file_diskname;
       file
   with _ ->
       if !verbose_share then
-        lprintf_nl () "New file with md4: %s" (Md4.to_string md4);
-      let file_exists = Unix32.file_exists file_diskname in
+        lprintf_nl "New file with md4: %s" (Md4.to_string md4);
 
       let t =
+(*  emulate_sparsefiles does not work, temporarily disabled
         if
 (* Don't use this for shared files ! *)
           writable &&
 (* Only if the option is set *)
           !!emulate_sparsefiles &&
 (* Only if the file does not already exists *)
-          not (Sys.file_exists file_diskname)
+          not (Unix32.file_exists file_diskname)
         then
-          Unix32.create_sparsefile file_diskname
+          Unix32.create_sparsefile file_diskname writable
         else
-          Unix32.create_diskfile file_diskname Unix32.rw_flag 0o666
+*)
+          Unix32.create_diskfile file_diskname writable
       in
       let file_size =
         if file_size = Int64.zero then
           try
-            Unix32.getsize file_diskname writable
+            Unix32.getsize file_diskname
           with _ ->
               failwith "Zero length file ?"
         else file_size
       in
 
       if file_size <> zero && writable then (* do not truncate if not writable *)
-        Unix32.ftruncate64 t file_size !!create_file_sparse;
+        begin
+    try
+            Unix32.ftruncate64 t file_size !!create_file_sparse
+    with e ->
+      (try
+        Unix32.remove t
+       with e ->
+    lprintf_nl "Unix32.remove %s exception %s"
+      (file_diskname) (Printexc2.to_string e));
+      Unix32.destroy t;
+      failwith (Printf.sprintf "file size %s is too big, exception: %s"
+        (size_of_int64 file_size) (Printexc2.to_string e))
+  end;
 
-      let nchunks = Int64.to_int (Int64.pred file_size // block_size) + 1 in
-      let md4s = if file_size <= block_size then
-          [md4]
-        else [] in
+      let md4s = if file_size < block_size then [md4] else [] in
       let rec file = {
           file_diskname = file_diskname;
           file_file = file_impl;
           file_shared = None;
           file_md4 = md4;
           file_swarmer = None;
-          file_nchunks = nchunks;
-          file_filenames = filenames;
+          file_nchunks = get_nchunks file_size;
+          file_nchunk_hashes = get_nchunk_hashes file_size;
           file_computed_md4s = Array.of_list md4s;
           file_format = FormatNotComputed 0;
           file_sources = DonkeySources.create_file_sources_manager
@@ -370,8 +406,9 @@ let new_file file_diskname file_state md4 file_size filenames writable =
           impl_file_ops = file_ops;
           impl_file_age = last_time ();
           impl_file_size = file_size;
-          impl_file_fd = t;
+          impl_file_fd = Some t;
           impl_file_best_name = Filename.basename file_diskname;
+          impl_file_filenames = (if filename = "" then [] else [filename]);
           impl_file_last_seen = last_time () - 100 * 24 * 3600;
         }
       in
@@ -381,27 +418,15 @@ let new_file file_diskname file_state md4 file_size filenames writable =
       (match file_state with
           FileShared -> ()
         | _ ->
-            let kernel = Int64Swarmer.create_swarmer file_diskname file_size zone_size in
-            let swarmer = Int64Swarmer.create kernel (as_file file) block_size
+            let kernel = CommonSwarming.create_swarmer file_diskname file_size in
+            let swarmer = CommonSwarming.create kernel (as_file file) block_size
             in
             file.file_swarmer <- Some swarmer;
-(*
-        Int64Swarmer.set_writer swarmer (fun offset s pos len ->
-(*
-      lprintf "DOWNLOADED: %d/%d/%d\n" pos len (String.length s);
-      AnyEndian.dump_sub s pos len;
-*)
-
-            if !!CommonOptions.buffer_writes then
-              Unix32.buffered_write_copy t offset s pos len
-            else
-              Unix32.write  t offset s pos len
-        ); *)
-            Int64Swarmer.set_verifier swarmer
+            CommonSwarming.set_verifier swarmer
               (if md4s = [] then VerificationNotAvailable else
                 Verification (Array.of_list (List.map (fun md4 -> Ed2k md4) md4s))
             );
-            Int64Swarmer.set_verified swarmer (fun nblocks num ->
+            CommonSwarming.set_verified swarmer (fun nblocks num ->
                 if nblocks = 1 then begin
                     new_shared_files := file :: !new_shared_files;
                     file_must_update file
@@ -443,14 +468,14 @@ let is_black_address ip port =
 (* lprintf "is black ="; *)
     not (Ip.reachable ip) || (Ip.matches ip !!server_black_list) ||
     (List.mem port !!port_black_list) ||
-    (match Ip_set.match_ip !Ip_set.bl ip with
+    (match !Ip.banned ip with
         None -> false
-      | Some br ->
+      | Some reason ->
           if !verbose_connect then
-            lprintf_nl () "%s:%d blocked: %s" (Ip.to_string ip) port br.Ip_set.blocking_description;
+            lprintf_nl "%s:%d blocked: %s" (Ip.to_string ip) port reason;
           true))
 
-let new_server ip port score =
+let new_server ip port =
   let key = (ip) in
   try
     let found = Hashtbl.find servers_by_key key in
@@ -471,11 +496,10 @@ let new_server ip port score =
         server_search_queries = Fifo.create ();
         server_users_queries = Fifo.create ();
         server_connection_control = new_connection_control ();
-        server_score = score;
+        server_score = 5;
         server_tags = [];
-        server_nfiles = Int64.zero;
-        server_nusers = Int64.zero;
-        server_max_users = 0;
+        server_nfiles = None;
+        server_nusers = None;
         server_name = "";
         server_description = "";
         server_banner = "";
@@ -489,6 +513,14 @@ let new_server ip port score =
         server_id_requests = Fifo.create ();
         server_flags = 0;
         server_has_zlib = false;
+        server_version = "";
+        server_lowid_users = None;
+        server_soft_limit = None;
+        server_hard_limit = None;
+        server_max_users = None;
+        server_last_ping = 0.;
+        server_ping = 0;
+
       }
       and server_impl =
         {
@@ -517,8 +549,7 @@ let remove_server ip port =
         NoConnection -> ()
       | ConnectionWaiting token -> cancel_token token
       | Connection sock ->
-          TcpBufferedSocket.shutdown sock Closed_by_user);
-    server_remove (as_server s.server_server)
+          TcpBufferedSocket.shutdown sock Closed_by_user)
   with _ -> ()
 
 let dummy_client =
@@ -528,21 +559,14 @@ let dummy_client =
       client_upload = None;
       client_kind = Direct_address (Ip.null, 0);
       client_source = DonkeySources.dummy_source;
-(*      client_sock = NoConnection; *)
       client_ip = Ip.null;
       client_md4 = Md4.null;
-(*      client_last_filereqs = 0; *)
-(*      client_chunks = [||]; *)
       client_download = None;
-(*      client_block = None; *)
-(*      client_zones = []; *)
-(*      client_connection_control =  new_connection_control_recent_ok ( ()); *)
       client_file_queue = [];
       client_tags = [];
       client_name = "";
+      client_last_asked_file = dummy_file;
       client_all_files = None;
-      client_next_view_files = last_time () - 1;
-(*      client_all_chunks = ""; *)
       client_rating = 0;
       client_brand = Brand_unknown;
       client_brand_mod = Brand_mod_unknown;
@@ -582,27 +606,18 @@ let create_client key =
   let module D = DonkeyProtoClient in
   let s = DonkeySources.find_source_by_uid key in
   let rec c = {
-(*      dummy_client with *)
-
       client_client = client_impl;
-(*      client_connection_control =  new_connection_control_recent_ok ( ()); *)
-      client_next_view_files = last_time () - 1;
       client_kind = key;
       client_upload = None;
       client_source = s;
-(*      client_sock = NoConnection; *)
       client_ip = Ip.null;
       client_md4 = Md4.null;
-(*      client_last_filereqs = 0; *)
-(*      client_chunks = [||]; *)
       client_download = None;
-(*      client_block = None; *)
-(*      client_zones = []; *)
       client_file_queue = [];
       client_tags = [];
       client_name = "";
+      client_last_asked_file = dummy_file;
       client_all_files = None;
-(*      client_all_chunks = ""; *)
       client_rating = 0;
       client_brand = Brand_unknown;
       client_brand_mod = Brand_mod_unknown;
@@ -628,8 +643,7 @@ let create_client key =
       client_sent_challenge = Int64.zero;
       client_public_key = None;
       client_sui_verified = None;
-      } and
-    client_impl = {
+      } and    client_impl = {
       dummy_client_impl with
       impl_client_val = c;
       impl_client_ops = client_ops;
@@ -666,9 +680,11 @@ let string_of_client c =
   Printf.sprintf "client[%d] %s(%s) %s" (client_num c)
   c.client_name (brand_to_string c.client_brand)
   (match c.client_kind with
-      Indirect_address _ | Invalid_address _ -> ""
+      Indirect_address (server_ip, server_port, ip, port, real_ip) ->
+        Printf.sprintf  " I[%s:%d]" (Ip.to_string real_ip) port;
     | Direct_address (ip,port) ->
-        Printf.sprintf  " [%s:%d]" (Ip.to_string ip) port;
+        Printf.sprintf  " D[%s:%d]" (Ip.to_string ip) port;
+    | Invalid_address _ -> ""
   )
 
 
@@ -776,15 +792,13 @@ end;
   Printf.bprintf buf "  overnet_connect_tags: %d\n" (List.length !overnet_connect_tags);
   Printf.bprintf buf "  clients_root: %d\n" (List.length !clients_root);
   Printf.bprintf buf "  servers_list: %d\n" (List.length !servers_list);
-  Printf.bprintf buf "  current_files: %d\n" (List.length !current_files);
   Printf.bprintf buf "  xs_servers_list: %d\n" (List.length !xs_servers_list);
   Printf.bprintf buf "  connected_server_list: %d\n" (List.length !connected_server_list);
+  Printf.bprintf buf "  connecting_server_list: %d\n" (List.length !connecting_server_list);
   Printf.bprintf buf "  udp_servers_list: %d\n" (List.length !udp_servers_list);
   Printf.bprintf buf "  interesting_clients: %d\n" (List.length !interesting_clients);
   Printf.bprintf buf "  shared_files: %d\n" (List.length !shared_files);
-  Printf.bprintf buf "  new_hsared_files: %d\n" (List.length !new_shared_files);
-  Printf.bprintf buf "  file_md4s_to_register: %d\n" (List.length !file_md4s_to_register);
-
+  Printf.bprintf buf "  new_shared_files: %d\n" (List.length !new_shared_files);
   Printf.bprintf buf "  servers_by_key: %d\n" (Hashtbl.length servers_by_key);
   Printf.bprintf buf "  banned_ips: %d\n" (Hashtbl.length banned_ips);
   Printf.bprintf buf "  old_requests: %d\n" (Hashtbl.length old_requests);
@@ -869,6 +883,8 @@ let left_bytes = "MLDK"
 let overnet_server_ip = ref Ip.null
 let overnet_server_port = ref 0
 
+let overnet_port_info = ref 0
+let kademlia_port_info = ref 0
 
 
 (*************************************************************
@@ -893,7 +909,8 @@ let _ =
 let client_id c =
   match c.client_kind with
     Direct_address (ip, port) -> (ip, port, zero)
-  | Indirect_address (ip, port, id) -> (ip, port, id)
+  | Indirect_address (server_ip, server_port, id, port, real_ip) ->
+     (server_ip, server_port, id)
   | Invalid_address _ -> (Ip.null, 0, zero)
 
 let save_join_queue c =
@@ -903,7 +920,7 @@ let save_join_queue c =
       ) c.client_file_queue in
     begin
       if c.client_debug then
-        lprintf_nl () "Saving %d files associated with %s"
+        lprintf_nl "Saving %d files associated with %s"
         (List.length files) (Md4.to_string c.client_md4);
       Hashtbl.add join_queue_by_md4 c.client_md4 (files, last_time ());
       try
@@ -931,36 +948,36 @@ let clean_join_queue_tables () =
   ) list
 
 let client_public_key = ref ""
+let key_check_started = ref false
 
 let _ =
   option_hook client_private_key (fun _ ->
-    client_public_key := Unix32.load_key (!!client_private_key);
-
-    (
-    let key_checked = ref false in
-    let key_check_again = ref false in
-    let rec check_client_private_key () =
-      key_checked := true;
-      if not (String.sub !!client_private_key 0 5 = "MIIBC") then
-        if !key_check_again then
-	  begin
-	    lprintf_nl () "can not create proper client_private_key, exiting...";
-	    exit 70
-	  end
-	else
-          begin
-            lprintf_nl () "bad client_private_key detected, creating new key";
-	    set_simple_option donkey_ini "client_private_key" (Unix32.create_key ());
-	    key_check_again := true
-	  end
-    in
-    if not !key_checked then check_client_private_key ();
-    if !key_check_again then
+    if Autoconf.donkey_sui_works () then
       begin
-	lprintf_nl () "re-checking private key";
-	check_client_private_key ()
+	if not (try String.sub !!client_private_key 0 4 = "MIIB" with e -> false) then
+	  if !key_check_started then
+	    begin
+	      let s1 =
+		Printf.sprintf "can not create valid client_private_key, bad value found: %s"
+		  !!client_private_key in
+	      let s2 = "CryptoPP code seems not to work properly, do not use insane CFLAGS, exiting..." in
+	      Printf.eprintf "%s[EDK] %s\n%!" (log_time ()) s1;
+	      Printf.eprintf "%s[EDK] %s\n%!" (log_time ()) s2;
+	      lprintf_nl "%s" s1;
+	      lprintf_nl "%s" s2;
+	      exit 70
+	    end
+	  else
+	    begin
+	      key_check_started := true;
+	      lprintf_nl "found bad client_private_key: %s, creating new key..." !!client_private_key;
+	      client_private_key =:= (DonkeySui.SUI.create_key ());
+	    end
+	else
+	  begin
+	    client_public_key := DonkeySui.SUI.load_key (!!client_private_key);
+	  end
       end
-    );
   )
 
 let server_accept_multiple_getsources s =
@@ -972,3 +989,9 @@ let server_send_multiple_replies s =
 let server_send_getsources2 s =
   (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.getsources2) <> 0
 
+let full_client_identifier c =
+    Printf.sprintf "%s (%s%s) '%s'"
+      (Ip.to_string c.client_ip)
+      (brand_to_string_short c.client_brand)
+      (if c.client_emule_proto.emule_release = "" then "" else " " ^ c.client_emule_proto.emule_release)
+      (String.escaped c.client_name)
