@@ -28,7 +28,7 @@ open DonkeyTypes
 open DonkeyMftp
 
 let dump_file filename =
-  let ic = open_in filename in
+  Unix2.tryopen_read filename (fun ic ->
   let s = String.create 20 in
   try
     lprintf "file: %s\n" filename; 
@@ -40,8 +40,7 @@ let dump_file filename =
       dump (String.sub s 0 n);
       pos := !pos + n;
     done
-  with _ ->
-      close_in ic
+  with End_of_file | Exit -> ())
       
 module Server = struct
     
