@@ -45,9 +45,10 @@ open DonkeyClient
 open DonkeyThieves
 open DonkeyOptions
 
-let lprintf_nl () =
-  lprintf "%s[EDK] "
-    (log_time ()); lprintf_nl2
+let log_prefix = "[EDK]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
 
 let _ =
   network.op_network_is_enabled <- (fun _ -> !!enable_donkey);
@@ -62,15 +63,13 @@ let hourly_timer timer =
   DonkeyClient.clean_groups ();
   DonkeyClient.clean_requests ();
   Hashtbl.clear udp_servers_replies;
-  DonkeyThieves.clean_thieves ();
-  DonkeyNeighbours.recover_downloads !current_files
+  DonkeyThieves.clean_thieves ()
     
 let quarter_timer timer =
   clean_join_queue_tables ()
 
 let fivemin_timer timer =
   DonkeyShare.send_new_shared ();
-  DonkeyChunks.duplicate_chunks ();
   clients_root := []
 
 let second_timer timer =
@@ -78,13 +77,26 @@ let second_timer timer =
       DonkeySources.connect_sources connection_manager;
     with e ->
         if !verbose_sources > 2 then 
-          lprintf_nl () "Exception %s while checking sources" 
+          lprintf_nl "Exception %s while checking sources" 
             (Printexc2.to_string e)
-  )
+  );
+  DonkeyServers.udp_walker_timer ()
 
-let halfmin_timer timer =
+let five_second_timer timer =
+  DonkeyServers.check_server_connections ();
+  DonkeyServers.walker_timer ();
+  DonkeyOneFile.check_files_downloaded ();
+  DonkeyShare.check_shared_files ()
+
+let min_timer timer =
   DonkeySources.clean_sources (); (* Moved here from fivemin_timer. *)
-  DonkeyServers.update_master_servers ()
+  DonkeyServers.update_master_servers ();
+  DonkeyServers.check_for_preferred_servers ();
+ (try
+      DonkeyServers.query_locations_timer ();
+    with _ -> ());
+  (List.iter (fun file -> DonkeyShare.must_share_file file) !new_shared_files;
+  new_shared_files := [])
 (*  DonkeyIndexer.add_to_local_index_timer () *)
 
 
@@ -98,7 +110,7 @@ let disable enabler () =
       is_enabled := false;
       enabler := false;
       if !!enable_donkey then enable_donkey =:= false;
-      Hashtbl2.safe_iter (fun s -> disconnect_server s Closed_by_user)
+      Hashtbl2.safe_iter (fun s -> DonkeyServers.disconnect_server s Closed_by_user)
       servers_by_key;
       H.iter (fun c -> DonkeyClient.disconnect_client c Closed_by_user) 
       clients_by_kind;
@@ -152,8 +164,7 @@ let reset_tags () =
   ];
 
   emule_info.DonkeyProtoClient.EmuleClientInfo.tags <- [
-    int_tag (Field_UNKNOWN "compression") 
-      (if !!emule_compression then m.emule_compression else 0);
+    int_tag (Field_UNKNOWN "compression") m.emule_compression;
     int_tag (Field_UNKNOWN "udpver") m.emule_udpver;
     int_tag (Field_UNKNOWN "udpport") (!!donkey_port+4);
     int_tag (Field_UNKNOWN "sourceexchange") m.emule_sourceexchange;
@@ -179,7 +190,10 @@ let enable () =
     let enabler = ref true in
     is_enabled := true;
     network.op_network_disable <- disable enabler;
-    client_public_key := Unix32.load_key (!!client_private_key);
+    if Autoconf.donkey_sui_works () then
+    (try
+      client_public_key := DonkeySui.SUI.load_key (!!client_private_key)
+    with _ -> ());
     if not !!enable_donkey then enable_donkey =:= true;
     
     try
@@ -212,9 +226,9 @@ let enable () =
                 try
                   let file_disk_name = file_disk_name file in
                   if Unix32.file_exists file_disk_name &&
-                    Unix32.getsize file_disk_name false <> Int64.zero then begin
+                    Unix32.getsize file_disk_name <> Int64.zero then begin
                     (* getsize writable=false is ok because file has state FileDownloaded *)
-                      lprintf_nl () "FILE DOWNLOADED"; 
+                      lprintf_nl "FILE DOWNLOADED"; 
                       
                       DonkeyOneFile.declare_completed_file file;
                     end
@@ -223,11 +237,10 @@ let enable () =
                     file_commit (as_file file)
               end
           with e ->
-              lprintf_nl () "Exception %s while recovering download %s"
+              lprintf_nl "Exception %s while recovering download %s"
                 (Printexc2.to_string e) (file_disk_name file); 
       ) files_by_md4;
       
-      let list = ref [] in
 (* Normally, we should check that downloaded files are still there.  *)    
       let list = ref [] in
       List.iter (fun file ->
@@ -266,7 +279,7 @@ be useful when users want to share files that they had already previously
               udp_sock := Some sock;
               UdpSocket.set_write_controler sock udp_write_controler;
             with e ->
-                lprintf_nl () "Exception %s while binding UDP socket"
+                lprintf_nl "Exception %s while binding UDP socket"
                   (Printexc2.to_string e);
           end;
           sock
@@ -285,14 +298,14 @@ be useful when users want to share files that they had already previously
         | _ -> failwith "Bad socket address"
       end;
       
-      let port = !!donkey_port in
-      
       reset_tags ();
       
       Options.option_hook global_login reset_tags;
       Options.option_hook login reset_tags;
-      Options.option_hook enable_sui reset_tags;
-
+      Options.option_hook enable_sui ( fun _ ->
+	if not (Autoconf.donkey_sui_works ()) && !!enable_sui then enable_sui =:= false;
+	reset_tags ());
+				
 (**** START TIMERS ****)
       add_session_option_timer enabler check_client_connections_delay 
         (fun _ ->
@@ -300,32 +313,12 @@ be useful when users want to share files that they had already previously
           DonkeyServers.udp_query_sources ()
          );
       
-      add_session_option_timer enabler buffer_writes_delay 
-        (fun _ -> Unix32.flush ());
-      
-      add_session_option_timer enabler check_connections_delay 
-        DonkeyServers.check_server_connections;
-      add_session_option_timer enabler compute_md4_delay 
-        (fun _ ->
-          DonkeyOneFile.check_files_downloaded ();
-          DonkeyShare.check_shared_files ()
-      );
-      add_session_timer enabler 5.0 DonkeyServers.walker_timer;
-      add_session_timer enabler 1.0 DonkeyServers.udp_walker_timer;
-      
       add_session_timer enabler 3600. hourly_timer;
-      add_session_timer enabler 30. halfmin_timer;
+      add_session_timer enabler 60. min_timer;
       add_session_timer enabler 300. fivemin_timer;
       add_session_timer enabler 900. quarter_timer;
       add_session_timer enabler 1. second_timer;
-      add_session_timer enabler 60. (fun _ ->
-          (try
-              DonkeyServers.query_locations_timer ();
-            with _ -> ());
-          List.iter (fun file -> DonkeyShare.must_share_file file) 
-          !new_shared_files;  
-          new_shared_files := [];
-      );
+      add_session_timer enabler 5. five_second_timer;
       add_session_option_timer enabler remove_old_servers_delay 
           DonkeyServers.remove_old_servers;
 
@@ -339,13 +332,13 @@ be useful when users want to share files that they had already previously
 *)
 
   with e ->
-      lprintf_nl () "Error: Exception %s during startup"
+      lprintf_nl "Error: Exception %s during startup"
         (Printexc2.to_string e)
 
 
 let rec update_options () =
   let update v =
-      lprintf_nl () "Updating options to version %i" v;
+      lprintf_nl "Updating options to version %i" v;
       options_version =:= v;
       update_options ()
   in
@@ -355,6 +348,14 @@ let rec update_options () =
       if !!max_sources_per_file = 20000 then
         max_sources_per_file =:= 5000;
       update 1
+  | 1 ->
+      if !!upload_timeout = 1800. then
+        upload_timeout =:= 60.;
+      update 2
+  | 2 ->
+      if !!upload_timeout = 60. then
+        upload_timeout =:= 600.;
+      update 3
   | _ -> ()
 
 
@@ -384,6 +385,13 @@ let _ =
         network_downloaded = !donkey_download_counter;
         network_connected = List.length (connected_servers ());
       });
+  network.op_network_ports <- (fun _ ->
+    [
+    !!donkey_port, "client_port TCP";
+    !!donkey_port+4, "client_port UDP";
+    !overnet_port_info, "overnet_port TCP+UDP";
+    !kademlia_port_info, "kademlia_port UDP";
+    ]);
   CommonInteractive.register_gui_options_panel "eDonkey" 
     gui_donkey_options_panel;
   CommonInteractive.register_gui_options_panel "Overnet" 
