@@ -275,6 +275,20 @@ type search_for =
 | KeywordSearch of CommonTypes.search
 | FillBuckets
 
+(* using "=" to structurally compare files (-> search_fors) 
+   will result in an OutOfMemory exception *)
+let search_for_equals sf1 sf2 =
+  match sf1 with 
+    | FileSearch f1 -> 
+      (match sf2 with 
+        | FileSearch f2 when f1 == f2 -> true
+        | _ -> false)
+    | KeywordSearch k1 ->
+      (match sf2 with
+        | KeywordSearch k2 when k1 == k2 -> true
+        | _ -> false)
+    | _ -> true
+
 type overnet_search = {
     search_md4 : Md4.t;
     mutable search_kind : search_for;
@@ -301,6 +315,8 @@ module Make(Proto: sig
       val overnet_tcpport : int Options.option_record
       val overnet_section : Options.options_section
 
+      val checking_kind_timeout : int
+
       val redirector_section : string
       val options_section_name : string
       val command_prefix : string
@@ -311,18 +327,18 @@ module Make(Proto: sig
         UdpSocket.t -> UdpSocket.event -> unit
 
       val web_info : string
+      val web_info_descr : string
     end) = struct
 
     open Proto
 
-    let lprintf_nl () =
-      lprintf "%s[%s] "
-      (log_time ())
-      (if Proto.redirector_section = "DKKO" then "Overnet" else "Kademlia"); lprintf_nl2
+    let log_prefix = (if Proto.redirector_section = "DKKO" then "[Overnet]" else "[Kademlia]")
 
-    let lprintf_n () =
-      lprintf "[%s] "
-      (if Proto.redirector_section = "DKKO" then "Overnet" else "Kademlia"); lprintf
+    let lprintf_nl fmt =
+      lprintf_nl2 log_prefix fmt
+
+    let lprintf_n fmt =
+      lprintf2 log_prefix fmt
 
 (********************************************************************
 
@@ -333,7 +349,7 @@ module Make(Proto: sig
 let max_peers_per_bucket = 20
 let max_peers_per_prebucket = 40
 (* how many peers a search may ask for more search peers *)
-let max_search_queries = 64 
+let max_search_queries = 50 
 (* how many peers we ask for results *)
 let max_search_requests = 20 
 let max_boot_peers = 200
@@ -428,7 +444,7 @@ let dummy_peer =
     peer_kind = 0;
     peer_expire = 0;
     peer_last_send = 0;
-    peer_created = last_time ();
+    peer_created = 0;
   }
 
 let connected_peers = ref 0
@@ -716,7 +732,7 @@ let checking_kind p =
   if (last_time () - p.peer_last_send) < 10 ||
      p.peer_kind = 4 then () else begin
     p.peer_kind <- p.peer_kind + 1;
-    p.peer_expire <- last_time () + 120;
+    p.peer_expire <- last_time () + Proto.checking_kind_timeout;
   end;
   ()
   
@@ -761,14 +777,15 @@ let new_peer p =
   let ip = p.peer_ip in
   let port = p.peer_port in
  
-  if ip <> Ip.localhost && is_overnet_ip ip && port <> 0 then
-
     try
       let pp = KnownPeers.find known_peers p in
       (* TODO: check for changes in ip:port? Or Hash? *)
       pp
     with _ ->
-    
+
+   if ip <> Ip.localhost && is_overnet_ip ip 
+     && port <> 0 && p.peer_created <> 0 then
+   
         let bucket = bucket_number p.peer_md4 in
 
 (* Add the peer to the table of known peers *)
@@ -787,7 +804,6 @@ restart. *)
 
             if Fifo.length prebuckets.(bucket) = max_peers_per_prebucket then
 	    begin
-              let l = last_time () - 1800 in
               let pp = Fifo.take prebuckets.(bucket) in
               Fifo.put prebuckets.(bucket)
 (* If the head of the bucket is not dead we should keep it *)
@@ -837,7 +853,7 @@ let get_closest_peers md4 nb =
         if p.peer_kind < 4 && p.peer_expire > last_time () && 
            p.peer_last_send <> 0 then begin
             if !verbose_overnet then begin
-            lprintf_nl () "Adding good search peer %s:%d"
+            lprintf_nl "Adding good search peer %s:%d"
               (Ip.to_string p.peer_ip) p.peer_port;
 	    end;
             decr nb;
@@ -910,7 +926,7 @@ let get_any_peers  nb =
 
 let add_search_peer s p =
   if p.peer_ip <> Ip.localhost && is_overnet_ip p.peer_ip && 
-     p.peer_port <> 0 then begin
+     p.peer_port <> 0 && p.peer_created <> 0 then begin
     let nbits = common_bits p.peer_md4 s.search_md4 in
     begin
       try
@@ -925,8 +941,9 @@ let add_search_peer s p =
   end
 
 let create_search kind md4 =
-  if !verbose_overnet then lprintf_nl () "create_search";
-  let s = {
+  if !verbose_overnet then lprintf_nl "create_search";
+  let starttime = last_time () + (2 * List.length !overnet_searches) in
+  let s = ref {
       search_md4 = md4;
       search_kind = kind;
       search_queries = 0;
@@ -934,17 +951,28 @@ let create_search kind md4 =
       search_waiting_peers = Array.init 129 (fun _ -> Fifo.create ());
       search_asked_peers = Array.init 129 (fun _ -> Fifo.create ());
       search_ok_peers = Array.init 129 (fun _ -> Fifo.create ());
-      search_start = last_time ();
-      search_last_query = last_time () + 
-        ((List.length !overnet_searches) mod (int_of_float !!overnet_query_peer_period));
+      search_start = (match kind with
+                       KeywordSearch s -> last_time ()
+                     | FillBuckets -> last_time ()
+                     | FileSearch s -> starttime);
+      search_last_query = (match kind with
+                       KeywordSearch s -> last_time ()
+                     | FillBuckets -> last_time ()
+                     | FileSearch s -> starttime);
       search_hits = 0;
       search_nresults = 0;
-      search_results = Hashtbl.create 13;
+      search_results = Hashtbl.create 64;
     } in
-  List.iter (add_search_peer s) (get_closest_peers md4 max_search_queries);
-  if !verbose_overnet then lprintf_nl () "create_search done";
-  overnet_searches := s :: !overnet_searches;
-  s
+  List.iter (fun ss ->
+    if ss.search_md4 = !s.search_md4 && (search_for_equals ss.search_kind !s.search_kind) then begin
+     ss.search_start <- !s.search_start;
+     s := ss;
+    end
+  ) !overnet_searches;
+  List.iter (add_search_peer !s) (get_closest_peers md4 max_search_queries);
+  if !verbose_overnet then lprintf_nl "create_search done";
+  overnet_searches := !s :: !overnet_searches;
+  !s
 
 let create_keyword_search w s =
   let md4 = Md4.string w in
@@ -966,7 +994,7 @@ let udp_client_handler t p =
   let other_ip = ip_of_udp_packet p in
   let other_port = port_of_udp_packet p in
   if !verbose_overnet then
-    lprintf_nl () "UDP FROM %s:%d type %s"
+    lprintf_nl "UDP FROM %s:%d type %s"
       (Ip.to_string other_ip) other_port
       (message_to_string t);
   (* Emule uses other_ip:other_port, so do we *)
@@ -977,13 +1005,14 @@ let udp_client_handler t p =
 
   | OvernetConnect p ->
       if is_overnet_ip sender.peer_ip && sender.peer_port <> 0 then
-        let sender = new_peer { p with peer_port = other_port ; peer_ip = other_ip } in
+        let sender = new_peer { p with peer_ip = other_ip } in
+        (* let sender = new_peer { p with peer_port = other_port ; peer_ip = other_ip } in *)
         new_peer_message sender;
         udp_send sender (OvernetConnectReply (get_any_peers 20))
        else
 	 begin
 	   if !verbose_overnet then
-	     lprintf_nl () "Connect: invalid IP %s:%d received from %s:%d"
+       lprintf_nl "Connect: invalid IP %s:%d received from %s:%d"
 	          (Ip.to_string p.peer_ip) p.peer_port (Ip.to_string other_ip) other_port;
 	     failwith "Message not understood"
 	 end
@@ -994,22 +1023,22 @@ let udp_client_handler t p =
         match list with
           [] -> ()
         | [p] ->
-            let sender = new_peer { p with peer_port = other_port ; peer_ip = other_ip } in
+            let sender = new_peer { p with peer_ip = other_ip } in
             new_peer_message sender
         | p :: tail ->
-            let p = new_peer p in
+            let _ = new_peer p in
             iter tail
       in
       iter ps;
 
   | OvernetPublicize p ->
-      let sender = new_peer { p with peer_port = other_port ; peer_ip = other_ip } in
+      let sender = new_peer { p with peer_ip = other_ip } in
       new_peer_message sender;
       if is_overnet_ip sender.peer_ip && sender.peer_port <> 0 then
         udp_send sender (OvernetPublicized (Some (my_peer ())))
        else begin
 	      if !verbose_overnet then
-	        lprintf_nl () "Publicize: invalid IP %s:%d received from %s:%d"
+          lprintf_nl "Publicize: invalid IP %s:%d received from %s:%d"
 	          (Ip.to_string p.peer_ip) p.peer_port (Ip.to_string other_ip) other_port;
 	      failwith "Message not understood"
 	    end
@@ -1038,8 +1067,8 @@ let udp_client_handler t p =
   | OvernetUnknown (opcode, s) ->
       if !verbose_unknown_messages then
         begin
-          lprintf_nl () "Unknown message from %s:%d " (Ip.to_string other_ip) other_port;
-          lprintf_nl () "\tCode: %d" opcode; dump s;
+          lprintf_nl "Unknown message from %s:%d " (Ip.to_string other_ip) other_port;
+          lprintf_nl "\tCode: %d" opcode; dump s;
         end
 
   | OvernetSearchFilesResults (md4, results) ->
@@ -1058,9 +1087,9 @@ let udp_client_handler t p =
                           s.search_hits <- s.search_hits + 1;
                           Hashtbl.add s.search_results r_md4 r_tags;
                           if !verbose_overnet then begin
-                              lprintf_nl () "FILE FOUND, TAGS:";
+                              lprintf_nl "FILE FOUND, TAGS:";
                               print_tags r_tags;
-                              lprintf_nl () ""
+                              lprintf_nl ""
                         end;
 
                           DonkeyOneFile.search_found true sss r_md4 r_tags;
@@ -1129,32 +1158,32 @@ let udp_client_handler t p =
 
   | OvernetFirewallConnectionACK(md4) ->   
        if !verbose_overnet && debug_client other_ip then   
-         lprintf_nl () "FIREWALL ACK for md4=%s" (Md4.to_string md4)   
+         lprintf_nl "FIREWALL ACK for md4=%s" (Md4.to_string md4)   
     
    | OvernetFirewallConnectionNACK(md4) ->   
        if !verbose_overnet && debug_client other_ip then   
-         lprintf_nl () "FIREWALL NACK for md4=%s" (Md4.to_string md4)   
+         lprintf_nl "FIREWALL NACK for md4=%s" (Md4.to_string md4)   
     
  (* send the answer *)   
    | OvernetGetMyIP other_port ->   
        if !verbose_overnet && debug_client other_ip then   
-         lprintf_nl () "GET MY IP (port=%d)\n" other_port;   
+         lprintf_nl "GET MY IP (port=%d)\n" other_port;   
  (* FIXME : should be able to flush the UDP buffer*)   
        udp_send sender (OvernetGetMyIPResult other_ip);   
        udp_send sender OvernetGetMyIPDone   
     
    | OvernetGetMyIPResult(ip) ->   
        if !verbose_overnet && debug_client other_ip then   
-         lprintf_nl () "GET MY IP RESULT (%s)\n" (Ip.to_string ip)   
+         lprintf_nl "GET MY IP RESULT (%s)\n" (Ip.to_string ip)   
     
    | OvernetGetMyIPDone ->   
        if !verbose_overnet && debug_client other_ip then   
-         lprintf_nl () "GET MY IP DONE\n"   
+         lprintf_nl "GET MY IP DONE\n"   
   
   | OvernetPeerNotFound peer ->
       begin
         if !verbose_overnet || debug_client other_ip then
-          lprintf_nl () "Peer NOT FOUND %s (%s:%d) kind: %d (msg 33)"
+          lprintf_nl "Peer NOT FOUND %s (%s:%d) kind: %d (msg 33)"
             (Md4.to_string peer.peer_md4) (Ip.to_string peer.peer_ip)
         peer.peer_port peer.peer_kind;
         let dp = { dummy_peer with peer_port = peer.peer_port ; peer_ip = peer.peer_ip } in
@@ -1169,6 +1198,7 @@ let udp_client_handler t p =
                 if p.peer_ip = peer.peer_ip && 
                     p.peer_port = peer.peer_port then begin
                   decr pre_connected_peers;
+                  KnownPeers.remove known_peers dp;
                 end else Fifo.put b p
               done;
             done;
@@ -1178,8 +1208,8 @@ let udp_client_handler t p =
 
    | OvernetUnknown21 peer ->
       if !verbose_overnet && debug_client other_ip then begin
-          lprintf_nl () "Unknown 21 message ...";
-          lprintf_nl () "From peer: %s ip: %s:%d kind: %d"
+          lprintf_nl "Unknown 21 message ...";
+          lprintf_nl "From peer: %s ip: %s:%d kind: %d"
             (Md4.to_string peer.peer_md4) (Ip.to_string peer.peer_ip) peer.peer_port peer.peer_kind
         end
 
@@ -1220,7 +1250,11 @@ let query_next_peers () =
       if s.search_last_query < timeout then begin
         s.search_last_query <- s.search_last_query + overnet_query_peer_period;
 (* Query next search peers *)
-        iter 128 2;
+        if s.search_queries < max_search_queries then
+          (if s.search_queries = 0 then
+            iter 128 3
+          else
+            iter 128 2);
 (* Request next results *)
         if s.search_requests < max_search_requests then begin
            let nrequests =
@@ -1256,8 +1290,7 @@ let query_next_peers () =
 
 let recover_file file =
   if file_state file = FileDownloading then
-    let search = create_search (FileSearch file) file.file_md4 in
-    ()
+    ignore (create_search (FileSearch file) file.file_md4)
 
 let check_current_downloads () =
   List.iter recover_file !DonkeyGlobals.current_files
@@ -1277,7 +1310,8 @@ let update_buckets () =
       else
       begin
         decr connected_peers;
-        if !verbose_overnet then lprintf_nl () "update_bucket1: removing %s:%d" (Ip.to_string p.peer_ip) p.peer_port;
+        KnownPeers.remove known_peers p;
+        if !verbose_overnet then lprintf_nl "update_bucket1: removing %s:%d" (Ip.to_string p.peer_ip) p.peer_port;
       end;
     done
 
@@ -1303,7 +1337,8 @@ let update_buckets () =
             (* bad peers are removed *)    
             end else if p.peer_kind = 4 && p.peer_expire <= last_time () then begin
               decr pre_connected_peers;
-              if !verbose_overnet then lprintf_nl () "update_bucket2: removing %s:%d" (Ip.to_string p.peer_ip) p.peer_port;
+              KnownPeers.remove known_peers p;
+              if !verbose_overnet then lprintf_nl "update_bucket2: removing %s:%d" (Ip.to_string p.peer_ip) p.peer_port;
             end else
             (* the rest returns in prebuckets *)
             Fifo.put pb p
@@ -1375,6 +1410,10 @@ let enable () =
       let sock = (UdpSocket.create (Ip.to_inet_addr !!client_bind_addr)
           (!!overnet_port) (Proto.udp_handler udp_client_handler)) in
       udp_sock := Some sock;
+      if Proto.redirector_section = "DKKO" then
+	overnet_port_info := !!overnet_port;
+      if Proto.redirector_section = "DKKA" then
+	kademlia_port_info := !!overnet_port;
       UdpSocket.set_write_controler sock udp_write_controler;
 
 (* copy all boot_peers to unknown_peers *)
@@ -1437,8 +1476,8 @@ let enable () =
 
           compute_to_ping ();
 
-          let l = last_time () - 300 in
-(* remove searches that are older than 5 minutes *)
+          let l = last_time () - 180 in
+(* remove searches that are older than 3 minutes *)
           overnet_searches := List.filter (fun s ->
               s.search_requests < max_search_requests &&
               s.search_start > l
@@ -1448,13 +1487,13 @@ let enable () =
 (* every 15min for light operations *)
       add_session_timer enabler 900. (fun _ ->
           if !!enable_overnet then begin
-              let s = create_search FillBuckets !!overnet_md4 in
+              let _ = create_search FillBuckets !!overnet_md4 in
               check_current_downloads ();
             end
       );
 
       begin
-        let s = create_search FillBuckets !!overnet_md4 in
+        let _ = create_search FillBuckets !!overnet_md4 in
         check_current_downloads ();
       end;
       
@@ -1476,6 +1515,10 @@ let disable () =
         | Some sock ->
             udp_sock := None;
             UdpSocket.close sock Closed_by_user);
+      if Proto.redirector_section = "DKKO" then
+	overnet_port_info := 0;
+      if Proto.redirector_section = "DKKA" then
+	kademlia_port_info := 0;
     end
 
 let _ =
@@ -1501,7 +1544,7 @@ let load_contact_dat filename =
     ) ss;
     List.length ss
   with e ->
-      lprintf_nl () "Exception %s while loading %s" (Printexc2.to_string e)
+      lprintf_nl "Exception %s while loading %s" (Printexc2.to_string e)
       filename;
       0
 
@@ -1550,7 +1593,7 @@ let _ =
            Fifo.put pb p;
          done;
          ""
-    ), ("<bucket_nr> :\t\t\tdumps a bucket");
+    ), ("<bucket_nr> :\t\tdumps a bucket");
 
     "dump_known_peers", Arg_none (fun o ->
          let buf = o.conn_buf in
@@ -1571,7 +1614,6 @@ let _ =
     ), ("<ip> <port> :\t\t\tadd an " ^ command_prefix_to_net ^ " peer");
 
     "link", Arg_multiple (fun args o ->
-        let buf = o.conn_buf in
         let url = String2.unsplit args ' ' in
         if parse_overnet_url url then
           "download started"
@@ -1595,10 +1637,10 @@ let _ =
            command_prefix_to_net ^ " store");
           ]);
         "";
-    ), (":\t\t\t\t" ^ command_prefix_to_net ^ " Stats commands");
+    ), (":\t\t\t" ^ command_prefix_to_net ^ " stats commands");
 
     "stats", Arg_none (fun o ->
-        let buf = o.conn_buf and sum = ref 0 in
+        let buf = o.conn_buf in
         if o.conn_output = HTML then
           begin
             Printf.bprintf buf "\\<div class=results\\>";
@@ -1670,7 +1712,7 @@ let _ =
           Printf.bprintf buf "\\</div\\>\n";
 
         "";
-    ), ":\t\t\t\t" ^ command_prefix_to_net ^ " Stats";
+    ), ":\t\t\t\t" ^ command_prefix_to_net ^ " stats";
 
     "web", Arg_multiple (fun args o ->
         let urls =
@@ -1686,10 +1728,9 @@ let _ =
             Printf.bprintf o.conn_buf "Loading %s\n" url;
             CommonWeb.load_url true web_info url) urls;
         "web boot started"
-    ), "<urls> :\t\t\t\tdownload .ocl URLS (no arg load default)";
+    ), "<urls> : \t\t\tdownload .ocl URLs (no arg load default)";
 
     "load", Arg_one (fun filename o ->
-        let buf = o.conn_buf in
         try
           let n = load_contact_dat filename in
           Printf.sprintf "%d overnet peers loaded" n;
@@ -1698,7 +1739,7 @@ let _ =
     ), "<filename> :\t\t\tload the peers from a contact.dat file";
 
     "md4", Arg_none (fun o -> "MD4 is " ^ (Md4.to_string !!overnet_md4);
-    ), ":\t\t\t\t\tget client MD4 address on the Overnet/Kademlia network";
+    ), ":\t\t\t\tget client MD4 address on the Overnet/Kademlia network";
 
     "store", Arg_none (fun o ->
         let buf = o.conn_buf in
@@ -1747,10 +1788,10 @@ let _ =
             (Ip.of_string ip)
           (int_of_string port)
           (OvernetUnknown (opcode,msg));
-          lprintf_nl () "Sending UDP message %d to %s:%s" opcode ip port;
-          dump msg; lprintf_nl () ""; "Sending UDP message"
+          lprintf_nl "Sending UDP message %d to %s:%s" opcode ip port;
+          dump msg; lprintf_nl ""; "Sending UDP message"
         with _ ->
-            lprintf_nl () "Unable to send UDP message"; "Unable to send UDP message"
+            lprintf_nl "Unable to send UDP message"; "Unable to send UDP message"
     ), ":\t\t\t\tsend UDP message (<ip> <port> <msg in hex>)";
     
     "buckets", Arg_none (fun o ->
@@ -1811,10 +1852,10 @@ let _ =
 let overnet_search (ss : search) =
   if !!overnet_search_keyword && !!enable_overnet then
     let q = ss.search_query in
-    if !verbose_overnet then lprintf_nl () "========= %s_search =========" command_prefix_to_net;
+    if !verbose_overnet then lprintf_nl "========= %s_search =========" command_prefix_to_net;
     let ws = keywords_of_query q in
     List.iter (fun w ->
-        if !verbose_overnet then lprintf_nl () "%s_search for %s" command_prefix_to_net w;
+        if !verbose_overnet then lprintf_nl "%s_search for %s" command_prefix_to_net w;
         let s = create_keyword_search w ss in
         Hashtbl.iter (fun r_md4 r_tags ->
             DonkeyOneFile.search_found true ss r_md4 r_tags) s.search_results;
@@ -1842,19 +1883,6 @@ let forget_search ss =
       | _ -> true) !overnet_searches
   end    
 
-let _ =
-  CommonWeb.add_redirector_info Proto.redirector_section (fun buf ->
-      update_buckets ();
-      let peers = get_any_peers 32 in
-      buf_int buf (List.length peers);
-      List.iter (fun p ->
-          buf_ip buf p.peer_ip;
-          buf_int16 buf p.peer_port;
-          buf_int16 buf p.peer_tcpport;
-      )
-      peers;
-  )
-
 let cancel_recover_file file =
    begin
 (* reset the Hashtbls and Fifos *)
@@ -1877,7 +1905,7 @@ let cancel_recover_file file =
    end   
 
 let _ =
-  CommonWeb.add_web_kind web_info (fun _ filename ->
+  CommonWeb.add_web_kind web_info web_info_descr (fun _ filename ->
       let s = File.to_string filename in
       let s = String2.replace s '"' "" in
       let lines = String2.split_simplify s '\n' in
@@ -1890,24 +1918,25 @@ let _ =
                 Ip.async_ip name (fun ip ->
                     let port = int_of_string port in
                     if !verbose_overnet then
-                      lprintf_nl () "Adding %s peer %s:%d" command_prefix_to_net name port;
+                      lprintf_nl "Adding %s peer %s:%d" command_prefix_to_net name port;
                     bootstrap ip port)
-            | _ -> lprintf_nl () "BAD LINE ocl: %s" s;
-          with _ -> lprintf_nl () "DNS failed";
+            | _ -> lprintf_nl "BAD LINE ocl: %s" s;
+          with _ -> lprintf_nl "DNS failed";
       ) lines
   );
 
   (* Add this kind of web_info only for overnet *)
   if Proto.redirector_section = "DKKO" then
-      CommonWeb.add_web_kind "contact.dat" (fun url filename ->
+    CommonWeb.add_web_kind "contact.dat" "List of Overnet boot peers" 
+      (fun url filename ->
         if !!enable_overnet && !!overnet_update_nodes then
           let n = load_contact_dat filename in
-            lprintf_nl () "contact.dat loaded from %s, added %d peers" url n;
+            lprintf_nl "contact.dat loaded from %s, added %d peers" url n;
 	else
 	  if not !!enable_overnet then
-	    lprintf_nl () "Overnet module is disabled, ignoring..."
+      lprintf_nl "Overnet module is disabled, ignoring..."
 	  else
-	    lprintf_nl () "Overnet_update_nodes is disabled, ignoring..."
+      lprintf_nl "Overnet_update_nodes is disabled, ignoring..."
         );
 
 (*************************************************************
