@@ -36,14 +36,6 @@ open DonkeyGlobals
 open DonkeyTypes
 open DonkeyMftp
 
-let lprintf_nl () =
-  lprintf "%s[EDK] "
-    (log_time ()); lprintf_nl2
-
-let lprintf_n () =
-  lprintf "%s[EDK] "
-    (log_time ()); lprintf
-
 let buf = TcpBufferedSocket.internal_buf
 
 let client_msg_to_string emule_version msg =
@@ -64,7 +56,7 @@ let server_msg_to_string msg =
   DonkeyProtoServer.write buf msg;
 
   if !verbose_msg_servers then begin
-      lprintf_nl () "MESSAGE TO SERVER:";
+      lprintf_nl "MESSAGE TO SERVER:";
       DonkeyProtoServer.print msg;
       lprint_newline ();
     end;
@@ -87,11 +79,13 @@ let direct_client_sock_send emule_version sock m =
 let client_send c m =
   let emule_version = c.client_emule_proto in
   if !verbose_msg_clients || c.client_debug then begin
-      lprintf_nl () "Sent to client[%d] %s(%s) %s" (client_num c)
+      lprintf_nl "Sent to client[%d] %s(%s) %s" (client_num c)
         c.client_name (brand_to_string c.client_brand)
       (match c.client_kind with
-          Indirect_address _ | Invalid_address _ -> ""
+          Indirect_address (server_ip, server_port, ip, port, real_ip) ->
+	    Printf.sprintf "[%s:%d]" (Ip.to_string (ip_of_id ip)) port
         | Direct_address (ip,port) -> Printf.sprintf "[%s:%d]" (Ip.to_string ip) port
+        | Invalid_address _ -> ""
       );
       DonkeyProtoClient.print m;
       lprint_newline ();
@@ -117,23 +111,21 @@ let servers_send socks m =
   List.iter (fun s -> write_string s m) socks
 
 let client_handler2 c ff f =
-  let emule_version = match !c with
-      None -> emule_proto ();
-    | Some c -> c.client_emule_proto
-  in
   let msgs = ref 0 in
   fun sock nread ->
 
-    if !verbose then lprintf_nl () "between clients %d" nread;
     let module M= DonkeyProtoClient in
     let b = TcpBufferedSocket.buf sock in
     try
       while b.len >= 5 do
+        let emule_version = match !c with
+            None -> emule_proto ();
+          | Some c -> c.client_emule_proto
+        in
         let opcode = get_uint8 b.buf b.pos in
         let msg_len = get_int b.buf (b.pos+1) in
         if b.len >= 5 + msg_len then
           begin
-            if !verbose then lprintf_nl () "client_to_client";
             let s = String.sub b.buf (b.pos+5) msg_len in
             buf_used b  (msg_len + 5);
             let t = M.parse emule_version opcode s in
@@ -150,7 +142,6 @@ lprint_newline (); *)
     with Not_found -> ()
 
 let cut_messages parse f sock nread =
-  if !verbose then lprintf_nl () "server to client %d" nread;
   let b = TcpBufferedSocket.buf sock in
   try
     while b.len >= 5 do
@@ -158,7 +149,6 @@ let cut_messages parse f sock nread =
       let msg_len = get_int b.buf (b.pos+1) in
       if b.len >= 5 + msg_len then
         begin
-          if !verbose then lprintf_nl () "server_to_client";
           let s = String.sub b.buf (b.pos+5) msg_len in
           buf_used b (msg_len + 5);
           let t = parse opcode s in
@@ -168,20 +158,24 @@ let cut_messages parse f sock nread =
     done
   with Not_found -> ()
 
-let udp_send t ip port msg =
+let really_udp_send t ip port msg isping =
 
   if !verbose_udp then begin
-      lprintf_nl () "Message UDP to %s:%d\n%s" (Ip.to_string ip) port
-        (DonkeyProtoUdp.print msg);
+      lprintf_nl "Message UDP%s to %s:%d\n%s" 
+        (if isping then "(PING)" else "") (Ip.to_string ip) 
+        port (DonkeyProtoUdp.print msg);
     end;
 
   try
     Buffer.reset buf;
     DonkeyProtoUdp.write buf msg;
     let s = Buffer.contents buf in
-    UdpSocket.write t false s ip port
+    UdpSocket.write t isping s ip port
   with e ->
-      lprintf_nl () "Exception %s in udp_send" (Printexc2.to_string e)
+      lprintf_nl "Exception %s in udp_send" (Printexc2.to_string e)
+
+let udp_send t ip port msg =
+  really_udp_send t ip port msg false
 
 let udp_handler f sock event =
   let module M = DonkeyProtoUdp in
@@ -210,7 +204,7 @@ let udp_basic_handler f sock event =
             if len = 0 ||
               int_of_char pbuf.[0] <> DonkeyOpenProtocol.udp_magic then begin
                 if !verbose_unknown_messages then begin
-                    lprintf_nl () "Received unknown UDP packet";
+                    lprintf_nl "Received unknown UDP packet";
                     dump pbuf;
                   end;
               end else begin
@@ -218,7 +212,7 @@ let udp_basic_handler f sock event =
                 f t p
               end
           with e ->
-              lprintf_nl () "Error %s in udp_basic_handler"
+              lprintf_nl "Error %s in udp_basic_handler"
                 (Printexc2.to_string e)
       ) ;
   | _ -> ()
@@ -237,7 +231,7 @@ let tag_file file =
       let name = if String2.starts_with name "hidden." then
           String.sub name 7 (String.length name - 7)
         else name in
-      if !verbose then lprintf_nl () "Sharing %s" name;
+      if !verbose_share then lprintf_nl "tag_file: Sharing %s" name;
       name
     ))::
   (int64_tag Field_Size file.file_file.impl_file_size) ::
@@ -246,8 +240,7 @@ let tag_file file =
         FormatNotComputed next_time when
         next_time < last_time () ->
           (try
-              if !verbose then lprintf_nl () "%s: Find format %s"
-                    (string_of_date (last_time ()))
+              if !verbose_share then lprintf_nl "Find format %s"
                   (file_disk_name file);
               file.file_format <- (
                 match
@@ -330,8 +323,6 @@ let make_tagged_server newer_server sock files =
     make_tagged sock files
 
 let server_send_share compressed sock msg =
-  if !verbose then
-      lprintf_nl () "Sending %d file(s) to server" (List.length msg);
   let max_len =
     !!client_buffer_size - 100
       - TcpBufferedSocket.remaining_to_write sock
@@ -347,8 +338,10 @@ let server_send_share compressed sock msg =
       let s = Buffer.contents buf in
       str_int s 0 nfiles;
       let s = String.sub s 0 prev_len in
-      if !verbose_share then
-         lprintf_nl () "Sending %d share(s) to server : " nfiles;
+      if !verbose_share || !verbose then
+         lprintf_nl "Sending %d share(s) to server %s:%d%s"
+	   nfiles (Ip.to_string (peer_ip sock)) (peer_port sock)
+	   (if compressed then "(zlib)" else "");
       Buffer.reset buf;
       let s_c =
         if compressed then
@@ -360,8 +353,6 @@ let server_send_share compressed sock msg =
          is smaller in that state. *)
       if compressed && ((String.length s_c) < (String.length s))  then
         begin
-          if !verbose_share then
-            lprintf_nl () "Using zlib";
           buf_int8 buf 0xD4;
           buf_int buf 0;
           buf_int8 buf 21; (* ShareReq *)
@@ -370,8 +361,6 @@ let server_send_share compressed sock msg =
         end
       else
         begin
-          if !verbose_share then
-            lprintf_nl () "No compression";
           buf_int8 buf 227;
           buf_int buf 0;
           buf_int8 buf 21; (* ShareReq *)
@@ -401,5 +390,29 @@ let client_send_files sock msg =
   str_int s 6 nfiles;
   write_string sock s
 
+let client_send_dir sock dir files =
+  let max_len = !!client_buffer_size - 100 -
+    TcpBufferedSocket.remaining_to_write sock in
+  Buffer.reset buf;
+  buf_int8 buf 227;
+  buf_int buf 0;
+  buf_int8 buf 96; (* ViewFilesDirReply *)
+  buf_string buf dir;
+  buf_int buf 0;
+  let pos = Buffer.length buf in
+  let nfiles, prev_len = DonkeyProtoClient.ViewFilesReply.write_files_max buf (
+      make_tagged (Some sock) files)
+    0 max_len in
+  let s = Buffer.contents buf in
+  let s = String.sub s 0 prev_len in
+  let len = String.length s - 5 in begin
+    str_int s 1 len;
+    str_int s (pos-4) nfiles;
+  end;
+  write_string sock s
+
 let udp_server_send s t =
   udp_send (get_udp_sock ()) s.server_ip (s.server_port+4) t
+
+let udp_server_send_ping s t =
+  really_udp_send (get_udp_sock ()) s.server_ip (s.server_port+4) t true
