@@ -47,10 +47,6 @@ open DonkeyUdp
 
 module Udp = DonkeyProtoUdp
 
-let lprintf_nl () =
-  lprintf "%s[EDK] "
-  (log_time ()); lprintf_nl2
-
 (* Constants *)
 let udp_requests_new  = 5
 let udp_requests_old  = 3
@@ -127,7 +123,7 @@ let query_locations_timer () =
               List.iter (
                 fun file ->
                   if !verbose_location then
-                    lprintf_nl () "TCP: Query Location of %s [%s] [%Ld] [%s]"
+                    lprintf_nl "TCP: Query Location of %s [%s] [%Ld] [%s]"
                       (file_best_name file) (Md4.to_string file.file_md4) (file_size file) (Ip.to_string (peer_ip sock));
                   let module M = DonkeyProtoServer in
                   let module E = M.QueryLocation in
@@ -182,7 +178,7 @@ let udp_query_sources () =
     ()
   done;
   if !verbose_location then
-    lprintf_nl () "udp_query_source to: %2d old %2d new servers"
+    lprintf_nl "udp_query_source to: %2d old %2d new servers"
       (List.length !old_servers) (List.length !new_servers);
 
   try
@@ -210,7 +206,7 @@ let udp_query_sources () =
     ) !old_servers;
 
   with e ->
-    lprintf_nl () "udp_query_sources: %s" (Printexc2.to_string e)
+    lprintf_nl "udp_query_sources: %s" (Printexc2.to_string e)
 
 
 let disconnect_server s reason =
@@ -223,10 +219,6 @@ let disconnect_server s reason =
     | Connection sock ->
         decr nservers;
         TcpBufferedSocket.close sock reason;
-        (*
-          lprintf "%s:%d CLOSED received by server\n"
-          (Ip.to_string s.server_ip) s.server_port;
-        *)
         connection_failed (s.server_connection_control);
         s.server_sock <- NoConnection;
         s.server_score <- s.server_score - 1;
@@ -239,6 +231,7 @@ let disconnect_server s reason =
           | _ -> ());
         s.server_banner <- "";
         s.server_sent_all_queries <- false;
+        remove_connecting_server s;
         remove_connected_server s
 
 
@@ -259,7 +252,7 @@ let client_to_server s t sock =
   s.server_last_message <- last_time ();
 
   if !verbose_msg_servers then begin
-    lprintf_nl () "Message from server:";
+    lprintf_nl "Message from server:";
     DonkeyProtoServer.print t; lprint_newline ()
   end;
 
@@ -270,10 +263,11 @@ let client_to_server s t sock =
           disconnect_server s (Closed_for_error "Low ID")
         else begin
           s.server_cid <- Some t.M.SetID.ip;
-	  s.server_realport <- t.M.SetID.port;
+          s.server_realport <- t.M.SetID.port;
           (* disconnect after (connected_server_timeout) seconds of silence *)
           set_rtimeout sock !!connected_server_timeout;
           set_server_state s Connected_initiating;
+          remove_connecting_server s;
           s.server_score <- s.server_score + 5;
           connection_ok (s.server_connection_control);
 
@@ -292,22 +286,27 @@ let client_to_server s t sock =
               s.server_name (Ip.to_string s.server_ip) s.server_port in
           CommonEvent.add_event (Console_message_event (Printf.sprintf "%s\n" server_header));
           if !CommonOptions.verbose_msg_servers then
-	    lprintf_nl () "%s" server_header;
+      lprintf_nl "%s" server_header;
           last_message_sender := server_num s
         end;
       s.server_banner <- s.server_banner ^ Printf.sprintf "%s\n" msg;
       let msg = Printf.sprintf "| %s" msg in
       CommonEvent.add_event (Console_message_event (Printf.sprintf "%s\n" msg));
       if !CommonOptions.verbose_msg_servers then
-        lprintf_nl () "%s" msg
+        lprintf_nl "%s" msg
 
   | M.ServerListReq l ->
-      if !verbose then lprintf_nl () "donkeyServers: Received serverlist";
-      if !!update_server_list_server then
+      if !!update_server_list_server then begin
+        if !verbose_msg_servers then lprintf_nl "Received serverlist from server";
         let module Q = M.ServerList in
         List.iter (fun s ->
             safe_add_server s.Q.ip s.Q.port
         ) l
+        end
+      else begin
+        if !verbose_msg_servers then lprintf_nl "Ignored serverlist from server"
+        end
+
 
   | M.ServerInfoReq t ->
       s.server_score <- s.server_score + 1;
@@ -321,7 +320,6 @@ let client_to_server s t sock =
                 s.server_description <- desc
             | _ -> ()
       ) s.server_tags;
-      printf_char 'S';
 
       (* nice and ugly, but it doesn't require any new fields *)
       set_server_state s (Connected
@@ -337,11 +335,11 @@ let client_to_server s t sock =
       s.server_queries_credit <-  0
 
   | M.InfoReq (users, files) ->
-      s.server_nusers <- Int64.of_int users;
-      s.server_nfiles <- Int64.of_int files;
+      s.server_nusers <- Some (Int64.of_int users);
+      s.server_nfiles <- Some (Int64.of_int files);
       if (users < !!min_users_on_server && not s.server_preferred) then
         begin
-          lprintf_nl () "%s:%d remove server min_users_on_server limit hit!"
+          lprintf_nl "%s:%d remove server min_users_on_server limit hit!"
             (Ip.to_string s.server_ip) s.server_port;
 
           disconnect_server s Closed_for_timeout;
@@ -353,7 +351,7 @@ let client_to_server s t sock =
       (* This can either be a reply to a QueryID or a indirect request for
          connection from another client. In this case, we should immediatly
          connect. *)
-      if !verbose then lprintf_nl () "QueryIDReplyReq: received";
+      if !verbose then lprintf_nl "QueryIDReplyReq: received";
       let module Q = M.QueryIDReply in
       if Ip.usable t.Q.ip then begin
         try
@@ -361,14 +359,14 @@ let client_to_server s t sock =
               None -> raise Not_found
             | Some file ->
                 if !verbose then
-                  lprintf_nl () "QueryIDReplyReq: This was a QueryID reply !?";
+                  lprintf_nl "QueryIDReplyReq: This was a QueryID reply !?";
                 let s = DonkeySources.find_source_by_uid
                   (Direct_address (t.Q.ip, t.Q.port)) in
                 DonkeySources.set_request_result s file.file_sources
                   File_new_source
         with _ ->
           if !verbose then
-            lprintf_nl () "QueryIDReplyReq: Calling back to %s:%d"
+            lprintf_nl "QueryIDReplyReq: Calling back to %s:%d"
               (Ip.to_string t.Q.ip) t.Q.port;
           let c = new_client (Direct_address (t.Q.ip, t.Q.port)) in
           DonkeyClient.reconnect_client c;
@@ -376,7 +374,7 @@ let client_to_server s t sock =
 
   | M.QueryIDFailedReq t ->
       if !verbose then
-        lprintf_nl () "QueryIDFailedReq:";
+        lprintf_nl "QueryIDFailedReq:";
       ignore (Fifo.take s.server_id_requests)
 
   | M.QueryReplyReq t ->
@@ -456,9 +454,8 @@ and remove clients whose server is deconnected. *)
       ()
 
 let connect_server s =
-  if !!enable_servers && can_open_connection connection_manager
+  if !!enable_servers && not (!Ip.banned s.server_ip <> None)
     && (not !!connect_only_preferred_server || s.server_preferred)
-    && not (Ip_set.ip_blocked s.server_ip)
   then
     match s.server_sock with
     | NoConnection ->
@@ -469,7 +466,6 @@ let connect_server s =
               try
                 (* lprintf "CONNECTING ONE SERVER\n"; *)
                 connection_try s.server_connection_control;
-                printf_char 's';
                 let sock = TcpBufferedSocket.connect token "donkey to server"
                     (Ip.to_inet_addr s.server_ip) s.server_port
                     (server_handler s) (* DonkeyProtoCom.server_msg_to_string*)  in
@@ -497,13 +493,9 @@ let connect_server s =
                     C.tags = !client_to_server_tags;
                   }
                 );
+                add_connecting_server s;
                 add_connected_server s;
               with e ->
-                (*
-                  lprintf "%s:%d IMMEDIAT DISCONNECT \n"
-                  (Ip.to_string s.server_ip) s.server_port;
-                  lprintf "DISCONNECTED IMMEDIATLY\n";
-                 *)
                 disconnect_server s (Closed_for_exception e)
           )
           in
@@ -519,7 +511,7 @@ let rec connect_one_server restart =
   (*  lprintf "connect_one_server\n";  *)
   if can_open_connection connection_manager then
     match !servers_list with
-	[] ->
+  [] ->
           if restart then begin
             servers_list := [];
             Hashtbl.iter (fun _ s ->
@@ -528,11 +520,11 @@ let rec connect_one_server restart =
             if !servers_list = [] then begin
               if !print_empty_list then begin
                 print_empty_list := false;
-                lprintf_nl () "There are no ED2K-servers in your servers.ini.";
-                lprintf_nl () "Please import servers from a server.met file.";
-                lprintf_nl () "Let MLDonkey use a file configured in web_infos";
-                lprintf_nl () "or enter this link into MLDonkey:";
-		lprintf_nl () "ed2k://|serverlist|http://www.gruk.org/server.met.gz|/"
+                lprintf_nl "There are no ED2K-servers in your servers.ini.";
+                lprintf_nl "Please import servers from a server.met file.";
+                lprintf_nl "Let MLDonkey use a file configured in web_infos";
+                lprintf_nl "or enter this link into MLDonkey:";
+	        lprintf_nl "ed2k://|serverlist|http://www.gruk.org/server.met.gz|/"
               end;
 
               raise Not_found;
@@ -585,8 +577,11 @@ let rec check_server_connections () =
  * clean serverlist
  *)
 let remove_old_servers () =
-  if !verbose then lprintf_nl () "REMOVE OLD SERVERS";
-  let t1 = Unix.gettimeofday () in
+  if last_time () - start_time > !!max_server_age * Date.day_in_secs then begin
+  
+  if !verbose then
+    lprintf_nl "old servers: Start check, remove servers not connected for %d days"
+      !!max_server_age;
 (*
 The new tactic: we sort the servers (the more recently connected first,
 the black-listed ones at the end), and we start checking from the last
@@ -605,8 +600,6 @@ position to the min_left_servers position.
             to_keep :=
               (connection_last_conn s.server_connection_control, s) :: !to_keep
   ) servers_by_key;
-  let t2 = Unix.gettimeofday () in
-  if !verbose then lprintf_nl () "Delay to detect black-listed servers: %2.2f" (t2 -. t1);
 
   if List.length !to_keep > !!min_left_servers then begin
     let array = Array.of_list !to_keep in
@@ -616,7 +609,7 @@ position to the min_left_servers position.
 
     if !verbose then
       Array.iteri (fun i (ls, s) ->
-        lprintf_nl () "server %d last_conn %d" (server_num s) ls) array;
+        lprintf_nl "old servers: server %d last_conn %d" (server_num s) ls) array;
 
     let min_last_conn =  last_time () - !!max_server_age * Date.day_in_secs in
 
@@ -625,7 +618,7 @@ position to the min_left_servers position.
         if ls < min_last_conn && s.server_sock = NoConnection
           && not s.server_preferred then begin
           if !verbose then
-            lprintf_nl () "Server too old: %s:%d"
+            lprintf_nl "old servers: Server too old: %s:%d"
               (Ip.to_string s.server_ip) s.server_port;
 
           to_remove := s :: !to_remove
@@ -633,17 +626,14 @@ position to the min_left_servers position.
     done
   end;
 
-  let t3 = Unix.gettimeofday () in
-  if !verbose then lprintf_nl () "Delay to detect old servers: %2.2f" (t3 -. t2);
-
   List.iter (fun s ->
-    remove_server s.server_ip s.server_port
+    DonkeyGlobals.remove_server s.server_ip s.server_port
   ) !to_remove;
 
-  let t4 = Unix.gettimeofday () in
-  if !verbose then lprintf_nl () "Delay to finally remove servers: %2.2f" (t4 -. t3);
   if !to_remove <> [] || !verbose then
-    lprintf_nl () "Removed %d old edonkey servers." (List.length !to_remove)
+    lprintf_nl "Removed %d old edonkey servers." (List.length !to_remove)
+
+  end
 
 
 (* Keep connecting to servers in the background. Don't stay connected to
@@ -684,7 +674,7 @@ let walker_timer () =
                   if connection_can_try s.server_connection_control then
                     begin
                       if !verbose then
-                        lprintf_nl () "WALKER: try connect %s"
+                        lprintf_nl "WALKER: try connect %s"
                           (Ip.to_string s.server_ip);
                       connect_server s
                     end
@@ -692,7 +682,7 @@ let walker_timer () =
                     begin
                       delayed_list := s :: !delayed_list;
                       if !verbose then
-                        lprintf_nl () "WALKER: connect %s delayed"
+                        lprintf_nl "WALKER: connect %s delayed"
                           (Ip.to_string s.server_ip);
                     end
               | _ -> ()
@@ -708,7 +698,6 @@ Each client issues 1 packet/4hour, so 100000 clients means 25000/hour,
 7 packets/second = 7 * 40 bytes = 280 B/s ...
 *)
 
-let udp_ping = String.create 6
 
 let udp_walker_timer () =
   match !udp_walker_list with
@@ -721,8 +710,19 @@ let udp_walker_timer () =
         end
   | s :: tail ->
       udp_walker_list := tail;
-      UdpSocket.write (get_udp_sock ())
-                  true udp_ping s.server_ip (s.server_port + 4)
+      let c = Int64.of_int (Random.int 65535) in
+      let challenge = 1437204480L ++ c in (* 0x55AA0000 *)
+      s.server_last_ping <- Unix.gettimeofday ();
+      udp_server_send_ping s (DonkeyProtoUdp.PingServerUdpReq challenge);
+
+      let module M = DonkeyProtoUdp in
+      let module E = M.ServerDescUdp in
+      udp_server_send s (M.ServerDescUdpReq {
+        E.ip = Ip.null;
+      })
+
+(*      UdpSocket.write (get_udp_sock ())
+                  true udp_ping s.server_ip (s.server_port + 4) *)
 
 (* sort the servers by preferred first
    then users count with decreasing order
@@ -730,34 +730,41 @@ let udp_walker_timer () =
 let compare_servers s2 s1 =
   let n = compare s1.server_preferred s2.server_preferred in
   if n = 0 then
-    Int64.to_int (s1.server_nusers -- s2.server_nusers)
+    Int64.to_int
+    ((match s1.server_nusers with None -> 0L | Some v -> v) --
+     (match s2.server_nusers with None -> 0L | Some v -> v))
   else n
 
 (* check connected servers *)
 let update_master_servers _ =
+  if !verbose_location then
+    lprintf_nl "master servers: start re-computing";
   let server_list = List.sort compare_servers (connected_servers ()) in
   let masters = ref [] in
+  let tag1 = ref true in
   List.iter (
+    let tag2 = ref true in
     fun s ->
       if s.server_master then
         match s.server_sock with
         | Connection _ ->
-            if !verbose then begin
-                lprintf_nl () "MASTER: OLD MASTER %s" (Ip.to_string s.server_ip);
-              end;
+            if !verbose_location then begin
+        if !tag2 then begin
+                lprintf_n "master servers (old):";
+    tag1 := false;
+    tag2 := false
+        end;
+        lprintf " %s" (Ip.to_string s.server_ip)
+      end;
             masters := s :: !masters
         | _ -> s.server_master <- false
   ) server_list;
+  if not !tag1 then lprint_newline ();
   let nmasters = ref (List.length !masters) in
 
-  if !verbose then
-    lprintf_nl () "MASTER: nmaster %d" !nmasters;
-
   let make_master s =
     (* normal servers don't have our SHARE, so send list if it becomes a master *)
     do_if_connected s.server_sock (fun sock ->
-        if !verbose then
-          lprintf_nl () "   MASTER: %s" (Ip.to_string s.server_ip);
         s.server_master <- true;
         incr nmasters;
 
@@ -778,8 +785,8 @@ let update_master_servers _ =
        connections *)
     if !nconnected_servers > max_allowed_connected_servers then
       begin
-        if !verbose then
-            lprintf_nl () "MASTER: DISCONNECT %s" (Ip.to_string s.server_ip);
+        if !verbose_location then
+            lprintf_nl "master servers: disconnect %s" (Ip.to_string s.server_ip);
         nconnected_servers := !nconnected_servers - 3;
         do_if_connected  s.server_sock (fun sock ->
             (* We will disconnect from this server.
@@ -797,41 +804,50 @@ let update_master_servers _ =
             last_time ()
             - connection_last_conn s.server_connection_control
           in
-          if !verbose then
-              lprintf_nl () "MASTER: Checking ip:%s ct:%d" (Ip.to_string s.server_ip) connection_time;
+          if !verbose_location then
+              lprintf_nl "master servers: Checking ip:%s, users: %Ld, ct:%d"
+          (Ip.to_string s.server_ip)
+	  (match s.server_nusers with None -> 0L | Some v -> v)
+	  connection_time;
           if not s.server_master
             && (s.server_preferred
                  || connection_time > !!become_master_delay
-                 || !!immediate_master
                )
             then
               begin
                 if (!nmasters < max_allowed_connected_servers) then
                   begin
-                    if !verbose then
-                        lprintf_nl () "   MASTER: RAISING %s (%Ld)"
-                                  (Ip.to_string s.server_ip) s.server_nusers;
+                    if !verbose_location then
+                        lprintf_nl "master servers: raising  %s"
+                                  (Ip.to_string s.server_ip);
                     make_master s
                   end
                 else if s.server_sent_all_queries then
                   match !masters with
                       [] -> disconnect_old_server s
                     | ss :: tail ->
+			let ss_nusers =
+			  match ss.server_nusers with
+			    None -> 0L
+			  | Some v -> v in
+			let s_nusers =
+			  match s.server_nusers with
+			    None -> 0L
+			  | Some v -> v in
                         (* check if the non-master has more users
                            or is a preferred one *)
                         if (s.server_preferred && not ss.server_preferred)
                           || (!!keep_best_server
-                               && mini ((Int64.to_int ss.server_nusers) + 1000)
-                                       ((Int64.to_int ss.server_nusers) * 5)
-                                  < (Int64.to_int s.server_nusers)
+                               && mini ((Int64.to_int ss_nusers) + 1000)
+                                       ((Int64.to_int ss_nusers) * 5)
+                                  < (Int64.to_int s_nusers)
                              )
                           then
                             begin
-                              if !verbose then
-                                lprintf_nl ()
-                                  "   MASTER: RAISING %s (%Ld) instead of %s (%Ld)"
-                                  (Ip.to_string s.server_ip) s.server_nusers
-                                  (Ip.to_string ss.server_ip) ss.server_nusers;
+                              if !verbose_location then
+                                lprintf_nl
+                                  "master servers: raising %s, disconnected %s"
+                                  (Ip.to_string s.server_ip) (Ip.to_string ss.server_ip);
                               ss.server_master <- false;
                               masters := tail;
                               make_master s
@@ -842,28 +858,24 @@ let update_master_servers _ =
               end
         )
     ) server_list;
-  if !verbose then
-      lprintf_nl () "MASTER: clean %d connected %d masters"
+  if !verbose_location then
+      lprintf_nl "master servers: %d connected %d masters - re-computing completed"
         !nconnected_servers !nmasters
 
+let check_for_preferred_servers () =
+  let found_preferred = ref false in
+  Hashtbl.iter (fun _ s -> if s.server_preferred then found_preferred := true) servers_by_key;
+  if not !found_preferred && !!connect_only_preferred_server then
+    begin
+      connect_only_preferred_server =:= false;
+      let message = "Set connect_only_preferred_server to false because no preferred server was found" in
+      lprintf_nl "%s" message;
+      startup_message := !startup_message ^ message
+    end
 
 open LittleEndian
 
 let _ =
-  udp_ping.[0] <- char_of_int 0xe3;
-  udp_ping.[1] <- char_of_int 0x96;
-  udp_ping.[2] <- char_of_int (Random.int 256);
-  udp_ping.[3] <- char_of_int (Random.int 256);
-  udp_ping.[4] <- char_of_int 0xAA;
-  udp_ping.[5] <- char_of_int 0x55;
-
-  CommonWeb.add_redirector_info "DKSV" (fun buf ->
-      buf_list (fun buf s ->
-          buf_ip buf s.server_ip;
-          buf_int16 buf s.server_port;
-      ) buf (connected_servers ())
-  );
-
   server_ops.op_server_sort <- ( fun s ->
     (3600 * s.server_score) + connection_last_conn s.server_connection_control
   )
