@@ -38,26 +38,14 @@ module ClientOption = struct
       | Module assocs ->
 
           let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv =
-            try conv (List.assoc name assocs) with _ -> []
-          in
           let client_hostname = get_value "client_hostname" value_to_string in
           let client_port = get_value "client_port" value_to_int in
+          let client_referer = try
+	      get_value "client_referer" value_to_string with _ -> "" in
           let client_proto = try
               get_value "client_proto" value_to_string with _ -> "http" in
           let proto = find_proto client_proto in
-          let c = new_client proto client_hostname client_port in
-
-(*
-          (try
-              c.client_user.user_speed <- get_value "client_speed" value_to_int
-            with _ -> ());
-
-          (try
-              if get_value "client_push" value_to_bool then
-                c.client_user.user_kind <- Indirect_location ("", client_uid)
-with _ -> ());
-  *)
+          let c = new_client proto client_hostname client_port client_referer in
           c
       | _ -> failwith "Options: Not a client"
 
@@ -66,10 +54,7 @@ with _ -> ());
         "client_hostname", string_to_value c.client_hostname;
         "client_port", int_to_value c.client_port;
         "client_proto", string_to_value c.client_proto.proto_string;
-(*
-            "client_speed", int_to_value u.user_speed;
-"client_push", bool_to_value false;
-  *)
+        "client_referer", string_to_value c.client_referer;
       ]
 
     let t =
@@ -86,9 +71,6 @@ let value_to_int32pair v =
 
 let value_to_file file_size file_state assocs =
   let get_value name conv = conv (List.assoc name assocs) in
-  let get_value_nil name conv =
-    try conv (List.assoc name assocs) with _ -> []
-  in
 
   let file_name = get_value "file_filename" value_to_string in
   let file_id =
@@ -102,13 +84,13 @@ let value_to_file file_size file_state assocs =
   (match file.file_swarmer with
       None -> ()
     | Some swarmer ->
-        Int64Swarmer.value_to_swarmer swarmer assocs;
+        CommonSwarming.value_to_frontend swarmer assocs;
   );
 
   (try
       ignore (get_value "file_sources" (value_to_list (fun v ->
               match v with
-              | SmallList [c; index] | List [c;index] ->
+              | SmallList [c; index] | List [c; index] ->
                   let s = ClientOption.value_to_client c in
                   add_download file s (Url.of_string (value_to_string index))
               | _ -> failwith "Bad source"
@@ -127,9 +109,7 @@ let file_to_value file =
       "file_id", string_to_value (Md4.to_string file.file_id);
       "file_sources",
       list_to_value (fun c ->
-          lprintf "SAVING: find_client...\n";
           let n = (find_download file c.client_downloads).download_url in
-          lprintf "SAVING: find_client...done...saving\n";
           SmallList [ClientOption.client_to_value c;
             string_to_value (Url.to_string n)]
       ) file.file_clients;
@@ -138,7 +118,7 @@ let file_to_value file =
   match file.file_swarmer with
     None -> assocs
   | Some swarmer ->
-      Int64Swarmer.swarmer_to_value swarmer assocs
+      CommonSwarming.frontend_to_value swarmer assocs
 
 let old_files =
   define_option fileTP_section ["old_urls"]
