@@ -46,15 +46,13 @@ open FileTPProtocol
 
 open FileTPClients
 
-(* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[HTTP] "
-    (log_time ()); lprintf_nl2
+let log_prefix = "[FileTP]"
 
-(* prints a new logline with date, module and does not start newline *)
-let lprintf_n () =
-  lprintf "%s[HTTP] "
-    (log_time ()); lprintf
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
 
 (*************************************************************************)
 (*                                                                       *)
@@ -68,8 +66,6 @@ let http_send_range_request c range sock d =
   let real_url =
     (Str.global_replace (Str.regexp " ") "%20" url.Url.full_file) in
 
-  let referer = d.download_referer in
-  
   let (x,y) = range in
   let range = Printf.sprintf "%Ld-%Ld" x (Int64.pred y) in
 
@@ -84,14 +80,19 @@ let http_send_range_request c range sock d =
                   Printf.bprintf buf "GET /get/%d/%s HTTP/1.1\r\n" index
                     name); *)
   Printf.bprintf buf "Host: %s\r\n" c.client_hostname;
-  Printf.bprintf buf "User-Agent: %s\r\n" user_agent;
-  Printf.bprintf buf "Referer: %s\r\n" (Url.to_string referer);
+  Printf.bprintf buf "User-Agent: %s\r\n" (get_user_agent ());
+  Printf.bprintf buf "Referer: %s\r\n" c.client_referer;
   Printf.bprintf buf "Range: bytes=%s\r\n" range;
   Printf.bprintf buf "Connection: Keep-Alive\r\n";
+  if url.Url.user <> "" then begin
+    let userpass = Printf.sprintf "%s:%s" url.Url.user url.Url.passwd in
+    let encoded = Base64.encode userpass in
+    Printf.bprintf buf "Authorization: Basic %s\r\n" encoded
+  end;
   Printf.bprintf buf "\r\n";
   let s = Buffer.contents buf in
   if !verbose_msg_clients then
-    lprintf_nl () "SENDING REQUEST to %s: %s"
+    lprintf_nl "SENDING REQUEST to %s: %s"
       c.client_hostname
       (String.escaped s);
   write_string sock s;
@@ -103,9 +104,26 @@ let http_send_range_request c range sock d =
 (*                                                                       *)
 (*************************************************************************)
 
+
+let parse_line header = 
+  let endline_pos = String.index header '\n' in
+  let http, code =
+    match String2.split (String.sub header 0 endline_pos) ' ' with
+    | http :: code :: ok :: _ ->
+      let code = int_of_string code in
+      if not (String2.starts_with (String.lowercase http) "http") 
+        then failwith "Not in http protocol"; 
+      http, code
+    | _ -> 
+      failwith "Not a HTTP header line"
+    in
+  http, code
+
+
+
 let rec client_parse_header c gconn sock header =
   if !verbose_msg_clients then
-    lprintf_nl () "CLIENT PARSE HEADER";
+    lprintf_nl "CLIENT PARSE HEADER";
   try
     set_lifetime sock 3600.;
     let d =
@@ -118,25 +136,15 @@ let rec client_parse_header c gconn sock header =
     connection_ok c.client_connection_control;
     set_client_state c Connected_initiating;
     if !verbose_msg_clients then begin
-        lprintf_nl () "HEADER FROM CLIENT:";
+        lprintf_nl "HEADER FROM CLIENT:";
         AnyEndian.dump_ascii header;
       end;
     let file = d.download_file in
     let size = file_size file in
 
-    let endline_pos = String.index header '\n' in
-    let http, code =
-      match String2.split (String.sub header 0 endline_pos
-        ) ' ' with
-      | http :: code :: ok :: _ ->
-          let code = int_of_string code in
-          if not (String2.starts_with (String.lowercase http) "http") then
-            failwith "Not in http protocol";
-          http, code
-      | _ -> failwith "Not a HTTP header line"
-    in
+    let http, code = parse_line header in
     if !verbose_msg_clients then
-      lprintf_nl () "GOOD HEADER FROM CONNECTED CLIENT\n";
+      lprintf_nl "GOOD HEADER FROM CONNECTED CLIENT\n";
 
     set_rtimeout sock 120.;
 (*              lprintf "SPLIT HEADER...\n"; *)
@@ -156,42 +164,19 @@ let rec client_parse_header c gconn sock header =
             unknown_header := !unknown_header || not (List.mem header known_download_headers)
         ) headers;
         if !unknown_header then begin
-            lprintf_n () "Download Header contains unknown fields";
-            lprintf_nl2 "    %s" first_line;
+            lprintf_n "Download Header contains unknown fields";
+            lprintf_nl "    %s" first_line;
             List.iter (fun (header, (value,header2)) ->
-                lprintf_nl () "    [%s] = [%s](%s)" header value header2;
+                lprintf_nl "    [%s] = [%s](%s)" header value header2;
             ) headers;
-            lprintf_nl () "end of header";
+            lprintf_nl "end of header";
           end;
       end;
 
-    (* I think this is already handeled with the new headder check before
-       download start code
-    (* If the header contains a redirection *)
-    if (code = 302) then begin
-      let (newurl, _) = List.assoc "location" headers in
-
-      remove_file file;
-      file_cancel (as_file file);
-      let u = Url.of_string newurl in
-
-      c.client_hostname <- u.Url.server;
-      c.client_port <- u.Url.port;
-
-      let file = new_file (Md4.random ()) u.Url.full_file zero in
-
-      lprintf_nl () "DOWNLOAD FILE %s" (file_best_name  file);
-      if not (List.memq file !current_files) then begin
-    current_files := file :: !current_files;
-      end;
-      add_download file c u r;
-      FileTPClients.get_file_from_source c file;
-
-     end;
-    *)
-
-    if  code < 200 || code > 299 then
+    if code < 200 || code > 299 then begin
+      pause_for_cause file (Printf.sprintf "%d" code);
       failwith "Bad HTTP code";
+    end;
 
     let start_pos, end_pos =
       try
@@ -221,7 +206,7 @@ let rec client_parse_header c gconn sock header =
             x, Int64.succ y
         with
         | e ->
-            lprintf_nl () "Exception %s for range [%s]"
+            lprintf_nl "Exception %s for range [%s]"
               (Printexc2.to_string e) range;
             raise e
       with e ->
@@ -229,14 +214,14 @@ let rec client_parse_header c gconn sock header =
             if code <> 206 && code <> 200 then raise Not_found;
             let (len,_) = List.assoc "content-length" headers in
             let len = Int64.of_string len in
-            if !verbose then lprintf_nl () "Specified length: %Ld" len;
+            if !verbose then lprintf_nl "Specified length: %Ld" len;
             match d.download_ranges with
               [] -> raise Not_found
             | (start_pos,end_pos,r) :: _ ->
-                lprintf_nl () "WARNING: Assuming client is replying to range";
+                lprintf_nl "WARNING: Assuming client is replying to range";
                 if len <> end_pos -- start_pos then
                   begin
-                    lprintf_nl () "ERROR: bad computed range: %Ld-%Ld/%Ld \n%s"
+                    lprintf_nl "ERROR: bad computed range: %Ld-%Ld/%Ld \n%s"
                       start_pos end_pos len
                       (String.escaped header);
                     raise Not_found
@@ -244,8 +229,8 @@ let rec client_parse_header c gconn sock header =
                 (start_pos, end_pos)
           with _ ->
 (* A bit dangerous, no ??? *)
-              if !verbose_hidden_errors then
-                lprintf_nl () "ERROR: Could not find/parse range header (exception %s), disconnect\nHEADER: %s"
+              if !verbose_unknown_messages then
+                lprintf_nl "ERROR: Could not find/parse range header (exception %s), disconnect\nHEADER: %s"
                     (Printexc2.to_string e)
                     (String.escaped header);
               disconnect_client c (Closed_for_error "Bad HTTP Range");
@@ -254,22 +239,23 @@ let rec client_parse_header c gconn sock header =
     (try
         let (len,_) = List.assoc "content-length" headers in
         let len = Int64.of_string len in
-        if !verbose then lprintf_nl () "Specified length: %Ld" len;
         if len <> end_pos -- start_pos then
           begin
-            failwith "\n\nERROR: bad computed range: %Ld-%Ld/%Ld \n%s\n"
-              start_pos end_pos len
-              (String.escaped header);
+            failwith (Printf.sprintf "ERROR: bad computed range: %Ld-%Ld/%Ld \n%s\n"
+              start_pos end_pos len (String.escaped header))
           end
       with _ ->
-          lprintf_nl () "[WARNING]: no Content-Length field\n%s\n"
+          lprintf_nl "[WARNING]: no Content-Length field\n%s\n"
             (String.escaped header)
     );
 
-    if !verbose then lprintf_nl () "Receiving range: %Ld-%Ld (len = %Ld)\n%s"
-      start_pos end_pos (end_pos -- start_pos)
-    (String.escaped header)
-    ;
+    (try
+        let (server,_) = List.assoc "server" headers in
+        c.client_software <- server;
+        client_must_update (as_client c);
+      with _ -> ()
+    );
+
     set_client_state c (Connected_downloading (file_num file));
     let counter_pos = ref start_pos in
 (* Send the next request *)
@@ -287,43 +273,41 @@ let rec client_parse_header c gconn sock header =
         let to_read = min (end_pos -- !counter_pos)
           (Int64.of_int b.len) in
 
-        if !verbose then lprintf_nl () "Reading: end_pos %Ld counter_pos %Ld len %d = to_read %Ld"
-end_pos !counter_pos b.len to_read;
-
         let to_read_int = Int64.to_int to_read in
-(*
-        if !verbose then lprintf "CHUNK: %s\n"
-          (String.escaped (String.sub b.buf b.pos to_read_int)); *)
-        let swarmer = match file.file_swarmer with
-            None -> assert false | Some sw -> sw
-        in
-        let old_downloaded =
-          Int64Swarmer.downloaded swarmer in
-(*        List.iter (fun (_,_,r) -> Int64Swarmer.free_range r)
-        d.download_ranges; *)
 
         begin
           try
             match d.download_uploader with
               None -> assert false
             | Some up ->
-                Int64Swarmer.received up
-                  !counter_pos b.buf b.pos to_read_int;
+    
+            let swarmer = CommonSwarming.uploader_swarmer up in
+
+            let old_downloaded = CommonSwarming.downloaded swarmer in
+
+            CommonSwarming.received up !counter_pos b.buf b.pos to_read_int;
+            let new_downloaded = CommonSwarming.downloaded swarmer in
+
+            c.client_downloaded <- c.client_downloaded ++ (new_downloaded -- old_downloaded);
+            client_must_update (as_client c);
+
+            if new_downloaded = file_size file then
+              download_finished file;
+
           with e ->
-              lprintf_nl () "Exception %s in Int64Swarmer.received"
+              lprintf_nl "Exception %s in CommonSwarming.received"
                 (Printexc2.to_string e)
         end;
+
         c.client_reconnect <- true;
 (*          List.iter (fun (_,_,r) ->
-              Int64Swarmer.alloc_range r) d.download_ranges; *)
-        let new_downloaded =
-          Int64Swarmer.downloaded swarmer in
+              CommonSwarming.alloc_range r) d.download_ranges; *)
 
         (match d.download_ranges with
-            [] -> lprintf_nl () "EMPTY Ranges!"
+            [] -> lprintf_nl "EMPTY Ranges!"
           | r :: _ ->
 (*
-              let (x,y) = Int64Swarmer.range_range r in
+              let (x,y) = CommonSwarming.range_range r in
               if !verbose then lprintf_nl "Received %Ld [%Ld] (%Ld-%Ld) -> %Ld"
                 !counter_pos to_read
                 x y
@@ -332,8 +316,6 @@ end_pos !counter_pos b.len to_read;
               ()
         );
 
-        if new_downloaded = file_size file then
-          download_finished file;
 (*
 lprintf "READ %Ld\n" (new_downloaded -- old_downloaded);
 lprintf "READ: buf_used %d\n" to_read_int;
@@ -348,17 +330,18 @@ lprintf "READ: buf_used %d\n" to_read_int;
                 lprintf "Ready for next chunk (version %s)\nHEADER:%s\n" http
                   (String.escaped header);
                 *)
-(*                Int64Swarmer.free_range r; *)
+(*                CommonSwarming.free_range r; *)
                 d.download_ranges <- tail;
 (* If we have no more range to receive, disconnect *)
-                if d.download_ranges = [] then raise Exit;
+                if d.download_ranges = [] then
+                  raise Exit;
                 gconn.gconn_handler <- HttpHeader (client_parse_header c);
           end)
 
   with e ->
-      if !verbose_hidden_errors then
+      if !verbose_unknown_messages then
         begin
-          lprintf_nl () "Exception %s in client_parse_header" (Printexc2.to_string e);
+          lprintf_nl "Exception %s in client_parse_header" (Printexc2.to_string e);
           AnyEndian.dump header
         end;
       disconnect_client c (Closed_for_exception e);
@@ -379,30 +362,33 @@ let http_set_sock_handler c sock =
 (*                                                                       *)
 (*************************************************************************)
 
-let http_check_size url start_download_file =
+let http_check_size file url start_download_file =
   let module H = Http_client in
   let r = {
       H.basic_request with
       H.req_url = url;
       H.req_proxy = !CommonOptions.http_proxy;
       H.req_request = H.HEAD;
-      H.req_user_agent = user_agent;
+      H.req_user_agent = get_user_agent ();
     } in
 
-  H.whead r (fun headers ->
-      if !verbose then lprintf_nl () "RECEIVED HEADERS";
+  H.whead2 r (fun headers ->
       let content_length = ref None in
       List.iter (fun (name, content) ->
           if String.lowercase name = "content-length" then
-            try
-              content_length := Some (Int64.of_string content)
-            with _ ->
-                lprintf_nl () "bad content length [%s]" content;
+        try content_length := Some (Int64.of_string content)
+        with _ -> lprintf_nl "bad content length [%s]" content;
       ) headers;
-      match !content_length with
+    (match !content_length with
         None -> failwith "Unable to start download (HEAD failed)"
-      | Some result_size ->
-          start_download_file result_size)
+    | Some result_size -> start_download_file result_size);
+  )
+  (fun c ->
+    match c with 
+     x when x < 200 || x > 299 -> 
+       pause_for_cause file (string_of_int x);
+    | _ -> ()
+  )
 
 (*************************************************************************)
 (*                                                                       *)
@@ -426,7 +412,6 @@ by the bandwidth manager... 2004/02/03: Normally, not true anymore, it should no
   even in this case... *)
 
         | CONNECTED ->
-          if !verbose then lprintf_nl () "CONNECTED !!! Asking for range...";
           f sock
         | _ -> ()
     )
