@@ -47,8 +47,11 @@ open Gettext
 let _s x = _s "FileTPInteractive" x
 let _b x = _b "FileTPInteractive" x
 
+module VB = VerificationBitmap
+
 let _ =
   network.op_network_connected <- (fun _ -> true);
+  network.op_network_ports <- (fun _ -> []);
   network.op_network_connected_servers <- (fun _ -> [])
 
 
@@ -66,42 +69,40 @@ let _ =
 
 module P = GuiTypes
 
+
+let clean_stop file =
+      CommonSwarming.remove_swarmer file.file_swarmer;
+      file.file_swarmer <- None;
+  List.iter (fun c ->
+    c.client_downloads <- List.filter 
+      (fun cd -> cd.download_file.file_id <> file.file_id) c.client_downloads;
+    if (List.length c.client_downloads) == 0 then begin
+      FileTPClients.disconnect_client c Closed_by_user;
+      client_remove c;
+    end;
+  ) file.file_clients
+
 let _ =
   file_ops.op_file_cancel <- (fun file ->
-      Int64Swarmer.remove_swarmer file.file_swarmer;
-      file.file_swarmer <- None;
+      clean_stop file;
       remove_file file;
   );
   file_ops.op_file_info <- (fun file ->
-      {
+      { (impl_file_info file.file_file) with
         P.file_fields = P.Fields_file_info.all;
-        P.file_comment = file_comment (as_file file);
-        P.file_name = file_best_name file;
-        P.file_num = (file_num file);
         P.file_network = network.network_num;
-        P.file_names = file.file_filenames;
-        P.file_md4 = Md4.null;
-        P.file_size = file_size file;
-        P.file_downloaded = file_downloaded file;
-        P.file_all_sources = 0;
-        P.file_active_sources = 0;
-        P.file_state = file_state file;
-        P.file_sources = None;
-        P.file_download_rate = file_download_rate file.file_file;
+        P.file_all_sources = (List.length file.file_clients);
+        P.file_active_sources = (List.length file.file_clients);
         P.file_chunks = (match file.file_swarmer with
-          None -> "" | Some swarmer ->
-            Int64Swarmer.verified_bitmap swarmer);
+        | None -> None 
+	| Some swarmer ->
+            Some (CommonSwarming.chunks_verified_bitmap swarmer));
         P.file_availability =
         [network.network_num,(match file.file_swarmer with
           None -> "" | Some swarmer ->
-            Int64Swarmer.availability swarmer)];
+            CommonSwarming.chunks_availability swarmer)];
         P.file_format = FormatNotComputed 0;
-        P.file_chunks_age = [|0|];
-        P.file_age = file_age file;
-        P.file_last_seen = BasicSocket.last_time ();
-        P.file_priority = file_priority (as_file file);
         P.file_uids = [Uid.create (FileTP file.file_id)];
-        P.file_sub_files = [];
       }
   )
 
@@ -111,28 +112,18 @@ let string_of_client_addr c = c.client_hostname
 
 let _ =
   client_ops.op_client_info <- (fun c ->
-      {
+      { (impl_client_info c.client_client) with
         P.client_network = network.network_num;
         P.client_kind = Known_location (Ip.from_name c.client_hostname,
           c.client_port);
         P.client_state = client_state (as_client c);
         P.client_type = client_type c;
-        P.client_tags = [];
         P.client_name = (Printf.sprintf "%s:%d"
           c.client_hostname c.client_port);
-        P.client_files = None;
         P.client_num = (client_num (as_client c));
-        P.client_rating = 0;
-        P.client_chat_port = 0 ;
         P.client_connect_time = BasicSocket.last_time ();
-        P.client_software = "";
-        P.client_release = "";
-        P.client_emulemod = "";
-        P.client_downloaded = zero;
-        P.client_uploaded = zero;
-        P.client_upload = None;
-        P.client_sui_verified = None;
-(*		P.client_sock_addr = (; *)
+        P.client_software = c.client_software;
+        P.client_downloaded = c.client_downloaded;
       }
   );
     client_ops.op_client_bprint <- (fun c buf ->
@@ -149,7 +140,7 @@ let _ =
         let cinfo = client_info cc in
         client_print cc o;
         Printf.bprintf buf "client: %s downloaded: %s uploaded: %s"
-          "fT" (* cinfo.GuiTypes.client_software *)
+          cinfo.GuiTypes.client_software
           (Int64.to_string cinfo.GuiTypes.client_downloaded)
         (Int64.to_string cinfo.GuiTypes.client_uploaded);
         Printf.bprintf buf "\nfilename: %s\n\n" info.GuiTypes.file_name;
@@ -159,6 +150,13 @@ let _ =
         let buf = o.conn_buf in
         let cc = as_client c in
         let cinfo = client_info cc in
+	let client_ip, client_port =
+	  match cinfo.GuiTypes.client_kind with
+	    Indirect_location (_, _, ip, port)
+	  | Known_location (ip, port) -> ip, port
+	in
+        let ccode,cname = Geoip.get_country (fst (client_ip,client_port)) in
+
         Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\"
     onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
 
@@ -173,8 +171,8 @@ let _ =
           ((string_of_connection_state (client_state cc)), "sr",
             (short_string_of_connection_state (client_state cc)));
           ("", "sr", cinfo.GuiTypes.client_name);
-          ("", "sr", "TP"); (* cinfo.GuiTypes.client_software *)
-          ("", "sr", ""); (* cinfo.GuiTypes.client_release *)
+          ("", "sr", cinfo.GuiTypes.client_software);
+          ("", "sr", cinfo.GuiTypes.client_release);
           ] @
           (if !show_emulemods_column then [("", "sr", "")] else [])
           @ [
@@ -183,80 +181,15 @@ let _ =
               (((last_time ()) - cinfo.GuiTypes.client_connect_time) / 60));
           ("", "sr", "D");
           ("", "sr", "N");
-          ("", "sr", (string_of_client_addr c));
+          ("", "sr", Printf.sprintf "%s:%d" (Ip.to_string client_ip) client_port);
+          ] @ (if !Geoip.active then [(cname, "sr", ccode)] else []) @ [
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
         true
     )
 
-(* As in bittorrent: make an initial connection just to know the complete
-  size of the file and disconnect immediatly after. *)
-
-  (*
-let rec start_download_file_from_mirror proto file url u result_size =
-  (*
-  lprintf "RECEIVED HEADERS\n";
-  let content_length = ref None in
-  List.iter (fun (name, content) ->
-      if String.lowercase name = "content-length" then
-        try
-          content_length := Some (Int64.of_string content)
-        with _ ->
-            lprintf "bad content length [%s]\n" content;
-  ) headers;
-  match !content_length with
-    None -> failwith "Unable to mirror download (HEAD failed)"
-  | Some result_size -> *)
-      if !verbose then
-        lprintf_nl "[FILETP]: STARTING DOWNLOAD WITH SIZE %Ld" result_size;
-      if file_size file <> result_size then
-        if !verbose then
-          lprintf_nl "[FILETP]: Error, unable to mirror download (files have different sizes)"
-      else
-      let client_hostname = url.Url.server in
-      let client_port = url.Url.port in
-      let c = new_client proto client_hostname client_port in
-      add_download file c url.Url.full_file referer.Url.full_file;
-      FileTPClients.get_file_from_source c file; 
-      ()  
-
-let test_mirrors file urls =
-  List.iter (fun url ->
-      try
-        let u = Url.of_string url in
-        let proto = match u.Url.proto with
-          | "http" -> FileTPHTTP.proto
-          | "ftp" -> FileTPFTP.proto
-          | "ssh" -> FileTPSSH.proto
-          | s -> failwith
-              (Printf.sprintf "Unknown URL protocol [%s]" s)
-        in
-        proto.proto_check_size u url
-          (start_download_file_from_mirror proto file)
-        (*
-        let module H = Http_client in
-        let r = {
-            H.basic_request with
-            H.req_url = u;
-            H.req_proxy = !CommonOptions.http_proxy;
-            H.req_request = H.HEAD;
-            H.req_user_agent = user_agent;
-          } in
-
-H.whead r (start_download_file_from_mirror file u)
-  *)
-      with _ -> ()) urls
-
-let
-let download_file_from_mirror file url =
-  test_mirrors file [url];
-  find_mirrors file url
-    *)
-
-
-
-let rec download_file_from_mirror file u r =
+let rec download_file_from_mirror file u referer =
 
   let proto = match u.Url.proto with
     | "http" -> FileTPHTTP.proto
@@ -268,8 +201,8 @@ let rec download_file_from_mirror file u r =
 
   let client_hostname = u.Url.server in
   let client_port = u.Url.port in
-  let c = new_client proto client_hostname client_port in
-  add_download file c u r;
+  let c = new_client proto client_hostname client_port referer in
+  add_download file c u;
   FileTPClients.get_file_from_source c file; 
   ()
 
@@ -293,7 +226,7 @@ and find_mirrors file u =
           let suffix = String.sub url namelen (urllen - namelen) in
           List.iter (fun name ->
               download_file_from_mirror file (Url.of_string
-                (name ^ suffix)) (Url.of_string name)) mirrors
+                (name ^ suffix)) name) mirrors
         else
           iter2 mirrors tail
   in
@@ -303,7 +236,6 @@ let previous_url = ref ""
   
 let download_file url referer = 
   let u = Url.of_string url in
-  let r = Url.of_string referer in
 
   if List.mem u !!old_files && !previous_url <> url then begin
       previous_url := url;
@@ -313,22 +245,20 @@ let download_file url referer =
   let file = new_file (Md4.random ()) u.Url.full_file zero in
   
   if !verbose then
-    lprintf_nl "[FILETP]: DOWNLOAD FILE %s" (file_best_name  file); 
+    lprintf_nl "Started new download: %s from %s" (file_best_name file) url; 
   if not (List.memq file !current_files) then begin
       current_files := file :: !current_files;
     end;
   
-  download_file_from_mirror file u r;
+  download_file_from_mirror file u referer;
   find_mirrors file u
 
 (* I think this is a real bad idea, we should check this by ensuring that the
    bt-url-handler is called first. *)
 let is_http_torrent headers url =
   let ext = String.lowercase (Filename2.last_extension url) in
-  if ext = ".torrent" || ext = ".tor"
+  ext = ".torrent" || ext = ".tor"
      || (String2.contains headers "content-type application/x-bittorrent")
-    then true
-  else false
 
 let get_regexp_int text r =
   ignore (Str.search_forward r text 0);
@@ -346,35 +276,29 @@ let get_regexp_string text r =
    It returns true if this file can be handled by fileTP,
    and false otherwise.
  *)
-let rec op_network_parse_url url =
-  if !verbose then
-    lprintf_nl "[FILETP] op_network_parse_url";
+let op_network_parse_url url =
   let location_regexp = "Location: \\(.*\\)" in
   let real_url = get_regexp_string url (Str.regexp location_regexp) in
-  if !verbose then
-    lprintf "real url: %s\n" real_url;
-  if (is_http_torrent url real_url) then false
+  if (is_http_torrent url real_url) && !!enable_bittorrent then
+    "this seems to be a torrent, ignoring...", false
   else
     if (String2.check_prefix real_url "http://") then (
-      if !verbose then
-        lprintf_nl "[FILETP]: http download";
       let length_regexp = "Content-Length: \\(.*\\)" in
        try let length = get_regexp_int url (Str.regexp length_regexp) in
          if (length > 0) then begin
-           download_file real_url ""; true
+           download_file real_url ""; "started FileTP download", true
          end
-         else raise Not_found
+         else "can not parse Content-Length", false
        with Not_found -> 
-           if !verbose then
-             lprintf_nl "[FILETP]: Unknown file length. Use a web browser";
-           false
-    )
-    else if (String2.check_prefix url "ftp://") || (String2.check_prefix url "ssh://") then (
-      download_file url "";
-      true
+           "Unknown file length. Use a web browser", false
     )
+    else 
+      if (String2.check_prefix real_url "ftp://") ||
+         (String2.check_prefix real_url "ssh://") then (
+      download_file real_url "";
+      "started FileTP download", true)
     else
-      false
+      "invalid URL", false
 
 let _ =
   network.op_network_parse_url <- op_network_parse_url
@@ -397,16 +321,23 @@ let commands = [
           Printf.bprintf buf "download started";
         _s ""
         with Not_found ->
-            if !verbose then
-              lprintf_nl "[FILETP]: Not enough parameters";
             let buf = o.conn_buf in
             if o.conn_output = HTML then
               html_mods_table_one_row buf "serversTable" "servers" [
                 ("", "srh", "Not enough parameters"); ]
             else
               Printf.bprintf buf "Not enough parameters";
-            _s ""  
-    ), " <url> <referer> :\t\t\t\tstart downloading this URL";
+            _s ""
+	| e ->
+	    let error = Printexc2.to_string e in
+            let buf = o.conn_buf in
+            if o.conn_output = HTML then
+              html_mods_table_one_row buf "serversTable" "servers" [
+                ("", "srh", error); ]
+            else
+              Printf.bprintf buf "%s" error;
+            _s ""
+    ), "<url> <referer> :\t\t\tstart downloading this URL";
 
     "mirror", "Network/FileTP", Arg_multiple (fun args o ->
         try
@@ -419,9 +350,8 @@ let commands = [
           | _ -> raise Not_found);
 
           if !verbose then
-            lprintf_nl "[FILETP]: MIRROR [%s] [%s]" !num !url;
+            lprintf_nl "MIRROR [%s] [%s]" !num !url;
           let u = Url.of_string !url in
-          let r = Url.of_string !referer in
 
           if List.mem u !!old_files && !previous_url <> !url then begin
               previous_url := !url;
@@ -430,13 +360,13 @@ let commands = [
           let num = int_of_string !num in
           Hashtbl.iter (fun _ file ->
               if !verbose then
-                lprintf_nl "[FILETP]: COMPARE %d/%d" (file_num file) num;
+                lprintf_nl "COMPARE %d/%d" (file_num file) num;
               if file_num file = num then begin
                   if !verbose then
-                    lprintf_nl "[FILETP]: Try HEAD from mirror";
+                    lprintf_nl "Try HEAD from mirror";
 
 
-                  download_file_from_mirror file u r;
+                  download_file_from_mirror file u !referer;
                   find_mirrors file u;
 
                   raise Exit
@@ -460,7 +390,7 @@ let commands = [
             _s ""
         | Not_found ->
             if !verbose then
-              lprintf_nl "[FILETP]: Not enough parameters";
+              lprintf_nl "Not enough parameters";
             let buf = o.conn_buf in
             if o.conn_output = HTML then
               html_mods_table_one_row buf "serversTable" "servers" [
@@ -468,17 +398,27 @@ let commands = [
             else
               Printf.bprintf buf "Not enough parameters";
             _s ""        
-    ), " <num> <url> <referer> :\t\t\tadd url as mirror for num";
+    ), "<num> <url> <referer> :\t\tadd URL as mirror for num";
     ]
 
 let _ =
   CommonNetwork.register_commands commands;
-  (* Shut up "Network.share not implemented by FileTP" *)
   network.op_network_share <- (fun fullname codedname size -> ());
-  (* Same with Network.search and Network.forget_search... *)
   network.op_network_search <- (fun ss buf -> ());
   network.op_network_download <- (fun r -> dummy_file);
-  file_ops.op_file_commit <- (fun file new_name -> ());
+  file_ops.op_file_commit <- (fun file new_name -> clean_stop file);
+  file_ops.op_file_pause <- (fun file -> 
+    List.iter (fun c ->
+      match c.client_connected_for with
+      | Some s when s.file_id = file.file_id -> 
+          FileTPClients.disconnect_client c Closed_by_user;
+      | _ -> ()
+    ) file.file_clients
+  );
+  file_ops.op_file_resume <- (fun file -> ());
+  file_ops.op_file_print_html <- (fun file buf -> ());
+  network.op_network_close_search <- (fun s -> ());
   network.op_network_forget_search <- (fun s -> ());
-  (* and Network.recover_temp *)
+  network.op_network_connect_servers <- (fun s -> ());
+  network.op_network_reset <- (fun _ -> ());
   network.op_network_recover_temp <- (fun s -> ())
