@@ -91,26 +91,31 @@ let segment_received c num s pos =
         match d.download_uploader with
           None -> assert false
         | Some up ->
+			  
+            let swarmer = CommonSwarming.uploader_swarmer up in
 
-            let swarmer = Int64Swarmer.uploader_swarmer up in
             let old_downloaded =
-              Int64Swarmer.downloaded swarmer in
-            Int64Swarmer.received up
+              CommonSwarming.downloaded swarmer in
+
+            CommonSwarming.received up
               pos s 0 (String.length s);
 
             let new_downloaded =
-              Int64Swarmer.downloaded swarmer in
+              CommonSwarming.downloaded swarmer in
+
+	    c.client_downloaded <- c.client_downloaded ++ (new_downloaded -- old_downloaded);
+            client_must_update (as_client c);
 
             if new_downloaded = file_size file then
               download_finished file;
 
       with e ->
-          lprintf "FT: Exception %s in Int64Swarmer.received\n"
+          lprintf "FT: Exception %s in CommonSwarming.received\n"
             (Printexc2.to_string e)
     end;
     c.client_reconnect <- true;
 (*          List.iter (fun (_,_,r) ->
-              Int64Swarmer.alloc_range r) d.download_ranges; *)
+              CommonSwarming.alloc_range r) d.download_ranges; *)
 (*
 lprintf "READ %Ld\n" (new_downloaded -- old_downloaded);
 lprintf "READ: buf_used %d\n" to_read_int;
@@ -118,7 +123,7 @@ lprintf "READ: buf_used %d\n" to_read_int;
 
     (match d.download_ranges with
       | (xx,yy,r) :: tail when  pos >= xx && pos <= yy ->
-          let (x,y) = Int64Swarmer.range_range r in
+          let (x,y) = CommonSwarming.range_range r in
           lprintf "Remaining: %Ld-%Ld/%Ld-%Ld\n"
             x y xx yy;
           if x = y then begin
@@ -134,7 +139,7 @@ lprintf "READ: buf_used %d\n" to_read_int;
               lprintf "\n ********** RANGE DOWNLOADED  ********** \n";
             end
       | (xx,yy,r) :: tail ->
-          let (x,y) = Int64Swarmer.range_range r in
+          let (x,y) = CommonSwarming.range_range r in
           lprintf "Bad range (%Ld): %Ld-%Ld/%Ld-%Ld\n"  pos
             x y xx yy;
       | [] ->
@@ -169,7 +174,7 @@ let ssh_set_sock_handler c sock =
 (*                                                                       *)
 (*************************************************************************)
 
-let ssh_check_size url start_download_file =
+let ssh_check_size file url start_download_file =
   let token = create_token unlimited_connection_manager in
   let shell, args = shell_command url.Url.server in
   lprintf "SHELL: ";
@@ -233,7 +238,6 @@ type segment =
 | SegmentX of int * int64 * int * int * string
 
 let ssh_connect token c f =
-  let ip = c.client_hostname in
   let shell, args = shell_command c.client_hostname in
   let sock, pid = exec_command token  shell args (fun _ _ -> ());
   in
