@@ -90,7 +90,7 @@ let clean_sources () =
 let download_finished file = 
   if List.memq file !current_files then begin
       file_completed (as_file file);
-      Int64Swarmer.remove_swarmer file.file_swarmer;
+      CommonSwarming.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
 (* TODO: maybe this remove file may have a bad effect on sharing *)
       GnutellaGlobals.remove_file file;
@@ -107,7 +107,7 @@ let download_finished file =
 (*************************************************************************)
     
 let check_finished swarmer file =
-  if Int64Swarmer.check_finished swarmer then
+  if CommonSwarming.check_finished swarmer then
     download_finished file
   
 (*************************************************************************)
@@ -243,7 +243,7 @@ save it on disk in the next version. *)
               | Some swarmer ->
                   if !verbose_msg_clients then
                     lprintf "[TTR] set_verifier\n";
-                  Int64Swarmer.set_verifier swarmer 
+                  CommonSwarming.set_verifier swarmer 
                     (Verification (Array.map (fun ttr -> TigerTree ttr) array))
             );
             
@@ -264,7 +264,6 @@ and read_some d c counter_pos b to_read_int =
   let up = match d.download_uploader with
       None -> assert false
     | Some up -> up in
-  let swarmer = Int64Swarmer.uploader_swarmer up in
   
   let file = d.download_file in
   
@@ -273,31 +272,16 @@ and read_some d c counter_pos b to_read_int =
       raise Exit;
     end;
 
-(*
-  lprintf "CHUNK: %s\n" 
-          (String.escaped (String.sub b.buf b.pos to_read_int)); *)
-  let old_downloaded = 
-    Int64Swarmer.downloaded swarmer in
-  
   begin
     try
-      Int64Swarmer.received up
+      CommonSwarming.received up
         counter_pos b.buf b.pos to_read_int;
     with e -> 
-        lprintf "FT: Exception %s in Int64Swarmer.received\n"
+        lprintf "FT: Exception %s in CommonSwarming.received\n"
           (Printexc2.to_string e);
 (* TODO: we should pause the download !!! *)
   end;
-  c.client_reconnect <- true;
-  
-  let new_downloaded = 
-    Int64Swarmer.downloaded swarmer in
-  
-(*
-lprintf "READ %Ld\n" (new_downloaded -- old_downloaded);
-lprintf "READ: buf_used %d\n" to_read_int;
-  *)
-  ()
+  c.client_reconnect <- true
 
 (*************************************************************************)
 (*                                                                       *)
@@ -325,8 +309,6 @@ and client_parse_header c gconn sock (first_line, headers) =
 
 (* The reply should be  "HTTP/1.1 200 OK" *)
     let space_pos = String.index first_line ' ' in
-    let slash_pos = String.index first_line '/' in
-    let proto = String.sub first_line (slash_pos+1) (space_pos - slash_pos) in
     let code = String.sub first_line (space_pos+1) 3 in
     let code = int_of_string code in
     
@@ -399,7 +381,7 @@ iff Content-Length = Requested Length and
                 (start_pos, end_pos)
           with _ ->  *)
 (* A bit dangerous, no ??? *)
-          if !verbose_hidden_errors then
+          if !verbose_unknown_messages then
             begin
               lprintf "[GDO] ERROR: Could not find/parse range header (exception %s), disconnect\n" 
                 (Printexc2.to_string e);
@@ -456,7 +438,7 @@ iff Content-Length = Requested Length and
           end)
   
   with e ->
-      if !verbose_hidden_errors then
+      if !verbose_unknown_messages then
         begin
           lprintf "[GDO] Exception %s in client_parse_header\n" (Printexc2.to_string e);
           print_head first_line headers;
@@ -495,9 +477,9 @@ and get_from_client sock (c: client) =
             let up = match d.download_uploader with
                 None -> 
                   let chunks = [ Int64.zero, file_size file ] in
-                  let up = Int64Swarmer.register_uploader swarmer 
+                  let up = CommonSwarming.register_uploader swarmer 
                       (as_client c)
-                    (AvailableRanges chunks) in
+                    (AvailableIntervals chunks) in
                   d.download_uploader <- Some up;
                   up                
               
@@ -518,7 +500,7 @@ and get_from_client sock (c: client) =
                 List.iter (fun req ->
                     match req with
                       RANGEReq (x,y,r) ->
-(*              let (x,y) = Int64Swarmer.range_range r               in *)
+(*              let (x,y) = CommonSwarming.range_range r               in *)
                         lprintf "%Ld-%Ld " x y
                     | HEADReq -> 
                         lprintf "HEAD"
@@ -527,38 +509,41 @@ and get_from_client sock (c: client) =
                 ) d.download_ranges;
                 
                 lprintf "\n  Current blocks: ";
-(*          List.iter (fun b -> Int64Swarmer.print_block b) d.download_blocks; *)
+(*          List.iter (fun b -> CommonSwarming.print_block b.up_block) d.download_blocks; *)
                 lprintf "\n\nFinding Range: \n";
               end;
             let range = 
               try
                 let rec iter () =
-                  match d.download_block with
-                    None -> 
+                  match d.download_blocks with
+                  | [] -> 
                       if !verbose_swarming then lprintf "No block\n";
-                      (try Int64Swarmer.verify_one_chunk swarmer with _ -> ());
-                      let b = Int64Swarmer.find_block up in
+                      (try CommonSwarming.verify_one_chunk swarmer with _ -> ());
+                      let _chunk, blocks = CommonSwarming.find_blocks up in
                       
                       if !verbose_swarming then begin
-                          lprintf "GOT BLOCK:\n";
-                          Int64Swarmer.print_uploaders swarmer;
-                          lprintf "Block Found: "; Int64Swarmer.print_block b;
+                          lprintf "GOT BLOCKS:\n";
+                          CommonSwarming.print_uploaders swarmer;
+                          lprintf "Blocks Found: "; 
+			  List.iter (fun b ->
+                            CommonSwarming.print_block b.up_block) blocks;
                         end;
                       
-                      d.download_block <- Some b;
+                      d.download_blocks <- blocks;
                       iter ()
-                  | Some b ->
+                  | blocks ->
 (*
                 if !verbose_swarming then begin
-                    lprintf "Current Block: "; Int64Swarmer.print_block b;
+                    lprintf "Current Block: "; CommonSwarming.print_block b;
 end;
   *)
                       try
-                        let (x,y,r) = Int64Swarmer.find_range up in 
+                        let (x,y,r) = 
+			  CommonSwarming.find_range up (Int64.of_int (256 * 1024)) in 
                         
                         if !verbose_swarming then begin
                             lprintf "GOT RANGE:\n";
-                            Int64Swarmer.print_uploaders swarmer;
+                            CommonSwarming.print_uploaders swarmer;
                           end;
                         d.download_ranges <- d.download_ranges @ 
                           [RANGEReq (x,y,r)];
@@ -566,12 +551,12 @@ end;
                       with Not_found ->
                           if !verbose_swarming then 
                             lprintf "Could not find range in current block\n";
-                          d.download_block <- None;
+                          d.download_blocks <- [];
                           iter ()
                 in
                 iter ()
               with Not_found -> 
-		if !verbose_hidden_errors then
+    if !verbose_unknown_messages then
                   lprintf_nl "Unable to get a block !!";
                   check_finished swarmer file;
                   raise Not_found
@@ -605,7 +590,7 @@ and disconnect_client c r =
   | Connection sock -> 
       (try
           if !verbose_msg_clients then
-              lprintf_nl "Disconnected from source for %s\n" (string_of_reason r); 
+              lprintf_nl "Disconnected from source for %s" (string_of_reason r); 
           c.client_requests <- [];
           connection_failed c.client_connection_control;
           set_client_disconnected c r;
@@ -621,8 +606,8 @@ and disconnect_client c r =
                 None -> ()
               | Some up ->
                   d.download_uploader <- None;
-                  Int64Swarmer.unregister_uploader up;
-                  d.download_block <- None;
+                  CommonSwarming.unregister_uploader up;
+                  d.download_blocks <- [];
                   d.download_ranges <- [];
           ) c.client_downloads;
           begin
@@ -681,7 +666,7 @@ should clearly be called). *)
       begin
         
         match c.client_user.user_kind with
-          Indirect_location (_, uid) -> 
+          Indirect_location (_, uid, _, _) -> 
             GnutellaProto.ask_for_push uid 
         | Known_location (ip, port) ->
             let token =
@@ -765,19 +750,22 @@ let push_handler cc gconn sock (first_line, headers) =
         lprintf "PARSING GIV HEADER\n"; 
       end;
     let colon_pos = String.index first_line ':' in
-    let slash_pos = String.index first_line '/' in
     let uid = Md4.of_string (String.sub first_line (colon_pos+1) 32) in
     let index = int_of_string (String.sub first_line 4 (colon_pos-4)) in
     if !verbose_msg_clients then begin
         lprintf "PARSED\n";
       end;
     let c = try
-        Hashtbl.find clients_by_uid (Indirect_location ("", uid)) 
+        Hashtbl.find clients_by_uid (Indirect_location ("", uid, ip, port)) 
       with _ ->
           try
             Hashtbl.find clients_by_uid (Known_location (ip,port))
           with _ ->
-              new_client  (Indirect_location ("", uid)) 
+              let c = new_client  (Indirect_location ("", uid, ip, port)) in
+              if String.length c.client_user.user_nick == 0 then
+                c.client_user.user_nick <- (Md4.to_string uid);
+              c
+            
     in
     c.client_host <- Some (ip, port);
     match c.client_sock with
@@ -876,7 +864,7 @@ let read_request url headers gconn sock =
         begin
           gconn.gconn_client_info_sent <- true;
           ("Remote-IP", Ip.to_string (fst (peer_addr sock)) ) ::
-          ("Server", user_agent) :: headers
+          ("Server", get_user_agent ()) :: headers
         end
     in
     make_http_header s headers
@@ -1110,7 +1098,7 @@ let listen () =
 (*************************************************************************)
       
 let push_connection guid index ip port =
-  let token =
+  let _ =
     add_pending_connection connection_manager (fun token ->
         let sh =Hashtbl.find shareds_by_id index in
         let sock = connect token "gnutella download" 
@@ -1143,7 +1131,7 @@ let push_connection guid index ip port =
         write_string sock 
           (Printf.sprintf "GIV %d:%s/%s\n\n" 
             index (Md4.to_string guid) sh.shared_codedname)
-    )    
+    )
 
   in
   ()
