@@ -47,6 +47,8 @@ open GnutellaGlobals
 open GnutellaComplexOptions
 open GnutellaProtocol
 
+module VB = VerificationBitmap
+
 (* Don't share files greater than 10 MB on Gnutella and limit to 200 files. 
  Why ? Because we don't store URNs currently, and we don't want mldonkey
  to compute hashes for hours at startup *)
@@ -64,7 +66,7 @@ type query =
 | QHasMinVal of string * int64
 | QHasMaxVal of string * int64
 | QNone (** temporary, used when no value is available ;
-	   must be simplified before transforming into strings *)
+     must be simplified before transforming into strings *)
 *)
 
 let recover_file file =
@@ -94,11 +96,17 @@ let find_search s =
 
 let result_download r =
   if !verbose then
-    lprintf ".............. result_download ..............\n";
+    lprintf_nl "result_download";
   let rec iter uids =
     match uids with
       [] -> raise IgnoreNetwork
     | uid :: tail ->
+        if !verbose then lprintf_nl "UID: %s Accept(BP: %B ED2K: %B MD5: %B)" 
+            (Uid.to_string uid) 
+            GnutellaNetwork.accept_bitprint
+            GnutellaNetwork.accept_ed2kuid
+            GnutellaNetwork.accept_md5ext;
+
         match Uid.to_uid uid with
           Sha1 _ | Bitprint _ when GnutellaNetwork.accept_bitprint ->
             download_file r
@@ -109,45 +117,49 @@ let result_download r =
         | _  -> iter tail
   in
   if !verbose then
-    lprintf "%d uids\n" (List.length r.result_uids);
+    lprintf_nl "%d uids" (List.length r.result_uids);
   iter r.result_uids
 
 let declare_file _ = should_update_shared_files := true
   
 let ask_for_uids sh =
+  if !verbose then lprintf_nl "ask_for_uids Accept(BP: %B ED2K: %B MD5: %B)" 
+     GnutellaNetwork.accept_bitprint
+     GnutellaNetwork.accept_ed2kuid
+     GnutellaNetwork.accept_md5ext;
   let info = IndexedSharedFiles.get_result sh.shared_info in
   if GnutellaNetwork.accept_ed2kuid then begin
       if !verbose then
-        lprintf "Ask for ED2K uid\n";
+        lprintf_nl "Ask for ED2K uid";
       CommonUploads.ask_for_uid sh ED2K (fun sh uid ->
           let uid = Uid.to_string uid in
           declare_word uid;
           declare_file ();
           if !verbose then
-            lprintf "Ed2k uid available (size %d)\n" 
+            lprintf_nl "Ed2k uid available (size %d)" 
             (Array.length info.CommonUploads.shared_md4s);
       );
     end;
   if GnutellaNetwork.accept_md5ext then begin
       if !verbose then
-        lprintf "Ask for MD5EXT uid\n";
+        lprintf_nl "Ask for MD5EXT uid";
       CommonUploads.ask_for_uid sh MD5EXT (fun sh uid ->
           let uid = Uid.to_string uid in
           declare_word uid;
           declare_file ();          
           if !verbose then
-            lprintf "Md5ext uid available: %s\n"  uid
+            lprintf_nl "Md5ext uid available: %s"  uid
       );
     end;
 
   if GnutellaNetwork.accept_bitprint then begin
       if !verbose then
-        lprintf "Ask for BITPRINT uid\n";
+        lprintf_nl "Ask for BITPRINT uid";
       CommonUploads.ask_for_uid sh BITPRINT (fun sh uid ->
           declare_word uid;
           declare_file ();
           if !verbose then
-            lprintf "Bitprint tree available (size %d)\n" 
+            lprintf_nl "Bitprint tree available (size %d)" 
               (Array.length info.CommonUploads.shared_tiger);
           ()
       );
@@ -156,13 +168,13 @@ let ask_for_uids sh =
           declare_word uid;
           declare_file ();
           if !verbose then
-            lprintf "Tiger tree available (size %d)\n" 
+            lprintf_nl "Tiger tree available (size %d)" 
               (Array.length info.CommonUploads.shared_tiger);
       );
       CommonUploads.ask_for_uid sh SHA1 (fun sh uid -> 
           let uid = Uid.to_string uid in
           if !verbose_share then
-            lprintf "Could share urn: %s\n" uid;
+            lprintf_nl "Could share urn: %s" uid;
 (* TODO : enter this shared file in the QRT *)
           declare_word uid;
           declare_file ();          
@@ -199,9 +211,14 @@ that we can reuse queries *)
   network.op_network_forget_search <- (fun s ->
       Hashtbl.remove searches_by_uid (find_search s).search_uid  
   );
+  network.op_network_reset <- (fun _ -> ());
   network.op_network_connected <- (fun _ ->
       !connected_servers <> []
   );
+  network.op_network_ports <- (fun _ ->
+    [
+    !!client_port, "client_port TCP+UDP";
+    ]);
   network.op_network_share <- (fun fullname codedname size ->
       (*
       lprintf "*************** op_network_share %s\n
@@ -242,7 +259,7 @@ let _ =
       match file.file_swarmer with
         None -> [CommonFile.as_file impl]
       | Some swarmer ->
-          Int64Swarmer.subfiles swarmer)
+          CommonSwarming.subfiles swarmer)
   ;
   file_ops.op_file_active_sources <- file_ops.op_file_all_sources;
   file_ops.op_file_check <- (fun file ->
@@ -252,7 +269,7 @@ let _ =
           match file.file_ttr with
             None -> failwith "No TTR for verification"
           | Some ttt ->
-              Int64Swarmer.verify_all_chunks swarmer true  
+              CommonSwarming.verify_all_chunks_immediately swarmer
   );  
   
   file_ops.op_file_recover <- (fun file ->
@@ -271,73 +288,53 @@ let _ =
 module P = GuiTypes
   
 let _ =
+  file_ops.op_file_print_html <- (fun file buf -> ());
   file_ops.op_file_cancel <- (fun file ->
-      Int64Swarmer.remove_swarmer file.file_swarmer;
+      CommonSwarming.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
       remove_file file;
       GnutellaProto.cancel_recover_files file
   );
   file_ops.op_file_commit <- (fun file new_name ->
-      Int64Swarmer.remove_swarmer file.file_swarmer;
+      CommonSwarming.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
   );
   file_ops.op_file_info <- (fun file ->
-      {
-        P.file_fields = P.Fields_file_info.all;
-        
-        P.file_comment = file_comment (as_file file);
-        P.file_name = file.file_name;
-        P.file_num = (file_num file);
+      { (impl_file_info file.file_file) with
         P.file_network = network.network_num;
-        P.file_names = [file.file_name, P.noips()];
-        P.file_md4 = Md4.null;
-        P.file_size = file_size file;
-        P.file_downloaded = file_downloaded file;
-        P.file_all_sources = 0;
-        P.file_active_sources = 0;
-        P.file_state = file_state file;
-        P.file_sources = None;
-        P.file_download_rate = file_download_rate file.file_file;
         
          P.file_chunks = (match file.file_swarmer with
-             None -> "" | Some swarmer ->
-               Int64Swarmer.verified_bitmap swarmer);
+         | None -> None 
+	 | Some swarmer -> 
+	     Some (CommonSwarming.chunks_verified_bitmap swarmer));
         P.file_availability =   [network.network_num,
            (match file.file_swarmer with
            None -> "" | Some swarmer ->
-                 Int64Swarmer.availability swarmer)];
+                 CommonSwarming.chunks_availability swarmer)];
         
-        P.file_format = FormatNotComputed 0;
         P.file_chunks_age = [|0|];
-        P.file_age = file_age file;
         P.file_last_seen = BasicSocket.last_time ();
-        P.file_priority = file_priority (as_file file);
         P.file_uids = file.file_uids;
-        P.file_sub_files = [];
       }    
   )
   
 let _ =
   server_ops.op_server_info <- (fun s ->
       if !!enable_gnutella then
-        {
-          P.server_num = (server_num s);
+        { (impl_server_info s.server_server) with
+
           P.server_network = network.network_num;
           P.server_addr = s.server_host.host_addr;
           P.server_port = s.server_host.host_port;
-	  P.server_realport = 0;
-          P.server_score = 0;
-          P.server_tags = [];
           P.server_nusers = s.server_nusers;
+          P.server_max_users = s.server_maxnusers;
           P.server_nfiles = s.server_nfiles;
-          P.server_state = server_state s;
           P.server_name = s.server_agent;
-          P.server_description = "";
-          P.server_users = None;
-          P.server_banner = "";
-          P.server_preferred = false;
-          } else
-        raise Not_found
+          P.server_version = s.server_vendor;
+          P.server_description = s.server_description;
+
+        } 
+        else raise Not_found
   );
   server_ops.op_server_connect <- (fun s ->
       GnutellaServers.connect_server s.server_host);
@@ -378,8 +375,8 @@ let _ =
       | "gnut://" :: "server" :: ip :: port :: _ ->  
           let ip = Ip.addr_of_string ip in
           let port = int_of_string port in
-          let s = new_server ip port in
-          true
+          let _ = new_server ip port in
+          "added Gnutella server", true
       | "gnut://" :: "friend" :: uid :: ip :: port :: _ ->  
           let ip = Ip.of_string ip in
           let port = int_of_string port in
@@ -387,12 +384,12 @@ let _ =
           let c = new_client (Known_location (ip, port)) in
           c.client_user.user_uid <- md4;
           friend_add (as_client c);
-          true
+          "added Gnutella friend", true
       | "gnut://" :: "friend" :: uid :: _ ->
           let md4 = Md4.of_string uid in
-          let c = new_client (Indirect_location ("", md4)) in
+          let c = new_client (Indirect_location ("", md4, Ip.null, 0)) in
           friend_add (as_client c);
-          true
+          "added Gnutella friend", true
       
       | _ -> 
           let (name, size, uids) =
@@ -407,43 +404,26 @@ let _ =
               let r = IndexedResults.get_result rs in
               let file = download_file r in
               CommonInteractive.start_download file;
-              true
+              "started Gnutella download", true
             end
-          else false
+          else "", false
   )
   
-let browse_client c = 
-  lprintf "Gnutella: browse client not implemented\n";
-  ()
-  
 let _ =
   client_ops.op_client_info <- (fun c ->
-      {
+      { (impl_client_info c.client_client) with
+
         P.client_network = network.network_num;
         P.client_kind = c.client_user.user_kind;
-        P.client_state = client_state (as_client c);
         P.client_type = client_type c;
-        P.client_tags = [];
-        P.client_name = c.client_user.user_nick;
-        P.client_files = None;
-        P.client_num = (client_num (as_client c));
-        P.client_rating = 0;
-        P.client_chat_port = 0 ;
-        P.client_connect_time = BasicSocket.last_time ();
-        P.client_software = "";
-        P.client_release = "";
-        P.client_emulemod = "";
-        P.client_downloaded = zero;
-        P.client_uploaded = zero;
-        P.client_upload = None;
-        P.client_sui_verified = None;
-(*        P.client_sock_addr = (match c.client_user.user_kind with
-                        | Known_location (ip,port) -> Ip.to_string ip
-                        | _ -> ""); *)
+        P.client_name = if c.client_user.user_speed > 0 
+                          then Printf.sprintf "%s (%d)" c.client_user.user_nick c.client_user.user_speed
+                          else c.client_user.user_nick;
+        P.client_software = c.client_user.user_software;
+
       }
   );
-  client_ops.op_client_browse <- (fun c immediate ->
-      browse_client c
+  client_ops.op_client_browse <- (fun _ _ -> lprintf_nl "Gnutella: browse client not implemented";
   );
 
     client_ops.op_client_bprint <- (fun c buf ->
@@ -495,6 +475,7 @@ let _ =
           ("", "sr", "D");
           ("", "sr", "N");
           ("", "sr", (string_of_kind cinfo.GuiTypes.client_kind));
+          ] @ (if !Geoip.active then [("?", "sr", "?")] else []) @ [
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
@@ -560,7 +541,7 @@ let commands = [
               (Queue.length waiting_udp_queue);
           end;
         ""
-    ), " :\t\t\t\tprint stats on Gnutella network";
+    ), ":\t\t\t\tprint stats on Gnutella network";
 
     ]
   
