@@ -22,10 +22,7 @@ open CommonOptions
 open Options
 open GnutellaNetwork
   
-let cmd_basedir = Autoconf.current_dir (* will not work on Windows *)
-
-let gnutella_ini = create_options_file (
-    Filename.concat file_basedir GnutellaNetwork.config_file)
+let gnutella_ini = create_options_file GnutellaNetwork.config_file
 
 let gnutella_section = file_section gnutella_ini ["Gnutella"] "Gnutella options"  
 
@@ -43,20 +40,39 @@ let client_port = define_option gnutella_section ["client_port"]
     "The port to bind the client to"
     int_option GnutellaNetwork.port
   
+let gnutella_experimental = define_option gnutella_section
+    ["experimental"]
+    "Activate this option if you want to test experimental features."
+    bool_option false
+
 let gnutella_hostfiles = define_option gnutella_section 
     ["hostfiles"]
   "A list of GWCache urls"
     (list_option string_option)
   [
-    "http://gwebcache.bearshare.net/gcache.php"
+    "http://cache.kicks-ass.net:8000/";
+    "http://g2cache.theg2.net/gwcache/lynnx.asp";
+    "http://galvatron.dyndns.org:59009/gwcache";
+    "http://gcache.sexter.com:8080/gwc/";
+    "http://gwcrab.sarcastro.com:8001";
+    "http://gwebcache.bearshare.net/";
+    "http://gwebcache.bearshare.net/gcache.php";
+    "http://intense.homelinux.coww.xolox.nl/gwebcache/";
+    "http://kisama.ath.cx:8080/";
+    "http://krill.shacknet.nu:20095/gwc";
+    "http://loot.alumnigroup.org/";
+    "http://pokerface.ibiza.bishopston.net:3558/";
+    "http://t.az.is.teh.r0x0r.gtkg.de/";
+    "http://tribaldance.ibiza.bishopston.net:3558/";
+    "http://www.deepnetexplorer.co.uk/webcache/";
   ]
 
 let urlfiles = define_option gnutella_section 
     ["urlfiles"]
-  "A list of GWCache urls"
+  "A list of urlfile urls"
     (list_option string_option)
   [
-    "http://gwebcache.bearshare.net/gcache.php"
+    "http://loot.alumnigroup.org/";
   ]
   
 let redirectors = define_option gnutella_section 
@@ -67,28 +83,22 @@ let redirectors = define_option gnutella_section
 (* (Ip.of_string "64.61.25.171")   *)
   
   
-let commit_in_subdir = define_option gnutella_section ["commit_in_subdir"]
-  "The subdirectory of temp/ where files should be moved to"
-    string_option "Gnutella"
-
-let user_agent = Printf.sprintf "MLDonkey %s" Autoconf.current_version
-
 let max_known_ultrapeers = define_option gnutella_section ["max_known_ultrapeers"]
   "Maximal number of ultrapeers remembered"
-    int_option 100
+    int_option 500
 
 let max_known_peers = define_option gnutella_section ["max_known_peers"]
   "Maximal number of peers remembered"
-  int_option 20
+  int_option GnutellaNetwork.max_known_peers_default
     
 let server_connection_timeout = 
   define_option gnutella_section ["server_connection_timeout"] 
-  "timeout when connecting to a server" float_option 10.
+  "timeout when connecting to a server" float_option 30.
 
 let client_uid = define_option gnutella_section ["client_uid"]
     "The UID of this client" Md4.option (Md4.random ())
   
-  let _	 =
+  let _  =
   option_hook client_uid (fun _ ->
      let s = Md4.direct_to_string !!client_uid in
      s.[8] <- '\255';
@@ -105,24 +115,49 @@ let verbose_servers =
   define_option gnutella_section ["verbose_servers"] 
     "level of verbosity when communicating with servers" int_option 0
     *)
-
-let network_options_prefix = define_option gnutella_section
-    ["options_prefix"] "The prefix which is appended to options names
-    when they are used in the telnet/WEB interfaces"
-    string_option GnutellaNetwork.options_prefix
   
 let max_available_slots = define_option gnutella_section
     ["max_available_slots"] "The maximal number of slots for upload by Gnutella clients"
     int_option 5
   
 let shortname o =
-  Printf.sprintf "%s%s" !!network_options_prefix (shortname o)
+  Printf.sprintf "%s%s" GnutellaNetwork.options_prefix (shortname o)
+
+let sha1_verification_threshold = define_option gnutella_section
+    ["sha1_verification_threshold"]
+  "Under this threshold, files are verified using the SHA1 digest
+(always available), without using the tiger-tree (experimental)."
+    int64_option Int64ops.zero
   
 let deflate_connections = define_option gnutella_section 
   ["deflate_connections"]
   "(only for development tests)"  
     bool_option false
   
+let keep_alive = define_option gnutella_section 
+    ["keep_alive"]
+  "(only for development tests)"  
+    bool_option true
+
+let dont_connect = define_option gnutella_section 
+    ["dont_connect"]
+  "(only for development tests)"  
+    bool_option false
+
+let supernode_enabled = define_option gnutella_section 
+  ["supernode_enabled"]
+  "(only for development tests)"  
+    bool_option false
+
+let incoming_data_log = define_option gnutella_section
+    ["incoming_data_log"]
+    "(only for development tests)"
+    string_option ""
+
+let options_version = define_option gnutella_section ["options_version"]
+    "(internal option)"
+    int_option 0
+  
 let gui_gnutella_options_panel = 
   (*
   define_option gnutella_section ["gui_gnutella_options_panel"]
@@ -135,9 +170,12 @@ let gui_gnutella_options_panel =
 (*    "Max Connected Ultrapeers", shortname max_ultrapeers, "T"; 
     "Max Known Ultrapeers", shortname max_known_ultrapeers, "T";
     "Max Known Peers", shortname max_known_peers, "T";    *)
-    "Commit Downloads In Incoming Subdir", shortname commit_in_subdir, "T";
     "Max Available Slots", shortname max_available_slots, "T";
     
     "          Max Connected Ultrapeers", shortname max_ultrapeers, "T"; 
   ]
-  
+
+let old_files =
+  define_option gnutella_section ["old_files"]
+    "" (list_option (tuple2_option (string_option, int64_option))) []
+
