@@ -22,10 +22,7 @@ open OpennapTypes
 open CommonTypes
 open Options
 
-let cmd_basedir = Autoconf.current_dir (* will not work on Windows *)
-
-let opennap_ini = create_options_file (
-    Filename.concat file_basedir "opennap.ini")
+let opennap_ini = create_options_file "opennap.ini"
 
 let opennap_section = file_section opennap_ini ["Opennap"] "Opennap options"
   
@@ -55,27 +52,22 @@ let use_napigator = define_option opennap_section ["use_napigator"]
 
 let servers_list_url = define_option opennap_section ["servers_list_url"]
     "The URL from which servers list is downloaded"
-    string_option  "http://beta34.napigator.com/servers/?sort=network&dir=asc"
+    string_option  "http://gotnap.com/index.php?loc=list"
   
 let server_connection_timeout = 
   define_option opennap_section ["server_connection_timeout"] 
-  "timeout when connecting to a server" float_option 10.
+  "timeout when connecting to a server" float_option 30.
     
 let max_shared_files = define_option opennap_section ["max_shared_files"]
   "The maximal number of files to share on a server"
     int_option 400
   
-let commit_in_subdir = define_option opennap_section ["commit_in_subdir"]
-  "The subdirectory of temp/ where files should be moved to"
-    string_option "Napster"
-      
-let network_options_prefix = define_option opennap_section
-    ["options_prefix"] "The prefix which is appended to options names
-    when they are used in the telnet/WEB interfaces"
-    string_option "OpenNap-"
-  
+let options_version = define_expert_option opennap_section ["options_version"]
+    "(internal option)"
+    int_option 0
+
 let shortname o =
-  Printf.sprintf "%s%s" !!network_options_prefix (shortname o)
+  Printf.sprintf "OpenNap-%s" (shortname o)
     
 let gui_opennap_options_panel = 
   (*
@@ -91,5 +83,4 @@ let gui_opennap_options_panel =
     "Max Connected Servers (<10)", shortname max_connected_servers, "T";
     "Use Napigator", shortname use_napigator, "B";
     "Napigator List URL", shortname servers_list_url, "T";
-    "Commit Downloads In Incoming Subdir", shortname commit_in_subdir, "T";
   ]
