@@ -103,15 +103,15 @@ let get_int8 s pos =
 let get_int16 s pos =
   let c1 = int_of_char s.[pos] in
   let c2 = int_of_char s.[pos+1] in
-  c1 + c2 * 256
+  c1 lor (c2 lsl 8)
 
 let buf_int8 buf i =
-  Buffer.add_char buf (char_of_int (i land 255))
+  Buffer.add_char buf (char_of_int (i land 0xff))
   
 let buf_int16 buf i =
-  let i = i land 65535 in
-  Buffer.add_char buf (char_of_int (i mod 256));
-  Buffer.add_char buf (char_of_int (i / 256))
+  let i = i land 0xffff in
+  Buffer.add_char buf (char_of_int (i land 0xff));
+  Buffer.add_char buf (char_of_int (i lsr 8))
   
 let dump s =
   let len = String.length s in
@@ -1106,7 +1106,7 @@ section 4.1 for more detailed information).
 open LittleEndian
 
 let server_msg_to_string msg =
-  Buffer.clear buf;
+  Buffer.reset buf;
   buf_int16 buf 0;
   write buf msg;
   let s = Buffer.contents buf in
@@ -1119,7 +1119,7 @@ let server_send sock m =
   write_string sock (server_msg_to_string m)
 
 let server_msg_to_string msg =
-  Buffer.clear buf;
+  Buffer.reset buf;
   buf_int16 buf 0;
   write buf msg;
   let s = Buffer.contents buf in
