@@ -19,6 +19,11 @@
 
 open Autoconf
 
+val syslog_oc : Syslog.t option ref
+
+val lprintf_original_output : out_channel option ref
+val keep_console_output : unit -> bool
+
 val log_time : unit -> string
 val cprintf : (string -> unit) -> ('a, unit, unit) format -> 'a
 (** [cprintf k format arguments] is the same as [printf format arguments],
@@ -26,27 +31,18 @@ val cprintf : (string -> unit) -> ('a, unit, unit) format -> 'a
     result of [k] is then returned as the result of [cprintf]. *)
 
 val lprintf :  ('a, unit, unit) format -> 'a
+val lprintf2 :  string -> ('a, unit, unit) format -> 'a
 val lprintf_nl :  ('a, unit, unit) format -> 'a
-val lprintf_nl2 :  ('a, unit, unit) format -> 'a
+val lprintf_nl2 : string -> ('a, unit, unit) format -> 'a
 val lprint_newline : unit -> unit  
 val lprint_char : char -> unit  
 val lprint_string : string -> unit  
 val lprint_int : int -> unit  
-
-(*
-val lprintf_to_stdout : bool ref  
-val lprintf_fifo : string Fifo.t
-val lprintf_max_size : int ref
-val lprintf_output : out_channel option ref
-val set_lprintf_handler : (string -> unit) -> unit
-*)
-
 val lprintf_max_size : int ref
   
 val detach : unit -> unit
 val log_to_file : out_channel -> unit
 val log_to_buffer : Buffer.t -> unit
-val set_logging : bool -> unit
 val close_log : unit -> unit
 
 val html_mods_big_header_start : Buffer.t -> string -> string list -> unit
