@@ -70,7 +70,8 @@ let split s c =
     try
       if pos = len then [""] else
       let pos2 = String.index_from s pos c in
-      (String.sub s pos (pos2-pos)) :: (iter (pos2+1))
+      if pos2 = pos then "" :: iter (pos+1) else
+        (String.sub s pos (pos2-pos)) :: (iter (pos2+1))
     with _ -> [String.sub s pos (len-pos)]
   in
   iter 0
@@ -255,23 +256,21 @@ let tokens s =
 
 external contains : string -> string -> bool = "ml_strstr"  
   
+let rec strneql s1 s2 len =
+  len = 0 || (
+    let len = len - 1 in
+    s1.[len] = s2.[len] && strneql s1 s2 len)
   
 let starts_with s1 s2 =
   let len1 = String.length s1 in
   let len2 = String.length s2 in
-  len2 <= len1 && String.sub s1 0 len2 = s2
+  len2 <= len1 && strneql s1 s2 len2
 
 let replace_char s c1 c2 =
   for i = 0 to String.length s - 1 do
     if s.[i] == c1 then s.[i] <- c2
   done
 
-let shorten max s =
-  let len = String.length s in
-  if len > max then 
-    String.sub s 0 max
-  else s
-
 let stem s =
   let s = String.lowercase (String.copy s) in
   for i = 0 to String.length s - 1 do
@@ -297,4 +296,29 @@ let iteri f s =
   for i = 0 to len-1 do
     f i s.[i]
   done
-  
\ No newline at end of file
+  
+let init n f =
+  let s = String.create n in
+  for i = 0 to n - 1 do
+    s.[i] <- f i 
+  done;
+  s
+
+let exists p s =
+  let l = String.length s in
+  let rec aux i =
+    i < l && (p s.[i] || aux (i+1)) in
+  aux 0
+
+let existsi p s =
+  let l = String.length s in
+  let rec aux i =
+    i < l && (p i s.[i] || aux (i+1)) in
+  aux 0
+
+let for_all p s =
+  let l = String.length s in
+  let rec aux i =
+    i >= l || p s.[i] && aux (i+1) in
+  aux 0
+
