@@ -38,7 +38,8 @@
   #endif
   #ifdef HAVE_LIBCHARSET_H
     #include <libcharset.h>
-  #else
+  #endif
+  #ifdef HAVE_LANGINFO_H
     #include <langinfo.h>
   #endif
   #include <iconv.h>
@@ -1198,7 +1199,7 @@ ml_win32_get_default_language (void)
 
 size_t 
 ml_iconv (iconv_t cd,
-          char    **inbuf,
+          const char    **inbuf,
           size_t  *inbytes_left,
           char    **outbuf,
           size_t  *outbytes_left)
@@ -1206,6 +1207,10 @@ ml_iconv (iconv_t cd,
     return iconv (cd, inbuf, inbytes_left, outbuf, outbytes_left);
 }
 
+#ifndef EILSEQ
+#define EILSEQ ENOENT
+#endif
+
 char*
 ml_convert_with_iconv (const char *str,
                        size_t     len,
@@ -1234,7 +1239,7 @@ ml_convert_with_iconv (const char *str,
 
  again:
   
-  err = ml_iconv (cd, (char **)&p, &inbytes_remaining, &outp, &outbytes_remaining);
+  err = ml_iconv (cd, (const char **)&p, &inbytes_remaining, &outp, &outbytes_remaining);
 
   if (err == (size_t) -1)
     {
