@@ -1,13 +1,20 @@
-#if !defined(_OCAMLFD_H)
-#define _OCAMLFD_H
 
-#if defined(__MINGW32__)
+#include "../../../config/config.h"
 
-#if !defined(PTW32_STATIC_LIB)
+/*******************************************************************
+
+
+                         MINGW specific
+
+
+*******************************************************************/
+
+#ifdef __MINGW32__
+
+#ifndef PTW32_STATIC_LIB
 #define PTW32_STATIC_LIB
 #endif
 
-
 #define FD_SETSIZE 1024
 
 #include <winsock.h>
@@ -28,55 +35,75 @@ typedef SOCKET OS_SOCKET;
 typedef unsigned int uint;
 extern void win32_maperr(unsigned long errcode);
 
+/*******************************************************************
 
 
-#else
+                         UNIX specific
 
 
+*******************************************************************/
+
+#else  /* __MINGW32__ */
+
 #define Fd_val(v) Int_val(v)
 #define Socket_val(v) Int_val(v)
 
 typedef int OS_FD;
 typedef int OS_SOCKET;
 
+#if defined(__OpenBSD__) || defined(__FreeBSD__)
+#include <string.h>
 #endif
 
-#include "caml/mlvalues.h"
-#include "caml/fail.h"
-#include "caml/alloc.h"
-#include "caml/memory.h"
+
+#endif  /* __MINGW32__ */
+
+/*******************************************************************
 
 
-#include <errno.h>
-#include <stdio.h>
+                         COMMON used 
+
+
+*******************************************************************/
+
+#include <caml/config.h>
+#include <caml/signals.h>
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 #include <caml/memory.h>
+#include <caml/fail.h>
+#include <caml/custom.h>
+#include <caml/callback.h>
+
+#include <errno.h>
+#include <stdio.h>
 #include <signal.h>
+#include <fcntl.h>
+
+#include <sys/types.h>
+#include <sys/time.h>
+#include <sys/stat.h>
 
 #ifdef HAS_SIGNALS_H
 #include <signals.h>
 #endif
 
-#include <sys/types.h>
+#ifdef HAS_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 
-// off_t is long on mingw, long long everywhere else
+
+/* off_t is long on mingw, long long everywhere else */
 #ifdef _OFF64_T_
 typedef off64_t OFF_T;
 #else
 typedef off_t OFF_T;
 #endif
 
-#include <sys/time.h>
-
-#ifdef HAS_SYS_SELECT_H
-#include <sys/select.h>
+#if !defined(__MINGW32__) && !defined(__BEOS__)
+#include <sys/mman.h>
 #endif
 
-#include <stdio.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
 #ifdef HAS_UNISTD
 #include <unistd.h>
 #endif
@@ -93,13 +120,6 @@ typedef off_t OFF_T;
 #define SEEK_END 2
 #endif
 
-#if defined(__OpenBSD__) || defined(__FreeBSD__)
-#include <string.h>
-#endif
-
-#if !defined(__MINGW32__) && !defined(__BEOS__)
-#include <sys/mman.h>
-#endif
 
 
 #define Nothing ((value) 0)
@@ -107,10 +127,6 @@ typedef off_t OFF_T;
 extern void unix_error (int errcode, char * cmdname, value arg) Noreturn;
 extern void uerror (char * cmdname, value arg) Noreturn;
 
-static int seek_command_table[] = {
-  SEEK_SET, SEEK_CUR, SEEK_END
-};
-
 extern OFF_T os_lseek(OS_FD fd, OFF_T pos, int dir);
 extern void os_ftruncate(OS_FD fd, OFF_T len, int sparse);
 extern ssize_t os_read(OS_FD fd, char *buf, size_t len);
@@ -119,10 +135,10 @@ extern int64 os_getfdsize(OS_FD fd);
 extern int64 os_getfilesize(char *path);
 extern void os_set_nonblock(OS_SOCKET fd);
 extern void os_uname(char buf[]);
+extern int os_os_supported();
 
 
 #define HASH_BUFFER_LEN 131072
 extern unsigned char hash_buffer[HASH_BUFFER_LEN];
 
 
-#endif
