@@ -119,7 +119,8 @@ let nb_sockets = ref 0
 let allow_read = ref true
 let allow_write = ref true
 
-let infinite_timeout = 3600. *. 24. *. 365. (* one year ! *)
+(*let infinite_timeout = 3600. *. 24. *. 365. (* one year ! *) *)
+let infinite_timeout = float_of_int Date.year_in_secs
 let current_time = ref (Unix.gettimeofday ())
 let last_time = ref (int_of_float (!current_time -. 1000000000.))
 
@@ -261,7 +262,7 @@ let sprint_socket s =
 let close t msg =
   if t.fd <> dummy_fd then begin
       if !debug then
-          lprintf_nl "[bS] CLOSING: %s" (sprint_socket t);
+          lprintf_nl "[bS] CLOSING: %s (%s)" (sprint_socket t) (string_of_reason msg);
       (try
           Unix.close t.fd;
           with _ -> ());
@@ -285,7 +286,7 @@ let dump_basic_socket buf = ()
 
 let create_blocking name fd handler =
 
-  let (fdnum : int) = get_fd_num fd in
+  let (_ : int) = get_fd_num fd in
 (*
   if fdnum >= Unix32.fds_size then begin
       Unix.close fd;
@@ -512,10 +513,17 @@ let loop () =
         lprintf_nl "[BW1] Resetting bandwidth counters";
       List.iter (fun f -> try f () with _ -> ()) !bandwidth_second_timers
   );
+  let loop_time = ref 0. in
   while true do
     try
-      if !loop_delay > 0. then (try select [] !loop_delay;  with _ -> ());
-      let time = update_time () in
+      let time = 
+ 	let new_time = update_time () in
+ 	let dynamic_loop_delay =  !loop_time +. !loop_delay -. new_time in
+        if dynamic_loop_delay > 0. then begin
+ 	  (try select [] dynamic_loop_delay;  with _ -> ());
+ 	  update_time ()
+ 	end else new_time in
+      loop_time := time;
       exec_tasks !fd_tasks;
       exec_hooks !after_select_hooks;
       exec_timers !timers;
@@ -548,7 +556,7 @@ timeout := 5.;
 *)
       exec_hooks !before_select_hooks;
 (*      lprintf "Tasks %d\n" (List.length !fd_tasks); *)
-      select !fd_tasks !timeout;
+      try select !fd_tasks !timeout with _ -> ();
     with
     | e ->
         lprintf_nl "Exception %s in Select.loop" (Printexc2.to_string e);
