@@ -99,7 +99,7 @@ val close_all : unit -> unit
   
 val start_time : int
   
-(* external setsock_iptos_throughput: Unix.file_descr -> int = "setsock_iptos_throughput"*)
+external setsock_iptos_throughput: Unix.file_descr -> int = "ml_setsock_iptos_throughput"
  
 val sock_num : t -> int
 val string_of_date : int -> string
@@ -110,11 +110,14 @@ val use_threads : bool ref
 external has_threads : unit -> bool = "ml_has_pthread"
 val get_rtimeout : t -> float * float
 val int64_time : unit -> int64
+val current_time : unit -> float
   
 val string_of_reason : close_reason -> string
   
 val loop_delay : float ref
 
+val socket_keepalive : bool ref
+
 (*
 0 : not verbose
 1 : summary every second
@@ -123,4 +126,4 @@ val loop_delay : float ref
 *)
 val verbose_bandwidth :  int ref
 val add_bandwidth_second_timer : (unit -> unit) -> unit
-  
\ No newline at end of file
+
