@@ -36,12 +36,14 @@ type request = {
     req_user_agent : string;
     req_accept : string;
     req_proxy : (string * int) option;
-
-    req_url : Url.url;
+    mutable req_url : Url.url;
+    mutable req_save_to_file_time : float;
+(* re-download a saved file only if newer *)
     req_request : http_request;
     req_referer : Url.url option;
     req_retry : int;
     req_max_retry : int;
+    req_save : bool;
   }
 
 type content_handler = 
@@ -49,9 +51,10 @@ type content_handler =
 
 val basic_request : request
   
-val get_page : request -> content_handler -> (unit -> unit) -> unit
+val get_page : request -> content_handler -> (unit -> unit) -> (int -> unit) -> unit
 val wget : request -> (string -> unit) -> unit
 val whead : request -> ( (string * string) list -> unit) -> unit
+val whead2 : request -> ( (string * string) list -> unit) -> (int -> unit) -> unit
 
 val wget_string : request -> (string -> unit) ->
   (int -> int -> unit) -> unit
