@@ -20,12 +20,14 @@
 open Printf2
 open BasicSocket
 open TcpBufferedSocket
+open Ip_set
 
 let verbose = ref false
 
-let lprintf_http_nl () =
-  lprintf "%s[HTTPsv] "
-  (log_time ()); lprintf_nl2
+let log_prefix = "[HTTPsv]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
 
 let html_escaped s =
   String2.convert false (fun b escaped c ->
@@ -90,6 +92,9 @@ type auth =
 | Read_auth
 | Write_auth
 
+type error_reason =
+| Blocked
+| Not_allowed
 
 type header =
   Unknown of string * string
@@ -97,9 +102,6 @@ type header =
 | Authorization of auth
 
 
-exception ProcessForked
-exception ThreadForked
-
 type options = {
     referer : Url.url option;
     content_length : int;
@@ -145,7 +147,8 @@ and config = {
     bind_addr : Unix.inet_addr;
     mutable port : int;
     requests : (string * handler) list;
-    mutable addrs : Ip.t list;
+    mutable addrs : Ip_set.blocking_list;
+    use_ip_block_list : bool;
     base_ref : string;
     default : handler;
   }
@@ -224,6 +227,31 @@ let split_head s =
   in
   iter 0 []
 
+let error_page code from_ip from_port my_ip my_port reason =
+  let error_text, error_text_long =
+    match code with
+    | "401" -> "Unauthorized", "<p>Please login using a valid username and password</p>"
+    | "403" -> "Forbidden", 
+		(match reason with
+		   Some Not_allowed -> Printf.sprintf
+"<p>Connection from %s rejected (see downloads.ini, <a href=\"http://mldonkey.sourceforge.net/Allowed_ips\">allowed_ips</a>)</p>\n"
+				    from_ip
+		 | Some Blocked -> Printf.sprintf "IP %s is blocked, its part of the used IP blocklist " from_ip
+		 | None -> "")
+    | _ -> Printf.sprintf "Unknown %s" code, ""
+  in
+  let reject_message = Printf.sprintf
+"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n<html>
+<head><title>%s %s</title></head>\n<h1>%s %s</h1>\n%s
+<hr><address>MLDonkey/%s at %s Port %s</address></html>"
+    code error_text code error_text error_text_long
+    Autoconf.current_version my_ip my_port
+  in
+  Printf.sprintf
+"HTTP/1.1 %s %s\nMLDonkey/%s\nConnection: close
+Content-Type: text/html; charset=iso-8859-1\nContent-length: %d\r\n"
+    code error_text Autoconf.current_version (String.length reject_message), reject_message
+
 let parse_head sock s =
   let h = split_head s in
 (*  List.iter (fun s -> lprintf_http_nl () "LINE: [%s]" (escaped s)) h; *)
@@ -264,7 +292,7 @@ let parse_head sock s =
               | _ -> options
             with e ->
                 if !debug then
-                    lprintf_http_nl () "Exception %s in header %s"
+                    lprintf_nl "Exception %s in header %s"
                       (Printexc2.to_string e) name;
                 options
 
@@ -789,7 +817,7 @@ let request_handler config sock nread =
   try
     iter new_pos
   with e ->
-      if !verbose then lprintf_http_nl () "Exception %s in request_handler"
+      if !verbose then lprintf_nl "Exception %s in request_handler"
         (Printexc2.to_string e);
       close sock (Closed_for_exception e)
 
@@ -797,20 +825,26 @@ let request_closer sock msg =
   ()
 
 
-open Ip_set
-
 let handler config t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET(from_ip, from_port)) ->
     (* check here if ip is OK *)
       let from_ip = Ip.of_inet_addr from_ip in
-      if Ip.matches from_ip config.addrs &&
-        (match Ip_set.match_ip !Ip_set.bl from_ip with
-           None -> true
-         | Some br ->
-             lprintf_http_nl () "%s:%d blocked: %s\n"
-               (Ip.to_string from_ip) from_port br.blocking_description;
-             false) then
+      let ip_is_allowed from_ip =
+	match Ip_set.match_ip config.addrs from_ip with
+	| Some br -> true
+	| None -> false
+      in
+      let ip_is_blocked from_ip =
+	if config.use_ip_block_list then
+	  match !Ip.banned from_ip with
+            None -> false
+           | Some reason -> lprintf_nl "%s:%d blocked: %s"
+               (Ip.to_string from_ip) from_port reason; true
+        else
+	  false
+      in 
+      if ip_is_allowed from_ip && not (ip_is_blocked from_ip) then
         let token = create_token unlimited_connection_manager in
         let sock = TcpBufferedSocket.create_simple
             token "http connection" s in
@@ -819,10 +853,22 @@ let handler config t event =
         TcpBufferedSocket.set_closer sock request_closer;
         TcpBufferedSocket.set_handler sock TcpBufferedSocket.BUFFER_OVERFLOW
           (fun _ -> TcpBufferedSocket.close sock Closed_for_overflow;
-	       lprintf_http_nl () "BUFFER OVERFLOW" );  ()
+         lprintf_nl "BUFFER OVERFLOW" );  ()
       else begin
-         lprintf_http_nl () "connection from %s rejected (see allowed_ips setting)"
-          (Ip.to_string from_ip);
+        lprintf_nl "connection from %s rejected (%s)"
+          (Ip.to_string from_ip)
+	  (if ip_is_blocked from_ip then "IP is blocked" else "see allowed_ips setting");
+        let token = create_token unlimited_connection_manager in
+        let sock = TcpBufferedSocket.create_simple token "http connection" s in
+	let s1,s2 = error_page "403"
+	    (Ip.to_string from_ip)
+	    (string_of_int from_port)
+	    (Ip.to_string (TcpBufferedSocket.my_ip sock))
+	    (string_of_int config.port)
+	    (if not (ip_is_allowed from_ip) then Some Not_allowed else Some Blocked)
+	in
+	TcpBufferedSocket.write_string sock (Printf.sprintf "%s\n%s" s1 s2);
+        shutdown sock Closed_connect_failed;
         Unix.close s
       end
   | _ -> ()
@@ -879,7 +925,7 @@ let parse_range range =
     x, Some y, Some z
   with
   | e ->
-      lprintf_http_nl () "Exception %s for range [%s]"
+      lprintf_nl "Exception %s for range [%s]"
         (Printexc2.to_string e) range;
       raise e
 
@@ -897,7 +943,7 @@ open Int64ops
 (*  Range: bytes=31371876- *)
 let request_range r =
   if !verbose then List.iter (fun (h, v1) ->
-      lprintf_http_nl () "HEADER [%s] = [%s]" h v1
+      lprintf_nl "HEADER [%s] = [%s]" h v1
   ) r.headers;
   let range = List.assoc "Range" r.headers in
   match parse_range range with
