@@ -17,12 +17,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 type auth = No_auth | Read_auth | Write_auth
+type error_reason = Blocked | Not_allowed
 and header =
     Unknown of string * string
   | Referer of Url.url
   | Authorization of auth
-exception ProcessForked
-exception ThreadForked
 type options = {
     referer : Url.url option;
     content_length : int;
@@ -61,7 +60,8 @@ and config = {
     bind_addr : Unix.inet_addr;
     mutable port : int;
     requests : (string * handler) list;
-    mutable addrs : Ip.t list;
+    mutable addrs : Ip_set.blocking_list;
+    use_ip_block_list : bool;
     base_ref : string;
     default : handler;
   } 
@@ -78,4 +78,5 @@ val parse_head : TcpBufferedSocket.t -> string -> request
 val verbose : bool ref 
   
 val request_range : request -> int64 * (int64 option)
-val parse_range : string -> int64 * int64 option * int64 option
\ No newline at end of file
+val parse_range : string -> int64 * int64 option * int64 option
+val error_page : string -> string -> string -> string -> string -> error_reason option -> string * string
