@@ -24,7 +24,7 @@ open BasicSocket
 
 let latencies = Hashtbl.create 2131
 
-let max_opened_connections = ref (fun () -> maxi 20 (MlUnix.max_sockets - 50))
+let max_opened_connections = ref (fun () -> 20)
 let max_connections_per_second = ref (fun () -> 50)
 
 let opened_connections = ref 0
@@ -179,7 +179,6 @@ let can_open_connection manager =
 
 let schedule_connections () =
   let max_wanted = !max_opened_connections () in
-  let current_connections = !opened_connections in
   let max_connections_per_second = !max_connections_per_second () in
 
   let rec iter todo_managers done_managers =
@@ -622,8 +621,7 @@ let can_read_handler t sock max_len =
             t.event_handler t BUFFER_OVERFLOW;
             lprintf "[OVERFLOW] in %s" (info sock);
             close t Closed_for_overflow;
-            raise exn_exit;
-            0
+            raise exn_exit
           )
         else
         if b.len + min_read_size < buf_len then
@@ -1023,7 +1021,6 @@ let close_after_write t =
 (*      lprintf "close_after_write: CLOSE\n";  - log output removed  *)
         shutdown t Closed_by_user)
 
-exception Http_proxy_error of string
 let http_proxy = ref None
 
 let set_reader t f =
@@ -1340,8 +1337,7 @@ let connect token name host port handler =
 (*Printf.bprintf buf "User-Agent: Mozilla/4.0 (compatible; MSIE 5.01; Windows NT; Hotbar 2.0)\n";*)
         Printf.bprintf buf "User-Agent: MLdonkey/%s\n" Autoconf.current_version;
         Printf.bprintf buf "\n";
-        let nw = MlUnix.write s (Buffer.contents buf) 0 (Buffer.length buf) in
-        ()
+        ignore (MlUnix.write s (Buffer.contents buf) 0 (Buffer.length buf))
       end;
     let t = create token name s handler in
 
@@ -1377,17 +1373,20 @@ let connect token name host port handler =
         forecast_upload t 0;    (* The TCP ACK packet *)
         t
     | Unix.Unix_error (Unix.ENETUNREACH,_,_) as e ->
+        (* log nothing here, but later in donkeyClient.ml *)
 	close t Closed_connect_failed;
         raise e
     | e ->
-        lprintf "For host %s:%d   "
-            (Unix.string_of_inet_addr host) port;
         close t Closed_connect_failed;
         raise e
   with
-    Unix.Unix_error (Unix.ENETUNREACH,_,_) as e -> raise e
+    Unix.Unix_error (Unix.ENETUNREACH,_,_) as e -> raise e (* avoid logging *)
+  | Unix.Unix_error (Unix.ENOBUFS,_,_) as e ->
+      if Autoconf.windows then lprintf_nl
+        "No more free buffers, read http://support.microsoft.com/kb/q196271/ to fix this problem";
+      raise e
   | e ->
-      lprintf "EXCEPTION %s  before connect to host %s:%d\n"
+      lprintf_nl "Exception (%s) before connect to host %s:%d"
           (Printexc2.to_string e) (Unix.string_of_inet_addr host) port;
       raise e
 
@@ -1807,27 +1806,20 @@ let join_stats titles values =
 
 let load_stats filename =
   try
-    let ic = open_in filename in
-    let rec iter_first ic =
-      let titles = input_line ic in
-      iter_second ic titles
-    and iter_second ic titles =
-      let values = input_line ic in
-      join_stats titles values;
-      iter_first ic
-
-    in
-    try
-      iter_first ic
-    with
-      End_of_file -> close_in ic
-    | e ->
-        lprintf "[BWS] Error %s reading %s\n" (Printexc2.to_string e) filename;
-        close_in ic; proc_net_fs := false
-  with
-  | e ->
-      lprintf "[BWS] Error %s opening %s\n" (Printexc2.to_string e) filename;
-      proc_net_fs := false
+    Unix2.tryopen_read filename (fun ic ->
+      try
+	let rec iter_first ic =
+	  let titles = input_line ic in
+	  iter_second ic titles
+	and iter_second ic titles =
+	  let values = input_line ic in
+	  join_stats titles values;
+	  iter_first ic in
+	iter_first ic
+      with End_of_file -> ())
+  with e ->
+    lprintf "[BWS] Error %s opening %s\n" (Printexc2.to_string e) filename;
+    proc_net_fs := false
 
 let proc_net_timer _ =
   if !proc_net_fs && !verbose_bandwidth > 0 then begin
@@ -1845,6 +1837,8 @@ let _ =
       Printf.bprintf buf "  read_bandwidth_controlers: %d\n" (List.length !read_bandwidth_controlers);
       Printf.bprintf buf "  write_bandwidth_controlers: %d\n" (List.length !write_bandwidth_controlers);
       Printf.bprintf buf "  to_deflate: %d\n" (List.length !to_deflate);
+      Printf.bprintf buf "  max_opened_connections: %d\n" (!max_opened_connections ());
+      Printf.bprintf buf "  max_connections_per_second: %d\n" (!max_connections_per_second ());
   );
   add_infinite_timer 1.0 proc_net_timer
 
