@@ -39,7 +39,7 @@ type buf = {
 type t = {
     mutable sock : BasicSocket.t;
     mutable rbuf : buf;
-    mutable wfifo : string Fifo2.t;
+    mutable wfifo : string Fifo.t;
     mutable wlen : int;
     mutable wfbuf : buf;
     mutable event_handler : handler;
@@ -108,8 +108,8 @@ end;
       t.wfbuf.buf <- "";
       close t.sock (Printf.sprintf "%s after %d/%d" s t.nread t.nwrite)
     with e ->
-        lprintf "Exception %s in TcpBufferedSocket.close" 
-          (Printexc2.to_string e); lprint_newline ();
+        lprintf "Exception %s in TcpBufferedSocket.close\n"
+          (Printexc2.to_string e);
         raise e
   end
   
@@ -120,11 +120,9 @@ let shutdown t s =
 end;
   *)
   (try BasicSocket.shutdown t.sock s with e -> 
-       lprintf "exception %s in shutdown" (Printexc2.to_string e);
-        lprint_newline () );
+       lprintf "exception %s in shutdown\n" (Printexc2.to_string e));
   (try close t s with  e -> 
-        lprintf "exception %s in shutdown" (Printexc2.to_string e);
-        lprint_newline ())
+        lprintf "exception %s in shutdown\n" (Printexc2.to_string e))
 
 let buf_create max = 
   {
@@ -212,15 +210,14 @@ let buf_add t b s pos1 len =
       end
     else
     if b.len + len > b.max_buf_size then begin
-        lprintf "BUFFER OVERFLOW %d+%d> %d" b.len len b.max_buf_size ; 
-        lprint_newline ();
+        lprintf "BUFFER OVERFLOW %d+%d> %d\n" b.len len b.max_buf_size ; 
         
         lprintf "MESSAGE [";
         for i = pos1 to pos1 + (mini len 20) - 1 do
           lprintf "(%d)" (int_of_char s.[i]);
         done;
         if len > 20 then lprintf "...";
-        lprintf "]"; lprint_newline ();
+        lprintf "]\n";
         
         t.event_handler t BUFFER_OVERFLOW;
       end
@@ -267,12 +264,11 @@ let write t s pos1 len =
 (*          if t.monitored then begin
               lprintf "write: direct written %d" nw; lprint_newline (); 
 end; *)
-          tcp_uploaded_bytes := Int64.add !tcp_uploaded_bytes (Int64.of_int nw);
+          tcp_uploaded_bytes := !tcp_uploaded_bytes ++ (Int64.of_int nw);
           (match t.write_control with
               None -> ()
             | Some bc ->
-                bc.moved_bytes <-
-                Int64.add bc.moved_bytes (Int64.of_int nw));
+                bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nw));
           t.nwrite <- t.nwrite + nw;
           if nw = 0 then (close t "closed on write"; pos2) else
             pos1 + nw
@@ -352,11 +348,10 @@ let can_read_handler t sock max_len =
           raise e
     
     in
-    tcp_downloaded_bytes := Int64.add !tcp_downloaded_bytes (Int64.of_int nread);
+    tcp_downloaded_bytes := !tcp_downloaded_bytes ++ (Int64.of_int nread);
     (match t.read_control with
         None -> () | Some bc ->
-          bc.moved_bytes <-
-          Int64.add bc.moved_bytes (Int64.of_int nread));        
+          bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nread));        
     
     t.nread <- t.nread + nread;
     if nread = 0 then begin
@@ -394,12 +389,11 @@ let can_write_handler t sock max_len =
 
 (*            if t.monitored then
 (lprintf "written %d" nw; lprint_newline ()); *)
-        tcp_uploaded_bytes := Int64.add !tcp_uploaded_bytes (Int64.of_int nw);
+        tcp_uploaded_bytes := !tcp_uploaded_bytes ++ (Int64.of_int nw);
         (match t.write_control with
             None -> ()
           | Some bc ->
-              bc.moved_bytes <-
-              Int64.add bc.moved_bytes (Int64.of_int nw));        
+              bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nw));        
         t.nwrite <- t.nwrite + nw;
         b.len <- b.len - nw;
         b.pos <- b.pos + nw;
@@ -475,6 +469,11 @@ let tcp_handler t sock event =
 let read_bandwidth_controlers = ref []
 let write_bandwidth_controlers = ref []
   
+let _ =
+  Heap.add_memstat "tcpClientSocket" (fun level buf ->
+      Printf.bprintf buf "  read_bandwidth_controlers: %d\n" (List.length !read_bandwidth_controlers);
+      Printf.bprintf buf "  write_bandwidth_controlers: %d\n" (List.length !write_bandwidth_controlers);
+  )
       
 let create_read_bandwidth_controler rate = 
   let bc = {
@@ -528,7 +527,7 @@ let set_write_controler t bc =
   set_allow_write t.sock bc.allow_io;
   bandwidth_controler t t.sock
   
-let max_buffer_size = ref 100000 
+let max_buffer_size = ref 1000000
 
 let dump_socket t buf = 
   print_socket buf t.sock;
@@ -536,7 +535,7 @@ let dump_socket t buf =
   
 let create name fd handler =
   if !debug then begin
-      lprintf "[fd %d %s]" (Obj.magic fd) name; lprint_newline ();
+      lprintf_nl "[fd %d %s]\n" (Obj.magic fd) name;
     end;
   MlUnix.set_close_on_exec fd;
   let t = {
@@ -553,7 +552,7 @@ let create name fd handler =
       write_power = 1;
       read_power = 1;
       wlen = 0;
-      wfifo = Fifo2.create ();
+      wfifo = Fifo.create ();
     } in
   let sock = BasicSocket.create name fd (tcp_handler t) in
   let name = (fun () ->
@@ -583,7 +582,7 @@ let create_blocking name fd handler =
       write_power = 1;
       read_power = 1;
       wlen = 0;
-      wfifo = Fifo2.create ();
+      wfifo = Fifo.create ();
     } in
   let sock = create_blocking name fd (tcp_handler t) in
   t.sock <- sock;
@@ -595,7 +594,7 @@ let create_simple name fd =
   
 let connect name host port handler =
   try
-   lprintf "CONNECT tcpClientSocket"; lprint_newline ();
+(*   lprintf "CONNECT tcpClientSocket\n";  *)
     let s = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
     let t = create name s handler in
     must_write (sock t) true;
@@ -604,14 +603,16 @@ let connect name host port handler =
       t
     with 
       Unix.Unix_error((Unix.EINPROGRESS|Unix.EINTR),_,_) -> t
+    | Unix.Unix_error (Unix.ENETUNREACH,_,_) as e ->
+        close t "connect failed";
+        raise e
     | e -> 
-        lprintf "For host %s port %d" (Unix.string_of_inet_addr host)
-        port; lprint_newline ();
+        lprintf "For host %s port %d\n"
+	  (Unix.string_of_inet_addr host) port;
         close t "connect failed";
         raise e
   with e -> 
-      lprintf "+++ Exception BEFORE CONNECT %s" (Printexc2.to_string e);
-      lprint_newline ();
+      lprintf "+++ Exception BEFORE CONNECT %s\n" (Printexc2.to_string e);
       raise e
       
   
@@ -743,7 +744,7 @@ let internal_buf = Buffer.create 17000
 let simple_send_buf buf sock = ()
   (*
   let s = Buffer.contents buf in
-  Buffer.clear buf;
+  Buffer.reset buf;
   buf_int8 buf 228;
   let len = String.length s in
   buf_int buf len;
@@ -753,7 +754,7 @@ let simple_send_buf buf sock = ()
   
 let value_send sock m = ()
   (*
-  Buffer.clear internal_buf;
+  Buffer.reset internal_buf;
   Buffer.add_string internal_buf (Marshal.to_string m []);
   simple_send_buf internal_buf sock
 *)
